#ifndef NOX11

# include "Fake_Keyboard.h"

void Fake_Keyboard::find(KeySym sym, int* keycode, bool* shift, bool* control) {
  if(mapping == 0)
    return;
  
  for(int i=0; i < (maxkeycode - minkeycode) * keysyms_per; i++)
    if(mapping[i] == sym) {
      *keycode = (i / keysyms_per) + minkeycode;
      int modifiers = i % keysyms_per;
      *shift = (modifiers == 1) ? true : false;
      *control = false;
      return;
    }
  
  // KeySym not found.
  *keycode = 0;
  return;
}

Fake_Keyboard::Fake_Keyboard(Display* d) : mapping(0) {
  set_display(d);
}

Fake_Keyboard::~Fake_Keyboard() {
  if(mapping != 0)
    XFree(mapping);
}

void Fake_Keyboard::set_display(Display* d) {
  display = d;
  
  if(mapping != 0) {
    XFree(mapping);
    mapping = 0;
  }
  
  XDisplayKeycodes(display, &minkeycode, &maxkeycode);
  
  
  //		XChangeKeyboardMapping(display, minkeycode, 2, intlsym, );  
  
  mapping = XGetKeyboardMapping(display,
                                minkeycode, maxkeycode - minkeycode, &keysyms_per);
  
  for(int i = 0; i < maxkeycode - minkeycode; i++) {
    //printf("%d: ", minkeycode + i);
    for(int j = 0; j < keysyms_per; j++) {
      //printf("%d(%c) ", mapping[(i * keysyms_per) + j], (char)mapping[(i * keysyms_per) + j]);
    }
    //printf("\n");
  }
}

void Fake_Keyboard::press(KeySym sym) {
  // Figure keycode and modifiers.
  int keycode;
  bool shift, control;
  find(sym, &keycode, &shift, &control);
  
  // No keycode available for this KeySym, so just return.
  if(keycode == 0)
    return;
  
  // Figure modifier keycodes.
  int shift_keycode = XKeysymToKeycode(display, XK_Shift_L);
  int control_keycode = XKeysymToKeycode(display, XK_Control_L);
  int meta_keycode = XKeysymToKeycode(display, XK_Meta_L);
  
  // Engage modifiers.
  if(shift)
    XTestFakeKeyEvent(display, shift_keycode, 1, 0);
  if(control)
    XTestFakeKeyEvent(display, control_keycode, 1, 0);
  if(meta_key == true)
    XTestFakeKeyEvent(display, meta_keycode, 1, 0);
  
  // Press and release key.
  XTestFakeKeyEvent(display, keycode, 1, 0);
  XTestFakeKeyEvent(display, keycode, 0, 0);
  
  // Disengage modifiers.
  if(shift)
    XTestFakeKeyEvent(display, shift_keycode, 0, 0);
  if(control)
    XTestFakeKeyEvent(display, control_keycode, 0, 0);
  if(meta_key == true) {
    XTestFakeKeyEvent(display, meta_keycode, 0, 0);
  }
  
  release_modifier();
  release_meta();
}

void Fake_Keyboard::press_modifier(KeySym sym) {
  // Figure keycode and modifiers.
  int keycode;
  bool shift, control;
  find(sym, &keycode, &shift, &control);
  
  // No keycode available for this KeySym, so just return.
  if(keycode == 0)
    return;
  
  // Press modifier
  XTestFakeKeyEvent(display, keycode, 1, 0);
  modifier_stack.push_back(keycode);
}

void Fake_Keyboard::release_modifier(KeySym sym) {
  // Figure keycode and modifiers.
  int keycode;
  bool shift, control;
  find(sym, &keycode, &shift, &control);
  
  // No keycode available for this KeySym, so just return.
  if(keycode == 0)
    return;
  
  // Release modifier
  XTestFakeKeyEvent(display, keycode, 0, 0);
}

void Fake_Keyboard::release_modifier() {
  for(std::vector<int>::iterator i = modifier_stack.begin(); i != modifier_stack.end(); i++) {
    // Release modifier
    XTestFakeKeyEvent(display, *i, 0, 0);			
  }
  modifier_stack.clear();
}

void Fake_Keyboard::press_meta() {
  meta_key = true;
}

void Fake_Keyboard::release_meta() {
  meta_key = false;
}

#endif
