// -*- mode: c; mode: fold -*-

#ifndef NOX11


# include "aqw.h"
# include <FL/Fl.H>
# include <stdlib.h>
# include "config.h"
# include <cstring>

# define _DEBUGKBD
# ifdef DEBUGKBD
#  include <iostream>
#  include <stdio.h>
# endif


typedef enum
{
   SHIFT_CHAR = 1,
   NUMBER_CHAR,
   SYMBOL_CHAR,
   UNSHIFT_CHAR,
   BACKSPACE_CHAR = XK_BackSpace,
   SPACE_CHAR = ' '
} special_chars;

typedef enum 
{
   LOWERCASE_MODE = 0,
   UPPERCASE_MODE,
   NUMBER_MODE,
   SYMBOL_MODE
} entry_modes;

typedef enum
{
   MODE_NORMAL = 0,
   MODE_ONCE,
   MODE_LOCK
} entry_mode_state;

static int char_sets[4][32] = 
{ 
# ifdef QUIKWRITING_1_0
    {'a', 's', 'k', NUMBER_CHAR, BACKSPACE_CHAR, SHIFT_CHAR, 'p', 'f', 'n', 'l', 'x', 'u', 't', 'y', 'j', 'r', 'i', 'd', 'b', SYMBOL_CHAR, SPACE_CHAR, UNSHIFT_CHAR, 'z', 'g', 'o', 'w', 'v', 'c', 'e', 'h', 'q', 'm' },
    {'A', 'S', 'K', NUMBER_CHAR, BACKSPACE_CHAR, SHIFT_CHAR, 'P', 'F', 'N', 'L', 'X', 'U', 'T', 'Y', 'J', 'R', 'I', 'D', 'B', SYMBOL_CHAR, SPACE_CHAR, UNSHIFT_CHAR, 'Z', 'G', 'O', 'W', 'V', 'C', 'E', 'H', 'Q', 'M' },
    {'0', '<', '>', NUMBER_CHAR, BACKSPACE_CHAR, SHIFT_CHAR, '(', ')', ',', '[', ']', '+', '=', '-', '9', '8', '.', '%', '$', SYMBOL_CHAR, SPACE_CHAR, UNSHIFT_CHAR, '7', '6', '5', '*', '/', '4', '3', '#', '2', '1' },
    {'/', '\\','|', NUMBER_CHAR, BACKSPACE_CHAR, SHIFT_CHAR, '(', ')', ',', '[', ']', '"', '\'','&', '!', '?', '.', '-', '_', SYMBOL_CHAR, SPACE_CHAR, UNSHIFT_CHAR, '}', '{', '*', '*', '^', ':', ';', '#', ' ', '@' }
# else
    {'a', 's', 'k',                      /*     XXX  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   NUMBER_CHAR, 't', SHIFT_CHAR,        /*     ...  XXX  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'x', 'f', 'n', 'r', 'p',             /*     ...  ...  XXX
                                               .  \     /  X
                                               .   \   /   X
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'u', SPACE_CHAR, 'y',                /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           X
					       .           X
					       .           X
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'j', 'l', 'i', 'd', 'b',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   X
                                               .  /     \  X
                                               ...  ...  XXX     */

   SYMBOL_CHAR, 'e', UNSHIFT_CHAR,      /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  XXX  ...     */ 

   'z', 'g', 'o', 'w', 'v',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               X   /   \   .
                                               X  /     \  .
                                               XXX  ...  ...     */

   'c', BACKSPACE_CHAR, 'h',            /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       X           .
					       X           .
					       X           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'q', 'm'                             /*     ...  ...  ...
                                               X  \     /  .
                                               X   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */
   },

   {'A', 'S', 'K',                      /*     XXX  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   NUMBER_CHAR, 'T', SHIFT_CHAR,        /*     ...  XXX  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'X', 'F', 'N', 'R', 'P',             /*     ...  ...  XXX
                                               .  \     /  X
                                               .   \   /   X
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'U', SPACE_CHAR, 'Y',                /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           X
					       .           X
					       .           X
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'J', 'L', 'I', 'D', 'B',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   X
                                               .  /     \  X
                                               ...  ...  XXX     */

   SYMBOL_CHAR, 'E', UNSHIFT_CHAR,      /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  XXX  ...     */

   'Z', 'G', 'O', 'W', 'V',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               X   /   \   .
                                               X  /     \  .
                                               XXX  ...  ...     */

   'C', BACKSPACE_CHAR, 'H',            /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       X           .
					       X           .
					       X           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   'Q', 'M'                             /*     ...  ...  ...
                                               X  \     /  .
                                               X   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   },

   {'1', '(', '[',                      /*     XXX  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */ 

   NUMBER_CHAR, '2', SHIFT_CHAR,        /*     ...  XXX  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   ']', ')', '3', '/', '*',             /*     ...  ...  XXX
                                               .  \     /  X
                                               .   \   /   X
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   '5', SPACE_CHAR, '7',                /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           X
					       .           X
					       .           X
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   '#', ',', '0', '.', '>',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   X
                                               .  /     \  X
                                               ...  ...  XXX     */

   SYMBOL_CHAR, '9', UNSHIFT_CHAR,      /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  XXX  ...     */

   '<', '=', '8', '-', '+',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               X   /   \   .
                                               X  /     \  .
                                               XXX  ...  ...     */

   '6', BACKSPACE_CHAR, '4',            /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       X           .
					       X           .
					       X           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   '$', '%'                             /*     ...  ...  ...
                                               X  \     /  .
                                               X   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   },

   {'/', '\\','|',                      /*     XXX  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   NUMBER_CHAR, '\'', SHIFT_CHAR,       /*     ...  XXX  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   '(', ')', ',', '[', ']',             /*     ...  ...  XXX
                                               .  \     /  X
                                               .   \   /   X
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   '"', '\'' ,'&',                      /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           X
					       .           X
					       .           X
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   '!', '?', '.', '-', '_',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   X
                                               .  /     \  X
                                               ...  ...  XXX     */

   SYMBOL_CHAR,SPACE_CHAR,UNSHIFT_CHAR, /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  XXX  ...     */

   '}', '{', '*', '*', '^',             /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               X   /   \   .
                                               X  /     \  .
                                               XXX  ...  ...     */

   ':', ';', '#',                       /*     ...  ...  ...
                                               .  \     /  .
                                               .   \   /   .
                                               ---       ---
					       X           .
					       X           .
					       X           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   ' ', '@'                             /*     ...  ...  ...
                                               X  \     /  .
                                               X   \   /   .
                                               ---       ---
					       .           .
					       .           .
					       .           .
                                               ---       ---
                                               .   /   \   .
                                               .  /     \  .
                                               ...  ...  ...     */

   }
# endif
};

static int tap_keys[9] =
{
   0,
   0,
   XK_Alt_L,      // Alt
   XK_Return,     // Ret
   XK_BackSpace,  // Bks
   0,
   XK_Escape,     // Esc
   XK_Tab,        // Tab
   XK_Control_L   // Ctl
};

int zone_data[32][3] =
{
   /* 0  */ { 6, 0, 0 },
   /* 1  */ { 6, 5, 0 },
   /* 2  */ { 6, 5, 4 },
   /* 3  */ { 5, 6, 0 },
   /* 4  */ { 5, 0, 0 },
   /* 5  */ { 5, 4, 0 },
   /* 6  */ { 4, 5, 6 },
   /* 7  */ { 4, 5, 0 },
   /* 8  */ { 4, 0, 0 },
   /* 9  */ { 4, 3, 0 },
   /* 10 */ { 4, 3, 2 },
   /* 11 */ { 3, 4, 0 },
   /* 12 */ { 3, 0, 0 },
   /* 13 */ { 3, 2, 0 },
   /* 14 */ { 2, 3, 4 },
   /* 15 */ { 2, 3, 0 },
   /* 16 */ { 2, 0, 0 },
   /* 17 */ { 2, 1, 0 },
   /* 18 */ { 2, 1, 8 },
   /* 19 */ { 1, 2, 0 },
   /* 20 */ { 1, 0, 0 },
   /* 21 */ { 1, 8, 0 },
   /* 22 */ { 8, 1, 2 },
   /* 23 */ { 8, 1, 0 },
   /* 24 */ { 8, 0, 0 },
   /* 25 */ { 8, 7, 0 },
   /* 26 */ { 8, 7, 6 },
   /* 27 */ { 7, 8, 0 },
   /* 28 */ { 7, 0, 0 },
   /* 29 */ { 7, 6, 0 },
   /* 30 */ { 6, 7, 8 },
   /* 31 */ { 6, 7, 0 }
};


AQW::AQW(Fake_Keyboard* fkbd) /*{{{*/
{ 
  fakeKbd = fkbd;
  
  input_mode_ = AQWMODE_LOWERCASE;
  input_lock_ = MODE_NORMAL;
  meta_key_   = 0;
  ctrl_key_   = 0;
  
  fRedraw  = false;
  uOffsetX = 0;
  uOffsetY = 0;
}
/*}}}*/

void AQW::offset(int X, int Y) /*{{{*/
{
  uOffsetX = X;
  uOffsetY = Y;
}
/*}}}*/

void AQW::hit_key(int key) /*{{{*/
{
# ifdef DEBUGKBD
  std::cout << "hit key " << key << "\n";
# endif
  if(ctrl_key_)
    {
      fakeKbd->press_modifier(XK_Control_L);
    }
  if(meta_key_)
    {
      fakeKbd->press_modifier(XK_Alt_L);
    }
  
  fakeKbd->press(key);
  
  if(ctrl_key_)
    {
      fakeKbd->release_modifier(XK_Control_L);
      ctrl_key_ = 0;
      //      redraw();
# ifndef AQW_SIMPLEINTERFACE
      fRedraw = true;
# endif
    }
  if(meta_key_)
    {
      fakeKbd->release_modifier(XK_Alt_L);
      meta_key_ = 0;
      //      redraw();
# ifndef AQW_SIMPLEINTERFACE
      fRedraw = true;
# endif
    }
}
/*}}}*/


const unsigned int sector2zone[16] =
{
  6, 6, 4, 4,
  8, 8, 2, 2, 
  5, 7, 5, 3,
  1, 7, 1, 3
};

/**
 * Indicate which zone the given coordinate is in.
 */
int AQW::which_zone(int zx, int zy) /*{{{*/
{
  /*
   // the old and expensive code:
  int zone;
  int a;
  int dist;
  int angle;
  int spacing_angle;
  int zero_zone_max=100;
  int dead_zone=15;
  
  // Adjust coords
  zx -= uOffsetX;
  zy -= uOffsetY;
  
  dist = (int)(10*sqrt(zx * zx + zy * zy));
  angle = (int)((atan2(zx, zy)*1000)+6283);
  angle=(angle+392)%6283;
  
  zone=0;
  if(dist > zero_zone_max) {
    zone=int((angle*8)/6283)+1;
    spacing_angle=30;
    if ( (angle % 785 ) > 785 - spacing_angle 
         ||angle % 785 < spacing_angle 
         || dist < zero_zone_max+dead_zone)
      {
        zone = 9;
      }
  }
   */
  
  // my new code, quite fast, no floating-point...
  // Jens Wilhelm Wulf, 29.10.2001
  int zone;
  int dist;
  int xabs, yabs;
  int sector = 0;
  
  // radius of zero_zone:
# define zero_zone_max (AQWSIZE / 6)
  // additional radius of dead_zone:
# define dead_zone     (AQWSIZE / 12)

# define SHIFTBY          16
  // TAN22p5sRev = (1/tan(22,5ÃÂ°)) * 2^(SHIFTBY)
# define TAN22p5sRev    158218  
  
# define ZZM      (zero_zone_max*zero_zone_max)
# define ZZMpDZ   ((zero_zone_max+dead_zone)*(zero_zone_max+dead_zone))
# define MAXDIST  (2*AQWSIZE/3)
  
  
  // Adjust coords
  zx -= uOffsetX;
  zy -= uOffsetY;
  
  xabs = abs(zx);
  yabs = abs(zy);
  
  if (xabs < MAXDIST && yabs < MAXDIST)
    {
  
      dist = zx * zx + zy * zy;
  
      if(dist > ZZM) 
        {
          if (dist < ZZMpDZ)
            zone = 9;
          else
            {
              if (xabs > yabs)
                {
                  sector |= 0x01;
                  
                  if (yabs == 0 || ((xabs<<SHIFTBY)/yabs) > TAN22p5sRev)
                    sector |= 0x08;
                }
              else
                {
                  if (xabs == 0 || ((yabs<<SHIFTBY)/xabs) > TAN22p5sRev)
                    sector |= 0x08;
                }
              
              if (zx > 0)
                sector |= 0x02;
              
              if (zy > 0)
                sector |= 0x04;
              
              zone = sector2zone[sector];              
            }
        }
      else 
        zone = 0;
    }
  else 
    zone = 0;
  
# ifdef DEBUGKBD
  printf("1:  sector:%d   dist:%d  zone: %d  \n",
         sector,dist,zone);
# endif
  
  return zone;
}
/*}}}*/


/**
 * Interpret the stroke entered and reset the buffer.
 */
int AQW::interpret_stroke() /*{{{*/
{
   int ret = -1;
   int n;

   // Look for a match in the set of zones.
   for(n = 0; n < 32; n++)
   {
      if(zone_list_[0] == zone_data[n][0] &&
         zone_list_[1] == zone_data[n][1] &&
         zone_list_[2] == zone_data[n][2])
      {
         ret = n;
         break;
      }
   }

   // Reset the zone list
   zone_ctr_ = 0;
   memset(zone_list_, 0, sizeof(zone_list_[0]) * MAX_ZONES);

   return ret;
}
/*}}}*/


/**
 * Check the input character to see if it should influence our
 * input mode.
 * \return 0 if the char didn't affect mode (so should be output)
 * or nonzero otherwise.
 */
int AQW::check_special_char(int ch) /*{{{*/
{
   int ret = 0;
   
   switch(ch)
   {
      case SHIFT_CHAR:
      {
         change_mode(AQWMODE_UPPERCASE);
         ret = 1;
         break;
      }
      case NUMBER_CHAR:
      {
         change_mode(AQWMODE_NUMBER);
         ret = 1;
         break;
      }
      case SYMBOL_CHAR:
      {
         change_mode(AQWMODE_SYMBOL);
         ret = 1;
         break;
      }
      case UNSHIFT_CHAR:
      {
         change_mode(AQWMODE_LOWERCASE);
         ret = 1;
         break;
      }
   }

   return ret;
}
/*}}}*/


/**
 * Change the input mode states, tracking multiple hits
 * against a mode button to lock/unlock the mode state.
 */
void AQW::change_mode(int mode) /*{{{*/
{
   if(input_mode_ != mode)
   {
      input_mode_ = mode;
      input_lock_ = MODE_ONCE;
   }
   else
   {
      input_lock_++;

      // Rotated through the set of locks, so reset.
      if(input_lock_ > MODE_LOCK)
      {
         input_mode_ = AQWMODE_LOWERCASE;
         input_lock_ = MODE_NORMAL;
      }
   }
}
/*}}}*/


/**
 * Retrieve the current character set for the current input
 * mode.
 */
int* AQW::get_current_set() /*{{{*/
{
   return(char_sets[input_mode_]);
}
/*}}}*/

int AQW::handle(int event) /*{{{*/
{
  int  wanted = 0;
  int* l = get_current_set();
  
  switch(event)
    {
     case FL_PUSH:
        {
          int cx = Fl::event_x();
          int cy = Fl::event_y();
          int new_zone = which_zone(cx, cy);
          
          if (new_zone == 9) 
            {
              break;
            }
          last_zone_ = new_zone;
          zone_ctr_ = 0;
          
          wanted = 1;
          break;
        }
     case FL_RELEASE:
        {
          int cx = Fl::event_x();
          int cy = Fl::event_y();
          
# ifdef DEBUGKBD
          std::cout << "release\n";
# endif
          if(which_zone(cx, cy) == last_zone_ && zone_ctr_ == 0)
            {
              switch(tap_keys[last_zone_])
                {
                 default:
                    {
                      hit_key(tap_keys[last_zone_]);
                      break;
                    }
                 case XK_Alt_L:
                    {
                      meta_key_ = !meta_key_;
                      //                          redraw();
# ifndef AQW_SIMPLEINTERFACE
                      fRedraw = true;
# endif
                      break;
                    }
                 case XK_Control_L:
                    {
                      ctrl_key_ = !ctrl_key_;
                      //                          redraw();
# ifndef AQW_SIMPLEINTERFACE
                      fRedraw = true;
# endif
                      break;
                    }
                }
            }
          
          wanted = 1;
          break;
        }
     case FL_DRAG:
        {
          int cx = Fl::event_x();
          int cy = Fl::event_y();
          int z = which_zone(cx, cy);
          
          if(z == 9) 
            {
              wanted = 1;
              break;
            }
          
# ifdef DEBUGKBD
          std::cout << "drag\n";
# endif
          // Check if we've gone from one zone to another.
          if(z != last_zone_)
            {
# ifdef DEBUGKBD
              std::cout << "stroke ended\n";
# endif
              // Ending a stroke, interpret it.
              if(z == 0)
                {
                  int hit = interpret_stroke();
                  
                  if(hit >= 0)
                    {
                      if(!check_special_char(l[hit]))
                        {
                          hit_key(l[hit]);
                          
                          // Check input modes, see if we need to reset
                          if(input_lock_ == MODE_ONCE)
                            {
                              input_lock_ = MODE_NORMAL;
                              input_mode_ = AQWMODE_LOWERCASE;
                              //                                  redraw();
                              fRedraw = true;
                            }
                        }
                      else
                        {
                          //                              redraw();
                          fRedraw = true;
                        }
                    }
                }
              else
                {
                  if(zone_ctr_ < MAX_ZONES)
                    zone_list_[zone_ctr_++] = z;
                  else
                    {
                      // zone_ctr_ ist gleich MAX_ZONES
                      for (zone_ctr_ = 0; zone_ctr_ < MAX_ZONES-1; zone_ctr_++)
                        zone_list_[zone_ctr_] = zone_list_[zone_ctr_+1];
                      zone_list_[MAX_ZONES-1] = z;
                      zone_ctr_ = MAX_ZONES;
                    }
                }
            }
          
          last_zone_ = z;
          
          wanted = 1;
          break;
        }
    }
  
  return wanted;
}
/*}}}*/

bool AQW::inputLock()    
{ 
  return(input_lock_ == MODE_LOCK); 
}
#endif
