// -*- mode: c; mode: fold -*-
// 
/**
 * Dieses Programm liest eine MuPO-Datei ein (erstes Argument),
 * modifiziert sie und schreibt sie dann wieder (zweites Argument).
 * 
 * Es werden nur die verschlüsselten Elemente gespeichert, alle anderen
 * werden entfernt.
 */

#include "todolistitem.h"
#include "todolistgroup.h"
#include "todolisttodo.h"
#include "tools.h"
#include "config.h"
#include <fstream>
#include <vector>
#include <iostream>

// set args /home/jens/.mupo /home/jens/.mupo_decrypt
// 
#define VERBOSITY       0

struct T_MuPOFile {
  ToDoListGroup*      pRoot;
  unsigned int        uSyncIDMax;
  unsigned int        uSyncIDMaxAfterSync;
  CryptMan            crypt;
};

T_MuPOFile         mupoFile;
T_Calendar         cal;
ToDoListGroup*     grWB;


void printUsage() 
{
  std::cout << "Usage:\n"
    "   crypttest infile outfile\n"
    "infile und outfile dürfen identisch sein.\n";  
}


int readFile(char*        name,
	     T_MuPOFile&  file) 
{
  std::ifstream      ifile;
  std::string        line = "";
  unsigned int  uDummy;
  
  ifile.open(name);
  if (ifile)
  {
    while ((line = readfLine(ifile)) != "g")
    {
      if (line.find("<idmax=") == 0)
      {
        line.replace(0, 7, "");
        line.replace(line.length()-1, 1, "");
        file.uSyncIDMax = atoi(line.c_str());
      }      
      if (line.find("<idmaxaftersync=") == 0)
      {
        line.replace(0, 16, "");
        line.replace(line.length()-1, 1, "");
        file.uSyncIDMaxAfterSync = atoi(line.c_str());
      }      
      if (line.find("<Crypt>") == 0)
      {
        if (file.crypt.Read(ifile))
        {
          std::cerr << "crypt\n";
          return(-1);
        }
      }
    };
    file.pRoot = new ToDoListGroup(0, "/", 0);
    file.pRoot->readContent(ifile, uDummy, file.uSyncIDMax, cal);
    
    ifile.close();
    return(0);
  }
  
  return(-1);
}


int writeFile(char*        name,
              T_MuPOFile&  file) 
{
  std::ofstream ofile;
  
  ofile.open(name);
  if (!ofile)
  {
    std::cerr << "Error writing file: " << name << "\n";
    return(-1);
  }
  ofile << "<data>\n";
  ofile << "<idmax=" << file.uSyncIDMax << ">\n";
  ofile << "<idmaxaftersync=" << file.uSyncIDMaxAfterSync << ">\n";
  file.crypt.Write(ofile);
  file.pRoot->writeContent(ofile, false, &file.crypt);
  ofile << "</data>\n";
  ofile.close();
  
  return(0);
}

int checkGroup(ToDoListGroup* gr)
{
  ToDoListGroup* g;
  ToDoListToDo*  t;
  std::string    summary;
  unsigned int   uDummy;
    
  grWB = gr;

  for (unsigned int n=0; n<gr->getNrOfTodos(); )
  {
    t = gr->getToDoAt(n);
    
    if (t->IsSaveCrypt())
    {
      // entschlüsseln
      const char* ptr;
      t->GetDescription(ptr, &(mupoFile.crypt));
      std::cout << "-------------------------------------------------------------\n";
      std::cout << t->getSummary() << "\n";
      std::cout << ptr << "\n\n";
      //
      n++;
    }
    else
    {
      // entfernen
      gr->delToDo(t, uDummy, cal);
    }
  }
    
  for (unsigned int n=0; n<gr->getNrOfGroups(); )
  {
    g = gr->getGroupAt(n);
    checkGroup(g);
    // Wenn diese Gruppe leer ist, kann man sie entfernen
    if (g->getNrOfTodos() == 0 && g->getNrOfGroups() == 0)
    {
      gr->delGroup(g, uDummy, cal);
    }
    else
      n++;
  }
  
  return (0);
}

int main(int argc, char* argv[]) 
{
  std::ofstream ofile;
  
  if (argc != 3)
  {
    printUsage();
    return(-1);
  }
  
  std::cout << "MuPO-File to read: " << argv[1] << "\n";
  if (readFile(argv[1], mupoFile))
  {
    std::cerr << "Error reading file: " << argv[1] << "\n";
    return(-1);
  }

  if (mupoFile.crypt.GetNumPwds() == 0)
  {
    std::cout << "Keine Schlüsseldaten enthalten.\n";
    return(-1);
  }
  else
  {
    std::string input = "";
    
    std::cout << "Das erste Passwort: ";
  
    char onechar;
    do
    {
      std::cin.get(onechar);
      if (onechar != '\n')
        input.push_back(onechar);
    }
    while (onechar != '\n');
    std::cout << "Eingabe: " << input << "\n";
    
    mupoFile.crypt.Login(0, input.c_str());
    if (mupoFile.crypt.IsLoggedIn() == false)
    {
      std::cerr << "Falsches Passwort\n";
      return(-1);
    }
  }
  
  // Rekursiv durch alles durchhangeln, was es da so gibt.
  checkGroup(mupoFile.pRoot);
  
  writeFile(argv[2], mupoFile);
  
  return(0);
}


