#include "descrdata.h"
#include <string.h>
#include <cstdio>
#include <cstdlib>

#include "tools.h"

#define DDDEBUG 0

DescrData::DescrData()
{
  bytes      = 0;
  nLen       = 0;
  nFlags     = 0;
  orig       = 0;
}

DescrData::~DescrData()
{
  if (bytes)
    free(bytes);
}

void DescrData::ForgetOrig()
{
  if (orig)
  {
    free(orig->bytes);
    delete orig;
    orig = 0;
  }
}

void DescrData::RememberOrig()
{
  if (nLen != 0 && bytes != 0)
  {
    orig = new OriginalData();
    orig->nLen   = nLen;
    orig->nFlags = nFlags;
    orig->bytes  = (char*)malloc(nLen);
    memcpy(orig->bytes, bytes, nLen);
  }
}

DescrData& DescrData::operator = (const DescrData& init)
{
  if (bytes)
    free(bytes);
  nLen       = init.nLen;
  nFlags     = init.nFlags;
  bytes      = (char*)malloc(nLen);
  memcpy(bytes, init.bytes, nLen);
  
  ForgetOrig();
  RememberOrig();
  
  return(*this);
}

void DescrData::CopyFrom(const DescrData* init)
{
  if (bytes)
    free(bytes);
  nLen       = init->nLen;
  nFlags     = init->nFlags;
  bytes      = (char*)malloc(nLen);
  memcpy(bytes, init->bytes, nLen);
  
  ForgetOrig();
  RememberOrig();
}

bool DescrData::operator == (const DescrData& init)
{
  if (nLen != init.nLen || nFlags != init.nFlags)
    return(false);
  
  if (bytes)
  {
    if (memcmp(bytes, init.bytes, abs(nLen)))
      return(false);
  }
  
  return(true);
}

bool DescrData::DescrIsBinary(const int& flags) const
{
  return(
         (
          flags & ((1<<FLAGS_ZIP) | (1<<FLAGS_CRYPT))
          ) 
         != 0);
}

bool DescrData::CurDescrIsBinary() const
{
  return(DescrIsBinary(nFlags));
}

/**
 * nFlags wurde vorher eingelesen
 */
int DescrData::ReadFromFile(int dlen, std::ifstream& ifile)
{
  if (bytes != (char*)0)
    free(bytes);
  
  bytes      = 0;
  nLen       = 0;
  
  int dlenToRead;
  
  if (DescrIsBinary(nFlags))
  {
    int            datalen;
    unsigned char* data    = ReadBinASCII(ifile, datalen, dlen);
    
    if (datalen < (int)dlen || datalen < 0)
    {
      printf("Expected %i byte, got %i\n", dlen, datalen);
      return(-1);
    }
    bytes = (char*)malloc(dlen);
    
    memcpy(bytes, data, dlen);
    
    free(data);
    nLen = dlen;
  }
  else
  {
    bytes = (char*)malloc(dlen+1);
    
    dlenToRead = dlen;
    ifile.read(bytes, dlen);
    if (dlenToRead > dlen)
      return(-1);
    bytes[dlen] = '\0';
    nLen = dlen+1;
  }
          
  // read newline
  ifile.get();

  if ((nFlags & (1<<FLAGS_ZIP)) != 0)
  {
    printf("unable to unzip\n");
    return(-1);    
  }

  // if this is encrypted, I should remember it:
  if ((nFlags & (1<<FLAGS_CRYPT)) != 0)
    RememberOrig();
  
  return(0);
}

void DescrData::SetSaveCrypt(bool fCrypt)
{
  if (fCrypt)
  {
    nFlags |= (1<<FLAGS_SAVE_CRYPT);
  }
  else
  {
    nFlags &= ~((1<<FLAGS_SAVE_CRYPT) | (1<<FLAGS_SAVE_ZIP));
  }
  
  ForgetOrig();  
}

void DescrData::SetNewChar(const char* aString)
{
  bool fSth = (aString != 0);
  int  nNewLen = 0;
  
  if (fSth)
  {
    nNewLen = strlen(aString);
    if (nNewLen == 0)
      fSth = false;
    else
      nNewLen++; // trailing 0x00
  }
  
  if (bytes)
  {
    if (fSth == false || nNewLen != nLen || memcmp(aString, bytes, nLen) != 0)
    {
#if DDDEBUG != 0    
      printf("ForgetOrig %i %i\n", nNewLen, nLen);
      if (nNewLen == nLen)
      {
        int n;
        for (n=0; n<nLen && aString[n] == bytes[n]; n++)
        {
        }      
        n -= 10;
        if (n<0)
          n=0;
        int count=20;
        if (count > nLen)
          count = nLen;
        for (int m=0; m<count; m++)
          printf("%c", bytes[n+m]);
        printf("\n");
        for (int m=0; m<count; m++)
          printf("%c", aString[n+m]);
        printf("\n");
      }
#endif    
      ForgetOrig();
    }
  }
  
  if (bytes)
  {
    free(bytes);
    bytes = 0;
    nLen  = 0;
  }
  
  if (fSth)
  {
    nLen  = strlen(aString) + 1;
    bytes = (char*)malloc(nLen);
    memcpy(bytes, aString, nLen);
  }
}
  
int DescrData::GetDescription(const char*& PtrToDescr,
                              CryptMan*    crypt)
{
  // Does it need to be decrypted?
  if ((nFlags & (1<<FLAGS_CRYPT)) != 0)
  {
    if (crypt->IsLoggedIn())
    {      
      char* tmp = crypt->Decrypt(bytes, nLen, (nFlags & (1<<FLAGS_CRYPT2)) != 0);
      free(bytes);
      bytes = tmp;
      nFlags &= ~(1<<FLAGS_CRYPT);
      nLen = strlen(bytes)+1;
    }
  }

  PtrToDescr = bytes;

  if ((nFlags & (1<<FLAGS_CRYPT)) != 0)
    return(1);
  else
    return(0);
}


int DescrData::GetDataToSave(char*& pDescrToSave, int& nDescrLenLocal, int& nFlagsToSave,
                             bool& fGotNewBuffer,
                             CryptMan* crypt)
{
  fGotNewBuffer  = false;
  const bool fCryptIfReq = true;

  // Soll es entschlüsselt gespeichert werden?
  if ((nFlags & (1<<FLAGS_SAVE_CRYPT)) == 0 || fCryptIfReq == false)
  {
    const char* tmp;
    if (GetDescription(tmp, crypt) != 0)
      return(-1);
  }
  else
  {
    // Wenn ich noch Originaldaten habe kann ich die zurückliefern, damit nicht der gleiche Inhalt
    // mit neuen Zufallsdaten verschlüsselt wird.
    // Flags brauche ich nicht zu prüfen, das passt so alles.
    if (orig)
    {
      pDescrToSave   = orig->bytes;
      nDescrLenLocal = orig->nLen;
      nFlagsToSave   = orig->nFlags;
      return(0);
    }
  }
      
  if (bytes == 0)
  {
    // Wenn etwas leer ist, möchte ich es auch nicht als gezipped oder verschlüsselt markieren.
    // Dann ist aber auch sichtbar, dass es leer ist!
    // Die Wunscheinstellungen jedoch müssen beibehalten werden.
    pDescrToSave   = 0;
    nDescrLenLocal = 0;
    nFlagsToSave   = (nFlags & 0xFFFF0000);
  }
  else
  {
    pDescrToSave   = bytes;
    nDescrLenLocal = nLen;
    nFlagsToSave   = nFlags;    
    
    // Does it need to be encrypted?
    if ((nFlagsToSave & (1<<FLAGS_CRYPT)) == 0 && (nFlagsToSave & (1<<FLAGS_SAVE_CRYPT)) != 0 && fCryptIfReq == true)
    {
      if (crypt->IsLoggedIn())
      {
        pDescrToSave = crypt->Encrypt(bytes, nDescrLenLocal, true);
        fGotNewBuffer = true;
        nFlagsToSave |= (1<<FLAGS_CRYPT) | (1<<FLAGS_CRYPT2);
      }
      else
        return(-1);
    }
  }
  
  return(0);
}
