#ifndef DESCRDATA_H
# define DESCRDATA_H

#include <fstream>
#include "cryptman.h"

/**
 * Data from file has to be kept (at least if it is encrypted) as long as it is unchanged.
 * Data from file is encrypted, it has to be kept until new data is set.
 * As soon as new data is set, original data is deleted.
 */
class DescrData
{
public:
  DescrData();
  
  ~DescrData();
  
  DescrData& operator = (const DescrData& init);
  bool operator == (const DescrData& init);
  bool operator != (const DescrData& todo2) { return(!(*this == todo2)); };

  void CopyFrom(const DescrData* init);
  
  bool DescrIsBinary(const int& flags) const;
  bool CurDescrIsBinary() const;
  
  /**
   * beim String wird die Anzahl der Zeichen inkl. 0x00 zurückgegeben!
   */
  int GetDataToSave(char*& pDescrToSave, int& nDescrLenLocal, int& nFlagsToSave,
                    bool& fGotNewBuffer,
                    CryptMan* crypt);
  
  int ReadFromFile(int dlen, std::ifstream& ifile);
  void SetSaveCrypt(bool fCrypt);

  void SetFlagsFromFile(int flags) { nFlags = flags; };
  
  void SetNewChar(const char* aString);
  

  int GetDescription(const char*& PtrToDescr,
                     CryptMan*    crypt);
  
  bool IsSaveCrypt() const { return(nFlags & (1<<FLAGS_SAVE_CRYPT)); };
   
  
public:
  /**
   * Bitpositions of nFlags.
   */
  enum {FLAGS_ZIP=0, FLAGS_CRYPT=1, FLAGS_CRYPT2=2,
        FLAGS_SAVE_ZIP=16+0, FLAGS_SAVE_CRYPT=16+1};

private:
  void ForgetOrig();
  void RememberOrig();
private:
  char* bytes;
  int   nLen;
  int   nFlags;

  struct OriginalData
  {
    char* bytes;
    int   nLen;
    int   nFlags;
  };
  
  OriginalData* orig;    
};

#endif
