#include "sha1.h"
#include <iostream>

/**
 * Port of sha1.java by Chuck McManis (cmcmanis@netcom.com), 
 * done by Jens Wilhelm Wulf. sha1.java is public domain and so is this...
 */

Hash_SHA1::Hash_SHA1()
{
  Init();
}

unsigned char Hash_SHA1::GetDigestByte(int n)
{
  return(aDigestBits[n]);
}

int Hash_SHA1::rol(unsigned int value, int bits)
{
  return (value << bits) | (value >> (32 - bits));
}

int Hash_SHA1::blk0(int i)
{
  aBlock[i] = (rol(aBlock[i],24)&0xFF00FF00) | (rol(aBlock[i],8)&0x00FF00FF);
  return aBlock[i];
}

int Hash_SHA1::blk(int i) 
{
  aBlock[i&15] = rol(aBlock[(i+13)&15]^aBlock[(i+8)&15]^
                    aBlock[(i+2)&15]^aBlock[i&15], 1);
  return (aBlock[i&15]);
}

void Hash_SHA1::R0(int data[], int v, int w, int x , int y, int z, int i) 
{
  data[z] += ((data[w] & (data[x] ^ data[y] )) ^ data[y]) +
    blk0(i) + 0x5A827999 + rol(data[v] ,5);
  data[w] = rol(data[w], 30);
}

void Hash_SHA1::R1(int data[], int v, int w, int x, int y, int z, int i) 
{
  data[z] += ((data[w] & (data[x] ^ data[y])) ^ data[y]) +
    blk(i) + 0x5A827999 + rol(data[v] ,5);
  data[w] = rol(data[w], 30);
}

void Hash_SHA1::R2(int data[], int v, int w, int x, int y, int z, int i) 
{
  data[z] += (data[w] ^ data[x] ^ data[y]) +
    blk(i) + 0x6ED9EBA1 + rol(data[v] ,5);
  data[w] = rol(data[w], 30);
}

void Hash_SHA1::R3(int data[], int v, int w, int x, int y, int z, int i) 
{
  data[z] += (((data[w] | data[x]) & data[y]) | (data[w] & data[x])) +
    blk(i) + 0x8F1BBCDC + rol(data[v] ,5);
  data[w] = rol(data[w], 30);
}

void Hash_SHA1::R4(int data[], int v, int w, int x, int y, int z, int i) 
{
  data[z] += (data[w] ^ data[x] ^ data[y]) +
    blk(i) + 0xCA62C1D6 + rol(data[v] ,5);
  data[w] = rol(data[w], 30);
}

void Hash_SHA1::Transform()
{
  /*
  for (int n=0; n<5; n++)
    std::cout << aState[n] << " ";
   std::cout << "\n";
   */
  
  /* Copy context->aState[] to working vars */
  aDD[0] = aState[0];
  aDD[1] = aState[1];
  aDD[2] = aState[2];
  aDD[3] = aState[3];
  aDD[4] = aState[4];
  /* 4 rounds of 20 operations each. Loop unrolled. */
  R0(aDD,0,1,2,3,4, 0);
  R0(aDD,4,0,1,2,3, 1);
  R0(aDD,3,4,0,1,2, 2);
  R0(aDD,2,3,4,0,1, 3);
  R0(aDD,1,2,3,4,0, 4);
  R0(aDD,0,1,2,3,4, 5);
  R0(aDD,4,0,1,2,3, 6);
  R0(aDD,3,4,0,1,2, 7);
  R0(aDD,2,3,4,0,1, 8);
  R0(aDD,1,2,3,4,0, 9);
  R0(aDD,0,1,2,3,4,10);
  R0(aDD,4,0,1,2,3,11);
  R0(aDD,3,4,0,1,2,12);
  R0(aDD,2,3,4,0,1,13);
  R0(aDD,1,2,3,4,0,14);
  R0(aDD,0,1,2,3,4,15);
  R1(aDD,4,0,1,2,3,16);
  R1(aDD,3,4,0,1,2,17);
  R1(aDD,2,3,4,0,1,18);
  R1(aDD,1,2,3,4,0,19);
  R2(aDD,0,1,2,3,4,20);
  R2(aDD,4,0,1,2,3,21);
  R2(aDD,3,4,0,1,2,22);
  R2(aDD,2,3,4,0,1,23);
  R2(aDD,1,2,3,4,0,24);
  R2(aDD,0,1,2,3,4,25);
  R2(aDD,4,0,1,2,3,26);
  R2(aDD,3,4,0,1,2,27);
  R2(aDD,2,3,4,0,1,28);
  R2(aDD,1,2,3,4,0,29);
  R2(aDD,0,1,2,3,4,30);
  R2(aDD,4,0,1,2,3,31);
  R2(aDD,3,4,0,1,2,32);
  R2(aDD,2,3,4,0,1,33);
  R2(aDD,1,2,3,4,0,34);
  R2(aDD,0,1,2,3,4,35);
  R2(aDD,4,0,1,2,3,36);
  R2(aDD,3,4,0,1,2,37);
  R2(aDD,2,3,4,0,1,38);
  R2(aDD,1,2,3,4,0,39);
  R3(aDD,0,1,2,3,4,40);
  R3(aDD,4,0,1,2,3,41);
  R3(aDD,3,4,0,1,2,42);
  R3(aDD,2,3,4,0,1,43);
  R3(aDD,1,2,3,4,0,44);
  R3(aDD,0,1,2,3,4,45);
  R3(aDD,4,0,1,2,3,46);
  R3(aDD,3,4,0,1,2,47);
  R3(aDD,2,3,4,0,1,48);
  R3(aDD,1,2,3,4,0,49);
  R3(aDD,0,1,2,3,4,50);
  R3(aDD,4,0,1,2,3,51);
  R3(aDD,3,4,0,1,2,52);
  R3(aDD,2,3,4,0,1,53);
  R3(aDD,1,2,3,4,0,54);
  R3(aDD,0,1,2,3,4,55);
  R3(aDD,4,0,1,2,3,56);
  R3(aDD,3,4,0,1,2,57);
  R3(aDD,2,3,4,0,1,58);
  R3(aDD,1,2,3,4,0,59);
  R4(aDD,0,1,2,3,4,60);
  R4(aDD,4,0,1,2,3,61);
  R4(aDD,3,4,0,1,2,62);
  R4(aDD,2,3,4,0,1,63);
  R4(aDD,1,2,3,4,0,64);
  R4(aDD,0,1,2,3,4,65);
  R4(aDD,4,0,1,2,3,66);
  R4(aDD,3,4,0,1,2,67);
  R4(aDD,2,3,4,0,1,68);
  R4(aDD,1,2,3,4,0,69);
  R4(aDD,0,1,2,3,4,70);
  R4(aDD,4,0,1,2,3,71);
  R4(aDD,3,4,0,1,2,72);
  R4(aDD,2,3,4,0,1,73);
  R4(aDD,1,2,3,4,0,74);
  R4(aDD,0,1,2,3,4,75);
  R4(aDD,4,0,1,2,3,76);
  R4(aDD,3,4,0,1,2,77);
  R4(aDD,2,3,4,0,1,78);
  R4(aDD,1,2,3,4,0,79);
  /* Add the working vars back into context.aState[] */
  aState[0] += aDD[0];
  aState[1] += aDD[1];
  aState[2] += aDD[2];
  aState[3] += aDD[3];
  aState[4] += aDD[4];
  /*
  for (int n=0; n<5; n++)
    std::cout << aState[n] << " ";
   std::cout << "\n";
   */
}

void Hash_SHA1::Init()
{
  fNonASCII    = false;
  lCount       = 0;
  fDigestValid = false;
  nBlockIndex  = 0;
  aState[0] = 0x67452301;
  aState[1] = 0xEFCDAB89;
  aState[2] = 0x98BADCFE;
  aState[3] = 0x10325476;
  aState[4] = 0xC3D2E1F0;
  
  for (int n=0; n<16; n++)
    aBlock[n] = 0;

  for (int n=0; n<HASH_LEN; n++)
    aDigestBits[n] = 0;
  
  for (int n=0; n<5; n++)
    aDD[n] = 0;
}

void Hash_SHA1::Update(const char* data, int len)
{
  for (int n=0; n<len; n++)
  {
    char b = data[n];
    int mask = (8 * (nBlockIndex & 3));
    
    // std::cout << "Update(" << (int)b << ")";
    
    lCount += 8;
    
    aBlock[nBlockIndex >> 2] &= ~(0xff << mask);
    aBlock[nBlockIndex >> 2] |= (b & 0xff) << mask;

    // std::cout << "aBlock[" << (nBlockIndex >> 2) << "] = " << aBlock[nBlockIndex >> 2] << "\n";
        
    if (++nBlockIndex == 64) 
    {
      Transform();
      nBlockIndex = 0;
    }  
  }
}

void Hash_SHA1::Finish()
{
  char bits[8];
  
  for (int i=0; i<8; i++)
  {
    bits[i] = (lCount >> (((7 - i) * 8))) & 0xff;
  }
  
  char tmp = 128;
  Update(&tmp, 1);
  
  tmp = 0;
  while (nBlockIndex != 56)
    Update(&tmp, 1);
  
  // This should cause a transform to happen.
  Update(bits, 8);
  for (int i = 0; i < HASH_LEN; i++) 
  {
    aDigestBits[i] = (char) ((aState[i>>2] >> ((3-(i & 3)) * 8) ) & 0xff);
    // std::cout << (int)(aDigestBits[i]) << " ";
  }
  // std::cout << "\n";
  
  fDigestValid = true;
}

void Hash_SHA1::Update(std::string data)
{
  for (std::string::size_type n=0; n<data.length(); n++)
  {
    char tmp = data[n];
  
    if (tmp < 32 || tmp > 126)
      fNonASCII = true;
  
    Update(&tmp, 1);
  }
}
