#ifndef SHA1_H
# define SHA1_H

#include <string>

/**
 * Port of sha1.java by Chuck McManis (cmcmanis@netcom.com), 
 * done by Jens Wilhelm Wulf. sha1.java is public domain and so is this...
 */
class Hash_SHA1
{
public:
  Hash_SHA1();
  
  void Init();
  
  void Update(std::string data);

  void Update(const char* data, int len);
  
  void Finish();

  unsigned char GetDigestByte(int n);
  
  /**
   * Care about this flag: you'll get different results for
   * non-ASCII-chars input in different charsets (on different
   * platforms, ...).
   */
  bool HadNonASCIIChar() { return(fNonASCII); };

public:
  enum {HASH_LEN=20};

private:
    
  int rol(unsigned int value, int bits);
  int blk0(int i);
  int blk(int i);

  void Transform();
  void R0(int data[], int v, int w, int x, int y, int z, int i);
  void R1(int data[], int v, int w, int x, int y, int z, int i);
  void R2(int data[], int v, int w, int x, int y, int z, int i);
  void R3(int data[], int v, int w, int x, int y, int z, int i);
  void R4(int data[], int v, int w, int x, int y, int z, int i);
  
private:    
  int           aState[5];
  int           aBlock[16];
  char          aDigestBits[HASH_LEN];
  bool          fDigestValid;
  long long int lCount;
  int           nBlockIndex;
  int           aDD[5];
  bool          fNonASCII;
};

#endif
