#ifndef __TODOLISTITEM_H
# define __TODOLISTITEM_H

# include <fstream>
# include <string>
# include <vector>
# include <time.h>
# include "cryptman.h"

# ifdef MUPOSYNC
#  include <iostream>
# endif

# define DIFF_SUMMARY       0x000002
# define DIFF_PRIORITY      0x000004
# define DIFF_SHOWCOMPL     0x000008
# define DIFF_COMPLLAST     0x000010 
# define DIFF_WTIMES        0x000020  // includes DIFF_RUNNING
# define DIFF_COMPLETED     0x000040
# define DIFF_DESCRIPTION   0x000080
# define DIFF_CHILD         0x000100
# define DIFF_NEW           0x000200
# define DIFF_TIMETHINGS    0x000400 // just one flag for begTime, endTime, showAs and recData.
# define DIFF_FLAGS         0x000800
// One could easily get incosistent data using single flags.
// 
// 
class ToDoListItem {
public:
  
  ToDoListItem(unsigned int ID, 
               std::string       pathInit);
  ToDoListItem(const ToDoListItem& init);
  virtual ~ToDoListItem();
  
  virtual int writeContent(std::ostream&  ofile,
                           bool           fExport,
                           CryptMan*      crypt,
                           bool           fIncludeYourself = true,
                           bool           fIncludeContent  = true) = 0;
  virtual void readContent(std::ifstream& ifile);
  unsigned int getID();
  void         setID(unsigned int);
  virtual void dump();
  std::string  getPath();
  void         setPath(std::string);
  
# ifdef MUPOSYNC
  virtual bool checkDiff(ToDoListItem* pOthersRoot);
  void         setMustBeKept();
  virtual void setShouldBeDeleted();
  bool         getMustBeKept();
  bool         getShouldBeDeleted();
  unsigned int getSyncFlag();
# endif
  unsigned int getSyncID();
  void         setSyncID(unsigned int n);
  void         setSummary(std::string n);
  std::string  getSummary();
  void         setPriority(unsigned int n);
  unsigned int getPriority();
  bool         hasBeenSaved();
  void         hasBeenSaved(bool fHBS);
  
protected:
# ifdef MUPOSYNC
  unsigned int   uSyncFlag;
  bool           fShouldBeDeleted;
  bool           fMustBeKept;
# endif
  std::string    path;
  unsigned int   uID;
  unsigned int   uSyncID;
  std::string    summary;         
  int            uPriority;
  bool           fHasBeenSaved;
};
typedef ToDoListItem* pToDoListItem;



#endif
