// -*- mode: c; mode: fold -*-

#include "tools.h"

#include <time.h>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include "config.h"


#define TOOLS_DEBUG 0

#if (TOOLS_DEBUG > 0)
# include <stdio.h>
#endif

std::string readfLine(std::istream& inDat) 
{
  char          Zeile[500];
  std::string        zeile;
  
  // Zeile lesen
  inDat.getline(Zeile, 499);
  zeile = Zeile;
      
  while (zeile.length() && zeile[0] == ' ')
    zeile.replace(0, 1, "");
  while (zeile.length() && zeile[zeile.length()-1] == ' ')
    zeile.replace(zeile.length()-1, 1, "");
      
  return(zeile);
}


std::string itoStr(long int nr, char fill, unsigned char digits, bool prettyneg) 
{
  std::string  tmp = "";
  char    digit;
  bool    neg = false;
  int     Cnt;
  
  
  if (nr < 0) 
    {
      neg = true;
      nr *= -1;
    }

  
  if (nr == 0)
    {
      for (int Cnt = 0; Cnt < digits-1; Cnt++) tmp += fill;
      tmp += '0';
    }
  else 
    {
      Cnt = 0;
      while (Cnt < digits || nr > 0)
        {
          if (nr > 0) digit = (nr % 10)+48;
          else 
	    {
	      if (neg) 
		{
		  if (prettyneg) digit = '-';
		  else 
		    {
		      tmp = '-' + tmp;
		      digit = fill;
		    }
		  neg = false;
		}
	      else digit = fill;
	    }
	    
          tmp = digit + tmp;
          nr /= 10;
	  Cnt++;
        }
    }
  
  return(tmp);
}


std::string strU(std::string in) 
{
  std::string tmp = "";
  
  for (unsigned int uCnt = 0; uCnt < in.length(); uCnt++)
    {
      tmp += toupper(in[uCnt]); 
    } 
  return(tmp); 
}


int Tag_des_Jahres(int tag, int monat, int jahr) 
{
  const int DaysToMonth[12] = {0, 31, 59, 90, 120, 151, 
                               181, 212, 243, 273, 304, 334};
  
  int Tage = DaysToMonth[monat-1] + tag;
  
  if (monat > 2 && (((jahr % 4 == 0) && (jahr % 100 != 0)) || jahr % 400 == 0))
    Tage++;
  
  return Tage;
} 


int CurrentDoomDay(int jahr) 
{
   const int CenturyDoomDay[4] = {5,4,2,0};
   int CC             = (jahr/100);
   int YY             = jahr%100;
   int CCDoomDay      = CenturyDoomDay[CC%4];
   int YYDoomDay      = 0;

   if (YY == 0)
    YYDoomDay = CCDoomDay;
   else
    {
      if (YY%12 == 0)
        YYDoomDay = (CCDoomDay + YY/12 - 1) % 7;
      else YYDoomDay = ( CCDoomDay
                         + ((YY/12)
                            + (YY%12)
                            + (((YY-1)%12)/4))
                         ) % 7;
    }
  if ((CC%4 == 0) && (YY != 0))
    YYDoomDay = YYDoomDay + 1;
  
  return YYDoomDay;
} 


int Wochentag(int tag, int monat, int jahr) 
{    
  return((CurrentDoomDay(jahr)+((Tag_des_Jahres(tag,monat,jahr) - 1)%7))%7);
}


int Kalenderwoche(int tag, int monat, int jahr) 
{
  int Woche      = 0;
  int Wchtag1Jan = CurrentDoomDay(jahr);
  int Tage       = Tag_des_Jahres(tag,monat,jahr)-1;

  if (Wchtag1Jan > 3)
    Tage = Tage - (7 - Wchtag1Jan);
  else 
    Tage = Tage + Wchtag1Jan;

  if (Tage < 0)
    {
      if ((Wchtag1Jan == 4) || (CurrentDoomDay(jahr-1) == 3))
        Woche = 53;
      else
        Woche = 52;
    }
  else 
    Woche = (Tage/7) + 1;

  if ((Tage > 360) && (Woche > 52)) 
    {
      if (Wchtag1Jan == 3)
        Woche = 53;
      else
        {
          if (CurrentDoomDay(jahr+1) == 4)
            Woche = 53;
          else 
            Woche = 1;
        }
    }

  return Woche;
} 


int MaxTag(int monat, int jahr) 
{
  const int DaysOfMonth[12] = {31, 28, 31, 30, 31, 30, 
                               31, 31, 30, 31, 30, 31};
  
  // Wenn es kein Februar ist, den Wert aus der Tabelle nehmen, ansonsten
  // prüfen, ob es ein Schaltjahr ist.
  // Schaltjahrdefinition: siehe Mail in DE/SCI/ELECTRONICS vom 6.11.:
  // ein Jahr ist ein Schaltjahr, wenn es 
  //   durch 4, aber nicht durch 100
  // oder
  //   durch 400
  // ganzzahlig teilbar ist.
 
 if (monat != 2) return DaysOfMonth[monat-1];
 else 
  {
    if (((jahr % 4 == 0) && (jahr % 100 != 0)) || jahr % 400 == 0) return 29;
    else return 28;
  } 
}


int daysOfYear(int y) 
{
    if (((y % 4 == 0) && (y % 100 != 0)) 
	|| y % 400 == 0) 
	return 366;
    else
	return 365;
}


int getNumberOfDay(int tag, int monat, int jahr) 
  // diese Nummer entspricht der Anzahl der Tage seit dem 1.1.1999 
  // naja, nicht strenggenommen, der 1.1.99 ist nämlich Nummer 1
  // nach dieser Zählung..
{
  int            WTagOffset;

  monat--;
  {
    WTagOffset = tag;
    // Anzahl der Tage seit 1.1.1999 ausrechnen
    const int DaysToMonth[12] = {0, 31, 59, 90, 120, 151, 
				 181, 212, 243, 273, 304, 334};
    
    WTagOffset += ((jahr-1999)*365) + DaysToMonth[monat]; 
    
    // Jetzt muß noch für jedes Schaltjahr seit 1999 ein Tag addiert werden.    
    for (int JahrCount = 1999; JahrCount < jahr; JahrCount++) 
      if (((JahrCount % 4 == 0) && (JahrCount % 100 != 0)) 
	  || JahrCount % 400 == 0) WTagOffset++;
    
    if ((monat > 1) && 
	(((jahr % 4 == 0) && (jahr % 100 != 0)) ||
	 (jahr % 400 == 0))) WTagOffset++;    
  } 
  return(WTagOffset);
}


void getDayByNumber(int nr, int& d, int& m, int& y) 
{
  d=1;
  m=1;
  y=1999;
  
  while (nr > daysOfYear(y))
    {
      nr -= daysOfYear(y);
      y++;
    }
  while (nr > MaxTag(m, y))
    {
      nr -= MaxTag(m, y);
      m++;
    }
  d = nr;
}


int getDateDiff(t_datetime dt1, t_datetime dt2) 
{
  return(getNumberOfDay(GETDAY(dt1), GETMONTH(dt1), GETYEAR(dt1)) -
         getNumberOfDay(GETDAY(dt2), GETMONTH(dt2), GETYEAR(dt2)));
}


void moveDate(t_datetime& dt1, int diff) 
{
  int d, m, y, nr;
  
  nr = getNumberOfDay(GETDAY(dt1), GETMONTH(dt1), GETYEAR(dt1)) + diff;
  getDayByNumber(nr, d, m, y);
  SETDATE(dt1, y, m, d);
}



void moveDateTime(t_datetime& dt, int nDiff) 
{
  int d, m, y, nr, h, min;
  
  h   = GETHOUR(dt);
  min = GETMINUTE(dt) + nDiff;
  
  if (min > 0)
    {
      h   += min / 60;
      min %= 60;
      nr  = getNumberOfDay(GETDAY(dt), GETMONTH(dt), GETYEAR(dt)) + (h / 24);
      h  %= 24;
    }
  else
    {
      h   += min / 60;
      min *= -1;
      min %= 60;
      if (min != 0)
        {
          h   -= 1;
          min = 60 - min;
        }
      
      if (h > 0)
        {
          nr = getNumberOfDay(GETDAY(dt), GETMONTH(dt), GETYEAR(dt));
        }
      else
        {
          h *= -1;
          
          nr = getNumberOfDay(GETDAY(dt), GETMONTH(dt), GETYEAR(dt)) - (h/24) - 1;
          
          h %= 24;
          
          h = 24 - h;
        }
      
    }
    
  
  getDayByNumber(nr, d, m, y);
  SETDATE(dt, y, m, d);
  SETTIME(dt, h, min);
}



t_datetime getTodayDate(t_datetime tmp) 
{
  time_t     curTime;
  tm*        curTimeBr;
  
  // get current time
  curTime = time(0);
  curTimeBr = localtime(&curTime);
  SETDATE(tmp, curTimeBr->tm_year+1900, curTimeBr->tm_mon+1, curTimeBr->tm_mday);
  
  return(tmp);
}


void ostern(int& nTag, int& nMonat, int nJahr) 
//  Viele weitere Kalenderdaten berechnen sich aus dem Osterdatum. 
//  So ist Aschermittwoch der 46. Tag vor, Christi Himmelfahrt der 39., 
//  Pfingstsonntag der 49. und Fronleichnam der 60. Tag nach dem Ostersonntag.
//  
//  
//  http://www.ptb.de/deutsch/org/4/43/432/oste.htm
//  
//  In den nachfolgenden Gleichungen bezeichnen  INT(a/b) den ganzzahligen Teil
//  des Quotienten zweier Zahlen a und b und MOD(a, b) den
//  nicht-negativen Rest den a beim Teilen durch b lässt.  Zu berechnen ist im
//   einzelnen für die Jahreszahl X : 
  
  
// 1.    K = INT( X / 100 );
// 2.    M = 15 + INT(( 3·K+3 ) / 4 ) - INT(( 8·K+13 ) / 25 );
// 3.    S = 2 - INT(( 3·K+3 ) / 4 );
// 4.    A = MOD( X, 19 );
// 5.    D = MOD( 19·A+M, 30 );
// 6.    R = INT( D / 29 ) + ( INT( D / 28 ) - INT( D / 29 )) · INT( A / 11 );
// 7.    OG = 21 + D - R; - Märzdatum des Ostervollmonds 
//                (= 14. Tag des ersten Monats im Mondkalender, genannt Nisanu)
// 8.    SZ = 7 - MOD( X + INT( X / 4 ) + S, 7 ); - Datum des 1. Sonntags im März
// 9.    OE = 7 - MOD( OG - SZ, 7 );


// Dann ist OS = OG + OE das Datum des Ostersonntags, als Datum im Monat März dargestellt. 
// Der 32. März entspricht dem 1. April usw. 

// Liegt der Ostertermin (Os) erst einmal fest, so berechnen sich daraus weitere besondere 
// Kalenderdaten, und zwar 

// Os - 46: Aschermittwoch,
// Os + 39: Christi Himmelfahrt,
// Os + 49: Pfingstsonntag,
// Os + 60: Fronleichnam  
{
  int K, M, S, A, D, R, OG, SZ, OE;
  
  K = nJahr / 100;
  M = 15 + (((3*K)+3) / 4) - (((8*K)+13) / 25);
  S = 2 - (((3*K)+3) / 4);
  A = nJahr % 19;
  D = ((19*A)+M) % 30;
  R = ( D / 29 ) + (((D / 28) - (D / 29)) * ( A / 11 ));
  OG = 21 + D - R;
  SZ = 7 - ( (nJahr + (nJahr / 4) + S) % 7 );
  OE = 7 - ( (OG - SZ) % 7 );
  
  nTag = OG + OE;
  nMonat = 3;
  
  if (nTag > 31)
    {
      nTag -= 31;
      nMonat = 4;
    }
}


bool existDat(std::string DateiPfad) 
{
  std::ifstream testDat;
  
  testDat.open((const char*)DateiPfad.c_str());
  
  if (!testDat) return(0); 
  else 
    { 
      testDat.close();
      return(1); 
    }
}

void WriteBinASCII(std::ostream& out, const unsigned char* data, int len)
{
  unsigned char cd[4];
  unsigned char in[3];
  int nBlocks = len/3;
  if (len%3)
    nBlocks++;
  
  unsigned char* datain = (unsigned char*)malloc(nBlocks*3);
  datain[nBlocks*3 -1] = 0;
  datain[nBlocks*3 -2] = 0;
  memcpy(datain, data, len);
  
  for (int n=0; n<nBlocks; n++)
  {
    // 76543210 76543210 76543210 
    // 33333322 22221111 11000000
    in[0] = datain[n*3+0];
    in[1] = datain[n*3+1];
    in[2] = datain[n*3+2];
    
    cd[0] = ( in[0] & 0x3F                    ) + '0';
    cd[1] = ((in[0] >> 6) | ((in[1]&0x0F)<<2) ) + '0';
    cd[2] = ((in[1] >> 4) | ((in[2]&0x03)<<4) ) + '0';
    cd[3] = ((in[2] >> 2)                     ) + '0';
    out.write((char*)cd, 4);
  }
  // printf("\n");
  out.write("!!!!", 4);
  free(datain);
}

unsigned char* ReadBinASCII(std::istream& in, int& len, unsigned int nExpectedNrOfBytes)
{
  if (nExpectedNrOfBytes == 0)
    nExpectedNrOfBytes = 256;
    
  unsigned char  cd[4];
  unsigned char  dd[3];
  int nBlocks = (nExpectedNrOfBytes / 3) + 2;
  unsigned char* inbuf = (unsigned char*)malloc( nBlocks * 3 );
  int nBlock=0;
  const unsigned char ucEOD = ('!' - '0')&0xFF;

#if (TOOLS_DEBUG > 0)
  printf("%i blocks\n", nBlocks);
#endif  
  
  do
  {
    //
    if (nBlock >= nBlocks)
    {
      // Speicher vergrößern
      nBlocks *= 2;
      inbuf = (unsigned char*)realloc(inbuf, nBlocks * 3);
    }    
    // block lesen
    in.read((char*)cd, 4);
    // dekodieren
    for (int n=0; n<4; n++)
      cd[n] -= '0';
    dd[0] =  cd[0]       | (cd[1] << 6);
    dd[1] = (cd[1] >> 2) | (cd[2] << 4);
    dd[2] = (cd[2] >> 4) | (cd[3] << 2);
    // 
    memcpy(&(inbuf[nBlock*3]), dd, 3);
    nBlock++;
  }
  while (in && cd[0] != ucEOD);

  if (!in)
    len = -1;
  else if (cd[0] != ucEOD || cd[1] != ucEOD || cd[2] != ucEOD || cd[3] != ucEOD)
    len = -2;
  else
    len = nBlock*3;
  
  return(inbuf);
}

std::string trim(std::string in)
{
  int nL  = in.length();
  int nCnt;
  
  // Die Anzahl der führenden Zeichen ermitteln
  for (nCnt=0; nCnt < nL && isspace(in[nCnt]); nCnt++)
  {
  }
  
  // Die führenden Zeichen löschen
  in.replace(0, nCnt, "");
  
  nL = in.length();
  if (nL > 0)
  {
    // Die Anzahl der abschliessenden Zeichen ermitteln
    for (nCnt = nL-1; nCnt > 0 && isspace(in[nCnt]); nCnt--)
    {
    }
    // Die abschliessenden Zeichen löschen
    in.replace(nCnt+1, nL-nCnt, "");     
  }
  
  return(in);  
}

std::string GetHomeDir()
{
  std::string tmp = "";
  
#ifdef WIN32
  tmp = getenv("USERPROFILE");
#else
  tmp = getenv("HOME");
#endif
  
  return(tmp);
}

