#include "cryptman.h"
#include <iostream>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include "tools.h"

#ifndef MUPOSYNC
# include "sha1.h"
# include "twofish.h"
#endif


#define CRYPT 1

PwdData::PwdData(std::string descr,
                 const unsigned char* cryptkey, int len_ck,
                 const unsigned char* saltA,    int len_sA,
                 const unsigned char* saltB,    int len_sB)
{
  keydescr = descr;
  
  for (int n=0; n<len_ck; n++)
    this->cryptkey.push_back(cryptkey[n]);
  
  for (int n=0; n<len_sA; n++)
    this->saltA.push_back(saltA[n]);
  
  for (int n=0; n<len_sB; n++)
    this->saltB.push_back(saltB[n]);
}

bool PwdData::Equals(const PwdData& b)
{
  return(keydescr == b.keydescr && 
         cryptkey == b.cryptkey &&
         saltA    == b.saltA    &&
         saltB    == b.saltB);
}

CryptMan::CryptMan()
{
  fLoggedIn    = false;
  fNeedsSaving = false;
  
  nRunden = 42;
}

CryptMan::~CryptMan()
{
#if 1==0
  if (fLoggedIn)
  {
    std::cout << "Masterkey: ";
    PrintBin(mkey, KEY_LEN);
    
    std::cout << "Der vercryptete Hash vom Masterkey: ";
    PrintBin(mkey_checkdata);
    
    for (unsigned int n=0; n<keys.size(); n++)
    {
      std::cout << "Schlüssel:\n";
      std::cout << " ";
      PrintBin(keys[n].cryptkey);
      std::cout << " ";
      PrintBin(keys[n].saltA);
      std::cout << " ";
      PrintBin(keys[n].saltB);
    }
  }
#endif
}

CryptMan& CryptMan::operator = (const CryptMan& init) 
{
  mkey_checkdata = init.mkey_checkdata;
  keys           = init.keys;
  fLoggedIn      = init.fLoggedIn;
  fNeedsSaving   = init.fNeedsSaving;
  nRunden        = init.nRunden;
  
#if 0==1
  if (MKeyEquals(init) == false)
  {
    std::cerr << "mkey_checkdata nicht korrekt übernommen?\n";
    PrintBin(mkey_checkdata);
    PrintBin(init.mkey_checkdata);
  }
  if (keys.size() != init.keys.size())
    std::cerr << "Anzahl der Schlüssel nicht korrekt übernommen?\n";
  else
    for (unsigned int n=0; n<keys.size(); n++)
      if (keys[n].Equals(init.keys[n]) == false)
        std::cerr << "Schlüssel nicht korrekt übernommen?\n";
#endif
  
  return(*this);
}

bool CryptMan::MKeyEquals(const CryptMan& b)
{
  return(mkey_checkdata == b.mkey_checkdata);
}

bool CryptMan::IsLoggedIn()
{
  return(fLoggedIn);
}

int  CryptMan::GetNumPwds()
{
  return(keys.size());
}

std::string CryptMan::GetPwdDescr(int n)
{
  return(keys[n].keydescr);
}

PwdData* CryptMan::GetPwdData(int n)
{
  return(&(keys[n]));
}

void CryptMan::Login(int nPwdNum, const char* pwd)
{
#ifndef MUPOSYNC
  unsigned char* key;
  int            nOutLen;
  
  key = DeriveKey(pwd, keys[nPwdNum].saltA, keys[nPwdNum].saltB);
  

  unsigned char* cryptkey = GetArray(keys[nPwdNum].cryptkey);
  unsigned char* possible_mkey = Decrypt(cryptkey, keys[nPwdNum].cryptkey.size(),
                                         key,
                                         nOutLen, false);
  free(cryptkey);

  //
  unsigned char* cryptcheck = GetArray(mkey_checkdata);
  unsigned char* mkeyhash   = Decrypt(cryptcheck, mkey_checkdata.size(),
                                      possible_mkey,
                                      nOutLen, false);
  free(cryptcheck);
  
  // der hash vom möglichen mkey wird gebraucht:
  Hash_SHA1* hash = new Hash_SHA1();
  hash->Init();
  hash->Update((const char*)possible_mkey, KEY_LEN);
  hash->Finish();

  fLoggedIn = true;
  for (int n=0; n<Hash_SHA1::HASH_LEN; n++)
    if (hash->GetDigestByte(n) != mkeyhash[n])
      fLoggedIn = false;
  
  if (fLoggedIn)
    for (int n=0; n<KEY_LEN; n++)
      mkey[n] = possible_mkey[n];
    
  //
  delete hash;
  free(possible_mkey);
  free(mkeyhash);

  // std::cout << "Login: " << nPwdNum << " " << pwd << "\n";
#endif
}

unsigned char* CryptMan::Encrypt(const unsigned char* input, int nInLen, const unsigned char* key, int& nOutLen, bool fIV)
{
#ifndef MUPOSYNC
# if CRYPT == 0
  int nBlocks = nInLen / KEY_LEN;
  if (nInLen % KEY_LEN)
    nBlocks++;
  
  nOutLen = nBlocks*KEY_LEN;
  
  unsigned char* out = (unsigned char*)malloc(nOutLen);
  unsigned char* in  = (unsigned char*)malloc(nOutLen);
  memcpy(in, input, nInLen);
  for (int n=nInLen; n<nOutLen; n++)
    in[n] = 0;
  
  for (int n=0; n<nBlocks; n++)
    for (int b=0; b<KEY_LEN; b++)
      out[n*KEY_LEN+b] = in[n*KEY_LEN+b] ^ key[b];
  
  free(in);
  
  return(out);
# endif
# if CRYPT == 1
  return(Twofish::Encrypt(input, nInLen, key, KEY_LEN, nOutLen, fIV));
# endif
#else
  return(0);
#endif
}

char* CryptMan::Encrypt(char* input, int& len, bool fIV)
{
  char* tmp = (char*)Encrypt((const unsigned char*)input, strlen(input)+1, mkey, len, fIV);
  
  return(tmp);
}

unsigned char* CryptMan::Decrypt(const unsigned char* input, int nInLen, const unsigned char* key, int& nOutLen, bool fIV)
{
#ifndef MUPOSYNC
# if CRYPT == 0
  return(Encrypt(input, nInLen, key, nOutLen, fIV));
# endif
# if CRYPT == 1
  return(Twofish::Decrypt(input, nInLen, key, KEY_LEN, nOutLen, fIV));
# endif
#else
  return(0);
#endif
}

char* CryptMan::Decrypt(char* input, int len, bool fIV)
{
  int nOutLen;
  char* tmp = (char*)Decrypt((const unsigned char*)input, len, mkey, nOutLen, fIV);

  return(tmp);
}

void CryptMan::SetMKeyCheckData(const unsigned char* data, int len)
{
  mkey_checkdata.clear();
  for (int n=0; n<len; n++)
    mkey_checkdata.push_back(data[n]);
}

void CryptMan::SetPwdData(std::string descr,
                          const unsigned char* cryptkey, int len_ck,
                          const unsigned char* saltA,    int len_sA,
                          const unsigned char* saltB,    int len_sB)
{
  keys.push_back(PwdData(descr,
                         cryptkey, len_ck,
                         saltA,    len_sA,
                         saltB,    len_sB));
}

void CryptMan::AddPwd(std::string descr,
                      std::string pwd)
{
#ifndef MUPOSYNC
  FILE* randfile = fopen("/dev/random", "r");
  
  // Create masterkey if needed
  if (mkey_checkdata.size() == 0)
  {
    // create mkey and mkey_checkdata
    if (randfile == 0)
    {
      // todocrypt: fill mkey with random data
    }
    else
    {
      int n=KEY_LEN;
      while (n)
        n -= fread( &(mkey[KEY_LEN-n]), 1, n, randfile );
    }
    //
    Hash_SHA1* hash = new Hash_SHA1();
    hash->Init();
    hash->Update((const char*)mkey, KEY_LEN);
    hash->Finish();
    
    // das hash-Ergebnis muss noch verschlüsselt werden
    unsigned char  hashbytes[Hash_SHA1::HASH_LEN];
    unsigned char* hashcrypt;
    int            hashcryptlen;
    for (int n=0; n<Hash_SHA1::HASH_LEN; n++)
      hashbytes[n] = hash->GetDigestByte(n);
    hashcrypt = Encrypt(hashbytes, Hash_SHA1::HASH_LEN, mkey, hashcryptlen, false);
    // 
    for (int n=0; n<hashcryptlen; n++)
      mkey_checkdata.push_back(hashcrypt[n]);
    //
    delete hash;
    free(hashcrypt);
    //
    fLoggedIn = true;
  }
  
  // The correct masterkey needs to be known to add a password
  if (fLoggedIn)
  {
    unsigned char* key;
    unsigned char  saltA[Hash_SHA1::HASH_LEN];
    unsigned char  saltB[KEY_LEN - Hash_SHA1::HASH_LEN];
    
    if (randfile)
    {
      int n, len;
      
      len = Hash_SHA1::HASH_LEN;
      n   = len;
      while (n)
        n -= fread( &(saltA[len-n]), 1, n, randfile );
      
      len = KEY_LEN - Hash_SHA1::HASH_LEN;
      n   = len;
      while (n)
        n -= fread( &(saltB[len-n]), 1, n, randfile );      
    }
    else
    {
      // todocrypt: fill with random bytes
    }

    key = DeriveKey(pwd, 
                    saltA, Hash_SHA1::HASH_LEN, 
                    saltB, KEY_LEN - Hash_SHA1::HASH_LEN);
    
    // create cryptmkey from mkey and key
    unsigned char* cryptmkey;
    int            nCryptKeyLen;
    cryptmkey = Encrypt(mkey, KEY_LEN,
                        key, 
                        nCryptKeyLen, false);
    
    SetPwdData(descr, 
               cryptmkey, nCryptKeyLen,
               saltA,     Hash_SHA1::HASH_LEN, 
               saltB,     KEY_LEN - Hash_SHA1::HASH_LEN);
    
    free(key);
    free(cryptmkey);
    //  
    // std::cout << "Neues Pwd: " << descr << " " << pwd << "\n";
    fNeedsSaving = true;
  }
  
  if (randfile)
    fclose(randfile);
#endif
}

void CryptMan::WriteVector(std::ostream& out, 
                           std::vector<unsigned char>& vect)
{
  unsigned char tmp;
  char          achar;
  
  out << vect.size() << "\n";
  for (unsigned int n=0; n<vect.size(); n++)
  {
    tmp = vect[n];
    achar = (tmp & 0x0F) + 'a';
    out.write(&achar, 1);
    achar = ((tmp >> 4) & 0x0F) + 'a';
    out.write(&achar, 1);
  }
  out << "\n";
}

void CryptMan::ReadVector(std::istream& in,
                          std::vector<unsigned char>& vect)
{
  std::string   tmps  = readfLine(in);
  unsigned int  nLen = atoi(tmps.c_str());
  unsigned char tmpc[2];
  
  for (unsigned int n=0; n<nLen; n++)
  {
    in.read((char*)(&tmpc), 2);
    tmpc[0] = (tmpc[0] - 'a') | ((tmpc[1] - 'a') << 4);
    vect.push_back(tmpc[0]);
  }
  in.read((char*)(&tmpc), 1); // newline
}

void CryptMan::Write(std::ostream& out)
{
  if (mkey_checkdata.size() != 0)
  {
    out << "<Crypt>\n";

    out << "0\n";
    out << nRunden << "\n";

    WriteVector(out, mkey_checkdata);
    
    out << keys.size() << "\n";
    
    for (unsigned int n=0; n<keys.size(); n++)
    {
      out << keys[n].keydescr << "\n";
      
      WriteVector(out, keys[n].cryptkey);
      WriteVector(out, keys[n].saltA);
      WriteVector(out, keys[n].saltB);      
    }
    
    out << "</Crypt>\n";
  }
}

int CryptMan::Read(std::istream& in)
{
  std::string tmp;
  unsigned int nKeys;
  
  readfLine(in);
  
  tmp = readfLine(in);
  nRunden = atoi(tmp.c_str());

  mkey_checkdata.clear();
  ReadVector(in, mkey_checkdata);

  tmp = readfLine(in);
  nKeys = atoi(tmp.c_str());
  keys.clear();
  for (unsigned int n=0; n<nKeys; n++)
  {
    keys.push_back(PwdData(readfLine(in), 0, 0, 0, 0, 0, 0));

    ReadVector(in, keys[n].cryptkey);
    ReadVector(in, keys[n].saltA);
    ReadVector(in, keys[n].saltB);      
  }

  tmp = readfLine(in);
  if (tmp.find("</Crypt>") != 0)
    return(-1);
  else
    return(0);
}

bool CryptMan::CanAddPwd()
{
  return(mkey_checkdata.size() == 0 || fLoggedIn);
}

bool CryptMan::CanLogIn()
{
  return(mkey_checkdata.size() != 0 && fLoggedIn == false);
}

unsigned char* CryptMan::DeriveKey(std::string                 PwdString,
                                   std::vector<unsigned char>& PwdSaltA, 
                                   std::vector<unsigned char>& PwdSaltB)
{
  unsigned char* sa = GetArray(PwdSaltA);
  unsigned char* sb = GetArray(PwdSaltB);
  
  unsigned char* tmp = DeriveKey(PwdString, 
                                 sa, PwdSaltA.size(),
                                 sb, PwdSaltB.size());
  
  free(sa);
  free(sb);
  
  return(tmp);
}


unsigned char* CryptMan::DeriveKey(std::string          PwdString,
                                   const unsigned char* PwdSaltA, int len_sA,
                                   const unsigned char* PwdSaltB, int len_sB)
{
#ifndef MUPOSYNC
  unsigned char* digest = (unsigned char*)malloc(KEY_LEN);
  
  // Aus PwdString und PwdSaltA werden die ersten 20 Byte des Schlüssels erzeugt:
  Hash_SHA1* hash = new Hash_SHA1();
  hash->Init();
  hash->Update(PwdString.c_str(), PwdString.length());
  hash->Update((const char*)PwdSaltA, len_sA);
  hash->Finish();
  for (int n=0; n<nRunden-1; n++)
  {
    for (int m=0; m<Hash_SHA1::HASH_LEN; m++)
      digest[m] = hash->GetDigestByte(m);
    hash->Init();
    hash->Update((const char*)digest, Hash_SHA1::HASH_LEN);
    hash->Finish();
  }    
  // Jetzt werden noch die übrigen Byte aus PwdSaltB angehängt:
  for (int n=Hash_SHA1::HASH_LEN; n<KEY_LEN; n++)
    digest[n] = PwdSaltB[n-Hash_SHA1::HASH_LEN];

  delete hash;
  
  return(digest);
#else
  return(0);
#endif
}

unsigned char* CryptMan::GetArray(std::vector<unsigned char>& vec)
{
  unsigned char* tmp = (unsigned char*)malloc(vec.size());
  for (unsigned int n=0; n<vec.size(); n++)
    tmp[n] = vec[n];
  return(tmp);
}

void CryptMan::PrintBin(const unsigned char* data, int len)
{
  for (int n=0; n<len; n++)
    std::cout << (int)data[n] << " ";
  std::cout << "\n";       
}

void CryptMan::PrintBin(const std::vector<unsigned char>& vec)
{
  for (unsigned int n=0; n<vec.size(); n++)
    std::cout << (int)vec[n] << " ";
  std::cout << "\n";       
}

