// -*- mode: c; mode: fold -*-
// 
/**
 * Dieses Programm liest eine MuPO-Datei ein (erstes Argument),
 * modifiziert sie und schreibt sie dann wieder (zweites Argument).
 * 
 * Es geht dabei um die Wocheneinträge im Kalender. Sie stehen in einer 
 * bestimmten Gruppe (per define unten benannt), heissen "JJJJ-WW".
 * Es sind Todos mit Start- und Enddatum gleich dem ersten Tag der Woche.
 * 
 * Das Format des Textes ist ebenfalls festgelegt, siehe Quelltext.
 */

#include "todolistitem.h"
#include "todolistgroup.h"
#include "todolisttodo.h"
#include "tools.h"
#include "config.h"
#include <fstream>
#include <vector>
#include <iostream>
#include <cstdlib>

// set args /home/jens/.mupo /home/jens/.mupo
// 
#define GROUPWHERETHINGSAREIN "Arbeitszeiten"
#define JWWWTOOLVERSION "08/15"
#define VERBOSITY       0

struct T_MuPOFile {
  ToDoListGroup*      pRoot;
  unsigned int        uSyncIDMax;
  unsigned int        uSyncIDMaxAfterSync;
  CryptMan            crypt;
};

T_MuPOFile         mupoFile;
T_Calendar         cal;
ToDoListGroup*     grWB;


void printUsage() 
{
  std::cout << "Usage:\n"
    "   jwwWTool infile outfile\n"
    "infile und outfile dürfen identisch sein.\n";  
}


int readFile(char*        name,
	     T_MuPOFile&  file) 
{
  std::ifstream      ifile;
  std::string        line = "";
  unsigned int  uDummy;
  
  ifile.open(name);
  if (ifile)
  {
    while ((line = readfLine(ifile)) != "g")
    {
      if (line.find("<idmax=") == 0)
      {
        line.replace(0, 7, "");
        line.replace(line.length()-1, 1, "");
        file.uSyncIDMax = atoi(line.c_str());
      }      
      if (line.find("<idmaxaftersync=") == 0)
      {
        line.replace(0, 16, "");
        line.replace(line.length()-1, 1, "");
        file.uSyncIDMaxAfterSync = atoi(line.c_str());
      }      
      if (line.find("<Crypt>") == 0)
      {
        if (file.crypt.Read(ifile))
        {
          std::cerr << "crypt\n";
          return(-1);
        }
      }
    };
    file.pRoot = new ToDoListGroup(0, "/", 0);
    file.pRoot->readContent(ifile, uDummy, file.uSyncIDMax, cal);
    
    ifile.close();
    return(0);
  }
  
  return(-1);
}


int writeFile(char*        name,
              T_MuPOFile&  file) 
{
  std::ofstream ofile;
  
  ofile.open(name);
  if (!ofile)
  {
    std::cerr << "Error writing file: " << name << "\n";
    return(-1);
  }
  ofile << "<data>\n";
  ofile << "<idmax=" << file.uSyncIDMax << ">\n";
  ofile << "<idmaxaftersync=" << file.uSyncIDMaxAfterSync << ">\n";
  file.crypt.Write(ofile);
  file.pRoot->writeContent(ofile, false, &file.crypt);
  ofile << "</data>\n";
  ofile.close();
  
  return(0);
}


/**
 * Den Inhalt von Description und der Begin-Zeit auswerten, irgendwohin schreiben.
 * Jede Zeile, die mit einem kleinen f gefolgt von einem Tabulatorzeichen beginnt,
 * wird als Radfahrt interpretiert.
 */
void wocheAuswerten(ToDoListToDo* t) 
{
  t_datetime   tag;
  int          pos;
  std::string  line;
  int          uTagInWoche = 0;
  std::string  descr;
  const char*  descr_c;
    
  if (t->GetDescription(descr_c, &mupoFile.crypt) == 0)
    descr = descr_c;
  else
    descr = "";  
  
#if VERBOSITY > 0  
  std::cout << "Auswerten von ";
  std::cout.flush();
  std::cout << t->getSummary() << "\n";
  std::cout.flush();
#endif
  
  tag = t->getDateTime(DATECODE_Begin);
  
  while (descr.length() > 0 && descr.find("f\t") != std::string::npos)
  {
    pos = descr.find("f\t");
    descr.replace(0, pos+2, "");
    pos = descr.find("\n");
    line = descr.substr(0, pos);
    descr.replace(0, pos, "");
    std::cout << GETDAY(tag)<< "." << GETMONTH(tag)<< "." << GETYEAR(tag)
      << ":  "
      << line << "\n";
    moveDate(tag, 1);
    uTagInWoche++;
  }
  
  if (uTagInWoche != 5)
  {
    std::cout << "Vorsicht: diese Woche (" << t->getSummary() << ") bestand offensichtlich nur aus " << uTagInWoche << " Tagen!\n";
    std::cout << "Die Wahrscheinlichkeit ist groß, dass ein Fehler vorliegt...\n";
  }
}


/**
 * Macht diverse Sachen...
 */
int checkGroup(ToDoListGroup* gr, t_datetime& latest) 
{
  unsigned int nMax = gr->getNrOfTodos();
  ToDoListGroup* g;
  ToDoListGroup* gy;
  ToDoListToDo*  t;
  ToDoListToDo*  nt;
  std::string    summary;
  t_datetime     begintime;
  int            kw;
  std::string    yearStr;
  
#if VERBOSITY > 0  
  std::cout << "checkGroup " << gr->getSummary() << "\n";
  std::cout.flush();
#endif
  
  if (gr->getSummary().find(GROUPWHERETHINGSAREIN) == 0)
  {
    grWB = gr;
    
    
    for (unsigned int n=0; n<nMax; n++)
    {
      t = gr->getToDoAt(n);
      // Dieses ToDo überprüfen
      summary = t->getSummary();
      begintime = t->getDateTime(DATECODE_Begin);
      if (summary.find("20") == 0  
          && t->getPriority() <= PRIORITY_TODOMAX
          && summary.length() > 6
          && summary[4] == '-'
          && ISDATESET(begintime))
      {
#if VERBOSITY > 0  
        std::cout << "jetzt: " << summary << "\n";
        std::cout.flush();
#endif
        // Welche Woche ist das?
        kw = Kalenderwoche(GETDAY(begintime), GETMONTH(begintime), GETYEAR(begintime));
        /*
         std::cout << summary << "  " 
         << GETDAY(begintime)<< "." << GETMONTH(begintime)<< "." << GETYEAR(begintime)
         << " KW " << kw << "\n";
         */
        // ist es das aktuellste?
        if (begintime > latest)
          latest = begintime;
        
#if VERBOSITY > 0  
        std::cout << "gleich: " << summary << "\n";
        std::cout.flush();
#endif
        wocheAuswerten(t);
        
#if VERBOSITY > 0  
        std::cout << "gemacht: " << summary << "\n";
        std::cout.flush();
#endif
        // Wenn dieses Ding schon "done" ist, kann man es in die Untergruppe für das
        // entsprechende Jahr übernehmen.
        if (t->isCompleted())
        {
          unsigned int uDummy = 1;
          yearStr = itoStr(GETYEAR(begintime), ' ', 4);
          
          // Gibt es schon eine Untergruppe für das entsprechende Jahr?
          gy = gr->getGroupPtrFromSumPath("", GROUPWHERETHINGSAREIN  "/" + yearStr + "/");
          if (gy == (ToDoListGroup*)0)
          {
            gy = gr->newGroup();
            gy->setSyncID(++mupoFile.uSyncIDMax);
            gy->setSummary(yearStr);
          }
#if VERBOSITY > 0  
          std::cout << "Verschiebe " << t->getSummary() << " nach " << yearStr << "\n";
          std::cout.flush();
#endif
          
          // in gy kann man dieses Element nun verschieben
          nt = gy->newToDo();
          uDummy = nt->getID();
          *nt = *t;
          nt->setID(uDummy);
          nt->setSyncID(++mupoFile.uSyncIDMax);
#if VERBOSITY > 0  
          std::cout << "bla\n";
          std::cout.flush();
#endif
          
          gr->delToDo(t, uDummy, cal);
#if VERBOSITY > 0  
          std::cout << "blubb\n";
          std::cout.flush();
#endif
          
          // Ich habe dieser Gruppe gerade das ToDo mit dem Index n genommen.
          // An der Stelle n liegt nach dem Löschen also das ToDo, welches
          // vorher den Index n+1 hatte. Außerdem gibt es in der Gruppe
          // nun ein Todo weniger. Also:
          n--;
          nMax--;
        }
      }
    }
    return (0);
  }
  
  nMax = gr->getNrOfGroups();
  for (unsigned int n=0; n<nMax; n++)
  {
    g = gr->getGroupAt(n);
    checkGroup(g, latest);
  }
  
  return(1);
}


/**
 * Templates für die folgenden acht Wochen (ab heute) sollen vorhanden sein. */
void templatesErstellen(t_datetime latest) 
{
  int            kw;
  std::string    newSumm;
  ToDoListToDo*  t;
  
  t_datetime ende;
  CLEARDATE(ende);
  CLEARTIME(ende);
  ende = getTodayDate(ende);
#if VERBOSITY > 0  
  std::cout << "Heute ist "
    << GETDAY(ende)<< "." << GETMONTH(ende)<< "." << GETYEAR(ende)
      << "\n";
  std::cout.flush();
#endif
  moveDate(ende, 7*8);
  
  
#if VERBOSITY > 0  
  std::cout << "Die letzte vorhandene Woche ist " 
    << GETDAY(latest)<< "." << GETMONTH(latest)<< "." << GETYEAR(latest)
      << "\n";
  std::cout << "Zu erstellen sind die Einträge bis "
    << GETDAY(ende)<< "." << GETMONTH(ende)<< "." << GETYEAR(ende)
      << "\n";
  std::cout.flush();
#endif  
  
  while (ende > latest)
  {
    moveDate(latest, 7);
    kw = Kalenderwoche(GETDAY(latest), GETMONTH(latest), GETYEAR(latest));
    
    newSumm = itoStr(GETYEAR(latest), '0', 4);
    newSumm += '-';
    newSumm += itoStr(kw, '0', 2);
    
#if VERBOSITY > 0  
    std::cout << "Erstelle " <<  newSumm << "\n";
    std::cout.flush();
#endif      
    t = grWB->newToDo();
    t->setSyncID(++mupoFile.uSyncIDMax);
    t->setSummary(newSumm);
    t->setDateDontCheck(DATECODE_Begin, GETYEAR(latest), GETMONTH(latest), GETDAY(latest), cal);
    t->setDateDontCheck(DATECODE_End, GETYEAR(latest), GETMONTH(latest), GETDAY(latest), cal);      
    t->setDescription("(:) : :-:\n"
                      "f\t|\n"
                      "(:) : :-:\n"
                      "f\t|\n"
                      "(:) : :-:\n"
                      "f\t|\n"
                      "(:) : :-:\n"
                      "f\t|\n"
                      "(:) : :-:\n"
                      "f\t|\n"
                      "\n\n"
                      "-=\n"
                      "-=\n"
                      "-=\n"
                      "-=\n"
                      "-=\n");
  }  
}


int main(int argc, char* argv[]) 
{
  std::ofstream ofile;
  t_datetime    latest;
  
  SETDATE(latest, 2000, 1, 1);
  
  std::cout << "jwwWTool " << JWWWTOOLVERSION << "   Jens Wilhelm Wulf <j.w.wulf@gmx.net>\n\n";
  if (argc != 3)
  {
    printUsage();
    return(-1);
  }
  
  std::cout << "MuPO-File to read: " << argv[1] << "\n";
  if (readFile(argv[1], mupoFile))
  {
    std::cerr << "Error reading file: " << argv[1] << "\n";
    return(-1);
  }
  
  // Rekursiv durch alles durchhangeln, was es da so gibt.
  checkGroup(mupoFile.pRoot, latest);
  
  // jetzt noch die neuen Dinger erstellen
  templatesErstellen(latest);
  
  writeFile(argv[2], mupoFile);
  
  return(0);
}


