// -*- mode: c; mode: fold -*-

#include "todolistgroup.h"

#include <stdlib.h>
#include <stdio.h>
#include "tools.h"
#include "config.h"
#include "datetime.h"


#pragma warning(disable: 4786)

#define _DEBUGSNOW
#define _DEBUGINITPATH

#ifdef VERBOSE
#include <iostream>
#endif

#ifdef DEBUGSNOW
#include <stdio.h>
#endif
#ifdef DEBUGINITPATH
#include <stdio.h>
#endif


ToDoListGroup::ToDoListGroup(unsigned int   ID,
			     std::string    pathInit,
			     unsigned int   uLevelInit) 
  : ToDoListItem(ID, pathInit)
{
#ifdef DEBUGSNOW
  printf("ToDoListGroup::ToDoListGroup() 1\n");
#endif
  uLevel = uLevelInit;
  fShowCompleted = INITIALSC;
  fCompletedLast = INITIALCL;
  uChildren      = 0;
  uChildrenDone  = 0;
#ifdef DEBUGSNOW
  printf("ToDoListGroup::ToDoListGroup() 2\n");
#endif
}


ToDoListGroup::ToDoListGroup(const ToDoListGroup& init) 
  : ToDoListItem(init)
{
  *this = init;
}


ToDoListGroup::~ToDoListGroup() 
{
  while (todos.size())
    {
      delete todos[todos.size()-1];
      todos.pop_back();
    }
  while (groups.size())
    {
      delete groups[groups.size()-1];
      groups.pop_back();
    }  
}


ToDoListGroup& ToDoListGroup::operator = (const ToDoListGroup& init) 
{  
  path          = init.path;
  uID           = init.uID;
  uSyncID       = init.uSyncID;
  summary       = init.summary;
  uPriority     = init.uPriority;
  
  return(*this);
}


bool ToDoListGroup::operator == (ToDoListGroup group2) 
{  
  if (group2.path          != path)
    return(false);
  if (group2.uID           != uID)
    return(false);
  if (group2.uSyncID       != uSyncID)
    return(false);
  if (group2.summary       != summary)
    return(false);         
  if (group2.uPriority     != uPriority)
    return(false);    
    
  return(true);
}


bool ToDoListGroup::operator != (ToDoListGroup group2) 
{
  return(!(*this == group2));
}


int           ToDoListGroup::writeContent(std::ostream&  ofile,
                                          bool           fExport,
                                          CryptMan*      crypt,
                                          bool           fIncludeYourself,
                                          bool           fIncludeContent) 
{  
  unsigned int i;
  int retcode = 0;
  
  if (fIncludeYourself)
    {
      ofile << "g\ns" << summary << "\n";
      if (!fExport)
	ofile << "I" << uSyncID << "\n";
      if (uPriority != INITIALPRIORITY)
	ofile << "p" << uPriority << "\n";
      
      if (fShowCompleted != INITIALSC)
	{
	  if (fShowCompleted)
	    ofile << "c1\n";
	  else
	    ofile << "c0\n";
	}
      
      if (fCompletedLast != INITIALCL)
	{
	  if (fCompletedLast)
	    ofile << "l1\n";
	  else
	    ofile << "l0\n";
	}
    }
  
  if (fIncludeContent)
    {
      unsigned int iMax = todos.size();
      for(i=0;i<iMax;i++)      
        retcode |= todos[i]->writeContent(ofile, fExport, crypt);

      iMax = groups.size();
      for(i=0;i<groups.size();i++)
        retcode |= groups[i]->writeContent(ofile, fExport, crypt);
    }
  
  if (fIncludeYourself)
    ofile << ">\n";
  
  if (!fExport)
    hasBeenSaved(true);
  
  return(retcode);
}


#ifdef OLDCODE
void           ToDoListGroup::readContentPre014(std::ifstream&     ifile,
                                                unsigned int& uNrActiveJobs,
                                                unsigned int& uSyncIDMax,
                                                T_Calendar&   cal) 
{
  ToDoListGroup*  pGroup;
  ToDoListToDo*   pToDo;
  std::string     line;
  unsigned int    uNextID = 0;
  
  while ((line = readfLine(ifile)) != "</g>")
    {
      if (line.find("<s=\"") == 0)
        {
          summary = line;
          summary.replace(0, 4, "");
          summary.replace(summary.length()-2, 2, "");
        }
      if (line.find("<p=") == 0)
        {
          line.replace(0, 3, "");
          line.replace(line.length()-1, 1, "");
          uPriority = atoi(line.c_str());
          if (uPriority > 5)
            uPriority = 5;
        }
      if (line.find("<g>") == 0)
        {
           pGroup = new ToDoListGroup(uNextID++, path+itoStr(uID, ' ', 0)+"/", uLevel+1);
          pGroup->readContentPre014(ifile, uNrActiveJobs, uSyncIDMax, cal);
          groups.push_back(pGroup);
        }
      if (line.find("<t>") == 0)
        {
          pToDo = new ToDoListToDo(uNextID++, path+itoStr(uID, ' ', 0)+"/");
          pToDo->readContentPre014(ifile, uNrActiveJobs, uSyncIDMax, cal);
          todos.push_back(pToDo);
        }
      if (line.find("<sc=0>") == 0)     
        fShowCompleted = false;      
      if (line.find("<sc=1>") == 0)
        fShowCompleted = true;
      if (line.find("<cl=0>") == 0)
        fCompletedLast = false;      
      if (line.find("<cl=1>") == 0)
        fCompletedLast = true;
      if (line.find("<I=") == 0)
        {
          line.replace(0, 3, "");
          line.replace(line.length()-1, 1, "");
          uSyncID = atoi(line.c_str());
        }      
    }
  if (uSyncID == 0)
    uSyncID = ++uSyncIDMax;
}

#endif

int            ToDoListGroup::readContent(std::ifstream&     ifile,
                                          unsigned int&      uNrActiveJobs,
                                          unsigned int&      uSyncIDMax,
                                          T_Calendar&        cal) 
{
  ToDoListGroup*  pGroup;
  ToDoListToDo*   pToDo;
  char*           line;
  int             retCode = 0;
  
  
  line = (char*)malloc(500);
  
  do
    {
      ifile.getline(line, 499);
           
      switch (line[0])
        {
         case 's':
          summary = line+1;
          break;
         case 'p':
	  uPriority = atoi(line+1);
	  if (uPriority > PRIORITY_TODOMAX)
	    uPriority = 1;
          break;
         case 'I':
	  uSyncID = atoi(line+1);
          break;
         case 'g':
	  pGroup = new ToDoListGroup(findFreeID(), path+itoStr(uID, ' ', 0)+"/", uLevel+1);
	  if (pGroup->readContent(ifile, uNrActiveJobs, uSyncIDMax, cal))
            {
              free(line);
              return(-1);
            }
	  groups.push_back(pGroup);
          break;
         case 't':
	  pToDo = new ToDoListToDo(findFreeID(), path+itoStr(uID, ' ', 0)+"/");
	  if (pToDo->readContent(ifile, uNrActiveJobs, uSyncIDMax, cal))
            {
              free(line);
              return(-1);
            }
	  todos.push_back(pToDo);
          break;
         case 'c':
          fShowCompleted = (line[1] == 1);
          break;
         case 'l':
          fCompletedLast = (line[1] == 1);
          break;
        }      
    }
  while (line[0] != '>' && !ifile.eof());

  if (line[0] != '>')
    retCode = -1;
  
  free(line);
  
  if (uSyncID == 0)
    uSyncID = ++uSyncIDMax;
  else
    {
      if (uSyncIDMax < uSyncID)
        uSyncIDMax = uSyncID;
      hasBeenSaved(true);
    }
  
  return(retCode);
}


void           ToDoListGroup::showContent(std::string                    group,
                                          T_Options                      options,
                                          std::vector<T_viewDataItem>&   viewData,
                                          std::vector<pToDoListItem>&    viewPtr,
                                          CryptMan*                      crypt)
{
  std::string   myChildrensPath;
  unsigned int  i, iMax;

  myChildrensPath = path + itoStr(uID, ' ', 0) + "/";
 
  if ((myChildrensPath.find(group) == 0 && (options.viewtype == flat || options.mainview == MAINVIEW_ONLYACTIVE)) || (myChildrensPath == group))
    {
      T_viewDataItem tmp;

      if (options.mainview == MAINVIEW_ONLYACTIVE)
	{
          iMax = todos.size();
	  for(i=0;i<iMax;i++)
	    if (todos[i]->isRunning())
	      {
		todos[i]->setViewData(tmp, crypt);
		viewPtr.push_back(todos[i]);
		viewData.push_back(tmp);
	      }
	}
      else
	{
	  // show my groups and todos
          iMax = groups.size();
	  for(i=0;i<iMax;i++)
	    if (options.fShowCompleted || !groups[i]->fCompleted)
	      {	    
		groups[i]->setViewData(tmp);
		viewPtr.push_back(groups[i]);
		viewData.push_back(tmp);
	      }
          iMax = todos.size();
	  for(i=0;i<iMax;i++)
	    if (options.fShowCompleted || !todos[i]->isCompleted())
	      {
		todos[i]->setViewData(tmp, crypt);
		viewPtr.push_back(todos[i]);
		viewData.push_back(tmp);
	      }
	}
            
      if (options.viewtype == flat || options.mainview == MAINVIEW_ONLYACTIVE)
	{
	  // show things below
          iMax = groups.size();
	  for(i=0;i<iMax;i++)
	    groups[i]->showContent(group, options, viewData, viewPtr, crypt);
	}    
    }
  else
    {
      if (group.find(myChildrensPath) == 0)
	{
	  // maybe show things below
          iMax = groups.size();
	  for(i=0;i<iMax;i++)
	    groups[i]->showContent(group, options, viewData, viewPtr, crypt);
	}    
    }
}


void           ToDoListGroup::dump() 
{
  unsigned int i, iMax;

  iMax = todos.size();
  for(i=0;i<iMax;i++)
    todos[i]->dump();

  // show my groups and todos
  iMax = groups.size();
  for(i=0;i<iMax;i++)
    groups[i]->dump();
  
}


bool           ToDoListGroup::checkCompleted() 
{ 
  unsigned int   tmp1, tmp2, i, iMax;
  
  fCompleted     = true;   
  uChildren      = 0;
  uChildrenDone  = 0;
  
  // I am completed if all my ToDos and subgroups are completed:
  iMax = groups.size();
  for(i=0;i<iMax;i++)
    {
      if (!groups[i]->checkCompleted())
        fCompleted = false;
      groups[i]->getCompletedNr(tmp1, tmp2);
      uChildrenDone += tmp1;
      uChildren     += tmp2;
    }
  
  iMax = todos.size();
  for(i=0;i<iMax;i++)
    if (todos[i]->getPriority() <= PRIORITY_TODOMAX)
      {
        uChildren++;
        if (todos[i]->isCompleted())
          uChildrenDone++;
        else          
          fCompleted = false;          
      }
  
  return(fCompleted);
}


void           ToDoListGroup::getCompletedNr(unsigned int& uDone, 
                                             unsigned int& uCnt) 
{
  uDone = uChildrenDone;
  uCnt  = uChildren;
}


void           ToDoListGroup::setViewData(T_viewDataItem& tmp) 
{
  tmp.fGroup    = true;
  tmp.uPriority = uPriority;
  tmp.summary   = new std::string(summary);
  tmp.fDone     = fCompleted;
  tmp.sortDate  = DATETIME_NODATE | DATETIME_NOTIME;
}


/**
 * Findet den kleinsten uID, der in dieser Gruppe noch frei ist.
 */
unsigned int   ToDoListGroup::findFreeID() 
  // previously has been =todos.size()+groups.size(), but
  // that can lead to errors...see bugfix 0.21
{
  unsigned int uFreeID = 0;
  unsigned int uCnt;
  unsigned int uCntMax;
  
  uCntMax = todos.size();
  for (uCnt = 0; uCnt < uCntMax; uCnt++)
    if (todos[uCnt]->getID() >= uFreeID)
      uFreeID = todos[uCnt]->getID() + 1;
  
  uCntMax = groups.size();
  for (uCnt = 0; uCnt < uCntMax; uCnt++)
    if (groups[uCnt]->getID() >= uFreeID)
      uFreeID = groups[uCnt]->getID() + 1;
  
  return(uFreeID);
}


ToDoListGroup* ToDoListGroup::newGroup() 
{
  ToDoListGroup*  pGroup;
  
  pGroup = new ToDoListGroup(findFreeID(), path+itoStr(uID, ' ', 0)+"/", uLevel+1);
  groups.push_back(pGroup);
  
  return(pGroup);
}


ToDoListToDo*  ToDoListGroup::newToDo() 
{
  ToDoListToDo*   pToDo;
  
  pToDo = new ToDoListToDo(findFreeID(), path+itoStr(uID, ' ', 0)+"/");
  todos.push_back(pToDo);
  
  return(pToDo);
}


std::string    ToDoListGroup::getPathText(std::string p) 
{
  std::string tmp = "", tmp2;
#ifdef DEBUGINITPATH
  std::cout << "getPathText(" << p << ") "  << path << " " << uID << "\n";
#endif  
   
  
  if (p == itoStr(uID, ' ', 0))
    {
#ifdef DEBUGINITPATH
      std::cout << "!<" << summary << "> " << uID << "\n";
#endif  
      if (uLevel != 0)
	tmp = summary;
      else
	tmp = "";
    }
  else
    if (p.find(itoStr(uID, ' ', 0) + "/") == 0)
      {
#ifdef DEBUGINITPATH
	std::cout << "?<" << summary << "> " << uID << "\n";
#endif  
	p.replace(0,p.find("/")+1,"");
        unsigned int iMax = groups.size();
	for(unsigned int i=0;i<iMax;i++)
	  {
	    tmp2 = groups[i]->getPathText(p);
	    if (tmp2 != "")
	      break;
	  }
	if (uLevel != 0)
	  tmp =  summary + "\n" + tmp2;
	else
	  tmp = tmp2;
      }
#ifdef DEBUGINITPATH
  std::cout << "----<" << tmp << ">\n";
#endif  
  
  return(tmp);
}


unsigned int   ToDoListGroup::getNrActiveJobs() 
{
  unsigned int uTmp = 0;
  unsigned int i;
  unsigned int iMax;
  
  iMax = todos.size();
  for(i=0;i<iMax;i++)
    if (todos[i]->isRunning())
      uTmp++;      

  iMax = groups.size();
  for(i=0;i<iMax;i++)
    uTmp += groups[i]->getNrActiveJobs();

  return(uTmp);
}


int            ToDoListGroup::delToDo(ToDoListToDo* pToDel,
                                      unsigned int& uNrActiveJobs,
                                      T_Calendar&   cal) 
{  
  unsigned int iMax = todos.size();
  for(unsigned int i=0;i<iMax;i++)
    if (todos[i] == pToDel)
      {
        todos.erase(todos.begin()+i);
        
        if (pToDel->isRunning())
          uNrActiveJobs--;
    
        pToDel->removeFromCalLists(cal);
        
        delete pToDel;

        return(0);
      }
  
  return(-1);
}


void           ToDoListGroup::removeFromCalLists(T_Calendar&    cal) 
{
  unsigned int i;
  unsigned int iMax;

  iMax = groups.size();
  for(i=0;i<iMax;i++)
    groups[i]->removeFromCalLists(cal);

  iMax = todos.size();
  for(i=0;i<iMax;i++)
    todos[i]->removeFromCalLists(cal);
}

                                       
void           ToDoListGroup::delGroup(ToDoListGroup* pToDel,
                                       unsigned int&  uNrActiveJobs,
                                       T_Calendar&    cal) 
{
  uNrActiveJobs -= pToDel->getNrActiveJobs();
  
  pToDel->removeFromCalLists(cal);
  
  delete pToDel;
     
  unsigned int iMax = groups.size();
  for(unsigned int i=0;i<iMax;i++)
    if (groups[i] == pToDel) 
      {
        groups.erase(groups.begin()+i);
        i = iMax;
      }
}


/**
 * Liefert den Zeiger auf die Gruppe, die zu einer bestimmten
 * Verkettung der uID-Werte gehört (das ist eindeutig).
 */
ToDoListGroup* ToDoListGroup::getGroupPtr(std::string aPath) 
{
  ToDoListGroup*  pTmp;
  std::string     myChildrensPath = path + itoStr(uID, ' ', 0) + "/";

  if (myChildrensPath == aPath)
    return(this);    

  if (aPath.find(myChildrensPath) == 0)
    {
      unsigned int iMax = groups.size();
      for(unsigned int i=0;i<iMax;i++)
        {
          pTmp = groups[i]->getGroupPtr(aPath);
          if (pTmp != (ToDoListGroup*)0)
            return(pTmp);
        }
    }

  return((ToDoListGroup*)0);
}


/**
 * Returns Pointer to a group. The group is given by the concatenated summaries.
 * 
 * Im Gegensatz zu getGroupPtr(std::string) nicht eindeutig!
 */
ToDoListGroup* ToDoListGroup::getGroupPtrFromSumPath(std::string root, std::string aPath) 
{
  ToDoListGroup*  pTmp;
  std::string     myChildrensPath = root + summary + "/";

  /*
  printf("Suche:\n  myChildrensPath: %s\n  root: %s\n  aPath: %s\n", 
         myChildrensPath.c_str(), root.c_str(), aPath.c_str());
  */
  if (myChildrensPath == aPath)
    return(this);    

  if (aPath.find(myChildrensPath) == 0)
    {
      unsigned int iMax = groups.size();
      for(unsigned int i=0;i<iMax;i++)
        {
          pTmp = groups[i]->getGroupPtrFromSumPath(myChildrensPath, aPath);
          if (pTmp != (ToDoListGroup*)0)
            return(pTmp);
        }
    }

  return((ToDoListGroup*)0);
}


void           ToDoListGroup::findToDosWithText(std::string                 toS, 
                                                std::vector<ToDoListToDo*>& vDPtr,
                                                CryptMan*                   crypt)
{
  unsigned int i;

  for(i=0;i<todos.size();i++)
    if (todos[i]->containsText(toS, crypt))
      vDPtr.push_back(todos[i]);
  
  for(i=0;i<groups.size();i++)
    groups[i]->findToDosWithText(toS, vDPtr, crypt);
}


double         ToDoListGroup::getTimeSum() 
{
  unsigned int i;
  unsigned int iMax;
  double dTmp = 0;
    
  iMax = todos.size();
  for(i=0;i<iMax;i++)
    dTmp += todos[i]->getTimeSum();
  
  iMax = groups.size();
  for(i=0;i<iMax;i++)
    dTmp += groups[i]->getTimeSum();

  return(dTmp);
}



ToDoListItem*  ToDoListGroup::getPtrToItemWithSyncID(unsigned int uASyncID,
                                                     bool&        fGroup,
                                                     bool         fRecursive) 
{
  ToDoListItem*  pItem;
  unsigned int   i;
  unsigned int   iMax;

  if (uASyncID == uSyncID)
    return(this);
  fGroup = false;
  
  iMax = todos.size();
  for(i=0;i<iMax; i++)
    if (todos[i]->getSyncID() == uASyncID)
      return((ToDoListItem*)todos[i]);
  fGroup = true;
  
  iMax = groups.size();
  for(i=0;i<iMax; i++)
    if (groups[i]->getSyncID() == uASyncID)
      return((ToDoListItem*)groups[i]);
  
  if (fRecursive)
    {
      iMax = groups.size();
      for(i=0;i<iMax;i++)
        {
          pItem = groups[i]->getPtrToItemWithSyncID(uASyncID, fGroup, true);
          if (pItem != (ToDoListItem*)0)
            return(pItem);
        }
    }
    
  return((ToDoListItem*)0);
}


#ifdef MUPOSYNC
bool           ToDoListGroup::checkDiff(ToDoListItem* pOthersRoot) 
{
  bool             fTmp;
  ToDoListGroup*   pItem;
    
  pItem = (ToDoListGroup*)((ToDoListGroup*)pOthersRoot)->getPtrToItemWithSyncID(uSyncID, fTmp);
  
  if (pItem == (ToDoListGroup*)0)
    {
      uSyncFlag |= DIFF_NEW;
#ifdef VERBOSE	  
      std::cout << "  " << uSyncID << " is new\n";
#endif
    }
  else
    {      
      for(unsigned int i=0;i<todos.size();i++)
	if (todos[i]->checkDiff(pItem))
	  uSyncFlag |= DIFF_CHILD;
      
      for(unsigned int i=0;i<groups.size();i++)
	if (groups[i]->checkDiff(pItem))
	  uSyncFlag |= DIFF_CHILD;
            
//      if (pItem->get() != get())
//	uSyncFlag |= DIFF_DUEDATE;
      if (pItem->getSummary() != getSummary())
	uSyncFlag |= DIFF_SUMMARY;
      if (pItem->getShowCompleted() != getShowCompleted())
	uSyncFlag |= DIFF_SHOWCOMPL;
      if (pItem->getCompletedLast() != getCompletedLast())
	uSyncFlag |= DIFF_COMPLLAST;
      if (pItem->getSummary() != getSummary())
	uSyncFlag |= DIFF_SUMMARY;
      if (pItem->getPriority() != getPriority())
	uSyncFlag |= DIFF_PRIORITY;
      
      if (uSyncFlag != 0)
	{
#ifdef VERBOSE	  
	  std::cout << "  " << uSyncID << " has been changed\n";
#endif
	  pItem->setMustBeKept();
	}
    }
  return(uSyncFlag != 0);
}


void           ToDoListGroup::checkDel(ToDoListItem* pOthersRoot) 
{
  ToDoListGroup*   pGroup;
  ToDoListGroup*   pOtherGroup;
  ToDoListToDo*    pItem;
  bool             fTmp;
  
  
  for(unsigned int i=0;i<todos.size();i++)
    {
      pItem = todos[i];
      
      if (((ToDoListGroup*)pOthersRoot)->getPtrToItemWithSyncID(pItem->getSyncID(), fTmp) == (ToDoListItem*)0)
	{
#ifdef VERBOSE	  
	  std::cout << uSyncID << " has been deleted\n";
#endif
	  pItem->setShouldBeDeleted();
	}
   }

  for(unsigned int i=0;i<groups.size();i++)
    {
      pGroup      = groups[i];
      pOtherGroup = (ToDoListGroup*)((ToDoListGroup*)pOthersRoot)->getPtrToItemWithSyncID(pGroup->getSyncID(), fTmp);
      if (pOtherGroup == (ToDoListItem*)0)
	{
#ifdef VERBOSE	  
	  std::cout << uSyncID << " has been deleted\n";
#endif
	  pGroup->setShouldBeDeleted();
	}
      else	
	pGroup->checkDel(pOtherGroup);      
   }
}


void           ToDoListGroup::syncDelete() 
{
  for(unsigned int i=0;i<todos.size();i++)
    if (todos[i]->getShouldBeDeleted() && !todos[i]->getMustBeKept())
      {
	delete todos[i];
        todos.erase(todos.begin()+i);
	i--;
      }
  
  for(unsigned int i=0;i<groups.size();i++)
    {
      if (groups[i]->getShouldBeDeleted() && !groups[i]->getMustBeKept())
	{
	  delete groups[i];
	  groups.erase(groups.begin()+i);
	  i--;
	}
      else
	groups[i]->syncDelete();
    }
}


void           ToDoListGroup::addNewItems(ToDoListGroup* pOthersRoot) 
{
  bool             fTmp;
  ToDoListGroup*   pItem;
  
  pItem = (ToDoListGroup*)pOthersRoot->getPtrToItemWithSyncID(uSyncID, fTmp);
  if (pItem != (ToDoListItem*)0)
    {    
      for(unsigned int i=0;i<todos.size();i++)
	if (todos[i]->getSyncFlag() & DIFF_NEW)
	  {
#ifdef VERBOSE
	    std::cout << todos[i]->getSyncID() << " added\n";
#endif
	    pItem->addToDoPtr(todos[i]);
	    todos.erase(todos.begin()+i);
	    i--;
	  }
      
      for(unsigned int i=0;i<groups.size();i++)
	{
	  if (groups[i]->getSyncFlag() & DIFF_NEW)
	    {
#ifdef VERBOSE
	      std::cout << groups[i]->getSyncID() << " added\n";
#endif
	      pItem->addGroupPtr(groups[i]);
	      groups.erase(groups.begin()+i);
	      i--;
	    }
	  else
	    groups[i]->addNewItems(pItem);
	}  
    }
}


void           ToDoListGroup::resetSyncID(unsigned int& uNewSyncID) 
{
  uSyncID = ++uNewSyncID;
  
  for(unsigned int i=0;i<todos.size();i++)
    todos[i]->setSyncID(++uNewSyncID);
  
  for(unsigned int i=0;i<groups.size();i++)
    groups[i]->resetSyncID(uNewSyncID);
}


void           ToDoListGroup::addToDoPtr(ToDoListToDo* pItem)   { todos.push_back(pItem);  }
void           ToDoListGroup::addGroupPtr(ToDoListGroup* pItem) { groups.push_back(pItem); }

void           ToDoListGroup::setShouldBeDeleted() 
{ 
  fShouldBeDeleted = true;
  
  for(unsigned int i=0;i<todos.size();i++)
    todos[i]->setShouldBeDeleted();
  
  for(unsigned int i=0;i<groups.size();i++)
    groups[i]->setShouldBeDeleted();
}


#endif
   
bool           ToDoListGroup::getShowCompleted()       { return(fShowCompleted); }
bool           ToDoListGroup::getCompletedLast()       { return(fCompletedLast); }
void           ToDoListGroup::setShowCompleted(bool n) { fShowCompleted = n; }
void           ToDoListGroup::setCompletedLast(bool n) { fCompletedLast = n; }

#ifdef GROUPINTERFACE
unsigned int            ToDoListGroup::getNrOfTodos()
{  
  return(todos.size());
}

unsigned int            ToDoListGroup::getNrOfGroups()
{  
  return(groups.size());
}

ToDoListToDo*  ToDoListGroup::getToDoAt(unsigned int index)
{
  if (index < todos.size())
    return(todos[index]);
  else
    return((ToDoListToDo*)0);
}

ToDoListGroup*  ToDoListGroup::getGroupAt(unsigned int index)
{
  if (index < groups.size())
    return(groups[index]);
  else
    return((ToDoListGroup*)0);
}
#endif
