#ifndef __TODOLISTGROUP_H
# define __TODOLISTGROUP_H

# include <fstream>
# include <string>
# include <vector>
# include <time.h>

# include "types.h"
# include "todolisttodo.h"


class ToDoListGroup : public ToDoListItem
{
public:
  ToDoListGroup(unsigned int ID, 
                std::string  pathInit,
                unsigned int uLevelInit);
  ToDoListGroup(const ToDoListGroup&);
  ToDoListGroup& operator =  (const ToDoListGroup& init);
  bool operator == (ToDoListGroup group2);
  bool operator != (ToDoListGroup group2);
  ~ToDoListGroup();
  
  virtual int writeContent(std::ostream&  ofile,
                           bool           fExport,
                           CryptMan*      crypt,
                           bool           fIncludeYourself = true,
                           bool           fIncludeContent  = true);
  int  readContent(std::ifstream&  ifile,
                   unsigned int&   uNrActiveJobs,
                   unsigned int&   uSyncIDMax,
                   T_Calendar&     cal);
# ifdef OLDCODE
  void readContentPre014(std::ifstream&  ifile,
                         unsigned int&   uNrActiveJobs,
                         unsigned int&   uSyncIDMax,
                         T_Calendar&     cal);
# endif
  void showContent(std::string                    group,
                   T_Options                      options,
                   std::vector<T_viewDataItem>&   viewData,
                   std::vector<pToDoListItem>&    viewPtr,
                   CryptMan*                      crypt);
  bool checkCompleted();
  void getCompletedNr(unsigned int& uDone, 
                      unsigned int& uCnt);
  void setViewData(T_viewDataItem& tmp);
  void dump();
  int  delToDo(ToDoListToDo*,
               unsigned int&,
               T_Calendar&);
  void delGroup(ToDoListGroup*,
                unsigned int&,
                T_Calendar&);
  ToDoListGroup* getGroupPtr(std::string);
  ToDoListGroup* newGroup();
  ToDoListToDo*  newToDo();
  std::string    getPathText(std::string p);
  
  bool getShowCompleted();
  bool getCompletedLast();
  void setShowCompleted(bool n);
  void setCompletedLast(bool n);
  
  
  void findToDosWithText(std::string                 toS, 
                         std::vector<ToDoListToDo*>& vDPtr,
                         CryptMan*                   crypt);
  
  unsigned int findFreeID();
  unsigned int getNrActiveJobs();
  double       getTimeSum();
  void         removeFromCalLists(T_Calendar&    cal);   
  
  ToDoListGroup* getGroupPtrFromSumPath(std::string root,
                                        std::string aPath);
  
  ToDoListItem* getPtrToItemWithSyncID(unsigned int uASyncID,
                                       bool&        fGroup,
                                       bool         fRecursive = false);
# ifdef MUPOSYNC
  virtual bool checkDiff(ToDoListItem* pOthersRoot);
  void         checkDel(ToDoListItem* pOthersRoot);
  void         syncDelete();
  void         addNewItems(ToDoListGroup* pOthersRoot);
  void         addToDoPtr(ToDoListToDo* pItem);
  void         addGroupPtr(ToDoListGroup* pItem);
  void         resetSyncID(unsigned int& uNewSyncID);
  
  virtual void setShouldBeDeleted();
# endif
# ifdef GROUPINTERFACE
  unsigned int   getNrOfTodos();
  unsigned int   getNrOfGroups();
  ToDoListToDo*  getToDoAt(unsigned int index);
  ToDoListGroup* getGroupAt(unsigned int index);
# endif   
protected:
  unsigned int                  uLevel;
  std::vector<ToDoListToDo*>    todos;
  std::vector<ToDoListGroup*>   groups;
  bool                          fCompleted;      
  bool                          fShowCompleted; 
  bool                          fCompletedLast;
  unsigned int                  uChildren;
  unsigned int                  uChildrenDone;
};


#endif
