// -*- mode: c; mode: fold -*-
#ifndef __TODOLISTTODO_H
# define __TODOLISTTODO_H

# include <fstream>
# include <string>
# include <vector>
# include <time.h>
# include <stdio.h>

# include "types.h"
# include "datetime.h"
# include "config.h"
# include "todolistitem.h"
# include "cryptman.h"
# include "descrdata.h"



struct T_workingTimes {
  time_t   begin;
  time_t   end;
  
  bool operator == (T_workingTimes s)
  {
    return(begin    == s.begin &&
           end      == s.end);
  };
  
  bool operator != (T_workingTimes s)
  {
    return(!(*this == s));
  };
  
  T_workingTimes& operator = (const T_workingTimes& t)
  {
    begin     = t.begin;
    end       = t.end;
    return(*this);
  };
};

struct T_Calendar;

class ToDoListToDo : public ToDoListItem
{
public:    
  ToDoListToDo(unsigned int ID,
               std::string  pathInit);
  ToDoListToDo(const ToDoListToDo&);
  
  ToDoListToDo& operator = (const ToDoListToDo& init);
  bool operator == (ToDoListToDo todo2);
  bool operator != (ToDoListToDo todo2);
  ~ToDoListToDo();
  
  /**
   * returns zero if pointer to description is returned,
   * otherwise:
   *   1: Description has to be decrypted. PtrToDescr points
   *      to encrypted data.
   */
  int GetDescription(const char*& PtrToDescr,
                     CryptMan*    crypt);
  
  /**
   * 
   */  
  const DescrData* GetDescription() const { return(&descr); };
    
  bool DescrIsBinary(int flags) const;
  bool CurDescrIsBinary() const;
  
  bool IsSaveCrypt() const { return(descr.IsSaveCrypt()); };
  void SetSaveCrypt(bool fCrypt);
   
  /**
   * returns zero on success
   */
  virtual int writeContent(std::ostream&  ofile,
                           bool           fExport,
                           CryptMan*      crypt,
                           bool           fIncludeYourself = true,
                           bool           fIncludeContent  = true);
  int  readContent(std::ifstream&     ifile,
                   unsigned int&      uNrActiveJobs,
                   unsigned int&      uSyncIDMax,
                   T_Calendar&        cal);
  
  bool isCompleted();
  void setViewData(T_viewDataItem& tmp,
                   CryptMan*       crypt);
  
  /**
   * returns 
   *   1 if date is changed
   *   0 if date is unchanged to do end of recurrence period
   *   0 if date is unchanged as there is no recurrence data
   */
  int  setFollowingRecurrenceDate(t_datetime& dTB, 
                                  t_datetime& dTE);
  int  setCompleted(T_Calendar&   cal);
  int  clearCompleted();
  
  int  getShowTimeAs();
  void setShowTimeAs(int n, T_Calendar&    cal);
  void setCompletedDirect(T_completed);
  time_t getCompletedTime();
  T_completed getCompletedDirect();
  
  
  
  void setDate(int            n,  
               int            year,
               int            month, 
               int            day,
               T_Calendar&    cal);
  void setDateDontCheck(int            n,  
                        int            year,
                        int            month, 
                        int            day,
                        T_Calendar&    cal);
  t_datetime getDateTime(int n);
  void clearDate(int          n,
                 T_Calendar&    cal);
  void setTime(int            n,
               int            hour, 
               int            min,
               T_Calendar&    cal);
  void setTimeDontCheck(int            n,
                        int            hour, 
                        int            min,
                        T_Calendar&    cal);
  void clearTime(int            n,
                 T_Calendar&    cal);
  
  void setPriority(unsigned int   n,
                   T_Calendar&    cal);
  
  
  bool containsText(std::string toS,
                    CryptMan*   crypt);
  bool isRunning();
  void setRunningDirect(bool fNew);
  int  setRunning(bool fR);
  
  unsigned int getNrTimes();
  time_t       getTimeBegin(unsigned int n);
  time_t       getTimeEnd(unsigned int n);
  void         setRecData(T_recData      recdata,
                          T_Calendar&    cal);
  void         getRecData(T_recData& recdata);
  
  /**
   * check: recData, dateTimeBegin, dateTimeEnd, showTimeAs
   */
  void         sanitizeTimeThings();
  
  
  double getTimeSum();
  void   addTime(time_t begin,
                 time_t end);
  
  void setDescription(const DescrData* aDescr);
  void setDescription(const char* pd);
  void removeFromCalLists(T_Calendar&    cal);
  void putIntoCalList    (T_Calendar&    cal);
  
  bool hasIdenticalData(ToDoListToDo* pTodo);
  
# ifdef MUPOSYNC
  virtual bool checkDiff(ToDoListItem* pOthersRoot);
  void         clearWTimes();
# endif
  
protected:
  std::vector<T_workingTimes>   workingTimes;
  T_completed                   completed;
  int                           showTimeAs;
  bool                          fRunning;
  t_datetime                    dateTimeBegin;
  t_datetime                    dateTimeEnd;
  
  T_recData                     recData;
  
  DescrData descr;  
};

struct T_DatedCalListItem
{
  ToDoListToDo*   item;
  t_datetime      dt;
};

struct T_Calendar
{
  int                             date;        // starting to show from this day
  int                             days;        // how many days to show on one page
  std::vector<ToDoListToDo*>      recTodos;
  std::vector<T_DatedCalListItem> dateTodos;
  std::vector<ToDoListToDo*>      recApps;
  std::vector<T_DatedCalListItem> dateApps;
  std::vector<ToDoListToDo*>      recNotes;
  std::vector<T_DatedCalListItem> dateNotes;
};


#endif
