#ifndef __TYPES_H
#define __TYPES_H

#ifdef __FreeBSD__
#include <time.h>
#endif

#ifdef WIN32
# include <time.h>
#endif


#include "datetime.h"
#include <string>
#include <vector>

#define SHOW_AS_FREE  0
#define SHOW_AS_BUSY  1

typedef enum {flat, tree} t_viewtype;

#define SORT_PRIORITY 0
#define SORT_DATE     1

struct T_viewDataItem {
  bool           fGroup;
  t_datetime     showDate;
  t_datetime     sortDate;
  t_datetime     otherDate;
  unsigned int   uPriority;
  std::string*   summary;
  bool           fDone;
  bool           fFirstDay;
  bool           fLastDay;
};

struct T_Options {
  t_viewtype viewtype;
  int        mainview;      // was ist gerade in MAINVISIBLE_MAIN zu sehen?
  bool       fShowKbd;
  int        mainviewBeforeOnlyActive;
  int        sorttype;
  bool       fShowCompleted;
  bool       fCompletedLast;
};

struct T_Session {
  unsigned int uWinWidth;
  unsigned int uWinHeight;
};

struct T_recData {
  int         nType;
  int         nInterval;
  t_datetime  begin;
  t_datetime  end;
  
  bool operator == (T_recData s)
    {
      return(nType    == s.nType    && 
             nInterval == nInterval &&
             begin    == s.begin    &&
             end      == s.end);
    };
  
  bool operator != (T_recData t)
    {
      return(!(*this == t));
    };
  
  T_recData& operator = (const T_recData& t)
    {
      nType     = t.nType;
      nInterval = t.nInterval;
      begin     = t.begin;
      end       = t.end;
      return(*this);
    };
};

struct T_completed {
  bool         fDone;
  time_t       time;
    
  bool operator == (T_completed t)
    {
      return(fDone == t.fDone && 
             time  == t.time);
    };
  
  bool operator != (T_completed t)
    {
      return(!(*this == t));
    };
  
  T_completed& operator = (const T_completed& t)
    {
      fDone = t.fDone;
      time  = t.time;
      return(*this);
    };
};

class T_GUIData {
  public:
   int getListSize() { return(oldPosInList.size()); };
   int getPosAt(int n) {return(oldPosInList[n]); };
   
   void pushList(int pos, int sel) 
   { 
     oldPosInList.push_back(pos); 
     oldSelInList.push_back(sel); 
   };
   
   int  popListPos() 
   { 
     int tmp = 0;
         
     if (oldPosInList.size())
     {
       tmp = oldPosInList[oldPosInList.size()-1];
       oldPosInList.pop_back();
     }
     return(tmp);
   };
   
   int  popListSel()
   { 
     int tmp = 0;
         
     if (oldSelInList.size())
     {
       tmp = oldSelInList[oldSelInList.size()-1];
       oldSelInList.pop_back();
     }
     if (tmp < 1)
       return(1);
     else
       return(tmp);
   };
   
   void clearStack()
   {
     while (oldPosInList.size())
       oldPosInList.pop_back();
     while (oldSelInList.size())
       oldSelInList.pop_back();
   };
      
   int                     nLineCal;
   int                     nLineNormal;
   
  private:
   std::vector<int>        oldPosInList;
   std::vector<int>        oldSelInList;   
};

struct T_SpecDay
{
  char*     descr;
  int       nDay;
  int       nMonth;
};

#endif
