#define US_HOLIDAYS

// Calculates things like the second Sunday in June, first Thursday in April, etc.
// nth is 1=first, 2=second, 3=third, 4=fourth, 5 and up = last
// day is 0=Monday, 1=Tuesday, .. 6=Sunday
// month is 1..12, year is whatever Wochentag and MaxTag can handle
int nthWhichDayOfMonth(int nth, int day, int month, int year)
{
	// find the first whatever day of the month
	int temp = ((day - Wochentag(1, month, year) + 7) % 7 + 1);

	// bump up to the nth whateverday of the month
	temp += (nth - 1) * 7;

	// If we've fallen off the end of the month, drop back a week
	if (temp > MaxTag(month, year))
		temp -= 7;
	return temp;
}

void SpecialDays::fillVarLists(int year)
{
  int nOsterMonat, nOsterTag;

  clearList(varFree);
  clearList(var);
  
  nCurYear = year;
  
  ostern(nOsterTag, nOsterMonat, year);
  
  // 1. Berechnung von "variablen" Feiertagen
  //    Ostern
  //    Christi Himmelfahrt
  //    Pfingsten
  //    Fronleichnam
  
  
  // Ostersonntag
  addToList(varFree, nOsterTag,  nOsterMonat, "Easter");
  
  // Ostermontag
  nOsterTag++;
  if (nOsterTag > MaxTag(nOsterMonat, year))
    {
      nOsterTag -= MaxTag(nOsterMonat, year);
      nOsterMonat++;
    }
//  addToList(varFree, nOsterTag,  nOsterMonat, "Ostern");
  
  // Karfreitag
  nOsterTag -= 3;
  if (nOsterTag < 1)
    {
      nOsterMonat--;
      nOsterTag += MaxTag(nOsterMonat, year);
    }
  addToList(varFree, nOsterTag,  nOsterMonat, "Good Friday");

  // Karfreitag
  nOsterTag -= 5;
  if (nOsterTag < 1)
    {
      nOsterMonat--;
      nOsterTag += MaxTag(nOsterMonat, year);
    }
  addToList(varFree, nOsterTag,  nOsterMonat, "Palm Sunday");

  // Christi Himmelfahrt
  nOsterTag += (39 + 7);
  while (nOsterTag > MaxTag(nOsterMonat, year))
    {
      nOsterTag -= MaxTag(nOsterMonat, year);
      nOsterMonat++;
    }
  addToList(varFree, nOsterTag,  nOsterMonat, "Ascension Day");

  // Pfingstsonntag
  nOsterTag += 10;
  while (nOsterTag > MaxTag(nOsterMonat, year))
    {
      nOsterTag -= MaxTag(nOsterMonat, year);
      nOsterMonat++;
    }
  addToList(varFree, nOsterTag,  nOsterMonat, "Pentecost");

  // Pfingstmontag
  nOsterTag++;
  while (nOsterTag > MaxTag(nOsterMonat, year))
    {
      nOsterTag -= MaxTag(nOsterMonat, year);
      nOsterMonat++;
    }
//  addToList(varFree, nOsterTag,  nOsterMonat, "Pfingsten");
  
  // Fronleichnam
  nOsterTag += 10;
  while (nOsterTag > MaxTag(nOsterMonat, year))
    {
      nOsterTag -= MaxTag(nOsterMonat, year);
      nOsterMonat++;
    }
//  addToList(varFree, nOsterTag,  nOsterMonat, "Fronleichnam");
    

  // 2. Berechnung von anderen besonderen Tagen
  // Der Muttertag ist der zweite Sonntag im Mai
  // Umstellung Sommer/Winterzeit Die Sommerzeit beginnt also am letzten Sonntag 
  //       des Monats März und endet am letzten Sonntag des Monats Oktober.
  //       März: Uhr vorstellen!
  // Aschermittwoch
  ostern(nOsterTag, nOsterMonat, year);
  
  // Aschermittwoch
  nOsterTag -= 46;
  while (nOsterTag < 1)
    {
      nOsterMonat--;
      nOsterTag += MaxTag(nOsterMonat, year);
    }
  addToList(var, nOsterTag,  nOsterMonat, "Ash Wednesday");

  addToList(varFree, nthWhichDayOfMonth(3, 0,  1, year),  1, "Martin Luther King Day");
  addToList(varFree, nthWhichDayOfMonth(3, 0,  2, year),  2, "Presidents Day");
  addToList(varFree, nthWhichDayOfMonth(5, 0,  5, year),  5, "Memorial Day");
  addToList(varFree, nthWhichDayOfMonth(1, 0,  9, year),  9, "Labor Day");
  addToList(varFree, nthWhichDayOfMonth(2, 0, 10, year), 10, "Columbus Day");
  addToList(varFree, nthWhichDayOfMonth(4, 3, 11, year), 11, "Thanksgiving Day");

  // According to Terry Chan, this is correct at least for year <= 2006.
  if (year < 2002)
    addToList(varFree, nthWhichDayOfMonth(5, 6,  1, year),  1, "Superbowl Sunday");
  else
    addToList(varFree, nthWhichDayOfMonth(1, 6,  2, year),  2, "Superbowl Sunday");

  addToList(var, nthWhichDayOfMonth(1, 6,  4, year),  4, "DST begins");
  addToList(var, nthWhichDayOfMonth(1, 3,  5, year),  5, "National Day of Prayer");
  addToList(var, nthWhichDayOfMonth(2, 6,  5, year),  5, "Mother's Day");
  addToList(var, nthWhichDayOfMonth(3, 6,  6, year),  6, "Father's Day");
  addToList(var, nthWhichDayOfMonth(5, 6, 10, year), 10, "DST ends");
}

void SpecialDays::fillFixedLists()
{
  addToList(fixedFree,  1,  1, "New Years Day");
  addToList(fixedFree,  4,  7, "Independence Day");
  addToList(fixedFree, 11, 11, "Veterans Day");
  addToList(fixedFree, 25, 12, "Christmas");

  addToList(fixed,  6,  1, "Epiphany");
  addToList(fixed,  2,  2, "Groundhog Day");
  addToList(fixed, 14,  2, "St. Valentine's Day");
  addToList(fixed, 17,  3, "St. Patrick's Day");
  addToList(fixed,  1,  4, "April Fools' Day");
  addToList(fixed, 15,  4, "Income Tax Due");
  addToList(fixed, 22,  4, "Earth Day");
  addToList(fixed, 14,  6, "Flag Day");
  addToList(fixed, 31, 10, "Halloween");
  addToList(fixed, 24, 12, "Christmas Eve");
  addToList(fixed, 31, 12, "New Years Eve");
}
