// -*- mode: c; mode: fold -*-

#include "main.h"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <errno.h>
#include <fstream>

#if defined( WIN32 ) && !defined(__CYGWIN__)
# include <io.h>
# include <fcntl.h>
# include <basetsd.h>
# include <sys\stat.h>
//#include <winbase.h>
#else
# include <unistd.h>
#endif

#include <fcntl.h> // flags bei open

#include <FL/fl_ask.H>
#include <FL/Enumerations.H>

#include "tools.h"
#include "filetools.h"
#include "configfile.h"

#include "strings.h"

#define LOCKFILEPATH "/ram/tmp/mupo.running"

void cb_keypress_ESC(Fl_Widget * o, void *)
{
  // I don't want the escape key to close the app:
  if (Fl::event() == FL_KEYBOARD || Fl::event_key() != FL_Escape)
    mainWin->CB_Done();
}

void copyFile(const char* dest, const char* source) 
{
#if defined( WIN32 ) && !defined(__CYGWIN__)
  
//  CopyFile(source, dest, false);
  
#else
  std::string tmp;
  
  tmp = "cp -p " + std::string(source) + " " + std::string(dest);
  
  system(tmp.c_str());
#endif
}


void make_windows(const char* filename)                       
{
  unsigned int uWidth = 0, uHeight;
  char* title = (char*)malloc(sizeof(char)*strlen(filename)+6);
  strcpy(title, "MuPO: ");
  strcat(title, filename);

  mupo->windowSize(uWidth, uHeight);
  
  mainWin = new MuPOGUI (uWidth*SIZEMUL, uHeight*SIZEMUL, title);
  mainWin->callback(cb_keypress_ESC);

  
#ifdef IPAQLINUX
  Fl::set_font((Fl_Font)0, "Verdana");  // not sure about this, makes somethings nice and others worse ....  
#endif
  
  mainWin->labelsize (TEXTSIZE1);
  
#ifdef NOBORDER
  mainWin->border(0);
#endif
  Fl_Group::current()->resizable(mainWin);
  mainWin->makeWindows();
  mainWin->set_non_modal ();
}


void moveBackupFiles(std::string masterFileName) 
{
  std::string aFileNameS, aFileNameD;
  
  aFileNameD = masterFileName;
  aFileNameD += "b2";
  aFileNameS = masterFileName;
  aFileNameS += "b1";  
#ifdef PARANOIDBACKUPFILES
  copyFile(aFileNameD.c_str(), aFileNameS.c_str());
#endif
  
  aFileNameD = aFileNameS;
  aFileNameS = masterFileName;
  aFileNameS += "b0";
#ifdef PARANOIDBACKUPFILES
  copyFile(aFileNameD.c_str(), aFileNameS.c_str());
#endif
  
  aFileNameD = aFileNameS;
  aFileNameS = masterFileName;
  copyFile(aFileNameD.c_str(), aFileNameS.c_str());  
}


static void updateClock(void * bla)        
{
  Fl::repeat_timeout(30.0, updateClock);
  mainWin->updateClock();
}

int defaultFile(std::string& path)
{
  std::string homepath = GetHomeDir();
    
  for (unsigned int n=0; n<mupoconfig.defaultfiles.size(); n++)
  {
    if (mupoconfig.defaultfiles[n][0] == '~')
    {          
      if (homepath.length() != 0)
        path = homepath + mupoconfig.defaultfiles[n].substr(1);
    }     
    else
      path = mupoconfig.defaultfiles[n];
    
    if (existDat(path))
      return(0);
    else
    {
      printf("%s does not exist\n", path.c_str());
    }
  }
  
  return(-1);
}



int main (int argc, char **argv)
{  
  int         nRetVal = 0;
  int         tmpFile;
  char        tmpfilename[16];
  std::string impFile;
  std::string impPath;
  int         nDFile = 0;
  int         nIFile = 0;
  int         nGroup = 0;
  bool        fQuit      = false;
  bool        fForceCal  = false;
  bool        fForceSave = false;
  std::string filename;
  std::string errMsg = "";
  t_datetime  date   = DATETIME_NODATE | DATETIME_NOTIME;

  
#ifdef LOCKFILE
  tmpFile = open(LOCKFILEPATH, O_WRONLY | O_CREAT);
  if (tmpFile == -1)
  {
    fl_message("Unable to create lockfile. Maybe it does exist? Is MuPO already running?");
    return(-1);
  }
  close(tmpFile);
#endif
  
  ReadConfigFile();
  defaultFile(filename);
  
  {
    for (int nCnt = 1; nCnt < argc; nCnt++)
    {
      if (argv[nCnt][0] == '-')
      {
        switch (argv[nCnt][1])
        {
         case 'q':
          fQuit      = true;
          break;
         case 'c':
          fForceCal  = true;
          if (nCnt+1 < argc && argv[nCnt+1][0] != '-')
          {
            std::string datestr = argv[nCnt+1];
            SETDATE(date, 
                    atoi(datestr.substr(0, 4).c_str()),
                    atoi(datestr.substr(5, 7).c_str()),
                    atoi(datestr.substr(8, 10).c_str()));
            nCnt++;
          }
          else
          {
            date = getTodayDate(DATETIME_NOTIME);
          }
          break;
         case 's':
          fForceSave = true;
          break;
         case 'f':
          nDFile = 1;
          break;
         case 'i':
          nIFile = 1;
          break;
         case 'g':
          nGroup = 1;
          break;
         default:
          break;
        }              
      }
      else
      {
        if (nDFile == 1)
        {
          nDFile = 2;
          filename = argv[nCnt];
        }
        if (nIFile == 1)
        {
          nIFile = 2;
          impFile = argv[nCnt];
        }
        if (nGroup == 1)
        {
          nGroup = 2;
          impPath = argv[nCnt];
        }
      }
    }
  }
  

  mupo = new MuPO();
  switch (mupo->checkFile(filename.c_str()))
  {
   default:
    break;
    // todo
  }

  mupo->createRoot();

  if (mupo->readData(filename.c_str()))
  {
    // master data file corrupt
    // show warning message and exit
    fl_message("Master data file corrupt. Read Documentation about data recovery.");
#ifdef LOCKFILE
    FileTools::rm(LOCKFILEPATH);
#endif
    return(-1);
  }
  
  switch (mupo->incsave_readFiles(errMsg))
  {
   case 1:
    // files read, everything ok
    moveBackupFiles(filename);
    printf("write to file: %i\n", mupo->writeData(filename.c_str()));
    mupo->incsave_delFiles();
    break;
   case 0:
    // no files to read, everything ok
    break;
   default:
    // incremental files corrupt
    // show warning message and exit
    errMsg = "Incremental file corrupt:\n" + errMsg + "\nRead Documentation about data recovery.";
    fl_message(errMsg.c_str());
#ifdef LOCKFILE
    remove(LOCKFILEPATH);
#endif
    return(-1);
    break;
  }
  
  
  
#if defined( WIN32 ) && !defined(__CYGWIN__)
  strcpy(tmpfilename, "mupoXXXXXX.tmp");
  tmpFile = _open(tmpfilename, _O_TRUNC | _O_CREAT | _O_RDWR, _S_IREAD | _S_IWRITE);
#else
  strcpy(tmpfilename, "/tmp/mupoXXXXXX");
  tmpFile = mkstemp(tmpfilename);
#endif
  if (tmpFile == -1)
  {
    fl_message("Unable to create temporary file");
#ifdef LOCKFILE
    remove(LOCKFILEPATH);
#endif
    return(-1);
  }
  close(tmpFile);

  
#ifdef CMDIMPORT
  if (nGroup == 2 && nIFile == 2)
  {
    // import data
    mupo->setMainViewType(MAINVIEW_NORMAL);
    impPath.insert(0, "/<unnamed>");
    mupo->autoImport(impFile, impPath);
  }
#endif
  
  if (!fQuit)
  {
    // run GUI
    make_windows(filename.c_str());
    
    mainWin->init(mupo, tmpfilename);

    mainWin->showMain(MAINVISIBLE_MAIN);
    mainWin->setMenuActive();
    if (mupo->getMainViewType() == MAINVIEW_CALENDAR)
      mainWin->showList(mupo->GUIData.nLineCal);
    else
      mainWin->showList(mupo->GUIData.nLineNormal);
    
    if (fForceCal)
    {
      // In den Kalendermodus wechseln
      mainWin->CB_Menu_MuPOView(MAINVIEW_CALENDAR);
      // Den Tag anzeigen
      mainWin->mupo->setCalFirstDay(date);
      mainWin->showList();
    }
    
    mainWin->initFakeKbd();
      
    mainWin->show ();
      
    Fl::add_timeout(1.0, updateClock);
    mainWin->updateClock();
    
    nRetVal = Fl::run ();
  }  
  
  if ((mupo->incsave_getNrOfFiles() > 0) || fForceSave || mupo->crypt.NeedsSaving())
  {
    moveBackupFiles(filename);  
    printf("write to file: %i\n", mupo->writeData(filename.c_str()));
    mupo->incsave_delFiles();
  }
  
  delete mupo;
  
  remove(tmpfilename);
    
#ifdef LOCKFILE
  remove(LOCKFILEPATH);
#endif
  
  return nRetVal;
}
