// -*- mode: c; mode: fold -*-
// 
#include "mupo.h"
#include <fstream>
#include "tools.h"
#include "types.h"
#include <cstdio>
#include <cstdlib>
#include "version.h"

#ifndef WIN32
# include <unistd.h>
#endif


#define _DEBUGDELCOMPL
#ifdef DEBUGDELCOMPL
# include <iostream>
#endif

#define _DEBUGSNOW
#ifdef DEBUGSNOW
# include <stdio.h>
#endif


#define _DEBUGINITPATH
#ifdef DEBUGINITPATH
# include <stdio.h>
#endif

#define _DEBUGCAL
#ifdef DEBUGCAL
# include <stdio.h>
#endif

#define _DEBUGINCSAVE
#ifdef DEBUGINCSAVE
# include <iostream>
#endif

extern const char* dayName[];

MuPO::MuPO() 
{
  specDays = new SpecialDays();
#ifdef DEBUGSNOW
  printf("MuPO::MuPO() 1\n");
#endif
  pRootGroup          = 0;
  pSearchResGroup     = 0;
  uNrActiveJobs       = 0;
  uSyncIDMax          = 0;
  uSyncIDMaxAfterSync = 0;
  options.mainview    = MAINVIEW_NORMAL;
  options.fShowKbd    = false;
  cal.days            = 7;
  cal.date            = getTodayDate(DATETIME_NOTIME);
  GUIData.nLineNormal = 0;
  GUIData.nLineCal    = 0;
  prevGroup.str       = new std::string("");
#ifdef DEBUGSNOW
  if (prevGroup.str == NULL)
    printf("new failed\n");
#endif
  preSearchGroup.str  = new std::string("");
#ifdef DEBUGSNOW
  if (preSearchGroup.str == NULL)
    printf("new failed\n");
#endif
  curGroup.str        = new std::string("");
#ifdef DEBUGSNOW
  if (curGroup.str == NULL)
    printf("new failed\n");
#endif
  
  session.uWinHeight = 160 * DEFAULT_WINSIZE_MUL;
  session.uWinWidth  = 160 * DEFAULT_WINSIZE_MUL;
  
  pIncSaveTodo  = (ToDoListToDo*)0;
  pIncSaveGroup = (ToDoListGroup*)0;
}


MuPO::~MuPO() 
{
  delete specDays;
  
  while (cal.recTodos.size())
    cal.recTodos.pop_back();
  while (cal.dateTodos.size())
    cal.dateTodos.pop_back();
  while (cal.recApps.size())
    cal.recApps.pop_back();
  while (cal.dateApps.size())
    cal.dateApps.pop_back();
  while (cal.recNotes.size())
    cal.recNotes.pop_back();
  while (cal.dateNotes.size())
    cal.dateNotes.pop_back();
  
  
  if (pRootGroup != 0)
    delete pRootGroup;
  
  delete curGroup.str;
  delete prevGroup.str;
  delete preSearchGroup.str;
}


int         MuPO::checkFile(const char* fname) 
{
  // todo
  // check if i am allowed to write...
  return(0);
}


int         MuPO::createRoot() 
{
#ifdef DEBUGSNOW
  printf("MuPO::createRoot() 1\n");
#endif
  pRootGroup = new ToDoListGroup(0, "/", 0);
#ifdef DEBUGSNOW
  printf("MuPO::createRoot() 2\n");
#endif
  // SNOWTEST
  curGroup.str->assign("/0/");
#ifdef DEBUGSNOW
  printf("MuPO::createRoot() 3\n");
#endif
  curGroup.ptr = pRootGroup;
#ifdef DEBUGSNOW
  printf("MuPO::createRoot() 4\n");
#endif
  return(0);
}


int         MuPO::readData(const char* fname) 
{
  std::ifstream ifile;
  std::string   line = "";
  bool          fPre014 = false;
  std::string   begstr = "g";
  bool          fMupo = true;
  int           retCode = 0;
  
#ifdef DEBUGSNOW
  printf("MuPO::readData() 1\n");
#endif
  filename = fname;  
  
  options.mainview = MAINVIEW_NORMAL;
  options.viewtype = tree;
  options.sorttype = SORT_DATE;
#ifdef DEBUGSNOW
  printf("MuPO::readData() 2\n");
#endif
  ifile.open(fname, std::ios::binary);
#ifdef DEBUGSNOW
  printf("MuPO::readData() 3\n");
#endif
  if (ifile)
  {
#ifdef DEBUGSNOW
    printf("MuPO::readData() 4\n");
#endif
    while ((line = readfLine(ifile)) != begstr && !ifile.eof())
    {
#ifdef DEBUGSNOW
      printf("MuPO::readData() 5\n");
      printf("%s", line.c_str());
      printf("\n");
#endif
      if (line.find("<type=MuPO-Sync>") == 0)
      {
        fMupo = false;
      }
      
      if (line.find("<version=") == 0)
      {
        line.replace(0, 9, "");
        if (line[0] == '0')
        {
          line.replace(0, 2, "");
          line.replace(line.length()-1, 1, "");
          fPre014 = (atoi(line.c_str()) < 14) && fMupo;
          if (fPre014)
            begstr = "<g>";
        }
      }
      
      if (line.find("<viewtype=flat>") == 0)
        options.viewtype = flat;
      if (line.find("<viewtype=tree>") == 0)
        options.viewtype = tree;
      
      if (line.find("<ShowKbd>") == 0)
        options.fShowKbd = true;
      
      
      // JWWTODO: demnächst entfernen   ANFANG
      if (line.find("<mainview=normal>") == 0)
        options.mainview = MAINVIEW_NORMAL;
      if (line.find("<mainview=calendar>") == 0)                      
        options.mainview = MAINVIEW_CALENDAR;
      if (line.find("<onlyActive>") == 0)
        options.mainview = MAINVIEW_ONLYACTIVE;
      // JWWTODO: demnächst entfernen   ENDE
      // 
      if (line.find("<Session.uWinWidth=") == 0)
      {
        line.replace(0, 19, "");
        line.replace(line.length()-1, 1, "");
        session.uWinWidth = atoi(line.c_str());
      }
      if (line.find("<Session.uWinHeight=") == 0)
      {
        line.replace(0, 20, "");
        line.replace(line.length()-1, 1, "");
        session.uWinHeight = atoi(line.c_str());
      }
      if (line.find("<GUIData.nLineNormal=") == 0)
      {
        line.replace(0, 21, "");
        line.replace(line.length()-1, 1, "");
        GUIData.nLineNormal = atoi(line.c_str());
      }
      if (line.find("<GUIData.nLineCal=") == 0)
      {
        line.replace(0, 18, "");
        line.replace(line.length()-1, 1, "");
        GUIData.nLineCal = atoi(line.c_str());
      }
      if (line.find("<GUIData.oldPosInList=") == 0)
      {
        line.replace(0, 22, "");
        line.replace(line.length()-1, 1, "");
        GUIData.pushList(atoi(line.c_str()), 0);
      }
      
      if (line.find("<mainviewtype=") == 0)
      {
        line.replace(0, 14, "");
        line.replace(line.length()-1, 1, "");
        options.mainview = atoi(line.c_str());
      }
      
      
      if (line.find("<sorttype=priority>") == 0)
        options.sorttype = SORT_PRIORITY;
      if (line.find("<sorttype=date>") == 0)
        options.sorttype = SORT_DATE;
      if (line.find("<idmax=") == 0)
      {
        line.replace(0, 7, "");
        line.replace(line.length()-1, 1, "");
        uSyncIDMax = atoi(line.c_str());
      }      
      if (line.find("<cal.date=") == 0)
      {
        line.replace(0, 10, "");
        line.replace(line.length()-1, 1, "");
        cal.date = (t_datetime)atoi(line.c_str());
      }      
      if (line.find("<cal.days=") == 0)
      {
        line.replace(0, 10, "");
        line.replace(line.length()-1, 1, "");
        cal.days = (t_datetime)atoi(line.c_str());
      }      
      if (line.find("<curGroup=") == 0)
      {
        line.replace(0, 10, "");
        line.replace(line.length()-1, 1, "");
        // SNOWTEST
#ifdef DEBUGSNOW
        printf("%s\n", line.c_str());
#endif
        *(curGroup.str) = line;
#ifdef DEBUGSNOW
        printf("MuPO::readData() 6\n");
#endif
      }      
      if (line.find("<idmaxaftersync=") == 0)
      {
        line.replace(0, 16, "");
        line.replace(line.length()-1, 1, "");
        uSyncIDMaxAfterSync = atoi(line.c_str());
      }      

      if (line.find("<Crypt>") == 0)
      {
        if (crypt.Read(ifile))
          retCode = -1;
      }
    };
#ifdef DEBUGSNOW
    printf("MuPO::readData() 7\n");
#endif
    
    if (line != begstr)
    {
      retCode = -1;
    }
    else
    {
#ifdef OLDCODE
      if (fPre014)
        pRootGroup->readContentPre014(ifile, uNrActiveJobs, uSyncIDMax, cal);
      else
#endif
        if (pRootGroup->readContent(ifile, uNrActiveJobs, uSyncIDMax, cal))
          retCode = -1;
    }
    
#ifdef DEBUGSNOW
    printf("MuPO::readData() read content\n");
#endif
    if (*(curGroup.str) != "/0/")
      curGroup.ptr = pRootGroup->getGroupPtr(*(curGroup.str));
    
    if (curGroup.ptr == (ToDoListGroup*)0 || *(curGroup.str) == "/0/")
    {
      curGroup.ptr = pRootGroup;
      *(curGroup.str)  = "/0/";
    }
    ifile.close();
  }
  else
  {
    pRootGroup->setSyncID(1);
    uSyncIDMax = 1;
  }
  
  if (retCode == 0)
    pRootGroup->checkCompleted();
  
  return(retCode);
}


int         MuPO::insertDataFromFile(const char* fname) 
{
  std::ifstream ifile;
  std::string   line;
  int           retCode = 0;
  
  if (options.mainview != MAINVIEW_NORMAL)
    return(-1);
  
  ifile.open(fname, std::ios::binary);
  if (ifile)
  {
    do
    {	  
      line = readfLine(ifile);
      if (line == "g")
      {
        std::ofstream  odat;
        unsigned int   uParent;
        
        uParent = curGroup.ptr->getSyncID();
        
        newGroup();
        curGroup.ptr->readContent(ifile, uNrActiveJobs, uSyncIDMax, cal);
        
        // incsave
#ifdef DEBUGINCSAVE
        std::cout << "save group and content\n";
#endif
        incsave_getFile(odat);
        // items are new
        odat << "<new parent=" << uParent << ">\n";
        
        curGroup.ptr->writeContent(odat, false, &crypt, true, true);
        odat << ">\n</new>\n";
        incsave_closeFile(odat);
        
        leaveCurGroup();
      }
      if (line == "t")
      {
        std::ofstream  odat;
        unsigned int   uParent;
        
        uParent = curGroup.ptr->getSyncID();
        
        newToDo();
        pCurItem->readContent(ifile, uNrActiveJobs, uSyncIDMax, cal);
        
        // incsave
#ifdef DEBUGINCSAVE
        std::cout << "save todo\n";
#endif
        incsave_getFile(odat);              
        // item is new
        odat << "<new parent=" << uParent << ">\n";
        
        pCurItem->writeContent(odat, false, &crypt);
        odat << ">\n</new>\n";
        incsave_closeFile(odat);
      }
    }
    while (line != "</data>" && !ifile.eof());
    
    if (line != "</data>")
      retCode = -2;
    
    ifile.close();
    pRootGroup->checkCompleted();
  }
  else
    retCode = -3;
  
  return(retCode);
}

int         MuPO::exportGroupToFile(const char* fname) 
{
  std::ofstream ofile;
  std::string   line;
  
  if (options.mainview == MAINVIEW_ONLYACTIVE)
    return(-1);
  
  ofile.open(fname, std::ios::binary);
  if (ofile)
  {
    ofile << "<type=MuPO>\n<version="MUPOVERSION">\n<data>\n";
    if (options.mainview == MAINVIEW_SEARCHRESULTS)
    {
      ofile << "g\nssearch results\n";
      unsigned int uForMaxVal = viewDataPtr.size();
      for (unsigned int n=0; n<uForMaxVal; n++)
      {        
        ((ToDoListToDo*)viewDataPtr[n])->writeContent(ofile, true, &crypt);
      }
      ofile << ">\n";
    }
    else
    {
      
      curGroup.ptr->writeContent(ofile, true, &crypt);
    }
    ofile << "</data>\n";
    
    ofile.close();
  }
  return(0);
}


int         MuPO::exportContentToFile(const char* fname) 
{
  std::ofstream ofile;
  std::string   line;
  
  if (options.mainview == MAINVIEW_ONLYACTIVE)
    return(-1);
  
  ofile.open(fname, std::ios::binary);
  if (ofile)
  {
    ofile << "<type=MuPO>\n<version="MUPOVERSION">\n<data>\n";
    if (options.mainview == MAINVIEW_SEARCHRESULTS)
    {
      unsigned int uForMaxVal = viewDataPtr.size();
      for (unsigned int n=0; n<uForMaxVal; n++)
      {
        
        ((ToDoListToDo*)viewDataPtr[n])->writeContent(ofile, true, &crypt);
      }
    }
    else
    {
      
      curGroup.ptr->writeContent(ofile, true, &crypt, false);
    }
    ofile << "</data>\n";
    
    ofile.close();
  }
  return(0);
}


int         MuPO::exportToDoToFile(const char* fname) 
{
  std::ofstream ofile;
  std::string   line;
  
  ofile.open(fname, std::ios::binary);
  if (ofile)
  {
    ofile << "<type=MuPO>\n<version="MUPOVERSION">\n<data>\n";
    
    pCurItem->writeContent(ofile, true, &crypt);
    ofile << "</data>\n";
    
    ofile.close();
  }
  return(0);
}


int         MuPO::writeData(const char* fname) 
{
  std::ofstream ofile;
  std::string   line;
  
  ofile.open(fname, std::ios::binary);
  
  if (!ofile)
    return(-1);
  
  ofile << "<type=MuPO>\n<version="MUPOVERSION">\n<data>\n";
  ofile << "<idmax=" << uSyncIDMax << ">\n";
  ofile << "<idmaxaftersync=" << uSyncIDMaxAfterSync << ">\n";
  ofile << "<curGroup=" << *(curGroup.str) << ">\n";
  
  if (options.fShowKbd == true)
    ofile << "<ShowKbd>\n";
  
  switch (options.viewtype)
  {
    case flat:
      ofile << "<viewtype=flat>\n";
      break;
    case tree:
      ofile << "<viewtype=tree>\n";
      break;
  }
  if (options.mainview == MAINVIEW_NORMAL || options.mainview == MAINVIEW_CALENDAR)
    ofile << "<mainviewtype=" << options.mainview << ">\n";
  switch (options.sorttype)
  {
    case SORT_PRIORITY:
      ofile << "<sorttype=priority>\n";
      break;
    case SORT_DATE:
      ofile << "<sorttype=date>\n";
      break;
  }
  ofile << "<cal.date=" << (unsigned int)cal.date << ">\n";
  ofile << "<cal.days=" << (unsigned int)cal.days << ">\n";
  
  ofile << "<GUIData.nLineNormal=" << GUIData.nLineNormal << ">\n";
  ofile << "<GUIData.nLineCal=" << GUIData.nLineCal << ">\n";
  
  unsigned int nForMaxVal = GUIData.getListSize();
  for (unsigned int uCnt = 0; uCnt < nForMaxVal; uCnt++)
    ofile << "<GUIData.oldPosInList=" << GUIData.getPosAt(uCnt) << ">\n";
  
  if (session.uWinHeight != 0)
    ofile << "<Session.uWinHeight=" << session.uWinHeight << ">\n";
  if (session.uWinWidth != 0)
    ofile << "<Session.uWinWidth=" << session.uWinWidth << ">\n";
  
  crypt.Write(ofile);
  
  if (pRootGroup->writeContent(ofile, false, &crypt))
    std::cerr << "writeContent hat nicht Null zurückgegeben!\n";
  
  ofile << "</data>\n";
  
  return(0);
}


std::string MuPO::getCurGroupPathText() 
{
#ifdef DEBUGINITPATH
  std::cout << "<MuPO::getCurGroupPathText()\n";
#endif
  if (options.mainview == MAINVIEW_ONLYACTIVE)
    return("Active jobs");
  else
  {
    std::string tmp = *(curGroup.str);
    
    tmp.replace(0,1,"");
    tmp.replace(tmp.length()-1,1,"");
    
    return(pRootGroup->getPathText(tmp));
  }
}


void        MuPO::clearViewData() 
{
  if (options.mainview != MAINVIEW_SEARCHRESULTS)
    while (viewDataIndex.size())
    {
      if (viewData[viewDataIndex.size()-1].summary != (std::string*)0)
        delete viewData[viewDataIndex.size()-1].summary;
      viewData.pop_back();
      viewDataIndex.pop_back();
      viewDataPtr.pop_back();
    }
}



void        MuPO::getContentRecItems(t_datetime                  dtStart,
                                     t_datetime                  dtEnd,
                                     std::vector<ToDoListToDo*>* recList) 
{
  t_datetime       firstDay, firstDay2, day;
  t_datetime       lastDay,  lastDay2;
  T_viewDataItem   VDI;
  int              nItemCnt;
  bool             fWeitergesetzt;
  T_recData        rd;
  
  int nForMaxVal = recList->size();
  for (nItemCnt = 0; nItemCnt < nForMaxVal; nItemCnt++)
  {
    (*recList)[nItemCnt]->getRecData(rd);
    if ((!ISDATESET(rd.end) || rd.end >= dtStart)
        && (!ISDATESET(rd.begin) || rd.begin <= dtEnd))
    {
      firstDay = (*recList)[nItemCnt]->getDateTime(DATECODE_Begin);
      lastDay  = (*recList)[nItemCnt]->getDateTime(DATECODE_End);
      
      do
      {
        if (lastDay >= dtStart && firstDay <= dtEnd)
        {                                    
          std::string tmp;
          
          // insert item with given first/lastDay
          firstDay2 = firstDay;
          day = firstDay;
          CLEARTIME(day);
          if (firstDay2 < dtStart)                    
            firstDay2 = dtStart;
          
          lastDay2 = lastDay;
          CLEARTIME(firstDay2);
          CLEARTIME(lastDay2);
          (*recList)[nItemCnt]->setViewData(VDI, &crypt);
          VDI.fFirstDay = (day == firstDay2);
          
          tmp = VDI.summary->c_str();
          delete VDI.summary;
          
          while (firstDay2 <= dtEnd && firstDay2 <= lastDay2)
          {
            VDI.showDate = firstDay2;
            VDI.fLastDay = (lastDay2 == firstDay2);
            
            VDI.summary = new std::string(tmp);
            
            viewData.push_back(VDI);
            VDI.fFirstDay = false;
            viewDataPtr.push_back((*recList)[nItemCnt]);
            moveDate(firstDay2, 1);
          }
        }
        
        // set item to next recurring date
        fWeitergesetzt = ((*recList)[nItemCnt]->setFollowingRecurrenceDate(firstDay, lastDay) == 1);
      }
      while (firstDay <= dtEnd && fWeitergesetzt);
    }                    
  }  
}



void        MuPO::getContent() 
{
#ifdef DEBUGSNOW
  printf("MuPO::getContent() 1\n");
#endif
  if (options.mainview == MAINVIEW_CALENDAR)
  {
    t_datetime       today    = DATETIME_NODATE | DATETIME_NOTIME;
    t_datetime       firstDay;
    t_datetime       day;
    t_datetime       lastDay;
    T_viewDataItem   VDI;
    t_datetime       dtStart;
    t_datetime       dtEnd;
    unsigned int     nItemCnt;
    int              nDay, nMonth, nYear, nDOW;
    bool             fOverdueItems = false;
    bool             fFreeDay;
    unsigned int     uCnt;
    unsigned int     uForMaxVal;
    
    
#ifdef DEBUGSNOW
    printf("MuPO::getContent() c1\n");
#endif
    SETTIME(today, 0, 0);
    today = getTodayDate(today);
#ifdef DEBUGSNOW
    printf("MuPO::getContent() c2\n");
#endif
    
    // find items which are to be displayed in the given span
    dtStart = cal.date;
    SETTIME(dtStart, 0, 0);
    dtEnd   = cal.date;
    moveDate(dtEnd, cal.days-1);
    
    // notes
    nItemCnt = 0;
    if (cal.dateNotes.size())        
      while (nItemCnt < cal.dateNotes.size() && cal.dateNotes[nItemCnt].dt <= dtEnd)
      {
        if (cal.dateNotes[nItemCnt].item->getDateTime(DATECODE_End) >= dtStart)
        {
          std::string tmp;
          
          // This note appears somewhere in the span.
          // Create one entry for every day it appears in.
          firstDay = cal.dateNotes[nItemCnt].dt;
          day      = firstDay;
          CLEARTIME(day);
          if (firstDay < dtStart)
            firstDay = dtStart;
          lastDay = cal.dateNotes[nItemCnt].item->getDateTime(DATECODE_End);
          CLEARTIME(firstDay);
          CLEARTIME(lastDay);
          cal.dateNotes[nItemCnt].item->setViewData(VDI, &crypt);
          VDI.fFirstDay = (day == firstDay);
          
          tmp = VDI.summary->c_str();
          delete VDI.summary;
          
          while (firstDay <= dtEnd && firstDay <= lastDay)
          {
            VDI.showDate = firstDay;
            VDI.fLastDay = (lastDay == firstDay);
            
            VDI.summary = new std::string(tmp);
            
            viewData.push_back(VDI);
            VDI.fFirstDay = false;
            viewDataPtr.push_back(cal.dateNotes[nItemCnt].item);
            moveDate(firstDay, 1);
          }
          
        }
        nItemCnt++;
      }
#ifdef DEBUGSNOW
    printf("MuPO::getContent() c13\n");
#endif
    
    // appointments
    nItemCnt = 0;
    if (cal.dateApps.size())        
      while (nItemCnt < cal.dateApps.size() && cal.dateApps[nItemCnt].dt <= dtEnd)
      {
        if (cal.dateApps[nItemCnt].item->getDateTime(DATECODE_End) >= dtStart)
        {
          std::string tmp;
          
          // This note appears somewhere in the span.
          // Create one entry for every day it appears in.
          firstDay = cal.dateApps[nItemCnt].dt;
          day      = firstDay;
          CLEARTIME(day);
          if (firstDay < dtStart)
            firstDay = dtStart;
          lastDay = cal.dateApps[nItemCnt].item->getDateTime(DATECODE_End);
          CLEARTIME(firstDay);
          CLEARTIME(lastDay);
          cal.dateApps[nItemCnt].item->setViewData(VDI, &crypt);
          VDI.fFirstDay = (day == firstDay);
          
          tmp = VDI.summary->c_str();
          delete VDI.summary;
          
          while (firstDay <= dtEnd && firstDay <= lastDay)
          {                    
            VDI.showDate = firstDay;
            VDI.fLastDay = (lastDay == firstDay);
            
            VDI.summary = new std::string(tmp);
            
            viewData.push_back(VDI);
            VDI.fFirstDay = false;
            viewDataPtr.push_back(cal.dateApps[nItemCnt].item);
            moveDate(firstDay, 1);
          }
          
        }
        nItemCnt++;
      }
    
#ifdef DEBUGSNOW
    printf("MuPO::getContent() c23\n");
#endif
    // todos
    nItemCnt = cal.dateTodos.size()-1;
    if (cal.dateTodos.size())
      while (nItemCnt >= 0 && cal.dateTodos[nItemCnt].dt >= dtStart)
      {
        firstDay = cal.dateTodos[nItemCnt].item->getDateTime(DATECODE_Begin);
        if (!ISDATESET(firstDay) || firstDay <= dtEnd)
        {
          std::string tmp;
          
          // This note appears somewhere in the span.
          // Create one entry for every day it appears in.
          lastDay = cal.dateTodos[nItemCnt].dt;
          day     = lastDay;
          CLEARTIME(day);
          if (lastDay > dtEnd)
            lastDay = dtEnd;
          CLEARTIME(firstDay);
          CLEARTIME(lastDay);
          cal.dateTodos[nItemCnt].item->setViewData(VDI, &crypt);
          VDI.fLastDay = (day == lastDay);                
          
          tmp = VDI.summary->c_str();
          delete VDI.summary;
          
          while (lastDay >= dtStart && (lastDay >= firstDay || !ISDATESET(firstDay)))
          {
            VDI.showDate = lastDay;
            VDI.fFirstDay = (lastDay == firstDay);
            
            VDI.summary = new std::string(tmp);
            
            viewData.push_back(VDI);
            VDI.fLastDay = false;
            viewDataPtr.push_back(cal.dateTodos[nItemCnt].item);
            moveDate(lastDay, -1);
          }                
        }
        nItemCnt--;
      }
#ifdef DEBUGSNOW
    printf("MuPO::getContent() c33\n");
#endif
    
    // recurring items
    getContentRecItems(dtStart, dtEnd, &cal.recTodos);
    getContentRecItems(dtStart, dtEnd, &cal.recNotes);
    getContentRecItems(dtStart, dtEnd, &cal.recApps);      
    
#ifdef DEBUGSNOW
    printf("MuPO::getContent() c43\n");
#endif
    // ToDos anzeigen, die überfällig sind:
    if (dtStart == today)
    {
      VDI.showDate  = today-1;
      VDI.fFirstDay = false;
      VDI.fLastDay  = false;
      
      uForMaxVal = cal.dateTodos.size();
      for (nItemCnt = 0; nItemCnt < uForMaxVal; nItemCnt++)
        if (!cal.dateTodos[nItemCnt].item->isCompleted() && cal.dateTodos[nItemCnt].dt < today)
        {
          cal.dateTodos[nItemCnt].item->setViewData(VDI, &crypt);
          viewData.push_back(VDI);
          viewDataPtr.push_back(cal.dateTodos[nItemCnt].item);
          fOverdueItems = true;
        }
      
      uForMaxVal = cal.recTodos.size();
      for (nItemCnt = 0; nItemCnt < uForMaxVal; nItemCnt++)
        if (!cal.recTodos[nItemCnt]->isCompleted() 
            && cal.recTodos[nItemCnt]->getDateTime(DATECODE_End) < today)
        {
          cal.recTodos[nItemCnt]->setViewData(VDI, &crypt);
          viewData.push_back(VDI);
          viewDataPtr.push_back(cal.recTodos[nItemCnt]);
          fOverdueItems = true;
        }
    }
    
    if (fOverdueItems)
    {
      VDI.uPriority = PRIORITY_BLANK;
      VDI.summary   = new std::string("");
      VDI.showDate  = today-1;
      viewData.push_back(VDI);
      viewDataPtr.push_back((pToDoListItem)0);
    }
    
#ifdef DEBUGSNOW
    printf("MuPO::getContent() c53\n");
#endif
    // insert entries for the days            
    firstDay      = cal.date;
    SETTIME(firstDay, 0, 0);
    
    nDay   = GETDAY(firstDay);
    nMonth = GETMONTH(firstDay);
    nYear  = GETYEAR(firstDay);
    nDOW   = Wochentag(nDay, nMonth, nYear);
    
    VDI.fGroup    = false;
    VDI.fDone     = false;
    
    if (nYear != specDays->nCurYear)
    {
      specDays->clearList(specDays->var);
      specDays->clearList(specDays->varFree);
      specDays->fillVarLists(nYear);
    }
    
    for (int nDayCnt = 0; nDayCnt < cal.days; nDayCnt++)
    {
      VDI.showDate = firstDay;
      VDI.sortDate = firstDay;
      
      // check if there's something special about this day
      fFreeDay = false;
      VDI.uPriority = PRIORITY_DAYINFO;
      
      uForMaxVal = specDays->varFree.size();
      for (uCnt = 0; uCnt < uForMaxVal; uCnt++)
        if (specDays->varFree[uCnt].nDay == nDay && specDays->varFree[uCnt].nMonth == nMonth)
        {
          fFreeDay = true;
          viewDataPtr.push_back((pToDoListItem)0);
          VDI.summary = new std::string();
          VDI.summary->assign(std::string("@r")
                              + std::string(specDays->varFree[uCnt].descr));
          VDI.summary->insert(0, FREEDAYSTRING);                
          viewData.push_back(VDI);
        }
      
      uForMaxVal = specDays->fixedFree.size();
      for (uCnt = 0; uCnt < uForMaxVal; uCnt++)
        if (specDays->fixedFree[uCnt].nDay == nDay && specDays->fixedFree[uCnt].nMonth == nMonth)
        {
          fFreeDay = true;
          viewDataPtr.push_back((pToDoListItem)0);
          VDI.summary = new std::string();
          VDI.summary->assign(std::string("@r")
                              + std::string(specDays->fixedFree[uCnt].descr));
          VDI.summary->insert(0, FREEDAYSTRING);                
          viewData.push_back(VDI);
        }
      
      uForMaxVal = specDays->var.size();
      for (uCnt = 0; uCnt < uForMaxVal; uCnt++)
        if (specDays->var[uCnt].nDay == nDay && specDays->var[uCnt].nMonth == nMonth)
        {
          viewDataPtr.push_back((pToDoListItem)0);
          VDI.summary = new std::string();
          VDI.summary->assign(std::string("@r")
                              + std::string(specDays->var[uCnt].descr));
          if (fFreeDay)
            VDI.summary->insert(0, FREEDAYSTRING);                
          viewData.push_back(VDI);
        }
      
      uForMaxVal = specDays->fixed.size();
      for (uCnt = 0; uCnt < uForMaxVal; uCnt++)
        if (specDays->fixed[uCnt].nDay == nDay && specDays->fixed[uCnt].nMonth == nMonth)
        {
          viewDataPtr.push_back((pToDoListItem)0);
          VDI.summary = new std::string();
          VDI.summary->assign(std::string("@r")
                              + std::string(specDays->fixed[uCnt].descr));
          if (fFreeDay)
            VDI.summary->insert(0, FREEDAYSTRING);                
          viewData.push_back(VDI);
        }
      
      // put in day:
      viewDataPtr.push_back((pToDoListItem)0);
      VDI.uPriority = PRIORITY_NEWDAY;
      VDI.summary = new std::string();
      VDI.summary->assign(std::string("@r")
                          + std::string(dayName[nDOW])
                          + std::string(" ")
                          + itoStr(GETDAY(firstDay), '0', 2)
                          + "."
                          + itoStr(GETMONTH(firstDay), '0', 2) 
                          + "." 
                          + itoStr(GETYEAR(firstDay), '0', 4)
                          + "  "
                          + itoStr(Kalenderwoche(GETDAY(firstDay), GETMONTH(firstDay), GETYEAR(firstDay)), ' ', 1));
      if (nDOW == 6 || nDOW == 5 || fFreeDay)
        VDI.summary->insert(0, FREEDAYSTRING);
      if (firstDay == today)
        VDI.summary->insert(0, "@i");
      
      viewData.push_back(VDI);
      
      
      // insert a blank line
      VDI.uPriority = PRIORITY_BLANK;
      CLEARTIME(VDI.showDate);          
      CLEARTIME(VDI.sortDate);
      VDI.summary = new std::string("");
      viewData.push_back(VDI);
      viewDataPtr.push_back((pToDoListItem)0);
      
      
      // prepare for next day:
      moveDate(firstDay, 1);
      nDay   = GETDAY(firstDay);
      nMonth = GETMONTH(firstDay);
      ++nDOW;
      nDOW   = nDOW % 7;
      if (nYear != (int)GETYEAR(firstDay))
      {
        // Wie kam die folgende Zeile hierher? Vertippt? Cut&Paste-Fehler?
        // Gut, dass Wolfram Kuss sie bemerkt hat...(Mail 25.11.2001)
        //              nYear != GETYEAR(firstDay);
        specDays->clearList(specDays->var);
        specDays->clearList(specDays->varFree);
        specDays->fillVarLists(nYear);
      }          
    }
    
    
    uForMaxVal = viewData.size();
    for (unsigned int i=0; i<uForMaxVal; i++)
      viewDataIndex.push_back(i);
    
  }
  else
  {
    if (options.mainview == MAINVIEW_SEARCHRESULTS)
    {
      T_viewDataItem          tmp;
      
      // update data (necessary when items in the pSearchResGroup have been edited)
      while (viewDataIndex.size())
      {
        if (viewData[viewDataIndex.size()-1].summary != (std::string*)0)
          delete viewData[viewDataIndex.size()-1].summary;
        viewData.pop_back();
        viewDataIndex.pop_back();
      }
      unsigned int uForMaxVal = viewDataPtr.size(); 
      for (unsigned int i=0; i<uForMaxVal; i++)
      {
        ((ToDoListToDo*)viewDataPtr[i])->setViewData(tmp, &crypt);
        viewData.push_back(tmp);
        viewDataIndex.push_back(i);
      }      
    }
    else
    {
      if (options.mainview == MAINVIEW_ONLYACTIVE)
        pRootGroup->showContent("/0/", options, viewData, viewDataPtr, &crypt);
      else
        pRootGroup->showContent(*(curGroup.str), options, viewData, viewDataPtr, &crypt);
      
      unsigned int uForMaxVal = viewData.size(); 
      for (unsigned int i=0; i<uForMaxVal; i++)
        viewDataIndex.push_back(i);
    }
  }
}


bool        MuPO::cmpViewData(unsigned int d1, 
                              unsigned int d2) 
{
  // todo: consider due date
  /*
   if ((viewData[d1].fGroup != viewData[d2].fGroup) && options.viewtype == flat)
   {
   if (viewData[d1].fGroup && !viewData[d2].fGroup)
   return(true);
   else
   return(false);
   }
   else 
   {  */
  
  
  if ((viewData[d1].fDone != viewData[d2].fDone) && options.fCompletedLast)
  {
    if (viewData[d1].fDone && !viewData[d2].fDone)
      return(true);
    else
      return(false);
  }
  else
  {
    if (options.sorttype == SORT_DATE)
    {
      if (viewData[d1].sortDate != viewData[d2].sortDate)
      {
        if (viewData[d1].sortDate > viewData[d2].sortDate)
          return(true);
        else
          return(false);
      }
      else
      {
        if (viewData[d1].uPriority != viewData[d2].uPriority)
        {
          
          if (viewData[d1].uPriority > viewData[d2].uPriority)
            return(true);
          else
            return(false);
        }
        else
        {
          if (*viewData[d1].summary > *viewData[d2].summary)
            return(true);
          else
            return(false);
        }
      }
    }
    else         // SORT_PRIORITY
    {
      if (viewData[d1].uPriority != viewData[d2].uPriority)
      {
        if (viewData[d1].uPriority > viewData[d2].uPriority)
          return(true);
        else
          return(false);
      }
      else
      {
        if (viewData[d1].sortDate != viewData[d2].sortDate)
        {
          if (viewData[d1].sortDate > viewData[d2].sortDate)
            return(true);
          else
            return(false);
        }
        else
        {
          if (*viewData[d1].summary > *viewData[d2].summary)
            return(true);
          else
            return(false);
        }
      }
    }
  }    
  
  //    }
}


bool        MuPO::cmpViewDataCal(unsigned int d1, 
                                 unsigned int d2) 
{    
  if (viewData[d1].showDate != viewData[d2].showDate)
  {
    if (viewData[d1].showDate > viewData[d2].showDate)
      return(true);
    else
      return(false);
  }
  else
  {
    if (viewData[d1].uPriority != viewData[d2].uPriority)
    {
      if (viewData[d1].uPriority > viewData[d2].uPriority 
          || (viewData[d2].uPriority == PRIORITY_NEWDAY)
          || (viewData[d1].uPriority != PRIORITY_NEWDAY && viewData[d2].uPriority == PRIORITY_DAYINFO))
        return(true);
      else
        return(false);
    }
    else
    {
      if (viewData[d1].sortDate > viewData[d2].sortDate)
        return(true);
      else
        return(false);
    }
  }
}


void        MuPO::sortContent() 
{
  bool         fChanged = true;
  unsigned int uTmp;
  
  if (options.mainview == MAINVIEW_CALENDAR)
    while (fChanged)
    {
      fChanged = false;
      for (unsigned int i=0; i+1<viewDataIndex.size();i++)
        if (cmpViewDataCal(viewDataIndex[i], viewDataIndex[i+1]))
        {
          uTmp               = viewDataIndex[i];
          viewDataIndex[i]   = viewDataIndex[i+1];
          viewDataIndex[i+1] = uTmp;
          fChanged = true;
        }
    }    
  else
    while (fChanged)
    {
      fChanged = false;
      for (unsigned int i=0; i+1<viewDataIndex.size();i++)
        if (cmpViewData(viewDataIndex[i], viewDataIndex[i+1]))
        {
          uTmp               = viewDataIndex[i];
          viewDataIndex[i]   = viewDataIndex[i+1];
          viewDataIndex[i+1] = uTmp;
          fChanged = true;
        }
    }
}


int         MuPO::delCurGroup() 
{
  if (curGroup.ptr != pRootGroup && options.mainview == MAINVIEW_NORMAL)
  {
    ToDoListGroup* pTmp;
    std::ofstream  odat;
    
    // incsave:
    if (curGroup.ptr->hasBeenSaved())
    {
      incsave_getFile(odat);
      odat << "<deleted>\n" << curGroup.ptr->getSyncID() << "\n</deleted>\n";
      incsave_closeFile(odat);
    }
    
    pTmp = curGroup.ptr;
    leaveCurGroup();      
    curGroup.ptr->delGroup(pTmp, uNrActiveJobs, cal);
    
    pRootGroup->checkCompleted(); 
    return(0);
  }
  else
    return(-1);
}


int         MuPO::leaveCurGroup() 
{
  if (options.mainview == MAINVIEW_SEARCHRESULTS)
  {
    // todo: allow del/new/paste again
    pRootGroup->delGroup(pSearchResGroup, uNrActiveJobs, cal);
    pSearchResGroup = 0;
    curGroup.ptr  = preSearchGroup.ptr;
    *(curGroup.str)  = *(preSearchGroup.str);
    options.mainview = MAINVIEW_NORMAL;
#ifdef DEBUGGROUPS
    showGroups();
#endif
    return(0);
  }
  if (curGroup.ptr != pRootGroup && options.mainview == MAINVIEW_NORMAL)
  {
    curGroup.str->replace(curGroup.str->length()-1, 1, "");
    while (curGroup.str->at(curGroup.str->length()-1) != '/')
      curGroup.str->replace(curGroup.str->length()-1, 1, "");
    curGroup.ptr = pRootGroup->getGroupPtr(*(curGroup.str));
#ifdef DEBUGGROUPS
    showGroups();
#endif
    return(0);
  }
  else
    return(-1);
}


int         MuPO::turnCurItem2Group() 
{
  ToDoListGroup*  pGroup;
  ToDoListToDo*   pItem;
  t_datetime      abegin, aend;
  T_recData       arecdata;
  std::ofstream   odat;
  unsigned int    uParent;
  
#ifdef DEBUGGROUPS
  showGroups();
#endif
  if (options.mainview != MAINVIEW_NORMAL)
    return(-1);
  
  uParent = curGroup.ptr->getSyncID();
  
  // create new group and todo with data of the existent todo
  pGroup = curGroup.ptr->newGroup();
  pItem  = pGroup->newToDo();
  
  if (pCurItem->getSummary() != "")    
    pGroup->setSummary(pCurItem->getSummary());
  else
    pGroup->setSummary("<unnamed>");
  pGroup->setSyncID(++uSyncIDMax);
  
  
  // Daten besorgen:
  pCurItem->getRecData(arecdata);
  abegin = pCurItem->getDateTime(DATECODE_Begin);
  aend   = pCurItem->getDateTime(DATECODE_End);
  
  // Daten setzen:
  pItem->setSummary(pCurItem->getSummary());
  pItem->setDescription(pCurItem->GetDescription());
  pItem->setSyncID(++uSyncIDMax);
  for (unsigned int n=0; n<pCurItem->getNrTimes();n++)
    pItem->addTime(pCurItem->getTimeBegin(n), pCurItem->getTimeEnd(n));
  if (pCurItem->isRunning())
  {
    pItem->setRunningDirect(true);
    uNrActiveJobs++;
  }
  pItem->setCompletedDirect(pCurItem->getCompletedDirect());
  
  pItem->setPriority(pCurItem->getPriority(), cal);
  if (ISDATESET(aend))
    pItem->setDateDontCheck(DATECODE_End, GETYEAR(aend), GETMONTH(aend), GETDAY(aend), cal);
  if (ISTIMESET(aend))
    pItem->setTimeDontCheck(DATECODE_End, GETHOUR(aend), GETMINUTE(aend), cal);
  if (ISDATESET(abegin))
    pItem->setDateDontCheck(DATECODE_Begin, GETYEAR(abegin), GETMONTH(abegin), GETDAY(abegin), cal);
  if (ISTIMESET(abegin))
    pItem->setTimeDontCheck(DATECODE_Begin, GETHOUR(abegin), GETMINUTE(abegin), cal);
  pItem->setShowTimeAs(pCurItem->getShowTimeAs(), cal);
  pItem->setRecData(arecdata, cal);
  
  // incsave:
#ifdef DEBUGINCSAVE
  std::cout << "turn2group: delete one, save others\n";
#endif
  incsave_getFile(odat);
  odat << "<deleted>\n" << pCurItem->getSyncID() << "\n</deleted>\n";
  incsave_closeFile(odat);
  
  incsave_getFile(odat);
  // group and item are new
  odat << "<new parent=" << uParent << ">\n";
  
  pGroup->writeContent(odat, false, &crypt, true, true);
  odat << ">\n</new>\n";
  incsave_closeFile(odat);
  
  
  // delete old todo
  curGroup.ptr->delToDo(pCurItem, uNrActiveJobs, cal);
  
  
  
  
  setGroup(pGroup);
#ifdef DEBUGGROUPS
  showGroups();
#endif
  
  return(0);
}


int         MuPO::search(std::string toS) 
{
  std::vector<ToDoListToDo*>   vDPtr; 
  T_viewDataItem          tmp;
  
  if (options.mainview != MAINVIEW_NORMAL && options.mainview != MAINVIEW_SEARCHRESULTS)
    return(-1);
  
  if (options.mainview == MAINVIEW_NORMAL)
  {
    preSearchGroup.ptr = curGroup.ptr;
    *(preSearchGroup.str) = *(curGroup.str);
    // todo: inhibit del/new/paste
    // 
    clearViewData();
    // fill vector vDPtr:
    // don't fill in groups, only todos!!
    curGroup.ptr->findToDosWithText(toS, vDPtr, &crypt);
    
    pSearchResGroup = pRootGroup->newGroup();
    pSearchResGroup->setSummary("search results");
    
    setGroup(pSearchResGroup);
  }
  else
  {
    // fill vector vDPtr:
    // don't fill in groups, only todos!!            
    while (viewDataIndex.size())
    {
      if (((ToDoListToDo*)viewDataPtr[viewDataIndex.size()-1])->containsText(toS, &crypt))
        vDPtr.push_back((ToDoListToDo*)viewDataPtr[viewDataIndex.size()-1]);
      
      if (viewData[viewDataIndex.size()-1].summary != (std::string*)0)
        delete viewData[viewDataIndex.size()-1].summary;
      viewData.pop_back();
      viewDataIndex.pop_back();
      viewDataPtr.pop_back();
    }
  }
  
  
  // fill vectors viewDataXXX from vDPtr  
  
  while (vDPtr.size())
  {
    vDPtr[vDPtr.size()-1]->setViewData(tmp, &crypt);
    viewData.push_back(tmp);
    viewDataIndex.push_back(vDPtr.size()-1);
    viewDataPtr.push_back(vDPtr[vDPtr.size()-1]);
    vDPtr.pop_back();
  }
  options.mainview = MAINVIEW_SEARCHRESULTS;
  return(0);
}


void        MuPO::getGroupCompletedNr(unsigned int& uDone,
                                      unsigned int& uCnt) 
{
  curGroup.ptr->getCompletedNr(uDone, uCnt);
}


double      MuPO::getGroupTimeSum() 
{
  return(curGroup.ptr->getTimeSum());
}


double      MuPO::getItemTimeSum() 
{
  return(pCurItem->getTimeSum());
}


int         MuPO::newToDo(unsigned int uP, 
                          unsigned int uNum) 
{
  if (options.mainview == MAINVIEW_NORMAL)
  {
    setItem(curGroup.ptr->newToDo());
    pCurItem->setSyncID(++uSyncIDMax);
    return(0);
  }
  else
  {
    if (viewData.size() <= uNum)
      return(-1);
    
    if (options.mainview == MAINVIEW_CALENDAR && ISDATESET(viewData[uNum].showDate))
    {
      ToDoListGroup* pSomeGroup;
      int            day, month, year;
      
      // change to group which contains calendar entries      
      rememberGroup();
      
      pSomeGroup = pRootGroup->getGroupPtrFromSumPath("/", "/<unnamed>/calendar/");
      
      if (pSomeGroup != (ToDoListGroup*)0)
        setGroup(pSomeGroup);
      else
      {
        setGroup(pRootGroup);
        setGroup(curGroup.ptr->newGroup());
        curGroup.ptr->setSyncID(++uSyncIDMax);
        curGroup.ptr->setSummary("calendar");
      }
      setItem(curGroup.ptr->newToDo());
      pCurItem->setSyncID(++uSyncIDMax);
      
      day   = GETDAY(viewData[uNum].showDate);
      month = GETMONTH(viewData[uNum].showDate);
      year  = GETYEAR(viewData[uNum].showDate);
      setItemPriority(uP);          
      switch (uP)
      {
        case PRIORITY_APPOINTMENT:
          setDate(DATECODE_Begin, year, month, day);
          setDate(DATECODE_End,   year, month, day);
          setTime(DATECODE_Begin, 8, 0);
          setTime(DATECODE_End,   8, 30);
          setShowTimeAs(SHOW_AS_BUSY);
          break;
        case PRIORITY_NOTE:
          setDate(DATECODE_Begin, year, month, day);
          setDate(DATECODE_End,   year, month, day);
          setItemSummary("");
          break;
        default:
          setDate(DATECODE_End,   year, month, day);
          setDate(DATECODE_Begin, year, month, day);
          break;
      }
      
      enterPrevGroup();            
      
      return(0);
    }
    else
      return(-1);
  }
}


void        MuPO::setItem2ViewItem(int n) 
{
  setItem((ToDoListToDo*)(viewDataPtr[n]));
}


int         MuPO::setItemCompleted() 
{
  if (pCurItem->isRunning())
    uNrActiveJobs--;
  return(pCurItem->setCompleted(cal));
}


void        MuPO::setItemRunning(bool fR) 
{
  if (pCurItem->setRunning(fR) == 0)
  {
    if (fR)
      uNrActiveJobs++;
    else
      uNrActiveJobs--;
  }	 
}


int         MuPO::newGroup() 
{ 
  if (options.mainview == MAINVIEW_NORMAL)
  {
#ifdef DEBUGGROUPS
    showGroups();
#endif
    setGroup(curGroup.ptr->newGroup());
    curGroup.ptr->setSyncID(++uSyncIDMax);
#ifdef DEBUGGROUPS
    showGroups();
#endif
    return(0);
  }
  else
    return(-1);
}


bool        MuPO::toggleGroupShowCompleted() 
{
  curGroup.ptr->setShowCompleted(!curGroup.ptr->getShowCompleted());
  return(curGroup.ptr->getShowCompleted());
}


bool        MuPO::toggleGroupCompletedLast() 
{
  curGroup.ptr->setCompletedLast(!curGroup.ptr->getCompletedLast());
  return(curGroup.ptr->getCompletedLast());
}

ToDoListGroup* MuPO::GetParentOfItem(ToDoListItem*  pItem)
{
  return(pRootGroup->getGroupPtr(pItem->getPath()));
}

int         MuPO::delToDo() 
{
  if (options.mainview == MAINVIEW_NORMAL || options.mainview == MAINVIEW_CALENDAR)
  {
    DeleteItem(pCurItem);

    return(0);
  }
  else
    return(-1);  
}


int         MuPO::delCompleted() 
{
  unsigned int uCnt;
  
  for (uCnt = 0; uCnt < viewData.size(); uCnt++)
    if (viewData[uCnt].fDone && 
        !viewData[uCnt].fGroup && 
        viewData[uCnt].uPriority <= PRIORITY_TODOMAX)
    {
      if (DeleteItem(viewDataPtr[uCnt]))
        return(-1);
    }
  return(0);
}



int         MuPO::setMainViewType(int newtype) 
{
  int retVal;
  
  if (newtype != options.mainview)
    retVal = 0;
  else
    retVal = -1;
  
  switch (newtype)
  {
    case MAINVIEW_ONLYACTIVE:
      options.mainviewBeforeOnlyActive = options.mainview;
      if (uNrActiveJobs)
        options.mainview = MAINVIEW_ONLYACTIVE;
      break;
    case MAINVIEW_BEFOREONLYACTIVE:
      options.mainview = options.mainviewBeforeOnlyActive;
      break;
    default:
      options.mainview = newtype;
      break;
  }
  return(retVal);
}


void        MuPO::setGroup(ToDoListGroup* n) 
{ 
  curGroup.ptr = n;       
  *(curGroup.str) = curGroup.ptr->getPath();
  *(curGroup.str) += itoStr(curGroup.ptr->getID(), ' ', 0);
  *(curGroup.str) += "/";
#ifdef DEBUGGROUPS
  showGroups();
#endif
}


void        MuPO::setDate(int n, int year, int month, int day) 
{
  pCurItem->setDate(n, year, month, day, cal);
}


void        MuPO::setDateToday(int n) 
{
  time_t     curTime;
  // get current time
  curTime = time(0);
  
  tm*  curTimeBr;  
  curTimeBr = localtime(&curTime);
  
  // get currently set datetime
  pCurItem->setDate(n, curTimeBr->tm_year+1900, curTimeBr->tm_mon+1, curTimeBr->tm_mday, cal);
}


t_datetime  MuPO::getDateTime(int n) 
{
  return(pCurItem->getDateTime(n));
}


void        MuPO::clearDate(int n) 
{
  pCurItem->clearDate(n, cal);
}


void        MuPO::setTime(int n, int hour, int min) 
{
  pCurItem->setTime(n, hour, min, cal);
}


void        MuPO::clearTime(int n) 
{
  pCurItem->clearTime(n, cal);
}


void        MuPO::setCalSpan(int days) 
{
  cal.days = days;
}


int         MuPO::getCalSpan() 
{
  return(cal.days);
}


void        MuPO::setCalFirstDay(t_datetime date) 
{
  cal.date = date;
}


t_datetime  MuPO::getCalFirstDay() 
{
  return(cal.date);
}


void        MuPO::setItemPriority(unsigned int n) 
{
  pCurItem->setPriority(n, cal);   
}


unsigned int MuPO::getItemPriority() 
{
  return(pCurItem->getPriority());
}


void         MuPO::setItemSummary(std::string n) 
{ 
  pCurItem->setSummary(n);        
}


std::string  MuPO::getItemSummary() 
{
  return(pCurItem->getSummary()); 
}

int MuPO::getItemDescription(const char*& PtrToDescr)
{
  int nState = pCurItem->GetDescription(PtrToDescr, &crypt);
  
  if (nState == 0 && PtrToDescr == 0)
    return(-1);
  else
    return(nState);    
}


void         MuPO::setItemDescription(const char* pd) 
{
  pCurItem->setDescription(pd);          
}


int          MuPO::getShowTimeAs() 
{
  return(pCurItem->getShowTimeAs()); 
}


void         MuPO::setShowTimeAs(int n) 
{
  pCurItem->setShowTimeAs(n, cal);   
}


bool         MuPO::isItemCompleted() 
{
  return(pCurItem->isCompleted());      
}


time_t       MuPO::getItemCompletedTime() 
{
  return(pCurItem->getCompletedTime()); 
}


int          MuPO::clearItemCompleted() 
{
  return(pCurItem->clearCompleted());
}


bool         MuPO::isItemRunning() 
{
  return(pCurItem->isRunning());
}


int    MuPO::getItemNrTimes() 
{
  return(pCurItem->getNrTimes());
}


time_t MuPO::getItemTimeBegin(unsigned int n) 
{
  return(pCurItem->getTimeBegin(n));
}


time_t MuPO::getItemTimeEnd(unsigned int n) 
{
  return(pCurItem->getTimeEnd(n));
}


void   MuPO::getRecData(T_recData& recdata) 
{
  pCurItem->getRecData(recdata);
}


void   MuPO::setRecData(T_recData recdata) 
{
  pCurItem->setRecData(recdata, cal);
}


void   MuPO::clearRecData() 
{
  T_recData tmp;
  tmp.nType = RECTYPE_NONE;
  tmp.begin = INITIAL_DATETIME;
  tmp.end   = INITIAL_DATETIME;
  tmp.nInterval   = 1;
  pCurItem->setRecData(tmp, cal);
}


void   MuPO::sanitizeTimeThings() 
{
  pCurItem->sanitizeTimeThings();
}


void   MuPO::setGroup2ViewItem(int n) 
{
  setGroup((ToDoListGroup*)(viewDataPtr[n]));
#ifdef DEBUGGROUPS
  showGroups();
#endif
}


void         MuPO::setGroupPriority(unsigned int n) 
{
  curGroup.ptr->setPriority(n);        
}


unsigned int MuPO::getGroupPriority() 
{
  return(curGroup.ptr->getPriority()); 
}


void         MuPO::setGroupSummary(std::string n) 
{
  curGroup.ptr->setSummary(n);        
}


std::string  MuPO::getGroupSummary() 
{
  return(curGroup.ptr->getSummary()); 
}


bool         MuPO::getGroupShowCompleted() 
{ 
  return(curGroup.ptr->getShowCompleted()); 
}


bool         MuPO::getGroupCompletedLast() 
{
  return(curGroup.ptr->getCompletedLast()); 
}


void         MuPO::rememberGroup() 
{
  *(prevGroup.str) = *(curGroup.str);
  prevGroup.ptr = curGroup.ptr;
#ifdef DEBUGGROUPS
  showGroups();
#endif
}


void         MuPO::rememberGroupOfCurrentItem() 
{
  *(prevGroup.str) = pCurItem->getPath();
  prevGroup.ptr = pRootGroup->getGroupPtr(*(prevGroup.str));
#ifdef DEBUGGROUPS
  showGroups();
#endif
}


void         MuPO::enterPrevGroup() 
{
  *(curGroup.str) = *(prevGroup.str);
  curGroup.ptr = prevGroup.ptr;
#ifdef DEBUGGROUPS
  showGroups();
#endif
}


bool         MuPO::inRootGroup() 
{ 
  return((curGroup.ptr == pRootGroup) || (options.mainview == MAINVIEW_ONLYACTIVE)); 
}


void         MuPO::checkCompleted() 
{
  pRootGroup->checkCompleted(); 
}


unsigned int MuPO::getNrActiveJobs() 
{
  return(uNrActiveJobs);
}


bool         MuPO::getOptShowCompleted() 
{
  return(options.fShowCompleted);
}


int          MuPO::setOptShowCompleted(bool fNew) 
{
  options.fShowCompleted = fNew;
  return(0);
}


bool         MuPO::getOptShowKbd() 
{
  return(options.fShowKbd);
}


int          MuPO::setOptShowKbd(bool fNew) 
{
  options.fShowKbd = fNew;
  return(0);
}


bool         MuPO::getOptCompletedLast() 
{
  return(options.fCompletedLast);
}


int          MuPO::setOptCompletedLast(bool fNew) 
{
  options.fCompletedLast = fNew;
  return(0);
}


void         MuPO::setOptsSort(int item) 
{
  options.sorttype = item;
}


int          MuPO::getOptsSort() 
{
  return options.sorttype;
}


t_viewtype   MuPO::getOptViewtype() 
{
  return(options.viewtype);
}


int          MuPO::setOptViewtype(t_viewtype newtype) 
{
  options.viewtype = newtype;
  return(0);
}


int          MuPO::getMainViewType() 
{
  return(options.mainview);
}


void         MuPO::setItem(ToDoListToDo* n) 
{ 
  pCurItem = n; 
}



#ifdef CMDIMPORT
void MuPO::autoImport(std::string impFile, 
                      std::string impPath) 
{
  ToDoListGroup* pSomeGroup;
  
  pSomeGroup = pRootGroup->getGroupPtrFromSumPath("/", impPath);
  
  if (pSomeGroup != (ToDoListGroup*)0)
  {
    setGroup(pSomeGroup);
    insertDataFromFile(impFile.c_str());	   
  }
}

#endif


#ifdef DEBUGGROUPS
void MuPO::showGroup(string  name, 
                     T_Group item) 
{
  string somePath;
  
  if (item.ptr != (ToDoListGroup*) 0)
  {
    
    somePath = item.ptr->getPath();
    somePath += itoStr(item.ptr->getID(), ' ', 0);
    somePath += "/";
    
    std::cout << "Group: " << name << " : " << item.ptr->getSummary() << " " << item.ptr->getID() << "\n"
      << " Path: " << *(item.str) << "  "
      //        << pRootGroup->getPathText(item.ptr) << " "
      << somePath << "\n"
      << " Ptr : " << item.ptr
      << "  " << pRootGroup->getGroupPtr(*(item.str))
        << "\n";
  }
  else
    std::cout << "Group: " << name << " is not set!\n";
}


void MuPO::showGroups() 
{
  showGroup("prevGroup",      prevGroup);
  showGroup("preSearchGroup", preSearchGroup);
  showGroup("curGroup",       curGroup);
  std::cout << "\n";
}

#endif

void MuPO::incsave_rememberTodo()    
{
  incsave_forgetTodo();
  pIncSaveTodo = new ToDoListToDo(*pCurItem);
  fIncSaveNew = false;
}


void MuPO::incsave_forgetTodo()      
{
  if (pIncSaveTodo != (ToDoListToDo*)0)
  {
    delete pIncSaveTodo;  
    pIncSaveTodo = (ToDoListToDo*)0;
  }
}


bool MuPO::incsave_todoHasChanged()  
{
  if (pIncSaveTodo != (ToDoListToDo*)0 &&
      pCurItem     != (ToDoListToDo*)0)    
    return((*pIncSaveTodo != *pCurItem) || fIncSaveNew);
  else
    return(fIncSaveNew);
}


void MuPO::incsave_rememberGroup()   
{
  if (pIncSaveGroup == (ToDoListGroup*)0)
    pIncSaveGroup = new ToDoListGroup(*(curGroup.ptr));
  fIncSaveNew = false;
}


void MuPO::incsave_forgetGroup()     
{
  pIncSaveGroup = (ToDoListGroup*)0;
}


bool MuPO::incsave_groupHasChanged() 
{
  if (pIncSaveGroup != (ToDoListGroup*)0 &&
      curGroup.ptr  != (ToDoListGroup*)0)    
    return((*pIncSaveGroup != *(curGroup.ptr)) || fIncSaveNew);
  else
    return(fIncSaveNew);
}


void MuPO::incsave_newItem()         
{
  fIncSaveNew = true;
}


int  MuPO::incsave_getFile(std::ofstream& outfile)   
{
  unsigned int uCnt=0;
  std::string  tmp;
  
  while (existDat(filename+itoStr(uCnt, '0', 1)))
    uCnt++;
  
#ifdef DEBUGINCSAVE
  std::cout << "next file is " << filename+itoStr(uCnt, '0', 1) << "\n";
  std::cout.flush();
#endif
  
  tmp =  filename;
  tmp += itoStr(uCnt, '0', 1);
  outfile.open(tmp.c_str(), std::ios::binary);
  
  outfile << "<type=MuPO>\n<version="MUPOVERSION">\n<data>\n";
  return(0);
}


int  MuPO::incsave_closeFile(std::ofstream& outfile) 
{  
  outfile << "</data>\n";
  outfile.close();
  return(0);
}


int         MuPO::incsave_saveCurItem()  
{
  std::ofstream odat;
  
  incsave_getFile(odat);
  if (fIncSaveNew)
  {
    ToDoListGroup* pSomeGroup;
    
    if (options.mainview == MAINVIEW_CALENDAR)
      pSomeGroup = pRootGroup->getGroupPtrFromSumPath("/", "/<unnamed>/calendar/");
    else 
      pSomeGroup = curGroup.ptr;
    
    odat << "<new parent=" << pSomeGroup->getSyncID() << ">\n";
  }
  else
    odat << "<changed>\n";
  if (pCurItem != (ToDoListToDo*)0)
  {
    
    pCurItem->writeContent(odat, false, &crypt);
  }
  if (fIncSaveNew)
    odat << ">\n</new>\n";
  else
    odat << "</changed>\n";
  incsave_closeFile(odat);
  
#ifndef WIN32
# ifdef USESYNC  
  sync();
# endif
#endif  
  
  return(0);
}


int         MuPO::incsave_saveCurGroup() 
{
  std::ofstream odat;
  
  incsave_getFile(odat);
  if (fIncSaveNew)
  {
    T_Group pg;
    
    pg.str = new std::string("");
    
    *(pg.str) = curGroup.ptr->getPath();
    
    //      std::cout << "curGroup.ptr->getPath() = " << *(pg.str) << "\n";
              
    pg.ptr    = pRootGroup->getGroupPtr(*(pg.str));
    
    
    //      std::cout << "pg.ptr->getSyncID() = " << pg.ptr->getSyncID() << "\n";
      
    odat << "<new parent=" << pg.ptr->getSyncID() << ">\n";
    
    delete pg.str;
  }
  else
    odat << "<changed>\n";
  if (curGroup.ptr != (ToDoListGroup*)0)
  {
    
    curGroup.ptr->writeContent(odat, false, &crypt, true, false);
  }
  if (fIncSaveNew)
    odat << ">\n</new>\n";
  else
    odat << "</changed>\n";
  incsave_closeFile(odat);
  
#ifndef WIN32
# ifdef USESYNC  
  sync();
# endif
#endif  
  
  return(0);
}


int         MuPO::incsave_delFiles()     
{
  unsigned int uCnt=0;
  std::string  tmp;
  
  while (existDat(filename+itoStr(uCnt, '0', 1)))
  {
    tmp =  filename;
    tmp += itoStr(uCnt, '0', 1);      
    remove(tmp.c_str());
    
    uCnt++;
  }
  
  return(0);
}


unsigned int MuPO::incsave_getNrOfFiles()     
{
  unsigned int uCnt=0;
  
  while (existDat(filename+itoStr(uCnt, '0', 1)))
    uCnt++;
  
  return(uCnt);
}


int         MuPO::incsave_readFiles(std::string& errMsg) 
  // returnCode:
  //   0: no file read, everything ok
  //  -1: some error
  //   1: files read, everything ok
{
  unsigned int    uCnt=0;
  std::string     tmp;
  int             retCode = 0;
  std::ifstream   ifile;
  bool            fFileEnd;
  bool            fEnd = false;
  std::string     line = "";
  
  
  while (!fEnd)
  {
    tmp =  filename;
    tmp += itoStr(uCnt, '0', 1);
    
    ifile.open(tmp.c_str(), std::ios::binary);
    fFileEnd = false;
    if (ifile)
      while (!ifile.eof() && !fFileEnd)
      {
        line = readfLine(ifile);
        if (line.find("<new parent=") == 0) 
        {
          unsigned int   uParent;
          bool           fGroup;
          ToDoListGroup* pGroup;
          
          line.replace(line.length()-1, 1, "");
          uParent = atoi(line.c_str()+12);
#ifdef DEBUGINCSAVE
          std::cout << "new item(s) in group " << uParent << "\n";
          std::cout.flush();
#endif
          pGroup  = (ToDoListGroup*)(pRootGroup->getPtrToItemWithSyncID(uParent, fGroup, true));
          if (pGroup != (ToDoListGroup*)0)
          {
            if (pGroup->readContent(ifile, uNrActiveJobs, uSyncIDMax, cal))
            {
              
#ifdef DEBUGINCSAVE
              std::cout << "fehler bei readcontent\n";
#endif
              
              ifile.close();
              errMsg = tmp;
              return(-1);
            }
            else
            {
              line = readfLine(ifile);
              if (line != "</new>")
              {
                
#ifdef DEBUGINCSAVE
                std::cout << "line ist: " << line << "\n";
#endif
                
                ifile.close();
                errMsg = tmp;
                return(-1);
              }
            }
          }
          else
          {
            
#ifdef DEBUGINCSAVE
            std::cout << "parent nicht gefunden\n";
#endif
            
            ifile.close();
            errMsg = tmp;
            return(-1);
          }
          
          fFileEnd = true;
          retCode  = 1;
#ifdef DEBUGINCSAVE
          std::cout << "done\n";
          std::cout.flush();
#endif
        }
        
        
        if (line.find("<deleted>") == 0)    
        {
          unsigned int   uToDel;
          ToDoListItem*  pItem;
          bool           fGroup;
          std::string    group;
          ToDoListGroup* pGroup;
          
          line = readfLine(ifile);
          uToDel = atoi(line.c_str());
          
          line = readfLine(ifile);
          if (line != "</deleted>")
          {
            ifile.close();
            errMsg = tmp;
            return(-1);
          }
          
          
#ifdef DEBUGINCSAVE
          std::cout << "deleting item " << uToDel<< "\n";
          std::cout.flush();
#endif
          // find group where this item is in
          pItem  = pRootGroup->getPtrToItemWithSyncID(uToDel, fGroup, true);
          if (pItem != (ToDoListItem*)0)
          {
            group  = pItem->getPath();
            pGroup = pRootGroup->getGroupPtr(group);
            if (pGroup != (ToDoListGroup*)0)
            {
              if (fGroup)
                pGroup->delGroup((ToDoListGroup*)pItem, uNrActiveJobs, cal);
              else
                pGroup->delToDo((ToDoListToDo*)pItem, uNrActiveJobs, cal);
            }
          }
          else
          {
            ifile.close();
            errMsg = tmp;
            return(-1);
          }
          
          retCode  = 1;
          fFileEnd = true;
#ifdef DEBUGINCSAVE
          std::cout << "done\n";
          std::cout.flush();
#endif
        }
        
        
        if (line.find("<changed>") == 0)    
        {                
#ifdef DEBUGINCSAVE
          std::cout << "changed ";
          std::cout.flush();
#endif
          
          // item or group?
          line = readfLine(ifile);
          if (line[0] == 't')
          {
#ifdef DEBUGINCSAVE
            std::cout << "item ";
            std::cout.flush();
#endif
            // item has changed
            ToDoListToDo* pTodo;
            ToDoListToDo* pMTodo;
            unsigned int  uNrActiveJobsDummy;
            unsigned int  uSyncIDMaxDummy;
            T_Calendar    calDummy;
            bool          fGroup;
            
            pTodo = new ToDoListToDo(0, "");
            if (pTodo->readContent(ifile, uNrActiveJobsDummy, uSyncIDMaxDummy, calDummy))
            {
              ifile.close();
              errMsg = tmp;
              return(-1);
            }
            
            pMTodo = (ToDoListToDo*)(pRootGroup->getPtrToItemWithSyncID(pTodo->getSyncID(), fGroup, true));
            if (pMTodo != (ToDoListItem*)0)
            {
              pMTodo->removeFromCalLists(cal);
              pTodo->setID(pMTodo->getID());
              pTodo->setPath(pMTodo->getPath());                                     
              *pMTodo = *pTodo;
              pMTodo->putIntoCalList(cal);
            }
            else
            {
              ifile.close();
              errMsg = tmp;
              return(-1);
            }
            
            delete pTodo;
          }
          else
          {
#ifdef DEBUGINCSAVE
            std::cout << "group ";
            std::cout.flush();
#endif
            // group has changed
            ToDoListGroup* pGroup;
            ToDoListGroup* pMGroup;
            unsigned int   uNrActiveJobsDummy;
            unsigned int   uSyncIDMaxDummy;
            T_Calendar     calDummy;
            bool           fGroup;
            
            pGroup = new ToDoListGroup(0, "", 0);
            if (pGroup->readContent(ifile, uNrActiveJobsDummy, uSyncIDMaxDummy, calDummy))
            {
              ifile.close();
              errMsg = tmp;
              return(-1);
            }
            
            
            pMGroup = (ToDoListGroup*)(pRootGroup->getPtrToItemWithSyncID(pGroup->getSyncID(), fGroup, true));
            if (pMGroup != (ToDoListGroup*)0)
            {
              pGroup->setID(pMGroup->getID());
              pGroup->setPath(pMGroup->getPath());                    
              *pMGroup = *pGroup;
            }
            else
            {
              ifile.close();
              errMsg = tmp;
              return(-1);
            }
            
            
            delete pGroup;
          }
          
          retCode  = 1;
          fFileEnd = true;                
#ifdef DEBUGINCSAVE
          std::cout << "done\n";
          std::cout.flush();
#endif
        }
        
        
      }
    else
      fEnd = true;
    
    ifile.close();
    
    uCnt++;
  }
  
  pRootGroup->checkCompleted();
  
  return(retCode);
}


void   MuPO::windowSize(unsigned int& width, unsigned int& height) 
{
  if (width == 0)
  {
    width  = session.uWinWidth;
    height = session.uWinHeight;
  }
  else
  {
    session.uWinWidth  = width;
    session.uWinHeight = height;
  }
}

int MuPO::exportToStream(std::ostream& out, unsigned int idx)
{
  return(
         viewDataPtr[viewDataIndex[idx]]->writeContent(out, true, &crypt, true, true)
         );
}

int MuPO::DeleteItem(unsigned int idx)
{
  return(DeleteItem(viewDataPtr[viewDataIndex[idx]]));
}

int MuPO::DeleteItem(ToDoListItem*  pItem)
{  
  // incsave:
  {
    std::ofstream odat;
    if (pItem->hasBeenSaved())
    {
      incsave_getFile(odat);
      odat << "<deleted>\n" << pItem->getSyncID() << "\n</deleted>\n";
      incsave_closeFile(odat);
    }
  }
  
  ToDoListGroup* pGroup = dynamic_cast<ToDoListGroup*>(pItem);
  
  if (pGroup != 0)
  {
    curGroup.ptr->delGroup(pGroup, uNrActiveJobs, cal);
    return(0);
  }
  else
    return(GetParentOfItem(pItem)->delToDo(dynamic_cast<ToDoListToDo*>(pItem), uNrActiveJobs, cal));
}
