#ifndef __MUPO_H
# define __MUPO_H

# include <string>
# include <vector>

# include "types.h"
# include "tools.h"
# include "datetime.h"
# include "config.h"
# include "todolistitem.h"
# include "todolisttodo.h"
# include "todolistgroup.h"
# include "specdays.h"
# include "cryptman.h"


# define _DEBUGGROUPS
# ifdef DEBUGGROUPS
#  include <iostream>
# endif


# define DATETIME_DATE_SET 1
# define DATETIME_TIME_SET 2

struct T_Group{
  std::string*    str;      // a string like "/" or "/2/" or "/4/23/" or "/2/34/2/" or ...
  ToDoListGroup*  ptr;
};

class MuPO {
public:
  std::vector<T_viewDataItem>  viewData;        // things currently shown, unsorted
  std::vector<unsigned int>    viewDataIndex;   // things currently shown, sorted
  
  T_GUIData  GUIData;
  CryptMan   crypt;
  
  
  MuPO();
  ~MuPO();
  
  void   windowSize(unsigned int& width, unsigned int& height);
  int    checkFile(const char* fname);
  int    createRoot();
  int    readData(const char* fname);
  int    insertDataFromFile(const char* fname);
  int    writeData(const char* fname);
  int    exportGroupToFile(const char* fname);
  int    exportContentToFile(const char* fname);
  int    exportToDoToFile(const char* fname);
  
  /**
   * idx is index to viewDataIndex
   */
  int    exportToStream(std::ostream& out, unsigned int idx);

  std::string getCurGroupPathText();
  void   clearViewData();
  void   getContent();
  bool   cmpViewData(unsigned int d1, 
                     unsigned int d2);
  bool   cmpViewDataCal(unsigned int d1, 
                        unsigned int d2);
  void   sortContent();
  int    delCurGroup();
  int    leaveCurGroup();
  int    turnCurItem2Group();
  
  int    delCompleted();
  
  /**
   * idx is index to viewDataIndex
   */
  int DeleteItem(unsigned int idx);
  
  // todos
  int  newToDo(unsigned int uP = 0, unsigned int uNum = 0);
  
  /**
   * Das Element an der Stelle n der aktuellen Liste wird als aktuelles Element gesetzt.
   */
  void setItem2ViewItem(int n);
  
  void         setItemPriority(unsigned int n);
  unsigned int getItemPriority();
  void         setItemSummary(std::string n);
  std::string getItemSummary();
  
  /**
   * returns zero if pointer to description is returned,
   * otherwise:
   *  -1: description is empty
   *   1: Description has to be decrypted. PtrToDescr points
   *      to encrypted data.
   */
  int getItemDescription(const char*& PtrToDescr);

  void        setItemDescription(const char* pd);
  
  int    getShowTimeAs();
  void   setShowTimeAs(int n);
  
  void   setDate(int, int, int, int);
  void   setDateToday(int);
  t_datetime getDateTime(int);
  void   clearDate(int);
  
  void   setTime(int, int, int);
  void   clearTime(int);
  
  
  double getItemTimeSum();
  
  bool   isItemCompleted();
  time_t getItemCompletedTime();
  int    setItemCompleted();
  int    clearItemCompleted();
  
  bool isItemRunning();
  void setItemRunning(bool fR);   
  int getItemNrTimes();
  time_t getItemTimeBegin(unsigned int n);
  time_t getItemTimeEnd(unsigned int n);
  void getRecData(T_recData& recdata);
  void setRecData(T_recData recdata);
  void clearRecData();
  void sanitizeTimeThings();

  bool isItemSaveCrypt() { return(pCurItem->IsSaveCrypt()); };
  void SetItemSaveCrypt(bool fCrypt) { pCurItem->SetSaveCrypt(fCrypt); };
    
  // groups      
  int newGroup();
  void setGroup2ViewItem(int n);
  
  void         setGroupPriority(unsigned int n);
  unsigned int getGroupPriority();
  void         setGroupSummary(std::string n);
  std::string  getGroupSummary();
  void         getGroupCompletedNr(unsigned int& uDone, 
                                   unsigned int& uCnt);
  
  double getGroupTimeSum();
  
  bool getGroupShowCompleted();
  bool getGroupCompletedLast();
  
  bool toggleGroupShowCompleted();
  bool toggleGroupCompletedLast();
  
  void rememberGroup();
  
  void rememberGroupOfCurrentItem();
  
  void enterPrevGroup();
  
  // misc   
  int delToDo();   
  bool inRootGroup();
  
  void checkCompleted();
  
  int search(std::string toS);
  
  unsigned int getNrActiveJobs();
  
  bool       getOptShowCompleted();
  int        setOptShowCompleted(bool fNew);
  bool       getOptShowKbd();
  int        setOptShowKbd(bool fNew);
  bool       getOptCompletedLast();
  int        setOptCompletedLast(bool fNew);
  void       setOptsSort(int item);
  int        getOptsSort();
  t_viewtype getOptViewtype();
  int        setOptViewtype(t_viewtype newtype);
  
  int        getMainViewType();
  
  int        setMainViewType(int newtype);
  
  
  
  void setCalSpan(int);
  int  getCalSpan();
  
  void       setCalFirstDay(t_datetime);
  t_datetime getCalFirstDay();
  
  
# ifdef CMDIMPORT
  void autoImport(std::string impFile, 
                  std::string impPath);
# endif
  
# ifdef DEBUGGROUPS
  void showGroup(string  name, 
                 T_Group item);
  void showGroups();
# endif
  
  // incremental save
  void        incsave_rememberTodo();
  void        incsave_forgetTodo();
  bool        incsave_todoHasChanged();
  void        incsave_rememberGroup();
  void        incsave_forgetGroup();
  bool        incsave_groupHasChanged();
  void        incsave_newItem();
  int         incsave_getFile(std::ofstream& outfile);
  int         incsave_closeFile(std::ofstream& outfile);
  int         incsave_saveCurItem();
  int         incsave_saveCurGroup();
  int         incsave_delFiles();
  unsigned int incsave_getNrOfFiles();
  int         incsave_readFiles(std::string& errMsg);
  
protected:
  ToDoListGroup*               pRootGroup;
  ToDoListGroup*               pSearchResGroup;
  ToDoListToDo*                pCurItem;
  SpecialDays*                 specDays;
  
  std::vector<pToDoListItem>   viewDataPtr;   // things currently shown(pointers), unsorted
  unsigned int                 uNrActiveJobs;
  unsigned int                 uSyncIDMax;
  unsigned int                 uSyncIDMaxAfterSync;
  T_Options                    options;
  T_Calendar                   cal;
  T_Session                    session;
  
  T_Group                      prevGroup;
  T_Group                      preSearchGroup;
  T_Group                      curGroup;
  
  // incremental save
  ToDoListToDo*                pIncSaveTodo;
  ToDoListGroup*               pIncSaveGroup;
  bool                         fIncSaveNew;
  std::string                  filename;
  
  
  // misc
  void setGroup(ToDoListGroup* n);
  void setItem(ToDoListToDo* n);
  void getContentRecItems(t_datetime                  dtStart,
                          t_datetime                  dtEnd,
                          std::vector<ToDoListToDo*>* recList);
  
  
  ToDoListGroup* GetParentOfItem(ToDoListItem*  pItem);
  
  int DeleteItem(ToDoListItem*  pItem);
};

#endif
