// -*- mode: c; mode: fold -*-
// 
#include <stdio.h>
#include <stdlib.h>
#include <cstring>
#if defined( WIN32 ) && !defined(__CYGWIN__)
# include <io.h>
# include <fcntl.h>
# include <basetsd.h>
# include <sys\stat.h>
#else
# include <fcntl.h>
# include <unistd.h>
#endif

#if !defined( WIN32 ) && !defined ( __FreeBSD__ )
# include <sys/sysinfo.h>
#endif

#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/fl_ask.H>
#include <FL/fl_draw.H>

#include "mupogui.h"
#include "dateselect.h"
#include "timeselect.h"
#include "recurrence.h"
#include "config.h"
#include "version.h"
#include "pwd_ask.h"
#include "pwd_add.h"
#include "filetools.h"
#include "tools.h"
#include "configfile.h"




#define _DEBUGSNOW
#ifdef DEBUGSNOW
# include <stdio.h>
#endif

#define _DEBUGKBD
#ifdef DEBUGKBD
# include <iostream>
# include <stdio.h>
#endif

#define _DEBUGINCSAVE
#ifdef DEBUGINCSAVE
# include <iostream>
#endif

// images
#ifndef NOX11
# ifdef QUIKWRITING_1_0
#  include "lower_case.xpm"
#  include "upper_case.xpm"
#  include "numbers.xpm"
# else
#  include "lower_case2.xpm"
#  include "upper_case2.xpm"
#  include "numbers2.xpm"
# endif
# include "symbols.xpm"
# ifndef AQW_SIMPLEINTERFACE
#  include "lock.xpm"
#  include "ctl.xpm"
#  include "alt.xpm"
# endif
#endif


#define INPUTUSE_SEARCH     0
#define INPUTUSE_EXPGROUP   1
#define INPUTUSE_EXPCONT    2
#define INPUTUSE_INSFILE    3
#define INPUTUSE_TRANSLATE  4

extern MuPOGUI* mainWin;

#include "mupogui_cb1.cpp"
#include "mupogui_menu.cpp"
#include "mupogui_cb2.cpp"
#include "mupogui_menu2.cpp"



void kill_selection(Fl_Text_Editor* e) 
{
  if (e->buffer()->selected()) 
  {
    e->insert_position(e->buffer()->primary_selection()->start());
    e->buffer()->remove_selection();
  }
}

int kf_backspace_word(int, Fl_Text_Editor* e)
{
  if (!e->buffer()->selected())
  {
    Fl_Text_Buffer* buf = e->buffer();
    
    int nMoves = 0;
    bool fGo = true;
    int nStep = 1;
    int pos = e->insert_position() - 1;
    if (pos < 0)
      pos = 0;
    // delete until some word-character is left of cursor
    while (fGo && nStep)
    {
      if (isalpha(buf->character(pos)) == 0)
      {
        nStep = e->move_left();
        nMoves += nStep;
      }
      else
        fGo = false;
      pos = e->insert_position() - 1;
      if (pos < 0)
        pos = 0;
    }
    // delete until some none-word-character is left of cursor
    fGo = true;
    nStep = 1;
    while (fGo && nStep)
    {
      if (isalpha(buf->character(pos)) != 0)
      {
        nStep = e->move_left();
        nMoves += nStep;
      }
      else
        fGo = false;
      pos = e->insert_position() - 1;
      if (pos < 0)
        pos = 0;
    }
     
    if (nMoves)
      e->buffer()->select(e->insert_position(), e->insert_position()+nMoves);
  }
  
  kill_selection(e);
  
  e->show_insert_position();
  e->set_changed();
  if (e->when()&FL_WHEN_CHANGED)
    e->do_callback();
  return 1;
}

void cb_edit(Fl_Widget* widget, void* userdata)
{
  Fl_Text_Editor* e = (Fl_Text_Editor*)(userdata);
  Fl_Text_Buffer* b = e->buffer();
  
  int pos = e->insert_position() - 1;
  int linepos = 0;
  int nIndent = 0;
  {
    int pos_table_start = 0;
    int pos_notspace = 0;
    while (pos > 0 && b->character(pos) != '\n')
    {
      if (b->character(pos) == '|')
        pos_table_start = pos;
      if (isspace(b->character(pos)) == 0)
        pos_notspace = pos;
      linepos++;
      pos--;
    }
    
    if (pos+1 == pos_table_start)
      linepos = 0; // don't break table lines
    nIndent = pos_notspace - pos - 1;
  }
  
  if (linepos > 80)
  {
    pos = e->insert_position() - 1;
    while (pos > 0 && b->character(pos) != '\n' && b->character(pos) != ' ')
      pos--;
    if (b->character(pos) == ' ')
      b->replace(pos, pos+1, "\n");
    for (; nIndent>0; nIndent--)
      b->insert(pos+1, " ");         
  }
}

void MuPOGUI::initFakeKbd() 
{
#ifndef NOX11
  bool dummy1, dummy2;
  
  fakeKbd = new Fake_Keyboard(fl_display);
  
  fakeKbd->find(XK_Up,    &nKeyUp,    &dummy1, &dummy2);
  fakeKbd->find(XK_Down,  &nKeyDown,  &dummy1, &dummy2);
  fakeKbd->find(XK_Left,  &nKeyLeft,  &dummy1, &dummy2);
  fakeKbd->find(XK_Right, &nKeyRight, &dummy1, &dummy2);
  
  aqw  = new AQW(fakeKbd);
  
  
  letter_templates_[AQWMODE_LOWERCASE] = new Fl_Pixmap(lower_case_xpm);
# ifdef DEBUGKBD
  if (letter_templates_[AQWMODE_LOWERCASE] == (Fl_Pixmap*)0)
    std::cout << "pixmap-error!\n";
# endif  
  letter_templates_[AQWMODE_UPPERCASE] = new Fl_Pixmap(upper_case_xpm);
# ifdef DEBUGKBD
  if (letter_templates_[AQWMODE_UPPERCASE] == (Fl_Pixmap*)0)
    std::cout << "pixmap-error!\n";
# endif
  
  letter_templates_[AQWMODE_NUMBER] = new Fl_Pixmap(numbers_xpm);
# ifdef DEBUGKBD
  if (letter_templates_[AQWMODE_NUMBER] == (Fl_Pixmap*)0)
    std::cout << "pixmap-error!\n";
# endif
  
  letter_templates_[AQWMODE_SYMBOL] = new Fl_Pixmap(symbols_xpm);
# ifdef DEBUGKBD
  if (letter_templates_[AQWMODE_SYMBOL] == (Fl_Pixmap*)0)
    std::cout << "pixmap-error!\n";
# endif
  
# ifndef AQW_SIMPLEINTERFACE
  lock_image_ = new Fl_Pixmap(lock_xpm);
#  ifdef DEBUGKBD
  if (lock_image_ == (Fl_Pixmap*)0)
    std::cout << "pixmap-error!\n";
#  endif
  meta_image_ = new Fl_Pixmap(alt_xpm);
  ctrl_image_ = new Fl_Pixmap(ctl_xpm);
# endif
#endif
}

MuPOGUI::MuPOGUI(int w, int h, const char* l) : 
Fl_Window(w, h, l) 
{
  uTextSize1      = TEXTSIZE1;
  fGroupQuickView = false;
  nMainVisible    = MAINVISIBLE_MAIN;
  uWinWidth       = w/SIZEMUL;
  uWinHeight      = h/SIZEMUL;
  size_range(160*SIZEMUL, 160*SIZEMUL, 1000*SIZEMUL, 1000*SIZEMUL);
}

void MuPOGUI::resize(int X, int Y, int W, int H) 
{
  uWinWidth  = W/SIZEMUL;
  uWinHeight = H/SIZEMUL;
  
  //  Fl_Widget::resize(0,0,uWinWidth*SIZEMUL, uWinHeight*SIZEMUL);
  Fl_Window::resize(X,Y,uWinWidth*SIZEMUL, uWinHeight*SIZEMUL);
  
  uTotalWidth  = uWinWidth;
  uTotalHeight = uWinHeight;
  
  // Remember as 'session data'
  mupo->windowSize(uTotalWidth, uTotalHeight);
  
  if (mupo->getOptShowKbd() && uWinHeight > 60
      && nMainVisible != MAINVISIBLE_MAIN
      && nMainVisible != MAINVISIBLE_ABOUT)
  {
    uWinHeight -= AQWSIZE/SIZEMUL;
#ifndef NOX11
    aqw->offset(AQWBRDRLEFT+(AQWSIZE/2), (uWinHeight*SIZEMUL)+(AQWSIZE/2));
#endif
  }
  
#ifndef NOX11
  KbdButtonLeft->resize(0, (uWinHeight*SIZEMUL)+15, AQWBRDRLEFT / 2, AQWBRDRLEFT / 2);
  KbdButtonRight->resize(AQWBRDRLEFT / 2, (uWinHeight*SIZEMUL)+15, AQWBRDRLEFT / 2, AQWBRDRLEFT / 2);
#endif
  hideKbdB->resize(0, uWinHeight*SIZEMUL, 15, 15);
  curDataBox->resize(AQWBRDRLEFT+AQWSIZE+AQWBRDRRIGHT,                     uWinHeight*SIZEMUL,
                     (uWinWidth*SIZEMUL)-AQWBRDRLEFT+AQWSIZE+AQWBRDRRIGHT, AQWSIZE);
  
  
  
  
  inputBox->resize(0, 0, uWinWidth*SIZEMUL, (uWinHeight - 30)*SIZEMUL);
  inputCancelB->resize(35*SIZEMUL, (uWinHeight-15)*SIZEMUL, 40*SIZEMUL, 15*SIZEMUL);
  inputLine->resize(0,(uWinHeight-30)*SIZEMUL,uWinWidth*SIZEMUL,15*SIZEMUL);
  
  
  
  
  nHeadHeight = 0;
  
  if (mupo->getMainViewType() != MAINVIEW_CALENDAR)
  {
    headstring = mupo->getCurGroupPathText();      
    if (headstring.length())
    {
      nHeadHeight = 1;
      unsigned int uForMaxVal = headstring.length();
      for (unsigned int i=0; i<uForMaxVal; i++)
        if (headstring[i] == '\n')
          nHeadHeight++;
    }
    nHeadHeight *= 13;
    headline->label(headstring.c_str());
  }
  
  headline->resize(0, 0, uWinWidth*SIZEMUL, nHeadHeight*SIZEMUL);
  if (nMainVisible == MAINVISIBLE_MAIN)
  {
    listView->resize(0, nHeadHeight*SIZEMUL, uWinWidth*SIZEMUL, (uWinHeight-15-nHeadHeight)*SIZEMUL);
    headline->redraw();
    listView->redraw();
  }
  
  
  mainAboutBox ->resize(0, 0, uWinWidth*SIZEMUL, (uWinHeight - 15)*SIZEMUL);
  mainMenuB->resize (0*SIZEMUL, (uWinHeight-15)*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL);
  mainDoneB->resize(0*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL);
  mainNewB->resize (33*SIZEMUL, (uWinHeight-15)*SIZEMUL, 50*SIZEMUL, 15*SIZEMUL);
  mainUpB->resize   (83*SIZEMUL, (uWinHeight-15)*SIZEMUL, 20*SIZEMUL, 15*SIZEMUL);
  mainActiveB->resize((uWinWidth-10)*SIZEMUL, (uWinHeight-15)*SIZEMUL, 10*SIZEMUL, 15*SIZEMUL);
  
  editSumLine->resize(0,0,uWinWidth*SIZEMUL,15*SIZEMUL);
  
  editToDoIsToDoCB->resize((uWinWidth-40)*SIZEMUL,15*SIZEMUL,40*SIZEMUL,15*SIZEMUL);
  
  editPrGr->resize(0,30*SIZEMUL,uWinWidth*SIZEMUL, 20*SIZEMUL);
  for (unsigned int i=0;i<PRIORITY_TODOMAX;i++)
    editPrB[i]->resize(25*i*SIZEMUL, 33*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL);
  
  editToDoDoneCB->resize(0,50*SIZEMUL,40*SIZEMUL,15*SIZEMUL);
  editToDoStateBox->resize(40*SIZEMUL, 50*SIZEMUL, 120*SIZEMUL, 15*SIZEMUL);
  
  if (nMainVisible == MAINVISIBLE_GROUP)
    editTimeSumBox->resize(0*SIZEMUL, 65*SIZEMUL, 120*SIZEMUL, 15*SIZEMUL);
  else
    editTimeSumBox->resize(0*SIZEMUL, 0*SIZEMUL, 120*SIZEMUL, 15*SIZEMUL);  
  
  editToDoDescr  ->resize(0,0,uWinWidth*SIZEMUL, (uWinHeight-15)*SIZEMUL);
#ifndef NOX11
  editToDoDescrUpB->resize((uWinWidth-30)*SIZEMUL,(uWinHeight-15)*SIZEMUL, 15*SIZEMUL, 15*SIZEMUL);
  editToDoDescrDwnB->resize((uWinWidth-15)*SIZEMUL,(uWinHeight-15)*SIZEMUL, 15*SIZEMUL, 15*SIZEMUL);
#endif
  editToDoMenuB->resize(35*SIZEMUL, (uWinHeight-15)*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL);
  editToDoOtherB->resize(68*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL);
  editToDoStrtStpB->resize(103*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL);
  editToDoExtEditB->resize(138*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL);
  
  editToDoBegDateB->resize(0, 70*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL);
  editToDoEndDateB->resize(0, 85*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL);
  editToDoBegTimeB->resize(33*SIZEMUL,70*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL);
  editToDoEndTimeB->resize(33*SIZEMUL,85*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL);
  editToDoBegBox  ->resize(66*SIZEMUL, 70*SIZEMUL, (160-66)*SIZEMUL, 15*SIZEMUL);
  editToDoEndBox  ->resize(66*SIZEMUL, 85*SIZEMUL, (160-66)*SIZEMUL, 15*SIZEMUL);
  editToDoShowAsBusy->resize(50*SIZEMUL, 100*SIZEMUL, 70*SIZEMUL, 15*SIZEMUL);
  editToDoRecurringB->resize(0*SIZEMUL,115*SIZEMUL,160*SIZEMUL,15*SIZEMUL);
#ifdef REMINDER
  editToDoReminderB->resize(0*SIZEMUL,130*SIZEMUL,160*SIZEMUL,15*SIZEMUL);
#endif  
  editToDoCryptCB->resize(0*SIZEMUL,145*SIZEMUL,160*SIZEMUL,15*SIZEMUL);
  if (nMainVisible == MAINVISIBLE_TODO_TIMES)
    editToDoTimeList->resize(0, 15*SIZEMUL, uWinWidth*SIZEMUL, (uWinHeight-15-15)*SIZEMUL);
  
  editGroupMenuB->resize(35*SIZEMUL, (uWinHeight-15)*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL);
  editGroupComplDetailBox->resize(0*SIZEMUL, 80*SIZEMUL, 120*SIZEMUL, 15*SIZEMUL);
  
  mainCalTodayB->resize(33*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL);
  mainCalSelB->resize(68*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL);
  mainCalPrevLB->resize(103*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL);
  mainCalPrevB->resize(114*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL);
  mainCalNextB->resize(125*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL);    
  mainCalNextLB->resize(136*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL);
}

void MuPOGUI::draw() 
{
#ifndef NOX11
  if (!aqw->needToRedraw())
#endif
    Fl_Window::draw();
  
#ifndef NOX11
  drawKbd();
#endif
}

#ifndef NOX11
void MuPOGUI::drawKbd() 
{
# ifdef DEBUGKBD
  std::cout << aqw->inputMode() << "\n";
  std::cout.flush();
# endif
  
  if (mupo->getOptShowKbd() == true)
    letter_templates_[aqw->inputMode()]->draw(AQWBRDRLEFT, uWinHeight*SIZEMUL);
  
# ifdef DEBUGKBD
  std::cout << "letter template done\n";
  std::cout.flush();
# endif
  
# ifndef AQW_SIMPLEINTERFACE  
  if (aqw->inputLock() == true)
  {
#  ifdef DEBUGKBD
    std::cout << "draw lock...";
    std::cout.flush();
#  endif
    if (lock_image_ == (Fl_Pixmap*)0)
    {
#  ifdef DEBUGKBD
      std::cout << "lock pixmap is NULL!!!!";
      std::cout.flush();
#  endif
    }
    else          
      lock_image_->draw(AQWBRDRLEFT+(AQWSIZE/2)-4, (uWinHeight*SIZEMUL)+(AQWSIZE/2)-4);
#  ifdef DEBUGKBD
    std::cout << "..done\n";
    std::cout.flush();
#  endif
  }
  
  if (aqw->meta() == true)
    meta_image_->draw(AQWBRDRLEFT+42, (uWinHeight*SIZEMUL)+48);
  
  if (aqw->ctrl() == true)
    ctrl_image_->draw(AQWBRDRLEFT+7, (uWinHeight*SIZEMUL)+48);
# endif
# ifdef DEBUGKBD
  std::cout << "draw done\n";
  std::cout.flush();
# endif
  aqw->redrawDone();
}
#endif

void MuPOGUI::OpenItem(int idx, bool fShowDescrIfExists)
{
  mupo->setItem2ViewItem(idx);
  
  int nDescrState; 
  {
    const char* pPtrToDescr;
    nDescrState = mupo->getItemDescription(pPtrToDescr);
  }            
  if (nDescrState == 1)
  {
    // Wenn man auf was verschlüsseltes geklickt hat, darf der Klick nicht
    // wirksam werden. Zuerst muss man sich erfolgreich einloggen.
    // Also hier einen Dlg mit Combobox (Nummer + Name) und Eingabefeld (Pwd) anbieten.
    // Das Eingabefeld soll ruhig mehrzeilig sein (Zeilenvorschub in langen Passwörtern/Sätzen ist toll)
    // und lesbar. Das hilft bei Fehleingaben und außerdem wird nach der Eingabe ohnehin was 'geheimes'
    // sichtbar. Vielleicht ein großes Klartextfeld und ein kleines Passwortfeld. Dasjenige mit Inhalt
    // wird benutzt.
    // Danach Entschlüsselungsversuch.
    pwd_ask_run_window(&(mupo->crypt));
    UpdateCryptMenu();
  }
  else
  {
    // INCSAVE: save data to compare it afterwards
    mupo->incsave_rememberTodo();
    
    show_editToDoWin(fShowDescrIfExists);
  }
}

int MuPOGUI::handle(int event) 
{
  int wanted = 0;
  
  if (event == FL_KEYBOARD)
  {
    switch (nMainVisible)
    {
      case MAINVISIBLE_MAIN:
        {
          int nSelectedLine = listView->value();
          int nIdx;
          
          wanted = 1;
          
          switch (Fl::event_key())
          {
            case FL_Up:
            case 65365: // page up
              if (nSelectedLine > 1)
              {
                listView->select(nSelectedLine,   0);
                listView->show(nSelectedLine-1);
                listView->select(nSelectedLine-1, 1);
              }
              break;            
            case FL_Down:
            case 65366: // page down
              if (nSelectedLine < listView->size())
              {
                listView->select(nSelectedLine,   0);
                listView->show(nSelectedLine+1);
                listView->select(nSelectedLine+1, 1);
              }
              break;
            case FL_Enter: // toggle
              if (nSelectedLine != 0 && mupo->viewData.size() != 0)
              {
                nIdx = mupo->viewDataIndex[nSelectedLine-1]+1;
                
                switch (mupo->viewData[nIdx-1].uPriority)
                {
                  case PRIORITY_BLANK:
                  case PRIORITY_NEWDAY:
                  case PRIORITY_DAYINFO:
                    break;
                  default:
                    if (!mupo->viewData[nIdx-1].fGroup)
                    {
                      toggleCompleted(nIdx-1, true);
                    }
                    break;
                }
              }
              break;        
            case FL_Right: // into group, open item
              if (nSelectedLine != 0 && mupo->viewData.size() != 0)
              {
                nIdx = mupo->viewDataIndex[nSelectedLine-1]+1;
                
                switch (mupo->viewData[nIdx-1].uPriority)
                {
                  case PRIORITY_BLANK:
                  case PRIORITY_NEWDAY:
                  case PRIORITY_DAYINFO:
                    break;
                  default:
                    if (mupo->viewData[nIdx-1].fGroup)
                    {
                      enterGroup(nIdx-1, true);
                    }
                    else
                    {
                      OpenItem(nIdx-1, false);
                    }
                    break;
                }
              }
              break;
            case FL_Left:  // group up
              CB_Up();
              break;
            default:
              {
                if (Fl::event_key(FL_Alt_L) || Fl::event_key(FL_Alt_R))
                {
                  switch (Fl::event_key())
                  {
                    case 'n': // new item
                      {
                        CB_Menu_newTodo();
                      }
                      break;
                    case 'm': // open menu
                      {
                        mainMenuB->popup();
                      }
                      break;
                  }
                }
                else if (Fl::event_key(FL_Control_L) || Fl::event_key(FL_Control_R))
                {
                  switch (Fl::event_key())
                  {
                    case 'q': // quit
                      {
                        CB_Done();
                      }
                      break;
                  }
                }
              }
              break;
          }
        }
        break;
        
      case MAINVISIBLE_TODO_BASIC:
      case MAINVISIBLE_TODO_TIMES:
      case MAINVISIBLE_TODO_DESCR:
        {
          if (Fl::event_key(FL_Alt_L) || Fl::event_key(FL_Alt_R))
          {
            switch (Fl::event_key())
            {
              case 'e':
                {
                  wanted = 1;
                  CB_editToDo_ExtEdit();
                }
                break;
              case 'm': // open menu
                {
                  editToDoMenuB->popup();
                }
                break;
              case 'c': // close
                {
                  CB_Done();
                }
                break;
            }
          }
          
        }
        break;
    }
  }
  else
  {
    wanted = Fl_Window::handle(event);
#ifndef NOX11
    if (!wanted)
    {
      wanted = aqw->handle(event);
      
      if (aqw->needToRedraw())
        redraw();
    }
#endif
  }
  
  return wanted;
}

void MuPOGUI::hideAll() 
{
  // ************************ main *******************
  // isn't there something like hide()???
  listView->resize(0,0,0,0);
  
  headline->hide();
  mainAboutBox->hide();    
  mainDoneB->hide();
  mainMenuB->hide();
  mainNewB->hide();  
  mainUpB->hide();
  mainActiveB->hide();
  
#ifndef NOX11
  KbdButtonLeft->hide();
  KbdButtonRight->hide();
#endif
  hideKbdB->hide();  
  curDataBox->hide();
  
  inputCancelB->hide();
  inputLine->hide();
  inputBox->hide();
  
  mainCalTodayB->hide();
  mainCalSelB->hide();
  mainCalPrevLB->hide();
  mainCalNextLB->hide();
  mainCalPrevB->hide();
  mainCalNextB->hide();
  
  
  // ******************** ToDo ************************
#ifndef NOX11
  editToDoDescrDwnB->hide();
  editToDoDescrUpB->hide();
#endif
  editToDoDescr->hide();
  editSumLine->hide();
  editPrGr->hide();
  editToDoStrtStpB->hide();
  editToDoExtEditB->hide();
  editToDoStateBox->hide();
  editToDoMenuB->hide();
  editToDoOtherB->hide();
  editToDoDoneCB->hide();
  editToDoIsToDoCB->hide();
  editToDoShowAsBusy->hide();
  editTimeSumBox->hide();
  editToDoBegDateB->hide();
  editToDoEndDateB->hide();
  editToDoEndTimeB->hide();  
  editToDoBegTimeB->hide();
  editToDoEndBox->hide();  
  editToDoBegBox->hide();
  editToDoRecurringB->hide();
#ifdef REMINDER
  editToDoReminderB->hide();
#endif
  editToDoCryptCB->hide();
  // isn't there something like hide()???
  editToDoTimeList->resize(0, 0, 0,0);
  
  // ******************** Group ************************
  editSumLine->hide();  
  editPrGr->hide();
  editGroupMenuB->hide();
  mainDoneB->hide();
  editTimeSumBox->hide();  
  editGroupComplDetailBox->hide();
}

int MuPOGUI::showMain(int nNewView) 
{
  int retVal = 0;
  
  hideAll();
  nMainVisible = nNewView;
  
  switch (nMainVisible)
  {
    case MAINVISIBLE_MAIN : 
      mainMenuB->show();
      mainNewB->show();
      mainUpB->show();
      headline->show();
      
      retVal = checkActiveJobs();
      break;
      
      
    case MAINVISIBLE_ABOUT : 
      mainAboutBox->show();
      mainDoneB->show();
      break;
      
      
    case MAINVISIBLE_INPUT : 
      inputCancelB->show();
      inputLine->show();
      inputBox->show();
      mainDoneB->show();
      inputLine->take_focus(); 
      break;
      
      
    case MAINVISIBLE_TODO_BASIC : 
      menu_EditToDoMenu[RB_MENU_BASICS].setonly();
      
      editToDoOtherB->label("Descr");
      editToDoOtherB->callback ((Fl_Callback *) cb_showScreen, (void*)MAINVISIBLE_TODO_DESCR);
      
      editSumLine->show();
      if (mupo->getItemPriority() <= PRIORITY_TODOMAX)
      {
        editPrGr->show();
        editToDoDoneCB->show();
        editToDoStateBox->show();
      }
      editToDoIsToDoCB->show();
      editToDoShowAsBusy->show();
      editToDoBegDateB->show();
      editToDoEndDateB->show();
      editToDoEndTimeB->show();
      editToDoBegTimeB->show();
      editToDoEndBox->show();
      editToDoBegBox->show();
      editToDoRecurringB->show();
#ifdef REMINDER
      editToDoReminderB->show();
#endif
      editToDoExtEditB->show();
      editToDoCryptCB->show();
      if (mupo->crypt.IsLoggedIn())
        editToDoCryptCB->activate();
      else
        editToDoCryptCB->deactivate();
      if (!mupo->isItemCompleted())
      {
        if (mupo->isItemRunning())
          editToDoStrtStpB->label("&Stop");
        else
          editToDoStrtStpB->label("&Start");
        editToDoStrtStpB->show();
      }
      
      
      mainDoneB->show();
      editToDoMenuB->show();
      editToDoOtherB->show();      
      editToDo_updateTimeThings();
      editToDo_updateRecThings();
      
      editSumLine->take_focus(); 
      break;
      
      
    case MAINVISIBLE_TODO_TIMES : 
      {
        menu_EditToDoMenu[RB_MENU_TIME].setonly();
        
        editToDoTimeList->clear();
        
        for (int n=0;n<mupo->getItemNrTimes();n++)
	{
	  tm* time2;
	  time_t time1;
	  char   time3[30];
          
	  time1 = mupo->getItemTimeBegin(n);
	  time2 = localtime(&time1);
	  strftime(time3, 42, DATEFORMATSTR" "TIMEFORMATSTR, time2); 
	  editToDoTimeList->add(time3);
	  if (n==mupo->getItemNrTimes()-1 && mupo->isItemRunning())
	    editToDoTimeList->add("   running");
	  else
          {	 
            time1 = mupo->getItemTimeEnd(n);
            time2 = localtime(&time1);
            strftime(time3, 42, "      -> "DATEFORMATSTR" "TIMEFORMATSTR, time2); 
            editToDoTimeList->add(time3);
          }
	}
        
        
        mainDoneB->show();
        editToDoMenuB->show();
        editToDoOtherB->label("Basics");
        editToDoOtherB->callback ((Fl_Callback *) cb_showScreen, (void*)MAINVISIBLE_TODO_BASIC);
        editToDoOtherB->show();      
        editTimeSumBox->show();
        editToDoExtEditB->show();
      }
      break;
      
      
    case MAINVISIBLE_TODO_DESCR : 
      menu_EditToDoMenu[RB_MENU_DESCR].setonly();
      
      editToDoOtherB->label("Basics");
      editToDoOtherB->callback ((Fl_Callback *) cb_showScreen, (void*)MAINVISIBLE_TODO_BASIC);
      
      editToDoDescr->show();
#ifndef NOX11
      editToDoDescrDwnB->show();
      editToDoDescrUpB->show();
#endif
      mainDoneB->show();
      editToDoMenuB->show();
      editToDoOtherB->show();
      editToDoExtEditB->show();
      break;
      
      
    case MAINVISIBLE_GROUP : 
      editSumLine->value(mupo->getGroupSummary().c_str(),mupo->getGroupSummary().length());
      editPrB[mupo->getGroupPriority()-1]->setonly();
      
      std::string       tmp;
      unsigned int      uTmp, uTmp2;
      
      uTmp = (unsigned int)(mupo->getGroupTimeSum() / 60); 
      tmp = "Time: " + itoStr(uTmp / 60, ' ', 0) + ":" + itoStr(uTmp % 60, '0', 2);
      strncpy(editTimeSum, tmp.c_str(), 20);
      editTimeSumBox->label(editTimeSum);
      editTimeSumBox->show();
      
      mupo->getGroupCompletedNr(uTmp, uTmp2);
      tmp = itoStr(uTmp, ' ', 0) + "/" + itoStr(uTmp2, ' ', 0) 
        + " (";
      if (uTmp2)
        tmp += itoStr(100*uTmp/uTmp2, ' ', 0) + " %) done";
      else
        tmp += " 100%) done";
      
      strncpy(editGroupComplDetail, tmp.c_str(), 30);
      editGroupComplDetailBox->label(editGroupComplDetail);
      editGroupComplDetailBox->show();
      
      editSumLine->show();
      editPrGr->show();
      editGroupMenuB->show();
      mainDoneB->show();
      editSumLine->take_focus();
      break;
      
      
  }
  setMenuActive();
  
  if (mupo->getOptShowKbd())
  {
    hideKbdB->show();
    curDataBox->show();
#ifndef NOX11
    KbdButtonLeft->show();
    KbdButtonRight->show();
#endif
  }
  
  resize(x(), y(), w(), h());
  
  return(retVal);
}

void MuPOGUI::show_editToDoWin(bool fShowDescrIfExists,
                               bool fForceShowDescr)
{
  const char* pPtrToDescr;
  int         nDescrState;
  
  nDescrState = mupo->getItemDescription(pPtrToDescr);
    
  nLineBeforeEditItem = listView->topline();
  
  editSumLine->value(mupo->getItemSummary().c_str(), mupo->getItemSummary().length());
  
  editToDoShowAsBusy->value(mupo->getShowTimeAs());
  
  if (mupo->getItemPriority() <= PRIORITY_TODOMAX)
  {
    editToDoIsToDoCB->set();
    editPrB[mupo->getItemPriority()-1]->setonly();
  }
  else  
    editToDoIsToDoCB->clear();
  
  if (mupo->isItemCompleted())
    editToDoDoneCB->set();
  else    
    editToDoDoneCB->clear();
  
  if (mupo->isItemSaveCrypt())
    editToDoCryptCB->set();
  else    
    editToDoCryptCB->clear();
  
  SetDescrBuffer((char*)pPtrToDescr);
  editToDo_updateTimeSum();  
  if (fForceShowDescr || (fShowDescrIfExists && pPtrToDescr != (char*)0))
    showMain(MAINVISIBLE_TODO_DESCR);
  else
    showMain(MAINVISIBLE_TODO_BASIC);
  
  showToDoState();
}

MuPOGUI::~MuPOGUI() 
{
#ifndef NOX11
  delete fakeKbd;
  
  delete aqw;
  delete letter_templates_[AQWMODE_LOWERCASE];
  delete letter_templates_[AQWMODE_UPPERCASE];
  delete letter_templates_[AQWMODE_SYMBOL];
  delete letter_templates_[AQWMODE_NUMBER];
# ifndef AQW_SIMPLEINTERFACE
  delete lock_image_;
  delete meta_image_;
  delete ctrl_image_;
# endif
#endif  
}

int MuPOGUI::checkActiveJobs() 
{
  if (mupo->getNrActiveJobs())
    mainActiveB->show();
  else
  {
    mainActiveB->hide();
    if (mupo->getMainViewType() == MAINVIEW_ONLYACTIVE)
    {
      mupo->setMainViewType(MAINVIEW_BEFOREONLYACTIVE);
      setMenuActive();
      return(nLineBeforeOnlyActive);
    }
  }
  return(0);
}

void MuPOGUI::setMenuActive() 
{
  if (mupo->getMainViewType() == MAINVIEW_CALENDAR)                   
    mainMenuB->menu (menu_CalMenu);
  else
    mainMenuB->menu (menu_Menu);
  
  // activate and show everything
  if (nMainVisible == MAINVISIBLE_MAIN)
  {
    mainCalTodayB->show();
    mainCalSelB->show();
    mainCalPrevLB->show();
    mainCalNextLB->show();
    mainCalPrevB->show();
    mainCalNextB->show();
    
    mainNewB->show();
    mainUpB->show();
  }
  
  menu_EditToDoMenu[B_MENU_TURN2GROUP].activate();
  menu_EditToDoMenu[B_MENU_DELETE].activate();
  
  menu_Menu[MENU_VIEW_CAL].activate();
  menu_Menu[CB_OPT_VIEW_COMPLETED].activate();
  menu_Menu[CB_OPT_VIEW_COMPLETEDL].activate();
  menu_Menu[RB_OPT_VIEW_TREE].activate();
  menu_Menu[RB_OPT_VIEW_FLAT].activate();
  
  menu_Menu[MENU_FILE].activate();
  menu_Menu[MENU_COPY].activate();
  menu_Menu[MENU_PASTE].activate();
  menu_Menu[MENU_GROUP].activate();
  menu_Menu[MENU_SEARCH].activate();
  
  
  // now disable things selectively
  if (mupo->getMainViewType() == MAINVIEW_CALENDAR)
  {
    menu_EditToDoMenu[B_MENU_TURN2GROUP].deactivate();
    mainNewB->hide();
    mainUpB->hide();
  }
  else
  {    
    mainCalTodayB->hide();
    mainCalSelB->hide();
    mainCalPrevLB->hide();
    mainCalNextLB->hide();
    mainCalPrevB->hide();
    mainCalNextB->hide();
  }
  
  if (mupo->getMainViewType() == MAINVIEW_ONLYACTIVE ||
      mupo->getMainViewType() == MAINVIEW_SEARCHRESULTS)
  {
    menu_EditToDoMenu[B_MENU_TURN2GROUP].deactivate();
    menu_EditToDoMenu[B_MENU_DELETE].deactivate();
    
    menu_Menu[CB_OPT_VIEW_COMPLETED].deactivate();
    menu_Menu[CB_OPT_VIEW_COMPLETEDL].deactivate();
    menu_Menu[RB_OPT_VIEW_TREE].deactivate();
    menu_Menu[RB_OPT_VIEW_FLAT].deactivate();
    
    menu_Menu[MENU_FILE].deactivate();
    menu_Menu[MENU_COPY].deactivate();
    menu_Menu[MENU_PASTE].deactivate();
    menu_Menu[MENU_GROUP].deactivate();
    mainNewB->hide();
  }
  
  if (mupo->getMainViewType() == MAINVIEW_ONLYACTIVE)
  {
    menu_Menu[MENU_VIEW_CAL].deactivate();
    menu_Menu[MENU_SEARCH].deactivate();            
    mainUpB->hide();
  }
}

void MuPOGUI::enterGroup(int nIdx, bool fKeyboard)      
{
  if (mupo->getOptViewtype() == tree)
    mupo->GUIData.pushList(listView->topline(), listView->value());
  mupo->setGroup2ViewItem(nIdx);
  showList();
  
  if (fKeyboard)
    listView->select(1);
}

/**
 * toggle completed on item nIdx. 
 * If <code>fKeyboard</code>, select the line near nIdx afterwards.
 */
void MuPOGUI::toggleCompleted(int  nIdx, 
                              bool fKeyboard) 
{
  int retcode;
  int nLineBeforeToggle = listView->value();
  
  mupo->setItem2ViewItem(nIdx);
  if (mupo->isItemCompleted())
    retcode = mupo->clearItemCompleted();
  else
  {
    retcode = mupo->setItemCompleted();
    setCompletedMessage(retcode);
  }
  
  // save new data if item has been successfully changed
  if (retcode >= 0)
  {
    // INCSAVE: just save
#ifdef DEBUGINCSAVE
    std::cout << "saving item\n";
#endif
    // item has changed
    mupo->incsave_saveCurItem();
  }
  
  int n = checkActiveJobs();                  
  mupo->checkCompleted();
  if (n)                    
    showList(n);
  else
    showList(listView->topline());
  
  if (fKeyboard)
  {
    int nSize = listView->size();
    if (nLineBeforeToggle <= nSize)
      listView->select(nLineBeforeToggle, 1);
    else
      listView->select(nSize, 1);
  }
}

void MuPOGUI::listViewClicked() 
{
  // check if something is selected
  int n = listView->value ();
  if (n == 0)
    return;
  
  if (nMainVisible != MAINVISIBLE_MAIN || mupo->viewData.size() == 0)
    return;

  if (!Fl::pushed() && Fl::event_button() == 1 
      && (Fl::event_state() & FL_CTRL) == 0 && (Fl::event_state() & FL_SHIFT) == 0)
  {
    n = mupo->viewDataIndex[n-1]+1;
    switch (mupo->viewData[n-1].uPriority)
    {
      case PRIORITY_BLANK:
        if (Fl::event_inside(0,nHeadHeight*SIZEMUL, (uWinWidth*SIZEMUL)>>1, (uWinHeight-15)*SIZEMUL))
        {
          // create a todo for this day
          mupo->newToDo(3, n-1);
          mupo->incsave_newItem();
          show_editToDoWin();
        }
        else
        {
          // create a note for this day
          mupo->newToDo(PRIORITY_NOTE, n-1);
          mupo->incsave_newItem();
          show_editToDoWin(false, true);
        }
        break;
      case PRIORITY_NEWDAY:
        // create an appointment for this day
        mupo->newToDo(PRIORITY_APPOINTMENT, n-1);
        mupo->incsave_newItem();
        show_editToDoWin();
        break;
      case PRIORITY_DAYINFO:
        // This is the holiday line
        break;          
      default:
        if (mupo->viewData[n-1].fGroup)
        {
          // group clicked
          if (Fl::event_inside(0,nHeadHeight*SIZEMUL, // in listViewCols[0] steht *SIZEMUL schon drin!
                               listViewCols[0],(uWinHeight-15)*SIZEMUL))
          { // Clicked group in the left column
            mupo->rememberGroup();
            mupo->setGroup2ViewItem(n-1);
            fGroupQuickView = true;
            nLineBeforeEditItem = listView->topline();
            
            // INCSAVE: save data to compare it afterwards
            mupo->incsave_rememberGroup();
            
            showMain(MAINVISIBLE_GROUP);
          }
          else
          { // clicked group on its text
            enterGroup(n-1, false);
          }
        }
        else
        {
          // Item clicked. React depending on where it was hit.
              
          if (Fl::event_inside(0,nHeadHeight*SIZEMUL,  // in listViewCols[0] steht *SIZEMUL schon drin!
                               listViewCols[0],(uWinHeight-15)*SIZEMUL))
          {
            toggleCompleted(n-1, false);
          }
          else
          {
            // clicked on summary
            OpenItem(n-1, 
                     Fl::event_inside(listViewCols[0] ,nHeadHeight*SIZEMUL, uWinWidth*SIZEMUL/2,(uWinHeight-15)*SIZEMUL) == 0 // in listViewCols[0] steht *SIZEMUL schon drin!
                     );
          }
        }          
        break;
    }      
  }
}

void MuPOGUI::showList(int nLine) 
{
  std::string                  line;
  bool                         fRowCol = false;      
  
#ifdef DEBUGSNOW
  printf("MuPOGUI::showList() 1\n");
#endif
  if (mupo->getMainViewType() == MAINVIEW_CALENDAR)
  {   // calendar
    mupo->clearViewData();
    
    listView->clear ();		// clear all
    
    nHeadHeight = 0;
    
    headline->resize(0, 0, uWinWidth*SIZEMUL, nHeadHeight*SIZEMUL);
    headline->redraw ();
    
    listView->resize(0, nHeadHeight*SIZEMUL, uWinWidth*SIZEMUL, (uWinHeight-15-nHeadHeight)*SIZEMUL);
    listView->redraw();
    
    mupo->getContent();
    mupo->sortContent();
    
    unsigned int uForMaxVal = mupo->viewDataIndex.size();
    for (unsigned int i=0; i<uForMaxVal;i++)
    {
      line = mupo->viewData[mupo->viewDataIndex[i]].summary->c_str();
      
      if (mupo->viewData[mupo->viewDataIndex[i]].fDone && 
          mupo->viewData[mupo->viewDataIndex[i]].uPriority <= PRIORITY_TODOMAX)
      {
        if (fRowCol)
          line = colStrLineA + "x\t"+ colStrLineA + "@." + line;
        else
          line = colStrLineB + "x\t"+ colStrLineB + "@." + line;
        fRowCol = !fRowCol;
      }
      else
      {
        
        if (mupo->viewData[mupo->viewDataIndex[i]].uPriority < PRIORITY_NEWDAY)
        {
          t_datetime dt1, dt2;
          std::string     t1 = "", t2 = "";
          
          if (mupo->viewData[mupo->viewDataIndex[i]].uPriority <= PRIORITY_TODOMAX)
          {
            dt1 = mupo->viewData[mupo->viewDataIndex[i]].otherDate;
            dt2 = mupo->viewData[mupo->viewDataIndex[i]].sortDate;
          }
          else
          {
            dt1 = mupo->viewData[mupo->viewDataIndex[i]].sortDate;
            dt2 = mupo->viewData[mupo->viewDataIndex[i]].otherDate;
          }
          if (mupo->viewData[mupo->viewDataIndex[i]].fFirstDay && ISTIMESET(dt1))
            t1 = itoStr(GETHOUR(dt1), ' ', 1) + ":" + itoStr(GETMINUTE(dt1), '0', 2);
          
          if (mupo->viewData[mupo->viewDataIndex[i]].fLastDay && ISTIMESET(dt2))
            t2 = itoStr(GETHOUR(dt2), ' ', 1) + ":" + itoStr(GETMINUTE(dt2), '0', 2);
          
          if (t1 != "" || t2 != "")
            line = "(" + t1 + "-" + t2 + ") " + line;
        }
        
        switch (mupo->viewData[mupo->viewDataIndex[i]].uPriority)
        {
          case PRIORITY_NEWDAY:                  
          case PRIORITY_DAYINFO:
            fRowCol = false;
            line = colStrHead +  "\t"  + colStrHead + line;
            break;
          case PRIORITY_APPOINTMENT:
            if (fRowCol)
              line = colStrLineA + "a\t"+ colStrLineA + "@." + line;
            else
              line = colStrLineB + "a\t"+ colStrLineB + "@." + line;
            fRowCol = !fRowCol;
            break;
          case PRIORITY_NOTE:
            if (fRowCol)
              line = colStrLineA + "i\t"+ colStrLineA + "@." + line;
            else
              line = colStrLineB + "i\t"+ colStrLineB + "@." + line;
            fRowCol = !fRowCol;
            break;
          case PRIORITY_BLANK:
            line = "";
            break;
          default:
            if (fRowCol)
              line = colStrLineA + "o\t"+ colStrLineA + "@." + line;
            else
              line = colStrLineB + "o\t"+ colStrLineB + "@." + line;
            fRowCol = !fRowCol;
            break;
        }
      }
      listView->add(line.c_str());
    }
    if (nLine != 0)
      listView->topline(nLine);
    
  }
  else
  {
    // SNOWTEST
    headstring = mupo->getCurGroupPathText();
    
    mupo->clearViewData();
    
    listView->clear ();		// clear all
    
    nHeadHeight = 0;
    if (headstring.length())
    {
      nHeadHeight = 1;
      
      unsigned int uForMaxVal = headstring.length();
      for (unsigned int i=0; i<uForMaxVal; i++)
      {
        if (headstring[i] == '\n')
          nHeadHeight++;
      }
    }
    nHeadHeight *= 13;
    
    headline->resize(0, 0, uWinWidth*SIZEMUL, nHeadHeight*SIZEMUL);
    headline->label(headstring.c_str());
    headline->redraw ();
    
    listView->resize(0, nHeadHeight*SIZEMUL, uWinWidth*SIZEMUL, (uWinHeight-15-nHeadHeight)*SIZEMUL);
    listView->redraw();
    
    
    
    if (mupo->getGroupShowCompleted())
      menu_Menu[CB_OPT_VIEW_COMPLETED].set();    
    else   
      menu_Menu[CB_OPT_VIEW_COMPLETED].clear();
    mupo->setOptShowCompleted(mupo->getGroupShowCompleted());
    
    if (mupo->getGroupCompletedLast())
      menu_Menu[CB_OPT_VIEW_COMPLETEDL].set();    
    else   
      menu_Menu[CB_OPT_VIEW_COMPLETEDL].clear();
    mupo->setOptCompletedLast(mupo->getGroupCompletedLast());
    
    
    
    
    
    mupo->getContent();
    mupo->sortContent();
    
    
    
    if (mupo->viewDataIndex.size())
    {
      std::string colStr;
      
      unsigned int uForMaxVal = mupo->viewDataIndex.size();
      for (unsigned int i=0; i<uForMaxVal; i++)
      {
        if (fRowCol)
          colStr = colStrLineA;
        else
          colStr = colStrLineB;
        fRowCol = !fRowCol;
        
        line = mupo->viewData[mupo->viewDataIndex[i]].summary->c_str();
        if (mupo->viewData[mupo->viewDataIndex[i]].fDone && 
            mupo->viewData[mupo->viewDataIndex[i]].uPriority <= PRIORITY_TODOMAX)
        {
          if (mupo->viewData[mupo->viewDataIndex[i]].fGroup)
            line = colStr + "@bX\t@b" + colStr + "@." + line;
          else		
            line = colStr + "x\t" + colStr + "@." + line;		
        }
        else
        {
          if (mupo->viewData[mupo->viewDataIndex[i]].fGroup)
            line = colStr + "@bO\t@b"+ colStr + "@." + line;
          else
          {
            switch (mupo->viewData[mupo->viewDataIndex[i]].uPriority)
            {
              case PRIORITY_APPOINTMENT:
                line = colStr + "a\t"+colStr + "@." + line;
                break;
              case PRIORITY_NOTE:
                line = colStr + "i\t"+colStr + "@." + line;
                break;
              default:
                line = colStr + "o\t"+colStr + "@." + line;
                break;
            }
          }
        }
        listView->add(line.c_str());
      }
      if (nLine != 0)
        listView->topline(nLine);
    }
    else
      listView->add("<empty>");
  }
}

void MuPOGUI::init(MuPO*       mymupo, 
                   const char* tmpFileName) 
{
  mupo = mymupo;
  
  strcpy(tmpfilename, tmpFileName);
  
  switch (mupo->getOptViewtype())
  {
    case flat:
      menu_Menu[RB_OPT_VIEW_FLAT].setonly();
      break;
    case tree:
      menu_Menu[RB_OPT_VIEW_TREE].setonly();
      break;
  }   
  switch (mupo->getOptsSort())
  {
    case SORT_PRIORITY:
      menu_Menu[RB_OPT_VIEW_SORTPRIORITY].setonly();
      break;
    case SORT_DATE:
      menu_Menu[RB_OPT_VIEW_SORTDATE].setonly();
      break;
  }   
  switch (mupo->getOptsSort())
  {
    case SORT_PRIORITY:
      menu_Menu[RB_OPT_VIEW_SORTPRIORITY].setonly();
      break;
    case SORT_DATE:
      menu_Menu[RB_OPT_VIEW_SORTDATE].setonly();
      break;
  }   
  switch (mupo->getCalSpan())
  {
    case 1:
      menu_CalMenu[RB_MENU_1DAY].setonly();
      break;
    case 2:
      menu_CalMenu[RB_MENU_2DAY].setonly();
      break;
    case 3:
      menu_CalMenu[RB_MENU_3DAY].setonly();
      break;
    case 7:
      menu_CalMenu[RB_MENU_7DAYS].setonly();
      break;
    default:
      menu_CalMenu[RB_MENU_42DAYS].setonly();
      break;
  }
  UpdateCryptMenu();
}

void MuPOGUI::CB_hideKbd() 
{ 
  mupo->setOptShowKbd(false);
  resize(x(), y(), w(), h());
  redraw();
}

void MuPOGUI::makeWindows() 
{
  // Farben festlegen  
  /*
   colStrHead  = "@B" + itoStr(fl_color_cube(0x99*FL_NUM_RED/256, 0xA9*FL_NUM_GREEN/256, 0xDB*FL_NUM_BLUE/256), ' ', 1);
   colStrLineA = "@B" + itoStr(fl_color_cube(0xA6*FL_NUM_RED/256, 0xC4*FL_NUM_GREEN/256, 0xE0*FL_NUM_BLUE/256), ' ', 1);
   colStrLineB = "@B" + itoStr(fl_color_cube(0xA6*FL_NUM_RED/256, 0xA4*FL_NUM_GREEN/256, 0xE0*FL_NUM_BLUE/256), ' ', 1);
   * */  
  colStrHead  = "@B" + itoStr(fl_gray_ramp(0xC0*FL_NUM_GRAY/256), ' ', 1);  
  colStrLineA = "@B" + itoStr(fl_gray_ramp(0xE8*FL_NUM_GRAY/256), ' ', 1);
  colStrLineB = "@B" + itoStr(fl_gray_ramp(0xFF*FL_NUM_GRAY/256), ' ', 1);
  
  begin();    
  
  // main
  {
#ifndef NOX11
    KbdButtonLeft  = new Fl_Button(0, (uWinHeight*SIZEMUL)+15, AQWBRDRLEFT / 2, AQWBRDRLEFT / 2, "l");
    KbdButtonLeft->labelsize (TEXTSIZEAQWSTATUS);
    KbdButtonLeft->callback ((Fl_Callback *) cb_kbdLeft);
    KbdButtonRight = new Fl_Button((AQWBRDRLEFT / 2), (uWinHeight*SIZEMUL)+15, AQWBRDRLEFT / 2, AQWBRDRLEFT / 2, "r");
    KbdButtonRight->labelsize (TEXTSIZEAQWSTATUS);
    KbdButtonRight->callback ((Fl_Callback *) cb_kbdRight);
#endif
    hideKbdB = new Fl_Button (0, uWinHeight*SIZEMUL, 15, 15, "X");
    hideKbdB->labelsize (TEXTSIZEAQWSTATUS);
    hideKbdB->callback ((Fl_Callback *) cb_hideKbd);
    
    curDataBox = new Fl_Box (AQWBRDRLEFT+AQWSIZE+AQWBRDRRIGHT,                     uWinHeight*SIZEMUL,
                             (uWinWidth*SIZEMUL)-AQWBRDRLEFT+AQWSIZE+AQWBRDRRIGHT, AQWSIZE);
    curDataBox->box (FL_FLAT_BOX);
    curDataBox->labelsize (TEXTSIZEAQWSTATUS);
    curDataBox->align (68 | FL_ALIGN_INSIDE);
        
    listView = new Fl_Browser (0, 15*SIZEMUL, uWinWidth*SIZEMUL, (uWinHeight - 15 - 15)*SIZEMUL);        
    listView->type (FL_MULTI_BROWSER);   
    
    listView->box (FL_FLAT_BOX);
    listView->labeltype (FL_NO_LABEL);
    listView->labelfont (1);
    listView->labelsize (uTextSize1);
    listView->textsize (uTextSize1);
    listView->column_widths(listViewCols);
    listView->callback ((Fl_Callback *) cb_listView);
    listView->align (FL_ALIGN_TOP);
    //  listView->has_scrollbar (Fl_Browser_::VERTICAL_ALWAYS);
    listView->when(FL_WHEN_RELEASE|FL_WHEN_CHANGED|FL_WHEN_RELEASE_ALWAYS);
    listView->scrollbar_width (10*SIZEMUL);
    
    
    headline = new Fl_Button (0, 0, uWinWidth*SIZEMUL, 15*SIZEMUL);
    //  headline->box (FL_BORDER_BOX);
  
    headline->labelfont (1);
    headline->labelsize (TEXTSIZE1);
    headline->align (68 | FL_ALIGN_INSIDE);
    headline->callback ((Fl_Callback *) cb_Up);
  }
  
  
  inputBox = new Fl_Box (0, 0, uWinWidth*SIZEMUL, (uWinHeight - 30)*SIZEMUL);
  inputBox->box (FL_FLAT_BOX);
  inputBox->labelsize (TEXTSIZE1);
  inputBox->align (68 | FL_ALIGN_INSIDE);
  
  inputCancelB= new Fl_Button (35*SIZEMUL, (uWinHeight-15)*SIZEMUL, 40*SIZEMUL, 15*SIZEMUL, "Cancel");
  inputCancelB->labelsize (TEXTSIZE1);
  inputCancelB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
  inputCancelB->callback ((Fl_Callback *) cb_Cancel); 
  
  inputLine = new Fl_Input(0,(uWinHeight-30)*SIZEMUL,uWinWidth*SIZEMUL,15*SIZEMUL);
  inputLine->textsize(TEXTSIZE1);
  
  
  
  mainAboutBox = new Fl_Box (0, 0, uWinWidth*SIZEMUL, (uWinHeight - 15)*SIZEMUL);
  mainAboutBox->box (FL_FLAT_BOX);
  mainAboutBox->labelsize (TEXTSIZE1);
  mainAboutBox->align (68 | FL_ALIGN_INSIDE);
  mainAboutBox->label("              MuPO v"MUPOVERSION"\n\n"
		      "    Multi-Purpose-Organizer\n\n"
		      "        Hierarchical todo-list,\n"
		      "        appointments and notes,\n"
                      "        calendar,\n"
		      "        project-time recorder.\n"
		      "        Fully searchable, encrypting.\n\n"
		      "        Copyright (C) 2001 - 2011\n"
		      "         Jens Wilhelm Wulf\n"
		      "        <j.w.wulf at gmx dot net>");
  //  mainAboutBox->hide();
  
  
  mainDoneB= new Fl_Button (0*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL, "&Close");
  mainDoneB->labelsize (TEXTSIZE1);
  mainDoneB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
  mainDoneB->callback ((Fl_Callback *) cb_Done);
  //  mainDoneB->hide();
  // 
  mainMenuB= new Fl_Menu_Button (0*SIZEMUL, (uWinHeight-15)*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL, "&Menu");
  mainMenuB->labelsize (TEXTSIZE1);
  mainMenuB->textsize (TEXTSIZE1);
  mainMenuB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
  mainMenuB->menu (menu_Menu);
  // mainMenuB->shortcut (-1, FL_ALT + 'm');
  
  mainNewB= new Fl_Button (33*SIZEMUL, (uWinHeight-15)*SIZEMUL, 50*SIZEMUL, 15*SIZEMUL, "&New Item");
  mainNewB->labelsize (TEXTSIZE1);
  mainNewB->callback ((Fl_Callback *) cb_Menu_newTodo);
  mainNewB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
  
  mainUpB= new Fl_Button (83*SIZEMUL, (uWinHeight-15)*SIZEMUL, 20*SIZEMUL, 15*SIZEMUL, "..");
  mainUpB->labelfont (1);
  mainUpB->labelsize (TEXTSIZE1);
  mainUpB->callback ((Fl_Callback *) cb_Up);
  mainUpB->align (FL_ALIGN_BOTTOM | FL_ALIGN_INSIDE);
  
  mainActiveB = new Fl_Button ((uWinWidth-10)*SIZEMUL, (uWinHeight-15)*SIZEMUL, 10*SIZEMUL, 15*SIZEMUL, "!");
  mainActiveB->labelfont (1);
  mainActiveB->labelsize (TEXTSIZE1);
  mainActiveB->callback ((Fl_Callback *) cb_Menu_MuPOView, (void*)MAINVIEW_ONLYACTIVE);
  mainActiveB->align (FL_ALIGN_BOTTOM | FL_ALIGN_INSIDE);
    
  // editToDo
  {
    editSumLine = new Fl_Input(0,0,uWinWidth*SIZEMUL,15*SIZEMUL);
    editSumLine->textsize(TEXTSIZE1);
    
    editToDoIsToDoCB  = new Fl_Check_Button((uWinWidth-40)*SIZEMUL,15*SIZEMUL,40*SIZEMUL,15*SIZEMUL, "&ToDo");
    editToDoIsToDoCB->down_box(FL_DOWN_BOX);
    editToDoIsToDoCB->labelsize (TEXTSIZE1);
    editToDoIsToDoCB->when(FL_WHEN_CHANGED);
    editToDoIsToDoCB->callback ((Fl_Callback *) cb_edit_isToDo);
    
    editPrGr = new Fl_Group(0,30*SIZEMUL,uWinWidth*SIZEMUL, 20*SIZEMUL, "Priority:");
    editPrGr->labelsize (TEXTSIZE1);
    editPrGr->align(FL_ALIGN_TOP + FL_ALIGN_LEFT);
    editPrGr->box(FL_THIN_DOWN_BOX);
        
    for (unsigned int i=0;i<PRIORITY_TODOMAX;i++)
    {
      editPrLabel[i] = "&" + itoStr(i+1, ' ', 1);
      (editPrB[i]) = new Fl_Check_Button (i*25*SIZEMUL, 33*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL, 
                                          editPrLabel[i].c_str());
      editPrB[i]->callback ((Fl_Callback *) cb_editToDo_Pr, (void*)(i+1));
      editPrB[i]->when(FL_WHEN_CHANGED);      
      
      editPrB[i]->labelsize (TEXTSIZE1);
      editPrB[i]->down_box(FL_DOWN_BOX);
      editPrB[i]->type (FL_RADIO_BUTTON);
    }
    editPrGr->end();
    
    editToDoDoneCB  = new Fl_Check_Button(0,50*SIZEMUL,40*SIZEMUL,15*SIZEMUL,"&Done:");
    editToDoDoneCB->down_box(FL_DOWN_BOX);
    editToDoDoneCB->labelsize (TEXTSIZE1);
    editToDoDoneCB->callback ((Fl_Callback *) cb_editToDo_compl);
    editToDoDoneCB->when(FL_WHEN_CHANGED);
    
    editToDoStateBox = new Fl_Box (40*SIZEMUL, 50*SIZEMUL, 120*SIZEMUL, 15*SIZEMUL);
    editToDoStateBox->box (FL_FLAT_BOX);  
    editToDoStateBox->labelfont (0);
    editToDoStateBox->labelsize (TEXTSIZE1);
    editToDoStateBox->align (68 | FL_ALIGN_INSIDE);
    
    editTimeSumBox = new Fl_Box (0*SIZEMUL, 65*SIZEMUL, 120*SIZEMUL, 15*SIZEMUL);
    editTimeSumBox->box (FL_FLAT_BOX);
    editTimeSumBox->labelfont (0);
    editTimeSumBox->labelsize (TEXTSIZE1);
    editTimeSumBox->align (68 | FL_ALIGN_INSIDE);
    
    editToDoBegDateB= new Fl_Menu_Button (0, 70*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL, "&Begin");
    editToDoBegDateB->labelsize (TEXTSIZE1);
    editToDoBegDateB->textsize (TEXTSIZE1);
    editToDoBegDateB->align (FL_ALIGN_INSIDE);
    editToDoBegDateB->menu (menu_EditToDoBegDate);
    
    editToDoEndDateB= new Fl_Menu_Button (0, 85*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL, "End");
    editToDoEndDateB->labelsize (TEXTSIZE1);
    editToDoEndDateB->textsize (TEXTSIZE1);
    editToDoEndDateB->align (FL_ALIGN_INSIDE);
    editToDoEndDateB->menu (menu_EditToDoEndDate);
    
    editToDoBegTimeB  = new Fl_Menu_Button(33*SIZEMUL,70*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL, "Time");
    editToDoBegTimeB->labelsize (TEXTSIZE1);
    editToDoBegTimeB->textsize (TEXTSIZE1);
    editToDoBegTimeB->align (FL_ALIGN_INSIDE);
    editToDoBegTimeB->menu (menu_EditToDoBegTime);
    
    editToDoEndTimeB  = new Fl_Menu_Button(33*SIZEMUL,85*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL, "Time");
    editToDoEndTimeB->labelsize (TEXTSIZE1);
    editToDoEndTimeB->textsize (TEXTSIZE1);
    editToDoEndTimeB->align (FL_ALIGN_INSIDE);
    editToDoEndTimeB->menu (menu_EditToDoEndTime);
    
    editToDoBegBox = new Fl_Box (66*SIZEMUL, 70*SIZEMUL, (160-66)*SIZEMUL, 15*SIZEMUL);
    editToDoBegBox->box (FL_FLAT_BOX);
    editToDoBegBox->labelfont (0);
    editToDoBegBox->labelsize (TEXTSIZE1);
    editToDoBegBox->align (68 | FL_ALIGN_INSIDE);
    
    editToDoEndBox = new Fl_Box (66*SIZEMUL, 85*SIZEMUL, (160-66)*SIZEMUL, 15*SIZEMUL);
    editToDoEndBox->box (FL_FLAT_BOX);
    editToDoEndBox->labelfont (0);
    editToDoEndBox->labelsize (TEXTSIZE1);
    editToDoEndBox->align (68 | FL_ALIGN_INSIDE);
    
    editToDoShowAsBusy = new Fl_Choice (50*SIZEMUL, 100*SIZEMUL, 70*SIZEMUL, 15*SIZEMUL, "show as:");
    editToDoShowAsBusy->menu (menu_EditToDoShowAsBusy);
    editToDoShowAsBusy->labelsize (TEXTSIZE1);
    editToDoShowAsBusy->textsize (TEXTSIZE1);
    editToDoShowAsBusy->align(FL_ALIGN_LEFT);
    
    editToDoRecurringB  = new Fl_Button(0*SIZEMUL,115*SIZEMUL,160*SIZEMUL,15*SIZEMUL, "Recurring:");
    editToDoRecurringB->labelsize (TEXTSIZE1);
    editToDoRecurringB->callback ((Fl_Callback *) cb_editToDo_recurring);
    
#ifdef REMINDER
    editToDoReminderB  = new Fl_Button(0*SIZEMUL,130*SIZEMUL,160*SIZEMUL,15*SIZEMUL, "Reminder:");
    editToDoReminderB->labelsize (TEXTSIZE1);
    editToDoReminderB->callback ((Fl_Callback *) cb_editToDo_reminder);
    editToDoReminderB->deactivate();
#endif
    editToDoCryptCB  = new Fl_Check_Button(0*SIZEMUL,145*SIZEMUL,160*SIZEMUL,15*SIZEMUL, "Cr&ypt");
    editToDoCryptCB->down_box(FL_DOWN_BOX);
    editToDoCryptCB->labelsize (TEXTSIZE1);
    editToDoCryptCB->callback ((Fl_Callback *) cb_editToDo_crypt);
    editToDoCryptCB->when(FL_WHEN_CHANGED);
    editToDoCryptCB->deactivate();
    
#ifdef USE_FLEDIT
    editToDoDescrBuf = new Fl_Text_Buffer();
    editToDoDescr  = new Fl_Text_Editor(0,0,uWinWidth*SIZEMUL, (uWinHeight-15)*SIZEMUL);
    editToDoDescr->buffer(editToDoDescrBuf);
    
    editToDoDescr->remove_key_binding(FL_BackSpace, FL_TEXT_EDITOR_ANY_STATE);
    editToDoDescr->add_key_binding(FL_BackSpace, 0, Fl_Text_Editor::kf_backspace);
    editToDoDescr->add_key_binding(FL_BackSpace, FL_CTRL, kf_backspace_word);
    
    editToDoDescr->callback( (Fl_Callback*)cb_edit, (void*)(editToDoDescr));
    editToDoDescr->when(FL_WHEN_CHANGED);
#else
    editToDoDescr  = new Fl_Multiline_Input(0,0,uWinWidth*SIZEMUL, (uWinHeight-15)*SIZEMUL);
#endif
    
    editToDoDescr->textsize(uTextSize1);
    editToDoDescr->textfont(FL_HELVETICA);
    //  editToDoDescr->scrollbar_width (10*SIZEMUL);
    
#ifndef NOX11
    editToDoDescrUpB  = new Fl_Repeat_Button((uWinWidth-30)*SIZEMUL,(uWinHeight-15)*SIZEMUL, 15*SIZEMUL, 15*SIZEMUL, "u");
    editToDoDescrUpB->labelsize (TEXTSIZE1);
    editToDoDescrUpB->callback ((Fl_Callback *) cb_editToDo_UpB);
    editToDoDescrUpB->down_box(FL_THIN_UP_BOX);
    editToDoDescrDwnB = new Fl_Repeat_Button((uWinWidth-15)*SIZEMUL,(uWinHeight-15)*SIZEMUL, 15*SIZEMUL, 15*SIZEMUL, "d");
    editToDoDescrDwnB->labelsize (TEXTSIZE1);
    editToDoDescrDwnB->callback ((Fl_Callback *) cb_editToDo_DwnB);
    editToDoDescrDwnB->down_box(FL_THIN_UP_BOX);
#endif
    
    editToDoMenuB= new Fl_Menu_Button (35*SIZEMUL, (uWinHeight-15)*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL, "&Menu");
    editToDoMenuB->labelsize (TEXTSIZE1);
    editToDoMenuB->textsize (TEXTSIZE1);
    editToDoMenuB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
    editToDoMenuB->menu (menu_EditToDoMenu);
    
    
    editToDoOtherB = new Fl_Button (68*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL, "Descr");
    editToDoOtherB->labelsize (TEXTSIZE1);
    editToDoOtherB->align (FL_ALIGN_BOTTOM | FL_ALIGN_INSIDE);
    
    editToDoStrtStpB = new Fl_Button (103*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL, "&Start");
    editToDoStrtStpB->labelsize (TEXTSIZE1);
    editToDoStrtStpB->callback ((Fl_Callback *) cb_editToDo_StrtStp);
    editToDoStrtStpB->align (FL_ALIGN_BOTTOM | FL_ALIGN_INSIDE);
    
    editToDoExtEditB = new Fl_Button (103*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL, "&Editor");
    editToDoExtEditB->labelsize (TEXTSIZE1);
    editToDoExtEditB->callback ((Fl_Callback *) cb_editToDo_ExtEdit);
    editToDoExtEditB->align (FL_ALIGN_BOTTOM | FL_ALIGN_INSIDE);
    
    
    editToDoTimeList = new Fl_Browser (0, 0, uWinWidth*SIZEMUL, (uWinHeight- 15)*SIZEMUL);
    editToDoTimeList->type (1);
    editToDoTimeList->box (FL_FLAT_BOX);
    editToDoTimeList->labeltype (FL_NO_LABEL);
    editToDoTimeList->labelfont (1);
    editToDoTimeList->labelsize (TEXTSIZE1);
    editToDoTimeList->textsize (TEXTSIZE1);
    editToDoTimeList->align (FL_ALIGN_TOP);
    //  editToDoTimeList->has_scrollbar (Fl_Browser_::VERTICAL_ALWAYS);
    editToDoTimeList->scrollbar_width (10*SIZEMUL);
  }
  
  // editGroup
  {
    editGroupMenuB= new Fl_Menu_Button (35*SIZEMUL, (uWinHeight-15)*SIZEMUL, 33*SIZEMUL, 15*SIZEMUL, "&Menu");
    editGroupMenuB->labelsize (TEXTSIZE1);
    editGroupMenuB->textsize (TEXTSIZE1);
    editGroupMenuB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
    editGroupMenuB->menu (menu_EditGroupMenu);   
    
    editGroupComplDetailBox = new Fl_Box (0*SIZEMUL, 80*SIZEMUL, 120*SIZEMUL, 15*SIZEMUL);
    editGroupComplDetailBox->box (FL_FLAT_BOX);
    editGroupComplDetailBox->labelfont (0);
    editGroupComplDetailBox->labelsize (TEXTSIZE1);
    editGroupComplDetailBox->align (68 | FL_ALIGN_INSIDE);  
  }
  
  // calendar
  {
    mainCalTodayB= new Fl_Button (33*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL, "Today");
    mainCalTodayB->labelsize (TEXTSIZE1);
    mainCalTodayB->callback ((Fl_Callback *) cb_CalFirstDay, (void*)CALFIRSTDAY_TODAY);
    mainCalTodayB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
    
    mainCalSelB= new Fl_Button (68*SIZEMUL, (uWinHeight-15)*SIZEMUL, 35*SIZEMUL, 15*SIZEMUL, "Select");
    mainCalSelB->labelsize (TEXTSIZE1);
    mainCalSelB->callback ((Fl_Callback *) cb_CalFirstDay, (void*)CALFIRSTDAY_SELECT);
    mainCalSelB->align (FL_ALIGN_BOTTOM_LEFT | FL_ALIGN_INSIDE);
    
    mainCalPrevLB= new Fl_Button (103*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL, "<");
    mainCalPrevLB->labelsize (TEXTSIZE1);
    mainCalPrevLB->callback ((Fl_Callback *) cb_CalFirstDay, (void*)CALFIRSTDAY_DECL);
    mainCalPrevLB->align (FL_ALIGN_INSIDE);
    
    mainCalPrevB= new Fl_Button (114*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL, "-");
    mainCalPrevB->labelsize (TEXTSIZE1);
    mainCalPrevB->callback ((Fl_Callback *) cb_CalFirstDay, (void*)CALFIRSTDAY_DEC);
    mainCalPrevB->align (FL_ALIGN_INSIDE);
    
    mainCalNextB= new Fl_Button (125*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL, "+");
    mainCalNextB->labelsize (TEXTSIZE1);
    mainCalNextB->callback ((Fl_Callback *) cb_CalFirstDay, (void*)CALFIRSTDAY_INC);
    mainCalNextB->align (FL_ALIGN_INSIDE);
    
    mainCalNextLB= new Fl_Button (136*SIZEMUL, (uWinHeight-15)*SIZEMUL, 11*SIZEMUL, 15*SIZEMUL, ">");
    mainCalNextLB->labelsize (TEXTSIZE1);
    mainCalNextLB->callback ((Fl_Callback *) cb_CalFirstDay, (void*)CALFIRSTDAY_INCL);
    mainCalNextLB->align (FL_ALIGN_INSIDE);
  }
  
  end();
}

void MuPOGUI::CB_Done () 
{
  switch (nMainVisible)
  {
    case MAINVISIBLE_MAIN:
      if (mupo->getMainViewType() == MAINVIEW_CALENDAR)
        mupo->GUIData.nLineCal = listView->topline();
      if (mupo->getMainViewType() == MAINVIEW_NORMAL)
        mupo->GUIData.nLineNormal = listView->topline();
      hide ();
      break;
    case MAINVISIBLE_TODO_BASIC:
    case MAINVISIBLE_TODO_TIMES:
    case MAINVISIBLE_TODO_DESCR:
      editToDo_Done ();
      break;
    case MAINVISIBLE_GROUP:
      editGroup_Done ();
      break;
      
    case MAINVISIBLE_INPUT:
      if (strlen(inputLine->value()) != 0)
      {
        switch (inputUse)
        {
          case INPUTUSE_SEARCH:
            {
              std::string sstr = inputLine->value();
              
              if (mupo->getMainViewType() != MAINVIEW_SEARCHRESULTS 
                  && mupo->getOptViewtype() == tree)
                mupo->GUIData.pushList(listView->topline(), listView->value());
              mupo->search(sstr);
              showMain(MAINVISIBLE_MAIN);      
              setMenuActive();
              showList();
            }              
            break;
          case INPUTUSE_EXPGROUP:
            mupo->exportGroupToFile(inputLine->value());
            showMain(MAINVISIBLE_MAIN);
            break;
          case INPUTUSE_EXPCONT:
            mupo->exportContentToFile(inputLine->value());
            showMain(MAINVISIBLE_MAIN);
            break;
          case INPUTUSE_INSFILE:
            mupo->insertDataFromFile(inputLine->value());
            showMain(MAINVISIBLE_MAIN);
            checkActiveJobs();
            showList();
            break;
#ifdef DIRTYTRANSHACK
          case INPUTUSE_TRANSLATE:
            {
              std::ifstream indat;
              std::string   tmp;
              unsigned int  uCnt;
              char          chTmp;
              
              tmp = "/home/default/bin/grep -w \"";
              tmp += inputLine->value();
              tmp += "\" /usr/share/dictionary/ger-eng.txt > /tmp/trans.txt";
              system(tmp.c_str());
              
              indat.open("/tmp/trans.txt");                  
              if (indat)
              {
                uCnt = 0;
                
                while (uCnt < TRANSRESULTSIZE-1 &&
                       !indat.eof())
                {
                  chTmp = (char)indat.get();
                  if (!indat.eof())
                    transresult[uCnt++] = chTmp;
                }
                transresult[uCnt] = '\0';
                indat.close();
              }
              
              mainAboutBox->label(transresult);
              showMain(MAINVISIBLE_ABOUT);
            }
            break;
#endif
          default:
            showMain(MAINVISIBLE_MAIN);
            break;
        }
      }
      else
        showMain(MAINVISIBLE_MAIN);
      break;
      
    default:
      showMain(MAINVISIBLE_MAIN);
  }  
}

void MuPOGUI::CB_Cancel () 
{
  showMain(MAINVISIBLE_MAIN);      
}

void MuPOGUI::CB_Up () 
{
  if (!mupo->leaveCurGroup())
  {
    setMenuActive();   // one does possibly leave "search results"
    showList(mupo->GUIData.popListPos());
    listView->select(mupo->GUIData.popListSel());
  }
}

void MuPOGUI::CB_Menu_newTodo   () 
{
  if (mupo->newToDo() == 0)
  {
#ifdef DEBUGSNOW
    printf("note created, now displaying it..\n");
#endif
    mupo->incsave_newItem();
    show_editToDoWin();
  }
}

void MuPOGUI::CB_Menu_newGroup  () 
{  
  if (!mupo->newGroup())
  {
    if (mupo->getOptViewtype() == tree)
    {
      mupo->GUIData.pushList(listView->topline(), listView->value());
    }
    
    mupo->incsave_newItem();
    
    nLineBeforeEditItem = 0;
    showMain(MAINVISIBLE_GROUP);
  }
}

void MuPOGUI::CB_Menu_editGroup () 
{
  if (!mupo->inRootGroup())
  {
    nLineBeforeEditItem = listView->topline();
    
    // INCSAVE: save data to compare it afterwards
    mupo->incsave_rememberGroup();
    
    showMain(MAINVISIBLE_GROUP);
  }
}

void MuPOGUI::CB_Menu_delGroup  () 
{
  if (fl_choice("Do you really want to delete this group?", 0, "Cancel", "Delete") == 2)
  {
    mupo->incsave_forgetGroup();
    mupo->exportGroupToFile(tmpfilename);
    if (!mupo->delCurGroup())
    { 
      int tmp;
      
      if (fGroupQuickView)
        tmp = nLineBeforeEditItem;
      else
      {
        tmp = mupo->GUIData.popListPos();
        mupo->GUIData.popListSel();
      }
      showMain(MAINVISIBLE_MAIN);
      showList(tmp);
    }
  }
}

void MuPOGUI::CB_Menu_delCompl  () 
{
  if (fl_choice("This will delete every visible todo that's finished. Really continue?", 0, "Cancel", "Delete") == 2)
  {
    if (!mupo->delCompleted())
      showList();
    else
      fl_message("Some error occured.");
  }
}

void MuPOGUI::CB_Menu_search    () 
{
  inputBox->label("string to search for:");
  inputUse = INPUTUSE_SEARCH;
  showMain(MAINVISIBLE_INPUT);
}

void MuPOGUI::CB_Menu_insFile   () 
{
  //  mupo->insertDataFromFile(fl_input("filename and path of MuPO-file to insert:"));
  mainWin->inputBox->label("filename and path of MuPO-file to insert:");
  mainWin->inputUse = INPUTUSE_INSFILE;
  mainWin->showMain(MAINVISIBLE_INPUT);
  //  checkActiveJobs();
  //  showList();
}

void MuPOGUI::CB_Menu_paste     () 
{
  mupo->insertDataFromFile(tmpfilename);
  checkActiveJobs();
  showList();
}

void MuPOGUI::CB_Menu_MuPOView(int item) 
{
  int   tmp = 0;
  bool  fTypeChanged = true;
  
  switch (item)
  {
    case MAINVIEW_ADD_KBD:
      fTypeChanged = false;
      mupo->setOptShowKbd(true);
      showKeyboard(true);
      resize(x(), y(), w(), h());
      redraw();      
      break;
      
    case MAINVIEW_CALENDAR:
      if (mupo->getMainViewType() == MAINVIEW_SEARCHRESULTS)
      {
        mupo->leaveCurGroup();
        mupo->GUIData.nLineNormal = mupo->GUIData.popListPos();
        mupo->GUIData.popListSel();
      }
      else
        mupo->GUIData.nLineNormal = listView->topline();
      tmp = mupo->GUIData.nLineCal;
      mupo->setMainViewType(MAINVIEW_CALENDAR);
      break;
      
    case MAINVIEW_NORMAL:
      if (mupo->getMainViewType() == MAINVIEW_CALENDAR)
        mupo->GUIData.nLineCal = listView->topline();
      tmp = mupo->GUIData.nLineNormal;
      mupo->setMainViewType(MAINVIEW_NORMAL);
      break;
      
    case MAINVIEW_ONLYACTIVE:
      if (mupo->getMainViewType() == MAINVIEW_ONLYACTIVE)
      {
        mupo->setMainViewType(MAINVIEW_BEFOREONLYACTIVE);
        tmp = nLineBeforeOnlyActive;
      }
      else
      {
        if (mupo->getMainViewType() == MAINVIEW_SEARCHRESULTS)
        {
          nLineBeforeOnlyActive = mupo->GUIData.popListPos();
          mupo->GUIData.popListSel();              
          mupo->leaveCurGroup();
        }
        else
          nLineBeforeOnlyActive =  listView->topline();
        mupo->setMainViewType(MAINVIEW_ONLYACTIVE);
      }
      break;
  }
  
  if (fTypeChanged)
  {
    setMenuActive();
    showList(tmp);
  }
}

void MuPOGUI::CB_Opts_sort(int item) 
{
  mupo->setOptsSort(item);
  
  showList();
}

void MuPOGUI::CB_Opts_View_flat () 
{
  mupo->GUIData.clearStack();
  
  mupo->setOptViewtype(flat);
  menu_Menu[RB_OPT_VIEW_FLAT].setonly();
  showList();
}

void MuPOGUI::CB_Opts_View_tree () 
{
  mupo->setOptViewtype(tree);
  menu_Menu[RB_OPT_VIEW_TREE].setonly();
  showList();
}

void MuPOGUI::CB_Opts_View_scompleted () 
{
  mupo->toggleGroupShowCompleted();
  showList();
}

void MuPOGUI::CB_Opts_View_completedl () 
{
  mupo->toggleGroupCompletedLast();
  showList();
}

void MuPOGUI::CB_Opts_View_toggleTextSize () 
{
  if (uTextSize1 == TEXTSIZE1)
    uTextSize1 = TEXTSIZE2;
  else
    uTextSize1 = TEXTSIZE1;
  
  listView->labelsize (uTextSize1);
  listView->textsize (uTextSize1);
  editToDoDescr->textsize(uTextSize1);
  
  showList();
}

std::string MuPOGUI::dateTime2str(t_datetime dt) 
{
  std::string tmp;
  
  if (ISDATESET(dt))
  {
    tmp = itoStr(GETDAY(dt), '0', 2) + "." +
      itoStr(GETMONTH(dt), '0', 2) + "." +
      itoStr(GETYEAR(dt), '0', 4);
    if (ISTIMESET(dt))
    {
      tmp += " " +
        itoStr(GETHOUR(dt), ' ', 1) + ":" +
        itoStr(GETMINUTE(dt), '0', 2);
    }
    else
    {
      tmp += " xx:xx";
    }
    return(tmp);
  }
  else 
    return("none");
}

void MuPOGUI::CB_editToDo_StrtStp () 
{
  mupo->setItemRunning(!mupo->isItemRunning());
  if (mupo->isItemRunning())
    editToDoStrtStpB->label("&Stop");
  else
  {
    editToDo_updateTimeSum();
    editToDoStrtStpB->label("&Start");
  }
}

void MuPOGUI::CB_editToDo_ExtEdit()
{
  int         tmpFile;
  char        tmpfilename[16];

  // save current content to temporary file
  const char* pPtrToDescr;
  int         nDescrState;
  
  nDescrState = mupo->getItemDescription(pPtrToDescr);
  
#if defined( WIN32 ) && !defined(__CYGWIN__)
  strcpy(tmpfilename, "mupoXXXXXX.tmp");
  tmpFile = _open(tmpfilename, _O_TRUNC | _O_CREAT | _O_RDWR, _S_IREAD | _S_IWRITE);
#else
  strcpy(tmpfilename, "/tmp/mupoXXXXXX");
  tmpFile = mkstemp(tmpfilename);
#endif
  if (tmpFile == -1)
  {
    fl_message("Unable to create temporary file");
  }
  
  char* buffer;
  int nFree;
  nFree = GetDescrBuffer(buffer);
  write(tmpFile, buffer, strlen(buffer));
  if (nFree)
    free(buffer);  
  close(tmpFile);
    
  // open temporary file with editor
  system((mupoconfig.editor + " " + std::string(tmpfilename)).c_str());
  
  // read file content into edit widget
  std::string bufstring = ReadFileIntoBuf(tmpfilename);
  SetDescrBuffer(bufstring.c_str());
  
  FileTools::rm(tmpfilename);
}

void MuPOGUI::editToDo_saveSettings() 
{  
  mupo->setItemSummary(editSumLine->value());

  char* buffer;
  int nFree = GetDescrBuffer(buffer);  
  mupo->setItemDescription(buffer);
  if (nFree)
    free(buffer);  
}

int MuPOGUI::GetDescrBuffer(char*& buffer)
{
#ifdef USE_FLEDIT
  buffer = editToDoDescrBuf->text();
  return(1);
#else
  buffer = (char*)editToDoDescr->value();
  return(0);
#endif        
}
  
void MuPOGUI::SetDescrBuffer(const char* buffer)
{
#ifdef USE_FLEDIT
  if (buffer)
    editToDoDescrBuf->text(buffer);
  else
    editToDoDescrBuf->text("");
  editToDoDescr->insert_position(0);
  editToDoDescr->scroll(0, 0);
  Fl::focus(editToDoDescr);
#else
  editToDoDescr->value(buffer);
  editToDoDescr->position(0);
  editToDoDescr->mark(0);
#endif
}

std::string MuPOGUI::ReadFileIntoBuf(char* filename)
{
  const int LINELEN = 500;
  char Zeile[LINELEN];
  
  std::ifstream infile;
  std::string   filestring = "";
  infile.open(filename);
  while (!infile.eof())
  {
    infile.getline(Zeile, LINELEN-1);    
    filestring += std::string(Zeile) + "\n";
  }
  infile.close();
  return(filestring);
}

void MuPOGUI::editToDo_Done () 
{
  int n;
  
  editToDo_saveSettings();
  
  // INCSAVE: compare data with the data saved before and save to 
  //          file if it has changed
  if (mupo->incsave_todoHasChanged())
  {
#ifdef DEBUGINCSAVE
    std::cout << "saving current item\n";
    std::cout.flush();
#endif
    // item is new or has changed
    mupo->incsave_saveCurItem();
  }
  mupo->incsave_forgetTodo();
  
  
  n = showMain(MAINVISIBLE_MAIN);
  mupo->checkCompleted();
  if (n)
    showList(n);
  else
    showList(nLineBeforeEditItem);
}

void MuPOGUI::showToDoState() 
{
  if (mupo->isItemCompleted())
  {
    tm* time2;
    time_t time1;
    
    time1 = mupo->getItemCompletedTime();
    time2 = localtime(&time1);
    strftime(complTime, 42, DATEFORMATSTR" "TIMEFORMATSTR, time2);
    
    editToDoStateBox->label(complTime);
    editToDoDoneCB->set();
  }
  else
  {
    editToDoStateBox->label("not yet...");
    editToDoDoneCB->clear();
  }
  editToDoStateBox->redraw();  
}


void MuPOGUI::CB_edit_Pr         (int nNr) 
{
  if (nNr == -1)
  {
    if (mupo->getItemPriority() <= PRIORITY_TODOMAX)
    {
      if (mupo->getShowTimeAs() == SHOW_AS_BUSY)
        mupo->setItemPriority(PRIORITY_APPOINTMENT);
      else
        mupo->setItemPriority(PRIORITY_NOTE);
      editPrGr->hide();
      editToDoDoneCB->hide();
      editToDoStateBox->hide();	  	  
    }
    else
    {
      editPrB[INITIALPRIORITY-1]->setonly();
      mupo->setItemPriority(INITIALPRIORITY);
      editPrGr->show();
      editToDoDoneCB->show();
      editToDoStateBox->show();	  	  
    }
  }
  else
  {
    editPrB[nNr-1]->setonly();
    if (nMainVisible == MAINVISIBLE_TODO_BASIC)
      mupo->setItemPriority(nNr);
    else
      mupo->setGroupPriority(nNr);
  }
}

void MuPOGUI::setCompletedMessage(int code) 
{
  switch (code)
  {
    case SETCOMPLCODE_CURDATE_BEFORE_RECSTART:
      fl_alert("Due to recurrence-settings, this ToDo is not active yet!");
      break;
    case SETCOMPLCODE_OK_RECTODO:
      fl_message("Recurrent ToDo:\nSet completed, activated for following date.");
      break;
    default:
      break;
  }
}

void MuPOGUI::CB_editToDo_compl         () 
{
  if (editToDoDoneCB->value() && !mupo->isItemCompleted())
  {
    setCompletedMessage(mupo->setItemCompleted());
    editToDoStrtStpB->hide();
    editToDo_updateTimeSum();
    editToDo_updateTimeThings();
  }
  else
    if (!editToDoDoneCB->value())
    {
      mupo->clearItemCompleted();
      if (mupo->isItemRunning())
        editToDoStrtStpB->label("&Stop");
      else
        editToDoStrtStpB->label("&Start");      
      editToDoStrtStpB->show();
    }
  
  showToDoState();
}

void MuPOGUI::CB_editToDo_UpB          () 
{
#ifndef NOX11
  if (editToDoDescr->position())
  {
    //    editToDoDescr->position(editToDoDescr->position()-1, editToDoDescr->position()-1);
    for (unsigned int uCnt = 0; uCnt < 10; uCnt++)
    {
      XTestFakeKeyEvent(fl_display, nKeyUp, 1, 0);
      XTestFakeKeyEvent(fl_display, nKeyUp, 0, 0);
    }
  }
#endif
}

void MuPOGUI::CB_editToDo_DwnB         () 
{  
#ifndef NOX11
  //  editToDoDescr->position(editToDoDescr->position()+1, editToDoDescr->position()+1);
  for (unsigned int uCnt = 0; uCnt < 10; uCnt++)
  {
    XTestFakeKeyEvent(fl_display, nKeyDown, 1, 0);
    XTestFakeKeyEvent(fl_display, nKeyDown, 0, 0);
  }
#endif
}

#ifndef NOX11
void MuPOGUI::CB_kbdLeft          () 
{  
  XTestFakeKeyEvent(fl_display, nKeyLeft, 1, 0);
  XTestFakeKeyEvent(fl_display, nKeyLeft, 0, 0);
}

void MuPOGUI::CB_kbdRight         () 
{  
  XTestFakeKeyEvent(fl_display, nKeyRight, 1, 0);
  XTestFakeKeyEvent(fl_display, nKeyRight, 0, 0);
}
#endif

void MuPOGUI::CB_editToDo_Menu_t2Group     () 
{
  editToDo_saveSettings();  
  if (!mupo->turnCurItem2Group())
  {
    if (mupo->getOptViewtype() == tree)
      mupo->GUIData.pushList(nLineBeforeEditItem, 0);
    showMain(MAINVISIBLE_MAIN);
    mupo->checkCompleted();
    showList(0);
  }
}

void MuPOGUI::CB_editToDo_Menu_del         () 
{
  if (fl_choice("Do you really want to delete this item?", 0, "Cancel", "Delete") == 2)
  {
    mupo->incsave_forgetTodo();
    mupo->exportToDoToFile(tmpfilename);
    if (!mupo->delToDo())
    {
      showMain(MAINVISIBLE_MAIN);
      mupo->checkCompleted();
      showList(nLineBeforeEditItem);
    }
    else
      fl_alert("Error trying to delete item!");
  }
}

void MuPOGUI::CB_editToDo_Menu_showGroup   () 
{
  if (mupo->getMainViewType() == MAINVIEW_NORMAL)
    editToDo_Done();
  else
  {
    // feststellen, in welche Group ich muss
    mupo->rememberGroupOfCurrentItem();
    // editToDo beenden
    editToDo_saveSettings();
    mupo->checkCompleted();
    
    switch (mupo->getMainViewType())
    {
      case MAINVIEW_SEARCHRESULTS:
        mupo->leaveCurGroup();
        break;
      case MAINVIEW_CALENDAR:
      case MAINVIEW_ONLYACTIVE:
        mupo->setMainViewType(MAINVIEW_NORMAL);
        break;
    }
    
    // reinwechseln
    mupo->enterPrevGroup();
    mupo->GUIData.clearStack();
    showMain(MAINVISIBLE_MAIN);
    setMenuActive();
    showList(0);
  }
}

void MuPOGUI::editGroup_Done () 
{
  mupo->setGroupSummary(editSumLine->value());
  
  // INCSAVE: compare data with the data saved before and save to 
  //          file if it has changed  
  if (mupo->incsave_groupHasChanged())
  {
#ifdef DEBUGINCSAVE
    std::cout << "save group\n";
#endif
    // group is new or has changed
    mupo->incsave_saveCurGroup();
  }
  mupo->incsave_forgetGroup();
  
  showMain(MAINVISIBLE_MAIN);
  mupo->checkCompleted();
  if (fGroupQuickView)
  {
    mupo->enterPrevGroup();
    fGroupQuickView = false;
  }
  showList(nLineBeforeEditItem);
}





void MuPOGUI::editToDo_updateRecThings() 
{
  T_recData recData;
  
  mupo->getRecData(recData);
  switch (recData.nType)
  {
    case RECTYPE_NONE:
      editToDoRecurringB->label("Recurrence: none");
      break;
    case RECTYPE_DAILY:
      editToDoRecurringB->label("Recurrence: daily");
      break;
    case RECTYPE_DAILYRESET:
      editToDoRecurringB->label("Recurrence: daily, resetable");
      break;
    case RECTYPE_WEEKLY:
      editToDoRecurringB->label("Recurrence: weekly");
      break;
    case RECTYPE_MONTHLYDAY:
      editToDoRecurringB->label("Recurrence: monthly, by day");
      break;
    case RECTYPE_MONTHLYDATE:
      editToDoRecurringB->label("Recurrence: monthly, by date");
      break;
    case RECTYPE_YEARLY:
      editToDoRecurringB->label("Recurrence: yearly");
      break;
  }
  editToDoRecurringB->redraw();
}

void MuPOGUI::editToDo_updateTimeThings() 
{
  // Alles, was mit Zeiten zu tun hat, bei dem Element in Einklang bringen:
  mupo->sanitizeTimeThings();
  
  // abhängig von isToDo und den eingestellten Zeiten 
  // Elemente darstellen oder nicht etc
  t_datetime edt, bdt;
  
  edt = mupo->getDateTime(DATECODE_End);
  bdt = mupo->getDateTime(DATECODE_Begin);
  
  editToDoEndDateB->hide();
  editToDoEndTimeB->hide();
  editToDoEndBox->hide();
  
  editToDoBegDateB->hide();
  editToDoBegTimeB->hide();
  editToDoBegBox->hide();
  
  editToDoRecurringB->hide();
#ifdef REMINDER
  editToDoReminderB->hide();
#endif	  
  editToDoShowAsBusy->hide();
  
  strncpy(editToDoBeginTime, dateTime2str(bdt).c_str(), 20);
  editToDoBegBox->label(editToDoBeginTime);
  strncpy(editToDoEndTime,   dateTime2str(edt).c_str(), 20);
  editToDoEndBox->label(editToDoEndTime);
  
  if (mupo->getItemPriority() <= PRIORITY_TODOMAX)
  {
    // es ist ein ToDo
    editToDoEndDateB->show();
    editToDoEndBox->show();
    
    if (ISDATESET(edt))
    {
      editToDoBegDateB->show();
      editToDoBegBox->show();
      editToDoEndTimeB->show();
      
      editToDoRecurringB->show();
#ifdef REMINDER
      editToDoReminderB->show();
#endif	  
      if (ISDATESET(bdt))
      {
        editToDoShowAsBusy->show();
        editToDoBegTimeB->show();
      }
      else
        CB_editToDo_showAs(SHOW_AS_FREE);
    }
    else
      CB_editToDo_showAs(SHOW_AS_FREE);
  }
  else
  {
    editToDoBegDateB->show();
    editToDoBegBox->show();
    
    if (ISDATESET(bdt))
    {
      editToDoBegTimeB->show();
      editToDoEndDateB->show();
      editToDoEndBox->show();
      
      editToDoRecurringB->show();
#ifdef REMINDER
      editToDoReminderB->show();
#endif	  
      if (ISDATESET(edt))
      {
        editToDoShowAsBusy->show();
        editToDoEndTimeB->show();
      }
      else
        CB_editToDo_showAs(SHOW_AS_FREE);
    }
    else
      CB_editToDo_showAs(SHOW_AS_FREE);
  }
  editToDo_updateRecThings();  
}

void MuPOGUI::editToDo_updateTimeSum() 
{
  std::string       tmp;
  unsigned int uTmp;
  
  uTmp = (unsigned int)(mupo->getItemTimeSum() / 60);
  tmp = "Time: " + itoStr(uTmp / 60, ' ', 0) + ":" + itoStr(uTmp % 60, '0', 2);
  strncpy(editTimeSum, tmp.c_str(), 20);
  
  editTimeSumBox->label(editTimeSum);
  editTimeSumBox->redraw();
}

#ifdef SHOWVOLTAGE
/* get the battery voltage*/
// returns 0..330 for 0 .. 3.3V
int MuPOGUI::getBatVoltage() 
{
  int fd;
  int in;
  unsigned short value[4];
  
  fd = open("/dev/adif", O_RDONLY);
  if (fd == -1)
    return 0;
  
  /* read all 4 registers even if we only care about the first one */
  in = read(fd, &value, 4*sizeof(short));
  close(fd);
  if (in == -1)
    return 0;
  
  return((value[0] * VOLTAGE_MUL) >> VOLTAGE_SH);
}
#endif

void MuPOGUI::updateClock() 
{
  // time 
  tm*      time2;
  time_t   time1;
  char     tmpStr[42];
  
  
  // uptime
#if !defined( WIN32 ) && !defined( __FreeBSD__  )
  struct sysinfo info;
  int updays, uphrs, upmin;
#endif
  
  // time
  time(&time1);
  time2 = localtime(&time1);
  strftime(curData, 30, DATEFORMATSTR"\n"TIMEFORMATSTR"\n", time2);
  
#ifdef SHOWVOLTAGE
  // voltage
  sprintf(tmpStr, "%i  V\n", getBatVoltage());
  // insert decimal point
  tmpStr[3] = tmpStr[2];
  tmpStr[2] = tmpStr[1];
  tmpStr[1] = '.';
  // cat
  strcat(curData, tmpStr);
#endif
  
  // uptime
#if !defined( WIN32 ) && !defined( __FreeBSD__  )
  sysinfo(&info);
  
  updays = (int) info.uptime / (60*60*24);
  upmin = (int) info.uptime / 60;
  uphrs = (upmin / 60) % 24;
  upmin %= 60;  
  sprintf(tmpStr, "%i d %02i:%02i", updays, uphrs, upmin);
  strcat(curData, tmpStr);  
#endif
  
  
  curDataBox->label(curData);
  curDataBox->redraw();
}

void MuPOGUI::showKeyboard(bool fDo)
{
  if (fDo)
  {
    hideKbdB->show();
    curDataBox->show();
#ifndef NOX11
    KbdButtonLeft->show();
    KbdButtonRight->show();
#endif
  }
}

void MuPOGUI::UpdateCryptMenu()
{
  if (mupo->crypt.CanLogIn())
    menu_Menu[MENU_CRYPT_LOGIN].activate();
  else
    menu_Menu[MENU_CRYPT_LOGIN].deactivate();
  if (mupo->crypt.CanAddPwd())
    menu_Menu[MENU_CRYPT_ADD_PWD].activate();
  else
    menu_Menu[MENU_CRYPT_ADD_PWD].deactivate();  
}

void MuPOGUI::ActOnSelected(int action)
{
  std::cout << "on selected: " << action << "\n";
  switch (action)
  {
    case ACT_COPY:
      {
        std::ofstream out;
        out.open(tmpfilename);
        out << "<type=MuPO>\n<version="MUPOVERSION">\n<data>\n";
        for (int n=1; n<=listView->size(); n++)
        {
          if (listView->selected(n))
            mupo->exportToStream(out, n-1);
        }
        out << "</data>\n";
        out.close();
      }
      break;
      
    case ACT_DEL:
      for (int n=listView->size(); n>0; n--)
        if (listView->selected(n))
          mupo->DeleteItem(n-1);
      mupo->checkCompleted();
      showList();
      break;
      
    default:
      break;
  }  
}

