// -*- mode: c; mode: fold -*-
// 
#ifndef _MUPOGUI_H_
# define _MUPOGUI_H_


# define USE_FLEDIT

# include <FL/Fl_Widget.H>
# include <FL/Fl_Window.H>
# include <FL/Fl_Pixmap.H>


# include <FL/Fl_Button.H>
# include <FL/Fl_Menu_Button.H>
# include <FL/Fl_Menu_Button.H>
# include <FL/Fl_Box.H>
# include <FL/Fl_Hold_Browser.H>


# ifndef NOX11
// fake keyboard:
#  define XK_LATIN1
#  define XK_MISCELLANY
#  include <X11/keysymdef.h>
#  include <X11/Xproto.h>
#  include "Fake_Keyboard.h"
#  include <FL/x.H>                    // fake keyboard
// #include <X11/extensions/XTest.h>    // fake keyboard
# endif

# include <FL/Fl_Input.H>
# include <FL/Fl_Check_Button.H>
# include <FL/Fl_Repeat_Button.H>

# ifdef USE_FLEDIT
#  include <FL/Fl_Text_Editor.H>
# else
#  include <FL/Fl_Multiline_Input.H>
# endif

# include <FL/Fl_Choice.H>



# include <string>
# include <vector>

# include "types.h"
# include "config.h"
# include "mupo.h"
# ifndef NOX11
#  include "aqw.h"
# endif


# define MAINVISIBLE_MAIN       0x00
# define MAINVISIBLE_ABOUT      0x01
# define MAINVISIBLE_TODO_BASIC 0x02
# define MAINVISIBLE_TODO_TIMES 0x03
# define MAINVISIBLE_TODO_DESCR 0x04
# define MAINVISIBLE_GROUP      0x05
# define MAINVISIBLE_INPUT      0x06

# define TRANSRESULTSIZE        500                                          


const int listViewCols[] = {
  13*SIZEMUL, 0
};

class MuPOGUI : public Fl_Window {
  
public:
  MuPOGUI(int w, int h, const char *l = 0);
  
  ~MuPOGUI();
  
  void resize(int X, int Y, int W, int H);
  
  
  
  void updateClock();
  void CB_Done ();
  void CB_Cancel ();
  void CB_Opts_sort(int item);
  void CB_Opts_View_flat    ();
  void CB_Opts_View_tree    ();
  void CB_Menu_MuPOView     (int);
  void CB_Opts_View_scompleted  ();
  void CB_Opts_View_completedl  ();
  void CB_Opts_View_toggleTextSize  ();
  
  void CB_Menu_newTodo   ();
  void CB_Menu_newGroup  ();
  void CB_Menu_editGroup ();
  void CB_Menu_delGroup  ();
  void CB_Menu_delCompl  ();
  void CB_Menu_search    ();
  void CB_Menu_insFile   ();
  void CB_Menu_paste     ();
  void CB_Up             ();
  
  void CB_hideKbd();

  void hideAll();
  int  showMain(int);
  void editToDo_Done();
  void editGroup_Done();
  
  void editToDo_updateTimeSum();
  void showToDoState();
  void editToDo_saveSettings();

  void UpdateCryptMenu();
  
  void setCompletedMessage(int code);
  
  int                     nMainVisible;  // welcher Bildschirm wird gezeigt?
  unsigned int            uWinHeight, uTotalHeight;
  unsigned int            uWinWidth, uTotalWidth;
  char                    tmpfilename[16];
  std::string             headstring;
  int                     nHeadHeight;
  
  /**
   * Used to remember that a group is edited by clicking on it. The difference
   * to a "normal" group-edit is that mupo->curGroup has to be changed to do it,
   * therefore this variable is needed.
   */
  bool                    fGroupQuickView;
  
  /**
   * Line in the current view before editing an item
   */
  int                     nLineBeforeEditItem;
  int                     nLineBeforeOnlyActive;
  
  T_GUIData               GUIData;   
  
  MuPO*                   mupo;
  
  Fl_Browser*             listView;
  Fl_Button*              headline;
  Fl_Button*              mainUpB; 
  Fl_Button*              mainDoneB;
  Fl_Menu_Button*         mainMenuB; 
  Fl_Button*              mainNewB;
  Fl_Button*              mainActiveB;
  Fl_Box*                 mainAboutBox;
  
  Fl_Button*              hideKbdB;
  
  
  Fl_Button*              mainCalTodayB;
  Fl_Button*              mainCalSelB;
  Fl_Button*              mainCalNextB;
  Fl_Button*              mainCalPrevB;
  Fl_Button*              mainCalNextLB;
  Fl_Button*              mainCalPrevLB;
  
  //
  Fl_Button*              inputCancelB;
  Fl_Input*               inputLine;
  Fl_Box*                 inputBox;
  int                     inputUse;
  
  
  
  const char* myInput(const char* text);
  
  int  checkActiveJobs();
  void setMenuActive();
  void initFakeKbd();
  void delCurGroup();
  void setHeadline();
  void showList(int nLine = 0);
  void makeWindows();
  void initOptions();
  void initVars();
  void listViewClicked();
  
  void init(MuPO*  mymupo, const char* tmpFileName);
  
  
  
# ifdef DIRTYTRANSHACK
  char                    transresult[TRANSRESULTSIZE];
# endif
  
  Fl_Input*               editSumLine;
  Fl_Box*                 editToDoStateBox;
  Fl_Box*                 editTimeSumBox;
  char                    complTime[20];
  char                    editTimeSum[20];
  Fl_Group*               editPrGr;
  Fl_Check_Button*        editPrB[PRIORITY_TODOMAX];
  std::string             editPrLabel[PRIORITY_TODOMAX];
  Fl_Choice*              editToDoShowAsBusy;
  Fl_Menu_Button*         editToDoBegDateB;
  Fl_Menu_Button*         editToDoEndDateB;
  Fl_Menu_Button*         editToDoBegTimeB;
  Fl_Menu_Button*         editToDoEndTimeB;
  Fl_Box*                 editToDoBegBox;
  Fl_Box*                 editToDoEndBox;
  char                    editToDoBeginTime[20];
  char                    editToDoEndTime[20];
  Fl_Check_Button*        editToDoDoneCB;
  Fl_Check_Button*        editToDoIsToDoCB;
  Fl_Button*              editToDoRecurringB;
# ifdef REMINDER
  Fl_Button*              editToDoReminderB;
# endif
  Fl_Check_Button*        editToDoCryptCB;
# ifndef NOX11
  Fl_Repeat_Button*       editToDoDescrUpB;
  Fl_Repeat_Button*       editToDoDescrDwnB;
  Fl_Button*              KbdButtonLeft;
  Fl_Button*              KbdButtonRight;
# endif
  
# ifdef USE_FLEDIT
  Fl_Text_Editor*         editToDoDescr;
  Fl_Text_Buffer*         editToDoDescrBuf;
# else
  Fl_Multiline_Input*     editToDoDescr;
# endif
  Fl_Button*              editToDoOtherB;
  Fl_Button*              editToDoExtEditB;
  Fl_Button*              editToDoStrtStpB;
  Fl_Menu_Button*         editToDoMenuB;
  Fl_Browser*             editToDoTimeList;
  
  void CB_editToDo_Menu_t2Group     ();
  void CB_editToDo_Menu_del         ();
  void CB_editToDo_Menu_showGroup   ();
  
  void CB_edit_Pr         (int);
  void CB_editToDo_showAs (int);
  
  void CB_editToDo_compl         ();
  
  void CB_editToDo_UpB          ();
  void CB_editToDo_DwnB         ();
  void CB_editToDo_StrtStp ();
  void CB_editToDo_ExtEdit ();
  
  void ActOnSelected(int action);
  
# ifndef NOX11
  void CB_kbdLeft();
  void CB_kbdRight();
# endif     
  
  
  void show_editToDoWin(bool fShowDescrIfExists = false,
                        bool fForceShowDescr    = false);
  std::string dateTime2str(t_datetime dt);
  void   editToDo_updateTimeThings();
  void   editToDo_updateRecThings();
  
  
  Fl_Menu_Button*         editGroupMenuB;
  Fl_Box*                 editGroupComplDetailBox;
  char                    editGroupComplDetail[30];
  
  
  
  void cb_HideKbd();
  
  
  
  void draw();
  int  handle(int);
  
  
private:
  void showKeyboard(bool fDo);
  int getBatVoltage();
# ifndef NOX11
  void  drawKbd();
# endif     
  
  void enterGroup(int nIdx, bool fKeyboard);
  void toggleCompleted(int nIdx, bool fKeyboard);
  void OpenItem(int idx, bool fShowDescrIfExists);
  
  /**
   * returns 1 if buffer has to be freed
   */
  int GetDescrBuffer(char*& buf);  
  void SetDescrBuffer(const char* buffer);
  std::string ReadFileIntoBuf(char* filename);
  
  std::string colStrHead;
  std::string colStrLineA;
  std::string colStrLineB;
  
  char                    curData[42];
  Fl_Box*                 curDataBox;
  unsigned int            uTextSize1;
  
# ifndef NOX11
  int                     nKeyUp, nKeyDown, nKeyLeft, nKeyRight;
  Fake_Keyboard*          fakeKbd;
  AQW*                    aqw;
  
  Fl_Pixmap*              letter_templates_[4];
#  ifndef AQW_SIMPLEINTERFACE
  Fl_Pixmap*              lock_image_;
  Fl_Pixmap*              ctrl_image_;
  Fl_Pixmap*              meta_image_;
#  endif
# endif   
};



#endif
