#ifndef CRYPTMAN_H
# define CRYPTMAN_H

#include <string>
#include <vector>
#include <iostream>

class PwdData
{
public:
  PwdData(std::string descr,
          const unsigned char* cryptkey, int len_ck,
          const unsigned char* saltA,    int len_sA,
          const unsigned char* saltB,    int len_sB);
public:
  
  bool Equals(const PwdData& b);
  
  /**
   * description of password (username or whatever)
   */
  std::string keydescr;

  /**
   * encrypted masterkey
   */
  std::vector<unsigned char> cryptkey;
  
  std::vector<unsigned char> saltA;
  std::vector<unsigned char> saltB;
};

class CryptMan
{
public:
  CryptMan();
  ~CryptMan();
  
  CryptMan& operator = (const CryptMan& init);
  
  int GetNumPwds();
  std::string GetPwdDescr(int n);
  PwdData* GetPwdData(int n);
  
  void Login(int nPwdNum, const char* pwd);
  bool IsLoggedIn();
  
  /**
   * encrypts a null-terminated string
   */
  char* Encrypt(char* input, int& len, bool fIV);
  
  /**
   * decrypts to a null-terminated string
   */
  char* Decrypt(char* input, int len, bool fIV);
  
  void AddPwd(std::string descr,
              std::string pwd);
  
  
  bool CanAddPwd();
  bool CanLogIn();

  bool NeedsSaving() { return(fNeedsSaving); };
  
  /**
   * Set encrypted hash of masterkey.
   * Used to check whether login has been successful.
   * See mkey_checkdata.
   */
  void SetMKeyCheckData(const unsigned char* data, int len);
  
  /**
   * 
   */
  void SetPwdData(std::string descr,
                  const unsigned char* cryptkey, int len_ck,
                  const unsigned char* saltA,    int len_sA,
                  const unsigned char* saltB,    int len_sB);
  
  void Write(std::ostream& out);
  int  Read(std::istream& in);

  bool MKeyEquals(const CryptMan& b);
  
private:
  void WriteVector(std::ostream& out, std::vector<unsigned char>& vect);
  void ReadVector(std::istream& in, std::vector<unsigned char>& vect);
  
  /**
   * Liefert einen Schlüssel mit einer Länge von KEY_LEN Byte.
   * Das Passwort wird mit PwdSaltA angereichert, der daraus resultierende Hash (Hash_SHA1::HASH_LEN Byte)
   * wird mit PwdSaltB angereichert. PwdSaltB sollte also (KEY_LEN - Hash_SHA1::HASH_LEN) Byte groß sein (mehr bringt nix).
   */
  unsigned char* DeriveKey(std::string PwdString, 
                           const unsigned char* PwdSaltA, int len_sA,
                           const unsigned char* PwdSaltB, int len_sB);

  unsigned char* DeriveKey(std::string                 PwdString,
                           std::vector<unsigned char>& PwdSaltA, 
                           std::vector<unsigned char>& PwdSaltB);
  
  /**
   * Der Schlüssel key hat die Länge KEY_LEN.
   */
  unsigned char* Encrypt(const unsigned char* input, int nInLen, const unsigned char* key, int& nOutLen, bool fIV);

  /**
   * Der Schlüssel key hat die Länge KEY_LEN.
   */
  unsigned char* Decrypt(const unsigned char* input, int nInLen, const unsigned char* key, int& nOutLen, bool fIV);

  unsigned char* GetArray(std::vector<unsigned char>& vec);

  void PrintBin(const unsigned char* data, int len);
  void PrintBin(const std::vector<unsigned char>& vec);
  
private:
  
  enum { KEY_LEN=32 };
  
  /**
   * Encrypted hash of Masterkey. 
   * Used to check whether login has been successful.
   * See SetMKeyCheckData.
   */
  std::vector<unsigned char> mkey_checkdata;
  
  std::vector<PwdData> keys;

  unsigned char mkey[KEY_LEN];

  bool fLoggedIn;
  
  bool fNeedsSaving;
  
  int  nRunden;
};

#endif
