// -*- mode: c; mode: fold -*-
// 
#include "todolistitem.h"
#include "todolistgroup.h"
#include "todolisttodo.h"
#include "tools.h"
#include "config.h"
#include <fstream>
#include <vector>
#include <iostream>
#include <cstdlib>


struct T_MuPOFile {
  ToDoListGroup*      pRoot;
  ToDoListGroup*      pGroup;
  ToDoListItem*       pItem;
  unsigned int        uSyncIDMax;
  unsigned int        uSyncIDMaxAfterSync;
  CryptMan            crypt;
};

T_MuPOFile         fileOld;
std::vector<T_MuPOFile> file;
ToDoListGroup*     pMsgGroup;
ToDoListGroup*     pMsgRoot;
ToDoListToDo*      pMsgToDo;
ToDoListGroup*     pCurFileRoot;
T_Calendar         cal;


#define _DEBUG
#define _DEBUG_MSG

void printUsage() 
{
  std::cout << "Usage:\n"
    "   muposync syncFile file1 ... \n"
    " You can use any number of files (even none). They all have to be created from\n"
    " the former syncFile. Copy the syncFile to all clients afterwards\n"
    " and keep it for the next sync.\n";
}


/**
 * Liest die Datei mit dem Namen "name" in die Struktur "file". Wenn
 * "fCheck" true ist, wird...
 */
int readFile(char*        name,
	     T_MuPOFile&  file,
	     bool         fCheck = true) 
{
  std::ifstream      ifile;
  std::string        line = "";
  unsigned int  uSyncIDMaxCheck = 0;
  unsigned int  uDummy;
  
  ifile.open(name);
  if (ifile)
  {
    while ((line = readfLine(ifile)) != "g")
    {
      if (line.find("<idmax=") == 0)
      {
        line.replace(0, 7, "");
        line.replace(line.length()-1, 1, "");
        file.uSyncIDMax = atoi(line.c_str());
        uSyncIDMaxCheck = file.uSyncIDMax;
      }      
      if (line.find("<idmaxaftersync=") == 0)
      {
        line.replace(0, 16, "");
        line.replace(line.length()-1, 1, "");
        file.uSyncIDMaxAfterSync = atoi(line.c_str());
      }      
      if (line.find("<Crypt>") == 0)
      {
        if (file.crypt.Read(ifile))
        {
          std::cerr << "crypt\n";
          return(-1);
        }
      }
    }
    file.pRoot = new ToDoListGroup(0, "/", 0);
    file.pRoot->readContent(ifile, uDummy, file.uSyncIDMax, cal);
    if (fCheck && file.uSyncIDMax != uSyncIDMaxCheck)
    {
      std::cerr << "There have been items in the file with (uSyncID == 0)!!\n";
      return(-1);
    }
    
    ifile.close();
    return(0);
  }
  
  return(-1);
}



int checkData(unsigned int uNrFiles) 
{
  if (uNrFiles)
  {
    for (unsigned int uCnt=0; uCnt < uNrFiles-1;uCnt++)
      if (file[uCnt].uSyncIDMaxAfterSync != file[uCnt+1].uSyncIDMaxAfterSync)
      {
        std::cerr << "Error: inconsistent data, uSyncIDMaxAfterSync not the same in\n"
          "(file" << uCnt << ", file" << uCnt+1 << ")\n";
        return(-1);
      }
    if (file[0].uSyncIDMaxAfterSync != fileOld.uSyncIDMaxAfterSync)
    {
      std::cerr << "Error: inconsistent data, uSyncIDMaxAfterSync not the same in\n"
        "(oldSyncFile, file1)\n";
      return(-1);
    }
  }
  return(0);
}

int checkCrypt(unsigned int uNrFiles)
{
  for (unsigned int uCnt=0; uCnt < uNrFiles;uCnt++)
  {
    std::cout << "########## checking crypt: file" << uCnt << "\n";
    // Wenn beide mit unterschiedlichen Schlüsseln verschlüsselt sind, hat
    // man ein Problem:
    if (fileOld.crypt.GetNumPwds() > 0 &&
        file[uCnt].crypt.GetNumPwds() > 0 &&
        file[uCnt].crypt.MKeyEquals(fileOld.crypt) == false)
    {
      std::cerr << "Files are encrypted using different Masterkeys!\n";
      return(-1);
    }
    // Verschlüsselung übernehmen, wenn sie dazugekommen ist, ebenso größere
    // Anzahl von Passworten. 
    // todo: das ist nicht wirklich gut; geht in die Hose, wenn Passworte entfernt werden!
    if (fileOld.crypt.GetNumPwds() < file[uCnt].crypt.GetNumPwds())
    {
      fileOld.crypt = file[uCnt].crypt;
    }    
  }
  
  return(0);
}

void checkDiff(unsigned int uNrFiles)
{
  for (unsigned int uCnt=0; uCnt < uNrFiles;uCnt++)
  {
    std::cout << "########## checking changes: file" << uCnt << "\n";
    file[uCnt].pRoot->checkDiff(fileOld.pRoot);
  }
}


void checkDel(unsigned int uNrFiles) 
{
  for (unsigned int uCnt=0; uCnt < uNrFiles;uCnt++)
  {
    std::cout << "########## checking deleted: file" << uCnt << "\n";
    fileOld.pRoot->checkDel(file[uCnt].pRoot);
  }
}



void msg(ToDoListItem* pItem,
	 bool          fGroup,
	 const char*   summary,
	 const char*   description) 
{
#ifdef DEBUG
  std::cout << "message(" << summary << ")(" << description;
#endif
  if (pMsgGroup == (ToDoListGroup*)0)
  {
    std::string path;
    
#ifdef DEBUG_MSG
    std::cout << "create pMsgGroup\n";
#endif
    
    pMsgGroup = pMsgRoot->newGroup();
    if (pMsgGroup == (ToDoListGroup*)0)
    {
      std::cerr << "Error on creating pMsgGroup in void msg(...);\n";
    }
    
    if (fGroup)
      path = "Group: ";
    else
      path = "ToDo: ";
#ifdef DEBUG_MSG
    std::cout << "getting summary\n";
#endif
    path += pItem->getSummary();      
    
#ifdef DEBUG_MSG
    std::cout << "setting summary\n";
#endif
    pMsgGroup->setSummary(path.c_str());
#ifdef DEBUG_MSG
    std::cout << "creating todo\n";
#endif
    pMsgToDo = pMsgGroup->newToDo();
#ifdef DEBUG_MSG
    std::cout << "setting summary\n";
#endif
    pMsgToDo->setSummary("path of this item:");
#ifdef DEBUG_MSG
    std::cout << "setting priority\n";
#endif
    pMsgToDo->setPriority(PRIORITY_NOTE, cal);
    if (fGroup)
    {
#ifdef DEBUG_MSG
      std::cout << "getting path: ";
#endif
      path = ((ToDoListGroup*)pItem)->getPath();
#ifdef DEBUG_MSG
      std::cout << path << "\n";
#endif          
      if (path.length())
        path.replace(0,1,"");
      if (path.length())
        path.replace(path.length()-1,1,"");
      
      path = pCurFileRoot->getPathText(path);
#ifdef DEBUG_MSG
      std::cout << path << "\n";
#endif          
    }
    else
    {
      path = "not implemented yet";
    }
    path = "not implemented yet";
#ifdef DEBUG_MSG
    std::cout << "setting description\n";
#endif
    pMsgToDo->setDescription(path.c_str());
#ifdef DEBUG_MSG
    std::cout << "pMsgGroup created\n";
#endif
  }
#ifdef DEBUG_MSG
  std::cout << "creating todo\n";
#endif
  pMsgToDo = pMsgGroup->newToDo();
#ifdef DEBUG_MSG
  std::cout << "setting summary\n";
#endif
  pMsgToDo->setSummary(summary);
#ifdef DEBUG_MSG
  std::cout << "setting priority\n";
#endif
  pMsgToDo->setPriority(PRIORITY_NOTE, cal);
#ifdef DEBUG_MSG
  std::cout << "setting description\n";
#endif
  pMsgToDo->setDescription(description);
#ifdef DEBUG
  std::cout << ")\n";
#endif
}


void syncOldItems(unsigned int uNrFiles) 
{
  bool               fGroup;
  unsigned int       uMask;
  
  std::cout << "\n" << fileOld.uSyncIDMaxAfterSync << " items to update\n";
  for (unsigned int uCnt=1; uCnt <= fileOld.uSyncIDMaxAfterSync; uCnt++)
  {
#ifdef DEBUG
    std::cout << uCnt << "\n";
#else
    std::cout << "\r" << uCnt;
#endif
    fileOld.pItem = fileOld.pRoot->getPtrToItemWithSyncID(uCnt, fGroup, true);
    if (fileOld.pItem != (ToDoListItem*)0)
    {
      pMsgGroup = (ToDoListGroup*)0;
      uMask     = 0;
      for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
      {
#ifdef DEBUG
        std::cout << "datei " << uFCnt << "\n";
#endif
        pCurFileRoot = file[uFCnt].pRoot;
        file[uFCnt].pItem = file[uFCnt].pRoot->getPtrToItemWithSyncID(uCnt, fGroup, true);
        if (file[uFCnt].pItem == (ToDoListItem*)0)
        {
          if (fileOld.pItem->getMustBeKept())
            msg(fileOld.pItem, fGroup, "deleted in one file" , "");		    
        }
        else
          // look if something has changed:
          uMask |= file[uFCnt].pItem->getSyncFlag();
      }
      if (uMask)
      {
#ifdef DEBUG
        std::cout << "Checking summary\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_SUMMARY)
            {
              if ((uMask & DIFF_SUMMARY) != DIFF_SUMMARY)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                msg(fileOld.pItem, fGroup, "summary", file[uFCnt].pItem->getSummary().c_str());
              }
              else
              {
                fileOld.pItem->setSummary(file[uFCnt].pItem->getSummary());
                uMask &= ~DIFF_SUMMARY;
              }
            }
        
#ifdef DEBUG
        std::cout << "Checking Flags\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_FLAGS)
            {
              if ((uMask & DIFF_FLAGS) != DIFF_FLAGS)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                msg(fileOld.pItem, fGroup, "flags", file[uFCnt].pItem->getSummary().c_str());
              }
              else
              {
                fileOld.pItem->setSummary(file[uFCnt].pItem->getSummary());
                uMask &= ~DIFF_FLAGS;
              }
            }
        
#ifdef DEBUG
        std::cout << "Checking priority\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_PRIORITY)
            {
              if ((uMask & DIFF_PRIORITY) != DIFF_PRIORITY)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                msg(fileOld.pItem, fGroup, "priority", itoStr(file[uFCnt].pItem->getPriority(), ' ', 1).c_str());
              }
              else
              {
                fileOld.pItem->setPriority(file[uFCnt].pItem->getPriority());
                uMask &= ~DIFF_PRIORITY;		    
              }
            }
        
#ifdef DEBUG
        std::cout << "Checking timethings\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_TIMETHINGS)
            {
              t_datetime  abegin, aend;
              T_recData   arecdata;
              int         ashowas;
              
              // Daten besorgen:
              ((ToDoListToDo*)file[uFCnt].pItem)->getRecData(arecdata);
              abegin = ((ToDoListToDo*)file[uFCnt].pItem)->getDateTime(DATECODE_Begin);
              aend   = ((ToDoListToDo*)file[uFCnt].pItem)->getDateTime(DATECODE_End);
              ashowas= ((ToDoListToDo*)file[uFCnt].pItem)->getShowTimeAs();
              // jetzt die Daten entweder übertragen, oder, falls diese Daten schon 
              // von einer anderen Seite verändert wurden, einen Hinweis machen, 
              // der die Daten enthält:
              if ((uMask & DIFF_TIMETHINGS) != DIFF_TIMETHINGS)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                msg(fileOld.pItem, fGroup, 
                    "show as, begin date/time, end date/time or recurrence data", 
                    "just compare those things in this item to the one left in its original place..");
                pMsgToDo->setShowTimeAs(ashowas, cal);
                if (ISDATESET(abegin))
                  pMsgToDo->setDateDontCheck(DATECODE_Begin, GETYEAR(abegin), 
                                             GETMONTH(abegin), GETDAY(abegin), cal);
                if (ISTIMESET(abegin))
                  pMsgToDo->setTimeDontCheck(DATECODE_Begin, GETHOUR(abegin), 
                                             GETMINUTE(abegin), cal);
                if (ISDATESET(aend))
                  pMsgToDo->setDateDontCheck(DATECODE_End, GETYEAR(aend), 
                                             GETMONTH(aend), GETDAY(aend), cal);
                if (ISTIMESET(aend))
                  pMsgToDo->setTimeDontCheck(DATECODE_End, GETHOUR(aend), 
                                             GETMINUTE(aend), cal);
                pMsgToDo->setRecData(arecdata, cal);
              }
              else
              {
                ((ToDoListToDo*)fileOld.pItem)->setShowTimeAs(ashowas, cal);
                
                if (ISDATESET(abegin))
                  ((ToDoListToDo*)fileOld.pItem)->setDateDontCheck(DATECODE_Begin, 
                                                                   GETYEAR(abegin), 
                                                                   GETMONTH(abegin), 
                                                                   GETDAY(abegin), 
                                                                   cal);
                if (ISTIMESET(abegin))
                  ((ToDoListToDo*)fileOld.pItem)->setTimeDontCheck(DATECODE_Begin, 
                                                                   GETHOUR(abegin), 
                                                                   GETMINUTE(abegin), 
                                                                   cal);
                if (ISDATESET(aend))
                  ((ToDoListToDo*)fileOld.pItem)->setDateDontCheck(DATECODE_End, 
                                                                   GETYEAR(aend), 
                                                                   GETMONTH(aend), 
                                                                   GETDAY(aend), 
                                                                   cal);
                if (ISTIMESET(aend))
                  ((ToDoListToDo*)fileOld.pItem)->setTimeDontCheck(DATECODE_End, 
                                                                   GETHOUR(aend), 
                                                                   GETMINUTE(aend), 
                                                                   cal);
                ((ToDoListToDo*)fileOld.pItem)->setRecData(arecdata, cal);
                
                uMask &= ~DIFF_TIMETHINGS;
              }
            }
        
#ifdef DEBUG
        std::cout << "Checking show completed\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
        {
#ifdef DEBUG
          std::cout << "datei " << uFCnt << "\n";
#endif                  
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_SHOWCOMPL)
            {
              if ((uMask & DIFF_SHOWCOMPL) != DIFF_SHOWCOMPL)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                if (((ToDoListGroup*)file[uFCnt].pItem)->getShowCompleted())
                  msg(fileOld.pItem, fGroup, "Show completed", "");
                else
                  msg(fileOld.pItem, fGroup, "Don't show completed", "");
              }
              else
              {
                ((ToDoListGroup*)fileOld.pItem)->setShowCompleted(((ToDoListGroup*)file[uFCnt].pItem)->getShowCompleted());
                uMask &= ~DIFF_SHOWCOMPL;
              }
            }
        }
        
#ifdef DEBUG
        std::cout << "Checking completed last\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_COMPLLAST)
            {
              if ((uMask & DIFF_COMPLLAST) != DIFF_COMPLLAST)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                if (((ToDoListGroup*)file[uFCnt].pItem)->getCompletedLast())
                  msg(fileOld.pItem, fGroup, "Completed last", "");
                else
                  msg(fileOld.pItem, fGroup, "Completed anywhere", "");
              }
              else
              {
                ((ToDoListGroup*)fileOld.pItem)->setCompletedLast(((ToDoListGroup*)file[uFCnt].pItem)->getCompletedLast());
                uMask &= ~DIFF_COMPLLAST;
              }
            }
        
#ifdef DEBUG
        std::cout << "Checking working times\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_WTIMES)
            {
              if ((uMask & DIFF_WTIMES) != DIFF_WTIMES)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                msg(fileOld.pItem, fGroup, "working times", "");
                pMsgToDo->setRunningDirect(((ToDoListToDo*)file[uFCnt].pItem)->isRunning());
                pMsgToDo->clearWTimes();
                for (unsigned int uTCnt=0; uTCnt<((ToDoListToDo*)file[uFCnt].pItem)->getNrTimes(); uTCnt++)
                  pMsgToDo->addTime(((ToDoListToDo*)file[uFCnt].pItem)->getTimeBegin(uTCnt), ((ToDoListToDo*)file[uFCnt].pItem)->getTimeEnd(uTCnt));
                
              }
              else
              {
                ((ToDoListToDo*)fileOld.pItem)->setRunningDirect(((ToDoListToDo*)file[uFCnt].pItem)->isRunning());		  
                ((ToDoListToDo*)fileOld.pItem)->clearWTimes();
                for (unsigned int uTCnt=0; uTCnt<((ToDoListToDo*)file[uFCnt].pItem)->getNrTimes(); uTCnt++)
                  ((ToDoListToDo*)fileOld.pItem)->addTime(((ToDoListToDo*)file[uFCnt].pItem)->getTimeBegin(uTCnt), ((ToDoListToDo*)file[uFCnt].pItem)->getTimeEnd(uTCnt));
                uMask &= ~DIFF_WTIMES;
              }
            }
        
#ifdef DEBUG
        std::cout << "Checking is completed\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_COMPLETED)
            {
              if ((uMask & DIFF_COMPLETED) != DIFF_COMPLETED)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                if (((ToDoListToDo*)file[uFCnt].pItem)->isCompleted())
                  msg(fileOld.pItem, fGroup, "Is Completed", "");
                else
                  msg(fileOld.pItem, fGroup, "Not completed", "");
                pMsgToDo->setCompletedDirect(((ToDoListToDo*)file[uFCnt].pItem)->getCompletedDirect());
              }
              else
              {
                ((ToDoListToDo*)fileOld.pItem)->setCompletedDirect(((ToDoListToDo*)file[uFCnt].pItem)->getCompletedDirect());
                uMask &= ~DIFF_COMPLETED;
              }
            }
        
#ifdef DEBUG
        std::cout << "Checking description\n";
#endif
        for (unsigned int uFCnt=0; uFCnt<uNrFiles; uFCnt++)
          if (file[uFCnt].pItem != (ToDoListItem*)0)
            if (file[uFCnt].pItem->getSyncFlag() & DIFF_DESCRIPTION)
            {
              if ((uMask & DIFF_DESCRIPTION) != DIFF_DESCRIPTION)
              {
                // check, as it already has been overwritten
                pCurFileRoot = file[uFCnt].pRoot;
                const char* descr_c;
                ((ToDoListToDo*)file[uFCnt].pItem)->GetDescription(descr_c, &(file[uFCnt].crypt));
                msg(fileOld.pItem, fGroup, "description", descr_c);
              }
              else
              {
                ToDoListToDo* bla = (ToDoListToDo*)file[uFCnt].pItem;

                ((ToDoListToDo*)fileOld.pItem)->setDescription(bla->GetDescription());
                uMask &= ~DIFF_DESCRIPTION;
              }	    
            }
      }
    }
  }
  std::cout << "\n";
}


void addNewItems(unsigned int uNrFiles) 
{
  for (unsigned int uCnt=0; uCnt < uNrFiles;uCnt++)
  {
    std::cout << "########## adding new: file" << uCnt << "\n";
    file[uCnt].pRoot->addNewItems(fileOld.pRoot);
  }
}


void createMsgGroup() 
{
  struct tm* time2;
  time_t time1;
  char   complTime[30];
  std::string name;
  
  time(&time1);
  time2 = localtime(&time1);
  strftime(complTime, 42, DATEFORMATSTR" "TIMEFORMATSTR, time2);
  name = "sync-log ";
  name += complTime;
  
  pMsgRoot = fileOld.pRoot->newGroup();
  pMsgRoot->setSummary(name);  
}



int main(int argc, char **argv) 
{
  std::ofstream ofile;
  
  std::cout << "MuPO-Sync " << MUPOSYNCVERSION << "   Jens Wilhelm Wulf <j.w.wulf@gmx.net>\n\n";
  if (argc < 2)
  {
    printUsage();
    return(-1);
  }
  
  std::cout << "file with result of previous sync: " << argv[1] << "\n";
  if (readFile(argv[1], fileOld, (argc != 2)))
  {
    std::cerr << "Error reading file: " << argv[1] << "\n";
    return(-1);
  }
  
  for (int nCnt=2; nCnt < argc; nCnt++)
  {
    std::cout << "input file to sync: " << argv[nCnt] << "\n";
    file.push_back(fileOld);
    if (readFile(argv[nCnt], file[nCnt-2]))
    {
      std::cerr << "Error reading file: " << argv[nCnt] << "\n";
      return(-1);
    }
  }
  
  if (checkData(argc-2))
    return(-1);

  if (checkCrypt(argc-2))
    return(-1);
  
  checkDiff(argc-2);
  checkDel(argc-2);
  if (argc > 2)
  {
    fileOld.pRoot->syncDelete();
    
    createMsgGroup();
    
    syncOldItems(argc-2);
    addNewItems(argc-2);
  }
  
  fileOld.uSyncIDMax = 0;
  fileOld.pRoot->resetSyncID(fileOld.uSyncIDMax);
  fileOld.uSyncIDMaxAfterSync = fileOld.uSyncIDMax;
    
  ofile.open(argv[1]);
  if (!ofile)
  {
    std::cerr << "Error writing file: " << argv[1] << "\n";
    return(-1);
  }
  ofile << "<type=MuPO-Sync>\n<version=" << MUPOSYNCVERSION << ">\n<data>\n";
  ofile << "<idmax=" << fileOld.uSyncIDMax << ">\n";
  ofile << "<idmaxaftersync=" << fileOld.uSyncIDMaxAfterSync << ">\n";
  fileOld.crypt.Write(ofile);
  fileOld.pRoot->writeContent(ofile, false, &fileOld.crypt);
  ofile << "</data>\n";
  ofile.close();
  
  delete fileOld.pRoot;
  for (int nCnt=2; nCnt < argc; nCnt++)
    delete file[nCnt-2].pRoot;
  
  return(0);
}


