// -*- mode: c; mode: fold -*-
// 
#include "todolisttodo.h"

#include "todolistgroup.h"
#include <cstdlib>
#include <string.h>
#include <ctype.h>
#include "tools.h"
#include "types.h"
#include "config.h"



#define _DEBUGSNOW
#ifdef DEBUGSNOW
# include <stdio.h>
#endif

#define _DEBUGOPERATORS
#ifdef DEBUGOPERATORS
# include <stdio.h>
#endif

#ifdef LARGESCREEN
# define DEBUG_RECRESET
# ifdef DEBUG_RECRESET
#  include <stdio.h>
# endif
#endif


ToDoListToDo::ToDoListToDo(unsigned int ID,
			   std::string  pathInit)
  : ToDoListItem(ID, pathInit)
{
  completed.fDone   = INITIALISC;
  fRunning          = false;
  dateTimeBegin     = INITIAL_DATETIME;
  dateTimeEnd       = INITIAL_DATETIME;
  showTimeAs        = SHOW_AS_FREE;
  
  recData.nType     = RECTYPE_NONE;
  recData.nInterval = 1;
  recData.begin     = INITIAL_DATETIME;
  recData.end       = INITIAL_DATETIME;
}

ToDoListToDo::ToDoListToDo(const ToDoListToDo& init) 
  : ToDoListItem(init)
{
  // 1. init
  
  // 2. set data
  *this = init;
}

ToDoListToDo::~ToDoListToDo()
{
  while (workingTimes.size())
    workingTimes.pop_back();
}

ToDoListToDo& ToDoListToDo::operator = (const ToDoListToDo& init) 
{
  T_workingTimes tmpSpan;
  
  descr = init.descr;
  
  unsigned int uForMaxVal = init.workingTimes.size();
  for (unsigned int uCnt=0; uCnt < uForMaxVal; uCnt++)
  {
    tmpSpan = init.workingTimes[uCnt];
    workingTimes.push_back(tmpSpan);     
  }
  completed     = init.completed;
  
  showTimeAs    = init.showTimeAs;
  fRunning      = init.fRunning;
  dateTimeBegin = init.dateTimeBegin;
  dateTimeEnd   = init.dateTimeEnd;
  recData       = init.recData;
  
  path          = init.path;
  uID           = init.uID;
  uSyncID       = init.uSyncID;
  summary       = init.summary;
  uPriority     = init.uPriority;
  
  return(*this);
}

bool ToDoListToDo::operator == (ToDoListToDo todo2) 
{
#ifdef DEBUGOPERATORS
  printf("operator ==\n");
#endif

  if (descr != todo2.descr)
  {
#ifdef DEBUGOPERATORS
    printf("  descr different\n");
#endif          
    return(false);
  }
  
  if (getNrTimes() != todo2.getNrTimes())
  {
#ifdef DEBUGOPERATORS
    printf("  nrtimes different\n");
#endif
    return(false);
  }
  
  unsigned int uForMaxVal = getNrTimes();
  for (unsigned int uCnt=0; uCnt < uForMaxVal; uCnt++)
    if (getTimeBegin(uCnt) != getTimeBegin(uCnt) ||
        getTimeEnd(uCnt)   != getTimeEnd(uCnt))
    {
#ifdef DEBUGOPERATORS
      printf("  time different\n");
#endif
      return(false);
    }
  
  if (todo2.getCompletedDirect() != getCompletedDirect())
  {
#ifdef DEBUGOPERATORS
    printf("  compl different\n");
#endif
    return(false);
  }
  
  if (todo2.showTimeAs != getShowTimeAs())
  {
#ifdef DEBUGOPERATORS
    printf("  showtimeas different\n");
#endif
    return(false);
  }
  
  if (todo2.fRunning != isRunning())
  {
#ifdef DEBUGOPERATORS
    printf("  frunning different\n");
#endif
    return(false);
  }
  
  if (todo2.getDateTime(DATECODE_Begin) != getDateTime(DATECODE_Begin))
  {
#ifdef DEBUGOPERATORS
    printf("  datetimebegin different\n");
#endif
    return(false);
  }
  
  if (todo2.getDateTime(DATECODE_End) != getDateTime(DATECODE_End))
  {
#ifdef DEBUGOPERATORS
    printf("  datetimeend different\n");
#endif
    return(false);
  }
  
  if (recData != todo2.recData)
  {
#ifdef DEBUGOPERATORS
    printf("  recdata different\n");
#endif
    return(false);
  }
  
  if (todo2.path          != path)
  {
#ifdef DEBUGOPERATORS
    printf("  path different: 1*%s* 2*%s*\n", path.c_str(), todo2.path.c_str());
#endif
    return(false);
  }
  if (todo2.uID           != uID)
  {
#ifdef DEBUGOPERATORS
    printf("   id different\n");
#endif
    return(false);
  }
  if (todo2.uSyncID       != uSyncID)
  {
#ifdef DEBUGOPERATORS
    printf("   syncid different\n");
#endif
    return(false);
  }
  if (todo2.summary       != summary)
  {
#ifdef DEBUGOPERATORS
    printf("   summary different\n");
#endif
    return(false);
  }
  if (todo2.uPriority     != uPriority)
  {
#ifdef DEBUGOPERATORS
    printf("   priority different\n");
#endif
    return(false);
  }
  
  return(true);
}

bool ToDoListToDo::operator != (ToDoListToDo todo2) 
{
  return(!(*this == todo2));
}

int ToDoListToDo::writeContent(std::ostream&  ofile,
                               bool           fExport,
                               CryptMan*      crypt,
                               bool           fIncludeYourself,
                               bool           fIncludeContent)
{
  char* pDescrToSave;
  int   nDescrLenLocal;
  int   nFlagsToSave;
  bool  fGotNewBuffer;

  sanitizeTimeThings();
  ofile << "t\ns" << summary << "\n";

  if (descr.GetDataToSave(pDescrToSave, nDescrLenLocal, nFlagsToSave,
                          fGotNewBuffer,
                          crypt))
    return(-1);
    
  if (nFlagsToSave != 0)
  {
    ofile.setf(std::ios::hex, std::ios::basefield);
    ofile << "F" << nFlagsToSave << "\n";
    ofile.setf(std::ios::dec, std::ios::basefield);
  }
  
  if (!fExport)
    ofile << "I" << uSyncID << "\n";
  
  if (uPriority != INITIALPRIORITY)
    ofile << "p" << uPriority << "\n";
  if (showTimeAs != SHOW_AS_FREE)  
    ofile <<  "S" << (int)showTimeAs << "\n";
  
  if (ISDATESET(dateTimeBegin) || ISTIMESET(dateTimeBegin))
    ofile << "m" << (unsigned int)dateTimeBegin << "\n";
  
  if (ISDATESET(dateTimeEnd) || ISTIMESET(dateTimeEnd))
    ofile << "n" << (unsigned int)dateTimeEnd << "\n";
  
  if (recData.nType != RECTYPE_NONE)
  {
    ofile << "1" << recData.nType << "\n";
    if (recData.nInterval != 1)
      ofile << "2" << recData.nInterval << "\n";
    if (ISDATESET(recData.begin))
      ofile << "3" << (unsigned int)recData.begin << "\n";
    if (ISDATESET(recData.end))
      ofile << "4" << (unsigned int)recData.end << "\n";
  }
  
  if (isRunning())
    ofile << "R\n";
  unsigned int uForMaxVal = workingTimes.size();
  for (unsigned int i=0;i<uForMaxVal; i++)
    ofile << "r" << (long int)  workingTimes[i].begin << 
    "," << (long int)  workingTimes[i].end << "\n";
  
  if (isCompleted() != INITIALISC)
  {
    if (isCompleted())
      ofile << "f" << (long int) completed.time << "\n";
    else
      ofile << "f\n";
  }
  
  if (pDescrToSave != (char*)0)
  {
    if (nDescrLenLocal)
    {
      if (descr.DescrIsBinary(nFlagsToSave))
      {
        ofile << "d" << nDescrLenLocal << "\n";
        WriteBinASCII(ofile, (const unsigned char*)pDescrToSave, nDescrLenLocal);
      }
      else
      {
        nDescrLenLocal--; // trailing 0x00
        ofile << "d" << nDescrLenLocal << "\n";
        ofile.write(pDescrToSave, nDescrLenLocal);
      }
      ofile.flush();
      ofile << "\n";
    }
  }
  
  if (fGotNewBuffer)
    free(pDescrToSave);
  
  ofile << ">\n";
  if (!fExport)
    hasBeenSaved(true);
  
  return(0);
}

int  ToDoListToDo::readContent(std::ifstream&         ifile,
			       unsigned int&          uNrActiveJobs,
			       unsigned int&          uSyncIDMax,
                               T_Calendar&            cal) 
{
  T_workingTimes  tmp;
  char*           line;
  std::string     lines;
  int             retCode = 0;
  
  line = (char*)malloc(500);
  
  do
  {
    ifile.getline(line, 499);
    
    switch (line[0])
    {
      case 's':
        summary = line+1;
        break;
      case 'R':
        fRunning = true;
        uNrActiveJobs++;
        break;          
      case 'r':
        lines = line+1;
        tmp.begin = (time_t)atoi(lines.substr(0,lines.find(",")).c_str());
        lines.replace(0, lines.find(",")+1,"");
        tmp.end = (time_t)atoi(lines.c_str());
        workingTimes.push_back(tmp);
        break;
      case 'f':
        if (line[1] != '\n')
        {
          completed.fDone = true;
          completed.time = (time_t)atoi(line+1);
        }
        else
          completed.fDone = false;
        break;
      case 'd':
        {
          unsigned int dlen;

          dlen = atoi(line+1);
          
          if (descr.ReadFromFile(dlen, ifile))
          {
            free(line);
            return(-1);
          }
        }
        break;
      case 'p':
        uPriority = atoi(line+1);
        if (uPriority > PRIORITY_TODOMAX && 
            uPriority != PRIORITY_APPOINTMENT && 
            uPriority != PRIORITY_NOTE)
          uPriority = 1;
        break;
      case 'm':
        dateTimeBegin = (t_datetime)atoi(line+1);
        break;
      case 'n':
        dateTimeEnd = (t_datetime)atoi(line+1);
        break;
      case 'I':
        uSyncID = atoi(line+1);
        break;
      case 'F':
        {
          char* ptr;
          descr.SetFlagsFromFile(strtol(line+1, &ptr, 16));
        }
        break;
      case 'S':
        showTimeAs = (int)atoi(line+1);
        break;
      case '1':
        recData.nType= atoi(line+1);
        break;
      case '2':
        recData.nInterval= atoi(line+1);
        break;
      case '3':
        recData.begin= atoi(line+1);
        break;
      case '4':
        recData.end= atoi(line+1);
        break;
    }                 
  }
  while (line[0] != '>' && !ifile.eof());
  
  if (line[0] != '>')
    retCode = -1;
  
  free(line);
  
  putIntoCalList(cal);
  
  if (uSyncID == 0)
    uSyncID = ++uSyncIDMax;
  else
  {
    if (uSyncIDMax < uSyncID)
      uSyncIDMax = uSyncID;
    hasBeenSaved(true);
  }
  
  return(retCode);
}

bool ToDoListToDo::isCompleted() 
{
  return(completed.fDone);
}

void ToDoListToDo::setViewData(T_viewDataItem& tmp,
                               CryptMan*       crypt)
{
  tmp.fGroup    = false;
  tmp.uPriority = uPriority;
  
  tmp.summary = new std::string();
  
  if (summary.length())
    tmp.summary->assign(summary);
  else
  {
    const char* pDescr;
    int   nDescrState = GetDescription(pDescr, crypt);

    if (nDescrState == 1)
      tmp.summary->assign("<encrypted>");
    else if (pDescr != (char*)0)
    {
      if (strlen(pDescr))
      {
        tmp.summary->assign(pDescr);
        unsigned int uForMaxVal = tmp.summary->length();
        for (unsigned int n=0; n<uForMaxVal; n++)
        {
          if (tmp.summary->at(n) == '\n')
            tmp.summary->at(n) = ' ';
          if (tmp.summary->at(n) == '\t')
            tmp.summary->at(n) = ' ';
        }
        while (tmp.summary->at(0) == ' ' && tmp.summary->length())
          tmp.summary->replace(0, 1, "");
        if (tmp.summary->length() > 24)
        {
          tmp.summary->replace(25, tmp.summary->length()-25, "");
          *(tmp.summary) += "...";
        }
      }
      else
        tmp.summary->assign("");
    }
    else
      tmp.summary->assign("");
  }
  
  tmp.fDone     = completed.fDone;
  if (uPriority <= PRIORITY_TODOMAX)
  {
    tmp.sortDate  = dateTimeEnd;
    tmp.otherDate = dateTimeBegin;
  }
  else
  {
    tmp.sortDate  = dateTimeBegin;
    tmp.otherDate = dateTimeEnd;
  }
}

int ToDoListToDo::setFollowingRecurrenceDate(t_datetime& dTB, t_datetime& dTE) 
{
  if (recData.nType != RECTYPE_NONE)
  {
    int         datediff = 0;
    t_datetime  dt1, dt2, dt;
    int         h = 0, m = 0;
    bool        fTime = false;
    
    
    // which Date to move?
    if (uPriority < PRIORITY_TODOMAX)
    {
      dt1 = dTE;
      dt2 = dTB;
    }
    else
    {
      dt1 = dTB;
      dt2 = dTE;
    }      
    
    // if the other date is set, too, what's the difference to
    // the important date?
    // keep time in memory
    if (ISDATESET(dt1) && ISDATESET(dt2))
    {
      datediff = getDateDiff(dt2, dt1);
      if (ISTIMESET(dt2))
      {
        fTime = true;
        h = GETHOUR(dt2);
        m = GETMINUTE(dt2);
      }
    }
    
    // move the date
    dt = dt1;
    switch(recData.nType)
    {
      case RECTYPE_DAILY:
        moveDate(dt, recData.nInterval);
        break;
      case RECTYPE_DAILYRESET:
        {
          dt = getTodayDate(dt);
          moveDate(dt, recData.nInterval);
        }
        break;
      case RECTYPE_WEEKLY:
        moveDate(dt, recData.nInterval*7);
        break;
      case RECTYPE_MONTHLYDAY:
        {
          int dow, fdow;
          int day,month,year;
          int n;
          
          day   = GETDAY(dt);
          month = GETMONTH(dt);
          year  = GETYEAR(dt);
          
          dow = Wochentag(day, month, year);
          n   = (day-1) / 7;
          
          do 
          {     
            month += recData.nInterval;
            while (month > 12)
            {
              month-= 12;
              year++;
            }
            fdow = Wochentag(1, month, year);
            day = dow + 1 - fdow;
            if (day < 1)
              day+= 7;
            day+= n*7;
          }
          while (day > MaxTag(month, year));
          SETDATE(dt, year, month, day);                            
        }
        break;
      case RECTYPE_MONTHLYDATE:
        {
          int day,month,year;
          day   = GETDAY(dt);
          month = GETMONTH(dt);
          year  = GETYEAR(dt);
          do 
          {
            month += recData.nInterval;
            while (month > 12)
            {
              month-= 12;
              year++;
            }
          }
          while (day > MaxTag(month, year));
          SETDATE(dt, year, month, day);
        }
        break;
      case RECTYPE_YEARLY:
        {
          int day,month,year;
          day   = GETDAY(dt);
          month = GETMONTH(dt);
          year  = GETYEAR(dt);
          do 
          {
            year++;
          }
          while (day == 29 && month == 2 && 
                 (year % 400 != 0) && (year % 4 != 0 || year % 100 == 0));
          SETDATE(dt, year, month, day);
        }
        break;
    }
    
    // check...
    if (!ISDATESET(recData.end) || (ISDATESET(recData.end) && dt <= recData.end))
    {
      dt1    = dt;
      
      // reconstruct the other date
      if (ISDATESET(dTB) && ISDATESET(dTE))
      {
        dt2 = dt1;
        moveDate(dt2, datediff);
        if (fTime)
          SETTIME(dt2, h, m);
      }
      
      // remap
      if (uPriority < PRIORITY_TODOMAX)
      {
        dTE   = dt1;
        dTB   = dt2;
      }
      else
      {
        dTE   = dt2;
        dTB   = dt1;
      }      
      return(1);
    }      
    else
      return(0);
  }
  else 
    return(0);
}

int  ToDoListToDo::setCompleted(T_Calendar&    cal) 
{
  // nur todos kann man completed setzen...
  if ((uPriority <= PRIORITY_TODOMAX) && (!completed.fDone))
  {
    int code = SETCOMPLCODE_OK;
    
    removeFromCalLists(cal);
    
    if (recData.nType != RECTYPE_NONE && ISDATESET(recData.begin))
    {
      t_datetime dt;
      
      dt = getTodayDate(INITIAL_DATETIME);
      if (dt < recData.begin)
        return(SETCOMPLCODE_CURDATE_BEFORE_RECSTART);
    }    
    if (setFollowingRecurrenceDate(dateTimeBegin, dateTimeEnd))       
      code = SETCOMPLCODE_OK_RECTODO;        
    else
    {
      completed.fDone = true;
      time(&(completed.time));
    }
    
    sanitizeTimeThings();
    putIntoCalList(cal);
    
    setRunning(false);
    return(code);
  }
  else    
    return(SETCOMPLCODE_ERROR);
}

double ToDoListToDo::getTimeSum() 
{
  double       dTmp       = 0;
  unsigned int uForMaxVal = workingTimes.size();
  
  if (isRunning())
    uForMaxVal--;
  
  for (unsigned int i=0;i<uForMaxVal; i++)
    dTmp += difftime(workingTimes[i].end, workingTimes[i].begin);
  
  return(dTmp);
}

int ToDoListToDo::setRunning(bool fR) 
{
  if (fR != fRunning)
  {
    T_workingTimes  tmp;
    if (fR)
    {
      time(&(tmp.begin));
      workingTimes.push_back(tmp);
    }
    else	     
      time(&(workingTimes[workingTimes.size()-1].end));	   
    fRunning = fR;
    return(0);
  }
  else
    return(-1);       
}

int ToDoListToDo::clearCompleted() 
{
  if ((uPriority <= PRIORITY_TODOMAX) && (completed.fDone))
  {
    completed.fDone = false;
    return(SETCOMPLCODE_OK);
  }
  else
    return(SETCOMPLCODE_ERROR);
}

bool ToDoListToDo::containsText(std::string toS,
                                CryptMan*   crypt) 
{
  std::string str1, str2;
  char*  str3;
  int    len;
  
  str1 = strU(toS);
  str2 = strU(getSummary());
  if (str2.find(str1) != std::string::npos)
    return(true);
  
  const char* pDescr;
  int         nDescrState = GetDescription(pDescr, crypt);
  if (nDescrState == 0 && pDescr != (char*)0)
  {
    len = strlen(pDescr);
    str3 = (char*)malloc(len+1);
    for (int n=0; n<=len; n++)
      str3[n] = toupper(pDescr[n]);
    if (strstr(str3, str1.c_str()) != (char*)0)
    {
      free(str3);
      return(true);
    }
    free(str3);
  }
  
  return(false);
}

void ToDoListToDo::addTime(time_t begin,
			   time_t end) 
{
  T_workingTimes tmp;
  
  tmp.begin = begin;
  tmp.end   = end;
  workingTimes.push_back(tmp);
}

#ifdef MUPOSYNC
bool ToDoListToDo::checkDiff(ToDoListItem* pOthersRoot) 
{
  bool            fTmp;
  ToDoListToDo*   pItem;
    
  pItem = (ToDoListToDo*)((ToDoListGroup*)pOthersRoot)->getPtrToItemWithSyncID(uSyncID, fTmp);
  
  if (pItem == (ToDoListToDo*)0)
  {
# ifdef VERBOSE	  
    std::cout << "     " << uSyncID << " is new\n";
# endif
    uSyncFlag |= DIFF_NEW;
  }
  else
  {
    {
      T_recData tmp;
      pItem->getRecData(tmp);
      if ((tmp.nType     != recData.nType)     ||
          (tmp.nInterval != recData.nInterval) ||
          (tmp.begin     != recData.begin)     ||
          (tmp.end       != recData.end))
        uSyncFlag |= DIFF_TIMETHINGS;
    }
    
    if (pItem->getDateTime(DATECODE_End) != getDateTime(DATECODE_End))
      uSyncFlag |= DIFF_TIMETHINGS;
    
    if (pItem->getDateTime(DATECODE_Begin) != getDateTime(DATECODE_Begin))
      uSyncFlag |= DIFF_TIMETHINGS;
    
    if (pItem->getShowTimeAs() != getShowTimeAs())
      uSyncFlag |= DIFF_TIMETHINGS;
    
    if (pItem->getSummary() != getSummary())
      uSyncFlag |= DIFF_SUMMARY;
    if (pItem->getPriority() != getPriority())
      uSyncFlag |= DIFF_PRIORITY;
    if (pItem->isCompleted() != isCompleted())
      uSyncFlag |= DIFF_COMPLETED;
    if (pItem->descr != descr)
      uSyncFlag |= DIFF_DESCRIPTION;
    
    if (pItem->isRunning() != isRunning() || pItem->getNrTimes() != getNrTimes())
      uSyncFlag |= DIFF_WTIMES;
    else
    {
      for (unsigned int uCnt= 0; uCnt < getNrTimes(); uCnt++)
        if (pItem->getTimeBegin(uCnt) != getTimeBegin(uCnt) || 
            pItem->getTimeEnd(uCnt)   != getTimeEnd(uCnt))
        {
          uSyncFlag |= DIFF_WTIMES;
          break;
        }
    }
    
    if (uSyncFlag != 0)
    {
# ifdef VERBOSE	  
      std::cout << "     " << uSyncID << " has been changed\n";
# endif
      pItem->setMustBeKept();
    }
  }
  return(uSyncFlag != 0);
}

void ToDoListToDo::clearWTimes() 
{
  while (workingTimes.size())
    workingTimes.pop_back();
}
#endif

void ToDoListToDo::sanitizeTimeThings() 
{
  t_datetime dt;
  bool       fResetRecData = false;
  bool       fHasRecData   = (recData.nType != RECTYPE_NONE);
  
  if (ISDATESET(dateTimeBegin) && ISDATESET(dateTimeEnd))
  {
    // end after begin?
    if (GETDATEONLY(dateTimeBegin) > GETDATEONLY(dateTimeEnd))
    {
      // Die Tage müssen in jedem Fall vertauscht werden. Gehe ich
      // jetzt davon aus, dass auch die Zeiten vertauscht sind?
      // Ja, mache ich mal. Dabei muss ich dann auch nicht beachten, 
      // ob die Zeiten überhaupt gesetzt sind.
      dt            = dateTimeBegin;
      dateTimeBegin = dateTimeEnd;
      dateTimeEnd   = dt;              
    }
    else if ((GETDATEONLY(dateTimeBegin) == GETDATEONLY(dateTimeEnd) &&
              ISTIMESET(dateTimeBegin) && 
              ISTIMESET(dateTimeEnd)   && 
              dateTimeBegin > dateTimeEnd))
    {
      // Gleicher Tag für Start und Ende, aber die Zeiten sind vertauscht.
      dt            = dateTimeBegin;
      dateTimeBegin = dateTimeEnd;
      dateTimeEnd   = dt;        
    }
    
  }
  else if (fHasRecData)
  {
    // there is no range to show
    showTimeAs = SHOW_AS_FREE;
    // 
    fResetRecData = true;
#ifdef DEBUG_RECRESET
    printf("Recurrence reset (1): no range [");
#endif
  }
  
  // Wiederholungsende und -beginn sollten in der richtigen Reihenfolge 
  // liegen:
  if (ISDATESET(recData.begin) && 
      ISDATESET(recData.end)   &&
      GETDATEONLY(recData.begin) > GETDATEONLY(recData.end))
  {
    dt            = recData.begin;
    recData.begin = recData.end;
    recData.end   = dt;
  }
  
  if (uPriority <= PRIORITY_TODOMAX)
  {
    // Bei einem todo ist das Enddatum der wichtigste Termin. Wenn
    // es den nicht gibt, kann man alles zurücksetzen.
    if (!ISDATESET(dateTimeEnd))
    {
      dateTimeBegin = INITIAL_DATETIME;
      dateTimeEnd   = INITIAL_DATETIME;
      if (fHasRecData)
      {
        fResetRecData = true;
#ifdef DEBUG_RECRESET
        printf("Recurrence reset (2): no end date for todo [");
#endif
      }
    }
    else
    {
      // Wenn es kein Datum gibt, auch die Zeit zurücksetzen:
      if (!ISDATESET(dateTimeBegin))
        dateTimeBegin = INITIAL_DATETIME;
      
      // Wiederholung zurücksetzen, wenn der Termin nicht innerhalb des 
      // Wiederholungszeitraums liegt.
      if (((GETDATEONLY(dateTimeEnd) < GETDATEONLY(recData.begin) && ISDATESET(recData.begin)) ||
           (GETDATEONLY(dateTimeEnd) > GETDATEONLY(recData.end)   && ISDATESET(recData.end))) && fHasRecData)
      {
        fResetRecData = true;
#ifdef DEBUG_RECRESET
        printf("Recurrence reset (3): date not in range [");
#endif
      }
    }
  }
  else
  {
    // Einen Termin kann man nicht zurücksetzen, also ist RECTYPE_DAILYRESET nicht erlaubt.
    if (recData.nType == RECTYPE_DAILYRESET)
      recData.nType = RECTYPE_DAILY;
    
    if (!ISDATESET(dateTimeBegin))
    {
      // Das Anfangsdatum ist nicht gesetzt, bei einem Termin ist das aber
      // das wichtigste. Also kann man alles andere auch zurücksetzen.
      dateTimeBegin = INITIAL_DATETIME;
      dateTimeEnd   = INITIAL_DATETIME;
      fResetRecData = true;
#ifdef DEBUG_RECRESET
      if (fHasRecData)
      {
        printf("Recurrence reset (4): type date without start date/time [");
      }
#endif
    }
    else
    {
      if (!ISDATESET(dateTimeEnd))
        dateTimeEnd = INITIAL_DATETIME;
      // Setzt Wiederholung zurück, wenn mindestens einer der beiden Fälle eintritt:
      //   -Wiederholungs-Start-Datum gesetzt, aber das Datum, an dem
      //    die Verabredung beginnen soll, liegt davor.
      //   -Wiederholungs-Ende-Datum gesetzt, aber das Datum, an dem
      //    die Verabredung beginnen soll, liegt danach.
      if (((GETDATEONLY(dateTimeBegin) < GETDATEONLY(recData.begin) && ISDATESET(recData.begin)) ||
           (GETDATEONLY(dateTimeBegin) > GETDATEONLY(recData.end)   && ISDATESET(recData.end))) && fHasRecData)
      {
        fResetRecData = true;
#ifdef DEBUG_RECRESET
        printf("Recurrence reset (5) [");
#endif
      }
    }
  }
  
  if (fResetRecData)
  {
    recData.begin     = INITIAL_DATETIME;
    recData.end       = INITIAL_DATETIME;
    recData.nInterval = 1;
    recData.nType     = RECTYPE_NONE;
  }
}

void ToDoListToDo::setDescription(const char* pd)
{
  descr.SetNewChar(pd);
}

void ToDoListToDo::setDescription(const DescrData* aDescr)
{
  descr.CopyFrom(aDescr);
}

void ToDoListToDo::putIntoCalList(T_Calendar& cal) 
{
  std::vector<T_DatedCalListItem>*  pList;
  
  if (ISDATESET(dateTimeBegin) 
      || ISDATESET(dateTimeEnd))
  {
    
    //      std::cout << "void ToDoListToDo::putIntoCalList() for " << summary << "\n";      
            
    if (recData.nType == RECTYPE_NONE || recData.nType == RECTYPE_DAILYRESET)
    {
      T_DatedCalListItem  it;
      unsigned int        uCnt;
      
      switch (uPriority)
      {
        case PRIORITY_NOTE:
          it.dt = dateTimeBegin;
          pList = &cal.dateNotes;
          break;
        case PRIORITY_APPOINTMENT:
          it.dt = dateTimeBegin;
          pList = &cal.dateApps;
          break;
        default:
          it.dt = dateTimeEnd;
          pList = &cal.dateTodos;
          break;
      }          
      it.item = this;
      uCnt = 0;
      while (uCnt < pList->size() && (*pList)[uCnt].dt < it.dt)
        uCnt++;
      pList->insert(pList->begin()+uCnt, it);          
    }
    else
    {
      switch (uPriority)
      {
        case PRIORITY_NOTE:
          cal.recNotes.push_back(this);
          break;
        case PRIORITY_APPOINTMENT:
          cal.recApps.push_back(this);
          break;
        default:
          cal.recTodos.push_back(this);
          break;
      }          
    }
  }
}

void ToDoListToDo::removeFromCalLists(T_Calendar&    cal) 
{
  if (ISDATESET(dateTimeBegin) 
      || ISDATESET(dateTimeEnd))
  {
    unsigned int                 uCnt;      
    
    
    //      std::cout << "void ToDoListToDo::removeFromCalLists() for " << summary << "\n";
      
    
    
    if (recData.nType == RECTYPE_NONE || recData.nType == RECTYPE_DAILYRESET)
    {
      std::vector<T_DatedCalListItem>*  pList;
      
      switch (uPriority)
      {
        case PRIORITY_NOTE:
          pList = &cal.dateNotes;
          break;
        case PRIORITY_APPOINTMENT:
          pList = &cal.dateApps;
          break;
        default:
          pList = &cal.dateTodos;
          break;
      }          
      uCnt = 0;
      while (uCnt < pList->size() && (*pList)[uCnt].item != this)
        uCnt++;
      if ((*pList)[uCnt].item == this && pList->size())
        pList->erase(pList->begin()+uCnt);
    }
    else
    { 
      std::vector<ToDoListToDo*>*  pList;
      
      switch (uPriority)
      {
        case PRIORITY_NOTE:
          pList = &cal.recNotes;
          break;
        case PRIORITY_APPOINTMENT:
          pList = &cal.recApps;
          break;
        default:
          pList = &cal.recTodos;
          break;
      }         
      uCnt = 0;
      while (uCnt < pList->size() && (*pList)[uCnt] != this)
        uCnt++;
      if ((*pList)[uCnt] == this && pList->size())
        pList->erase(pList->begin()+uCnt);
    }
  }
}

void ToDoListToDo::setCompletedDirect(T_completed someCompl) 
{
  completed = someCompl;
};

int  ToDoListToDo::getShowTimeAs()
{
  return(showTimeAs); 
}

time_t ToDoListToDo::getCompletedTime()
{
  return(completed.time); 
}

void ToDoListToDo::setShowTimeAs(int n, T_Calendar&    cal)
{ 
  showTimeAs = n;
  if (uPriority > PRIORITY_TODOMAX)
  {
    removeFromCalLists(cal);
    if (showTimeAs == SHOW_AS_FREE)
      uPriority = PRIORITY_NOTE;
    else
      uPriority = PRIORITY_APPOINTMENT;
    sanitizeTimeThings();
    putIntoCalList(cal);
  }
}

T_completed ToDoListToDo::getCompletedDirect() 
{
  return(completed);
}

void ToDoListToDo::setDate(int            n,  
                           int            year,
                           int            month, 
                           int            day,
                           T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  if (n == DATECODE_End)
    SETDATE(dateTimeEnd, year, month, day);
  else
    SETDATE(dateTimeBegin, year, month, day);
  sanitizeTimeThings();
  putIntoCalList(cal);
}

void ToDoListToDo::setDateDontCheck(int            n,  
                                    int            year,
                                    int            month, 
                                    int            day,
                                    T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  if (n == DATECODE_End)
    SETDATE(dateTimeEnd, year, month, day);
  else
    SETDATE(dateTimeBegin, year, month, day);
  putIntoCalList(cal);
}


t_datetime ToDoListToDo::getDateTime(int n) 
{
  if (n == DATECODE_End)
    return(dateTimeEnd);
  else
    return(dateTimeBegin);
}

void ToDoListToDo::clearDate(int            n,
                             T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  if (n == DATECODE_End)
    CLEARDATE(dateTimeEnd);
  else
    CLEARDATE(dateTimeBegin);       
  sanitizeTimeThings();
  putIntoCalList(cal);
}

void ToDoListToDo::setTime(int            n,
                           int            hour, 
                           int            min,
                           T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  if (n == DATECODE_End)
    SETTIME(dateTimeEnd, hour, min);
  else
    SETTIME(dateTimeBegin, hour, min);
  sanitizeTimeThings();
  putIntoCalList(cal);
}

void ToDoListToDo::setTimeDontCheck(int            n,
                                    int            hour, 
                                    int            min,
                                    T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  if (n == DATECODE_End)
    SETTIME(dateTimeEnd, hour, min);
  else
    SETTIME(dateTimeBegin, hour, min);
  putIntoCalList(cal);
}

void ToDoListToDo::clearTime(int            n,
                             T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  if (n == DATECODE_End)
    CLEARTIME(dateTimeEnd);
  else
    CLEARTIME(dateTimeBegin);
  sanitizeTimeThings();
  putIntoCalList(cal);
}

void ToDoListToDo::setPriority(unsigned int   n,
                               T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  uPriority = n;          
  sanitizeTimeThings();
  putIntoCalList(cal);
}

bool ToDoListToDo::isRunning() 
{
  return(fRunning); 
}

void ToDoListToDo::setRunningDirect(bool fNew)
{
  fRunning = fNew; 
}

unsigned int ToDoListToDo::getNrTimes() 
{
  return(workingTimes.size());
}

time_t ToDoListToDo::getTimeBegin(unsigned int n) 
{
  return(workingTimes[n].begin);
}

time_t ToDoListToDo::getTimeEnd(unsigned int n) 
{
  return(workingTimes[n].end);
}

void ToDoListToDo::setRecData(T_recData      recdata,
                              T_Calendar&    cal) 
{
  removeFromCalLists(cal);
  recData.nType     = recdata.nType;
  recData.nInterval = recdata.nInterval;
  recData.begin     = recdata.begin;
  recData.end       = recdata.end;
  sanitizeTimeThings();
  putIntoCalList(cal);
}

void ToDoListToDo::getRecData(T_recData& recdata) 
{
  recdata.nType     = recData.nType;
  recdata.nInterval = recData.nInterval;
  recdata.begin     = recData.begin;
  recdata.end       = recData.end;
}

int ToDoListToDo::GetDescription(const char*& PtrToDescr,
                                 CryptMan*    crypt)
{
  return(descr.GetDescription(PtrToDescr, crypt));
}

void ToDoListToDo::SetSaveCrypt(bool fCrypt)
{
  descr.SetSaveCrypt(fCrypt);
}

