// -*- mode: c; mode: fold -*-

#include <string>
#include <cstring>
#include <FL/Fl.H>
#include <FL/Fl_Repeat_Button.H>

#include "recurrence.h"
#include "config.h"
#include "tools.h"
#include "dateselect.h"

#define WINWIDTH  160
#define WINHEIGHT 160

extern const char* dayName[];


static void cb_Date    (Fl_Menu_ * o, void * code) /*{{{*/
{
  ((RecurrenceSelect*)(o->parent()->user_data()))->cb_Date_i(o, code);
}
/*}}}*/

Fl_Menu_Item menu_EndDate[] = { /*{{{*/
    {"none",    0, (Fl_Callback *) cb_Date,    (void*)DATECODE_EndNone,   0, 0, 0, TEXTSIZE1, 0},
    {"select",  0, (Fl_Callback *) cb_Date,    (void*)DATECODE_EndSelect, 0, 0, 0, TEXTSIZE1, 0},
    {0},
    {0}
};
/*}}}*/

Fl_Menu_Item menu_BegDate[] = { /*{{{*/
    {"none",    0, (Fl_Callback *) cb_Date,    (void*)DATECODE_BeginNone,   0, 0, 0, TEXTSIZE1, 0},
    {"select",  0, (Fl_Callback *) cb_Date,    (void*)DATECODE_BeginSelect, 0, 0, 0, TEXTSIZE1, 0},
    {0},                                                                           
    {0}
};
/*}}}*/

static void cb_recType(Fl_Choice* o, void* v) /*{{{*/
{
  ((RecurrenceSelect*)(o->parent()->user_data()))->cb_recType_i(o,v);
}
/*}}}*/

Fl_Menu_Item menu_recType[] = { /*{{{*/
    {"none",               0, (Fl_Callback *) cb_recType,   (void*)RECTYPE_NONE,        0, 0, 0, TEXTSIZE1, 0},
    {"daily",              0, (Fl_Callback *) cb_recType,   (void*)RECTYPE_DAILY,       0, 0, 0, TEXTSIZE1, 0},
    {"daily, resetable",   0, (Fl_Callback *) cb_recType,   (void*)RECTYPE_DAILYRESET,  0, 0, 0, TEXTSIZE1, 0},
    {"weekly",             0, (Fl_Callback *) cb_recType,   (void*)RECTYPE_WEEKLY,      0, 0, 0, TEXTSIZE1, 0},
    {"monthly, by day",    0, (Fl_Callback *) cb_recType,   (void*)RECTYPE_MONTHLYDAY,  0, 0, 0, TEXTSIZE1, 0},
    {"monthly, by date",   0, (Fl_Callback *) cb_recType,   (void*)RECTYPE_MONTHLYDATE, 0, 0, 0, TEXTSIZE1, 0},
    {"yearly",             0, (Fl_Callback *) cb_recType,   (void*)RECTYPE_YEARLY,      0, 0, 0, TEXTSIZE1, 0},
    {0},
    {0}
};
/*}}}*/


RecurrenceSelect::RecurrenceSelect(T_recData* recdata,
                                   int nday,
                                   int nmonth,
                                   int ndow,
                                   bool ftodo) /*{{{*/
{
  pRecData     = recdata;
  
  nDay   = nday;
  nMonth = nmonth;
  nDOW   = ndow;
  fToDo  = ftodo;
  
  Fl_Window* w;
    {
      Fl_Window* o = win = new Fl_Window(WINWIDTH*SIZEMUL,WINHEIGHT*SIZEMUL,"RecurrenceSelector");
      w = o;
      o->callback((Fl_Callback*)cb_Window, (void*)(this));
      w->hotspot(o);    
        {  
          recTypeBox = new Fl_Choice (2*SIZEMUL, 2*SIZEMUL, (WINWIDTH-2)*SIZEMUL, 15*SIZEMUL, "");
          recTypeBox->menu (menu_recType);
          recTypeBox->labelsize (TEXTSIZE1);
          recTypeBox->textsize (TEXTSIZE1);
          recTypeBox->align(FL_ALIGN_LEFT);
        }
        {  
          Fl_Box* o = new Fl_Box( 0*SIZEMUL,18*SIZEMUL, 50*SIZEMUL,15*SIZEMUL, "every");
          o->labelsize (TEXTSIZE1);
          o->box (FL_FLAT_BOX);          
        }
      
        {
          Fl_Repeat_Button* o = new Fl_Repeat_Button(50*SIZEMUL,18*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"-");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Int, (void*)-1);
        }
        {  
          Fl_Box* o = intBox=new Fl_Box(63*SIZEMUL,18*SIZEMUL, 38*SIZEMUL,15*SIZEMUL);
          o->labelsize (TEXTSIZE1);
          o->box (FL_FLAT_BOX);          
        }
        {
          Fl_Repeat_Button* o = new Fl_Repeat_Button(101*SIZEMUL,18*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"+");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Int, (void*)1);
        }
      
        {
          Fl_Box* o = timeBox = new Fl_Box( 114*SIZEMUL,18*SIZEMUL, 46*SIZEMUL,15*SIZEMUL);
          o->labelsize (TEXTSIZE1);
          o->box (FL_FLAT_BOX);          
        }
        {  
          Fl_Box* o = explBox = new Fl_Box( 0*SIZEMUL,35*SIZEMUL, WINWIDTH*SIZEMUL,(WINHEIGHT-80)*SIZEMUL);
          o->labelsize (TEXTSIZE1);
          o->box (FL_FLAT_BOX);          
        }
      
      BegDateB= new Fl_Menu_Button (0, (WINHEIGHT-45)*SIZEMUL, 60*SIZEMUL, 15*SIZEMUL, "Begin Date");
      BegDateB->labelsize (TEXTSIZE1);
      BegDateB->textsize (TEXTSIZE1);
      BegDateB->align (FL_ALIGN_INSIDE);
      BegDateB->menu (menu_BegDate);
      
      EndDateB= new Fl_Menu_Button (0, (WINHEIGHT-30)*SIZEMUL, 60*SIZEMUL, 15*SIZEMUL, "End Date");
      EndDateB->labelsize (TEXTSIZE1);
      EndDateB->textsize (TEXTSIZE1);
      EndDateB->align (FL_ALIGN_INSIDE);
      EndDateB->menu (menu_EndDate);


      BegBox = new Fl_Box (60*SIZEMUL, (WINHEIGHT-45)*SIZEMUL, (100)*SIZEMUL, 15*SIZEMUL);
      BegBox->box (FL_FLAT_BOX);
      BegBox->labelfont (0);
      BegBox->labelsize (TEXTSIZE1);
      BegBox->align (68 | FL_ALIGN_INSIDE);

      EndBox = new Fl_Box (60*SIZEMUL, (WINHEIGHT-30)*SIZEMUL, (100)*SIZEMUL, 15*SIZEMUL);
      EndBox->box (FL_FLAT_BOX);
      EndBox->labelfont (0);
      EndBox->labelsize (TEXTSIZE1);
      EndBox->align (68 | FL_ALIGN_INSIDE);

      
      
        {
          Fl_Button* o = doneB   = new Fl_Button(0*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 35*SIZEMUL,15*SIZEMUL,"Done");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Done);
        }
        {
          Fl_Button* o = new Fl_Button( 35*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 35*SIZEMUL,15*SIZEMUL,"Cancel");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Cancel);
        }
      o->set_modal();
      o->end();
    }
  dispInt();
  dispExpl();
  dispDates();
  check_active();
}
/*}}}*/


inline void RecurrenceSelect::cb_Window_i(Fl_Window*, void*) /*{{{*/
{
  win->hide();
}
/*}}}*/

void RecurrenceSelect::cb_Window(Fl_Window* o, void* v)      /*{{{*/
{ 
  ((RecurrenceSelect*)(o->user_data()))->cb_Window_i(o,v);
}
/*}}}*/


void RecurrenceSelect::check_active()      /*{{{*/
{ 
  if (pRecData->nType == RECTYPE_NONE)
    {
      BegDateB->deactivate();
      EndDateB->deactivate();
      pRecData->nInterval = 1;
    }
  else
    {
      BegDateB->activate();
      EndDateB->activate();
    }
}
/*}}}*/


inline void RecurrenceSelect::cb_recType_i(Fl_Choice*, void* inc) /*{{{*/
{
  pRecData->nType = (int)inc;
  if ((int)inc == RECTYPE_NONE)
    {
      pRecData->begin = INITIAL_DATETIME;
      pRecData->end = INITIAL_DATETIME;
      pRecData->nInterval = 1;
    }
  if ((int)inc == RECTYPE_DAILYRESET && !fToDo)
    pRecData->nType = RECTYPE_DAILY;
  check_active();
  dispExpl();
  dispDates();
  dispInt();
}
/*}}}*/



inline void RecurrenceSelect::cb_Date_i(Fl_Menu_*, void* inc) /*{{{*/
{
  int         n;
  int         year, month, day;
  t_datetime  dt;
  DateSelect* dateselect;
  
  n = (int)inc & 0xF0;
  
  switch ((int)inc & 0x0F)
    {
     case DATECODE_None:
      if (n == DATECODE_End)
        pRecData->end   = DATETIME_NODATE | DATETIME_NOTIME;
      else
        pRecData->begin = DATETIME_NODATE | DATETIME_NOTIME;
      break;
      
     case DATECODE_Select:
      if (n == DATECODE_End)
        dt = pRecData->end;
      else
        dt = pRecData->begin;
      if ISDATESET(dt)
        {
          year  = GETYEAR(dt);
          month = GETMONTH(dt);
          day   = GETDAY(dt);
        }
      else        
        day = -1;        
      
      dateselect = new DateSelect(&year, &month, &day);
      dateselect->show();
      while (dateselect->visible())
        Fl::wait();
      
      if (day != -1)
        SETDATE(dt, year, month, day);
      if (n == DATECODE_End)
        pRecData->end = dt;
      else
        pRecData->begin = dt;
           
      break;
    }
  
  dispDates();
}
/*}}}*/




inline void RecurrenceSelect::cb_Int_i(Fl_Repeat_Button*, void* inc) /*{{{*/
{
  if (pRecData->nType != RECTYPE_NONE)
    {
      pRecData->nInterval += (int)inc;
      if (pRecData->nInterval < 1)
        pRecData->nInterval = 1;
      if (pRecData->nInterval > 1000)
        pRecData->nInterval = 1000;
    }
  else
    pRecData->nInterval = 1;
  dispInt();
  dispExpl();
}
/*}}}*/

void RecurrenceSelect::cb_Int(Fl_Repeat_Button* o, void* v)      /*{{{*/
{
  ((RecurrenceSelect*)(o->parent()->user_data()))->cb_Int_i(o,v);
}
/*}}}*/



inline void RecurrenceSelect::cb_Cancel_i(Fl_Button*, void*) /*{{{*/
{
  pRecData->nType = -1;
  win->hide();
}
/*}}}*/

void RecurrenceSelect::cb_Cancel(Fl_Button* o, void* v)      /*{{{*/
{
  ((RecurrenceSelect*)(o->parent()->user_data()))->cb_Cancel_i(o,v);
}
/*}}}*/



inline void RecurrenceSelect::cb_Done_i(Fl_Button*, void*) /*{{{*/
{
  win->hide();
}
/*}}}*/

void RecurrenceSelect::cb_Done(Fl_Button* o, void* v)      /*{{{*/
{
  ((RecurrenceSelect*)(o->parent()->user_data()))->cb_Done_i(o,v);
}
/*}}}*/



void RecurrenceSelect::show() /*{{{*/
{
  win->show();
}
/*}}}*/

void RecurrenceSelect::hide() /*{{{*/
{
  win->hide();
}
/*}}}*/

int RecurrenceSelect::visible() /*{{{*/
{
  return win->visible();
}
/*}}}*/

void RecurrenceSelect::dispExpl() /*{{{*/
{
  std::string tmp = "explanation text\nhopefully longer\nthan one line";
  
  recTypeBox->value(pRecData->nType);
  
  switch (pRecData->nType)
    {
     case RECTYPE_NONE:
      timeBox->label("");
      tmp = "No recurrence.";
      break;
     case RECTYPE_DAILY:
      timeBox->label("day(s)");
      tmp = "Repeat every "
        + itoStr(pRecData->nInterval, ' ', 1)
          + " day(s).";
      break;
     case RECTYPE_DAILYRESET:
      timeBox->label("day(s)");
      tmp = "Repeat every "
        + itoStr(pRecData->nInterval, ' ', 1)
          + " days,\nreset when done.\nUseful if you have to do it\n AT LEAST every n day(s).";
      break;
     case RECTYPE_WEEKLY:
      timeBox->label("week(s)");
      tmp = "Repeat on \nevery "
        + itoStr(pRecData->nInterval, ' ', 1) 
          + " week(s).";
      tmp.insert(10, dayName[nDOW]);
      break;
     case RECTYPE_MONTHLYDAY:
      timeBox->label("month(s)");
      tmp = "Repeat on "+ itoStr(((nDay-1) / 7)+1, ' ', 1)+ ". " + std::string(dayName[nDOW]) + "\nevery "
        + itoStr(pRecData->nInterval, ' ', 1)
          + " month(s).";
      break;
     case RECTYPE_MONTHLYDATE:
      timeBox->label("month(s)");
      tmp = "Repeat on day "+ itoStr(nDay, ' ', 1)+"\nevery " 
        + itoStr(pRecData->nInterval, ' ', 1) 
          + " month(s).";
      break;
     case RECTYPE_YEARLY:
      timeBox->label("year(s)");
      tmp = "Repeat on "+ itoStr(nDay, ' ', 1)+"."+ itoStr(nMonth, ' ', 1)+".\n every "
        + itoStr(pRecData->nInterval, ' ', 1) 
          + " year(s).";
      break;
    }
  timeBox->redraw();      
  strcpy(explStr, tmp.c_str());
    
  explBox->label(explStr);
  explBox->redraw();
}
/*}}}*/

void RecurrenceSelect::dispDates() /*{{{*/
{
  std::string tmp;
  
  if (ISDATESET(pRecData->begin))
    tmp = itoStr(GETDAY(pRecData->begin), '0', 2) + "." 
    + itoStr(GETMONTH(pRecData->begin), '0', 2) + "." 
    + itoStr(GETYEAR(pRecData->begin), '0', 4);
  else 
    tmp = "none";
  strcpy(begStr, tmp.c_str());
  BegBox->label(begStr);
  BegBox->redraw();
  
  if (ISDATESET(pRecData->end))
    tmp = itoStr(GETDAY(pRecData->end), '0', 2) + "." 
    + itoStr(GETMONTH(pRecData->end), '0', 2) + "." 
    + itoStr(GETYEAR(pRecData->end), '0', 4);
  else 
    tmp = "none";
  strcpy(endStr, tmp.c_str());
  EndBox->label(endStr);
  EndBox->redraw();    
}
/*}}}*/

void RecurrenceSelect::dispInt() /*{{{*/
{
  std::string tmp;
  
  tmp = itoStr(pRecData->nInterval, ' ', 1);
  strcpy(intStr, tmp.c_str());
  intBox->label(intStr);
  intBox->redraw();
}
/*}}}*/

