// -*- mode: c; mode: fold -*-

#include <string>
#include <cstring>

#include <FL/Fl_Repeat_Button.H>


#include "timeselect.h"
#include "config.h"
#include "tools.h"

#define WINWIDTH  70
#define WINHEIGHT 70




TimeSelect::TimeSelect(int* hour, 
                       int* minute) /*{{{*/
{
  pnHour   = hour;
  pnMinute = minute;
  
  Fl_Window* w;
    {
      Fl_Window* o = win = new Fl_Window(WINWIDTH*SIZEMUL,WINHEIGHT*SIZEMUL,"TimeSelector");
      w = o;
      o->callback((Fl_Callback*)cb_Window, (void*)(this));
      w->hotspot(o);    
        {  
          Fl_Repeat_Button* o = new Fl_Repeat_Button( 10*SIZEMUL,25*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"+");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Hour, (void*)1);
        }
        {  
          Fl_Repeat_Button* o = new Fl_Repeat_Button( 10*SIZEMUL,38*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"-");
          o->labelsize (TEXTSIZE1);          
          o->callback((Fl_Callback*)cb_Hour, (void*)-1);
        }
        {  
          Fl_Box* o = timeBox = new Fl_Box( 2*SIZEMUL,0*SIZEMUL, 50*SIZEMUL,15*SIZEMUL);
          o->labelsize (TEXTSIZE1);
          o->box (FL_FLAT_BOX);          
        }
        {
          Fl_Repeat_Button* o = new Fl_Repeat_Button( 30*SIZEMUL,25*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"+");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Minute, (void*)10);
        }
        {  
          Fl_Repeat_Button* o = new Fl_Repeat_Button(30*SIZEMUL,38*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"-");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Minute, (void*)-10);
        }
        {
          Fl_Repeat_Button* o = new Fl_Repeat_Button( 43*SIZEMUL,25*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"+");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Minute, (void*)1);
        }
        {  
          Fl_Repeat_Button* o = new Fl_Repeat_Button(43*SIZEMUL,38*SIZEMUL, 13*SIZEMUL,13*SIZEMUL,"-");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Minute, (void*)-1);
        }
        {
          Fl_Button* o = doneB   = new Fl_Button(0*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 35*SIZEMUL,15*SIZEMUL,"Done");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Done);
        }
        {
          Fl_Button* o = new Fl_Button( 35*SIZEMUL,(WINHEIGHT-15)*SIZEMUL, 35*SIZEMUL,15*SIZEMUL,"Cancel");
          o->labelsize (TEXTSIZE1);
          o->callback((Fl_Callback*)cb_Cancel);
        }
      o->set_modal();
      o->end();
    }
  

  dispTime();
}
/*}}}*/

inline void TimeSelect::cb_Window_i(Fl_Window*, void*) /*{{{*/
{
  win->hide();
}
/*}}}*/

void TimeSelect::cb_Window(Fl_Window* o, void* v) { /*{{{*/
  ((TimeSelect*)(o->user_data()))->cb_Window_i(o,v);
}
/*}}}*/


inline void TimeSelect::cb_Hour_i(Fl_Button*, void* inc) /*{{{*/
{
  *pnHour += (int)inc;
  if (*pnHour < 0)
    *pnHour = 23;      
  if (*pnHour > 23)
    *pnHour = 0;
  dispTime();
}
/*}}}*/

void TimeSelect::cb_Hour(Fl_Button* o, void* v) /*{{{*/
{
  ((TimeSelect*)(o->parent()->user_data()))->cb_Hour_i(o,v);
}
/*}}}*/


inline void TimeSelect::cb_Minute_i(Fl_Button*, void* inc) /*{{{*/
{
  *pnMinute += (int)inc;
  if (*pnMinute < 0)
    {
      (*pnHour)--;
      if (*pnHour < 0)
        *pnHour = 23;      
      *pnMinute += 60;
    }
  if (*pnMinute > 59)
    {
      (*pnHour)++;
      if (*pnHour > 23)
        *pnHour = 0;
      *pnMinute -= 60;
    }
  dispTime();
}
/*}}}*/

void TimeSelect::cb_Minute(Fl_Button* o, void* v) /*{{{*/
{
  ((TimeSelect*)(o->parent()->user_data()))->cb_Minute_i(o,v);
}
/*}}}*/


inline void TimeSelect::cb_Cancel_i(Fl_Button*, void*) /*{{{*/
{
  *pnMinute = -1;
  win->hide();
}
/*}}}*/

void TimeSelect::cb_Cancel(Fl_Button* o, void* v) /*{{{*/
{
  ((TimeSelect*)(o->parent()->user_data()))->cb_Cancel_i(o,v);
}
/*}}}*/


inline void TimeSelect::cb_Done_i(Fl_Button*, void*) /*{{{*/
{
  win->hide();
}
/*}}}*/

void TimeSelect::cb_Done(Fl_Button* o, void* v) /*{{{*/
{
  ((TimeSelect*)(o->parent()->user_data()))->cb_Done_i(o,v);
}
/*}}}*/


void TimeSelect::show() /*{{{*/
{
  win->show();
}
/*}}}*/

void TimeSelect::hide() /*{{{*/
{
  win->hide();
}
/*}}}*/

int TimeSelect::visible() /*{{{*/
{
  return win->visible();
}
/*}}}*/

void TimeSelect::dispTime() /*{{{*/
{
  std::string tmp;
  
  tmp = itoStr(*pnHour, ' ', 2) + ":" + itoStr(*pnMinute, '0', 2);
  strcpy(tStr, tmp.c_str());
    
  timeBox->label(tStr);
  timeBox->redraw();
}
/*}}}*/

