#include "twofish.h"

#include <stdlib.h>
#include <string.h>
#include "aes.h"

unsigned char* Twofish::Encrypt(const unsigned char* input, int nInLen, 
                                const unsigned char* key,   int nKeyLen,
                                int& nOutLen,
                                bool fIV)
{
  keyInstance    ki;			/* key information, including tables */
  cipherInstance ci;			/* keeps mode (ECB, CBC) and IV */
  int            mode;
  
  if (fIV)
    mode = MODE_CBC;
  else
    mode = MODE_ECB;

  //todo
  //mode = MODE_ECB;
  
  if (makeKey(&ki, DIR_ENCRYPT, nKeyLen*8, NULL) != TRUE)
    return 0;
  
  if (cipherInit(&ci, mode, NULL) != TRUE)
    return 0;
  
  for (int n=0; n<nKeyLen/4; n++)
    ki.key32[n] = key[n*4+0] | (key[n*4+1] << 8) | (key[n*4+2] << 16) | (key[n*4+3] << 24);
  reKey(&ki);

  int nBlocks = nInLen / (BLOCK_SIZE/8);
  if (nInLen % (BLOCK_SIZE/8))
    nBlocks++;
  
  int nIdxDataStart=0;
  if (fIV)
  {
    nBlocks++;
    nIdxDataStart = BLOCK_SIZE/8;
  }
    
  nOutLen = nBlocks*(BLOCK_SIZE/8);
  
  unsigned char* in  = (unsigned char*)malloc(nOutLen);
  unsigned char* out = (unsigned char*)malloc(nOutLen);
  memcpy(&(in[nIdxDataStart]), input, nInLen);
  FILE* randfile = 0;
#ifdef PAD_DEV_RANDOM
  randfile = fopen("/dev/random", "r");
#endif
  // Block padding with random bytes if possible and random IV if requested.
  // If I can't get random data, I simply don't initialize it at all (which is
  // better than filling in zeroes).
  if (randfile != 0)
  {
    int n;

    // IV
    n = nIdxDataStart;
    while (n)
      n -= fread( &(in[nIdxDataStart-n]), 1, n, randfile );
    
    // block padding
    n = nOutLen-nIdxDataStart-nInLen;
    while (n)
      n -= fread( &(in[nOutLen-n]), 1, n, randfile );
    
    fclose(randfile);
  }
  
  if (fIV)
  {
    // Der IV wird hier quasi doppelt benutzt...war aufgrund der Änderung zu Version1 einfacher.
    memcpy(ci.iv32, in, sizeof(ci.iv32));
  }
  
  /* encrypt the bytes */
  if (blockEncrypt(&ci, &ki, in, nOutLen*8, out) != nOutLen*8)
    return 0;
    
  free(in);
  
  return(out);
}

unsigned char* Twofish::Decrypt(const unsigned char* input, int nInLen, 
                                const unsigned char* key,   int nKeyLen,
                                int& nOutLen,
                                bool fIV)
{
  keyInstance    ki;			/* key information, including tables */
  cipherInstance ci;			/* keeps mode (ECB, CBC) and IV */
  int            mode;
  
  if (fIV)
    mode = MODE_CBC;
  else
    mode = MODE_ECB;
    
  //todo
  //mode = MODE_ECB;

  if (makeKey(&ki, DIR_ENCRYPT, nKeyLen*8, NULL) != TRUE)
    return 0;
  
  if (cipherInit(&ci, mode, NULL) != TRUE)
    return 0;
  
  for (int n=0; n<nKeyLen/4; n++)
    ki.key32[n] = key[n*4+0] | (key[n*4+1] << 8) | (key[n*4+2] << 16) | (key[n*4+3] << 24);
  reKey(&ki);

  int nBlocks = nInLen / (BLOCK_SIZE/8);
  if (nInLen % (BLOCK_SIZE/8))
    nBlocks++;
  
  nOutLen = nBlocks*(BLOCK_SIZE/8);
    
  unsigned char* in  = (unsigned char*)malloc(nOutLen);
  unsigned char* out = (unsigned char*)malloc(nOutLen);
  memcpy(in, input, nInLen);
  for (int n=nInLen; n<nOutLen; n++)
    in[n] = 0;
  
  if (fIV)
  {
    // Der IV wird hier quasi doppelt benutzt...war aufgrund der Änderung zu Version1 einfacher.
    memcpy(ci.iv32, in, sizeof(ci.iv32));
  }
  
  /* decrypt the bytes */
  if (blockDecrypt(&ci, &ki, in, nOutLen*8, out) != nOutLen*8)
    return 0;
    
  free(in);  
  
  if (fIV)
  {
    nBlocks--;
    nOutLen = nBlocks*(BLOCK_SIZE/8);
    unsigned char* tmp  = (unsigned char*)malloc(nOutLen);
    memcpy(tmp, &(out[BLOCK_SIZE/8]), nOutLen);
    free(out);
    out = tmp;
  }
  
  return(out);
}
